/*	File: evaluation_A.cc

	This code checks that A(-0.5+10^{-23}) > A(26.54), see Lemma 4.12.

	Uses: Interval.hpp, involution.cc

	Author: Jordi-Lluis Figueras, Jordi Villadelprat, Warwick Tucker
	Latest edit: Thu June 21 10:25:00 CET 2012
*/


#include<stdio.h>
#include<time.h>
#include<math.h>
#include<stdlib.h>
#include "Interval.hpp"
#include "involution.cc"

#define DIGITSPRINT 40

int A(const INTERVAL& x, INTERVAL& res);

int main(int argc, char* argv[])
{
	if(argc < 4)
	{
		printf("Usage:\n\t[1]precision of the rigorous computations\n\t[2]Lower bound x\n\t[3]Upper bound x\n\n");
		printf("Execution:\n./evaluation_A.x 256 -0.49999999999999999999999 -0.49999999999999999999999\n\n");
		exit(1);
	}
	
	int prec = atoi(argv[1]);
	mpfr_t xmin, xmax;

	mpfr_set_default_prec(prec);

	INTERVAL X, RES;

	mpfr_init(xmin);
	mpfr_init(xmax);

	mpfr_set_str(xmin, argv[2], 10, GMP_RNDN);
	mpfr_set_str(xmax, argv[3], 10, GMP_RNDN);
	
	X.setfull(xmin, xmax);

	printf("x=\n");
	mpfi_out_str(stdout, 10, DIGITSPRINT, X.x);
	printf("\n");

	A(X, RES);

	printf("A(x)=\n");
	mpfi_out_str(stdout, 10, DIGITSPRINT, RES.x);
	printf("\n");
	printf("\n");
	
	mpfr_set_str(xmin, "26.54", 10, GMP_RNDN);
	mpfr_set_str(xmax, "26.54", 10, GMP_RNDN);
	
	X.setfull(xmin, xmax);

	A(X, RES);

	printf("A(26.54)=\n");
	mpfi_out_str(stdout, 10, DIGITSPRINT, RES.x);
	printf("\n");

	return 0;
}

int A(const INTERVAL& x, INTERVAL& res)
{
	res = (1.-exp(-2.*x)*(2.*x+1.))*0.5;
	return 0;
}
