/*	File: det_mpfi_near0.cc

	This code checks that the Wronskian W[l0, l1, l2] does not vanish in the interval 
	I_3=(-0.1, 0), see Lemma 4.9.

	Uses: Interval.hpp, involution.cc

	Author: Jordi-Lluis Figueras, Jordi Villadelprat, Warwick Tucker
	Latest edit: Thu June 21 10:25:00 CET 2012
*/

#include<stdio.h>
#include<time.h>
#include<math.h>
#include<stdlib.h>
#include "Interval.hpp"
#include "involution.cc"

#define DIGITSPRINT 40

int det_near0(const INTERVAL& x, INTERVAL& res);
void polynomial_determinantnear0(INTERVAL *coef, const INTERVAL& a, const INTERVAL& b);

INTERVAL *coef;

int main(int argc, char* argv[])
{
	if(argc < 2)
	{
		printf("Usage:\n\t[1]Precision of the rigorous computations\n\n");
		printf("Execution:\n./det_mpfi_near0 256 \n\n");
		exit(1);
	}
	
	int flagevaluation, flagfirstevaluation = 0;
	int prec = atoi(argv[1]), i;
	mpfr_t xmin, xmax, x, deltax, tolstep, aux;

	mpfr_set_default_prec(prec);

	INTERVAL X, Xinit, Xaux, Xaux2, PX, GoodXleft, GoodXright, z, CorrectX, CorrectPX;

	mpfr_init(xmin);
	mpfr_init(xmax);
	mpfr_init(x);
	mpfr_init(deltax);
	mpfr_init(tolstep);
	mpfr_init(aux);

	mpfr_set_str(tolstep, "1.e-75", 10, GMP_RNDN);
	mpfr_set_str(xmin, "-1.e-1", 10, GMP_RNDN);
	mpfr_set_str(xmax, "0", 10, GMP_RNDN);

	mpfr_sub(deltax, xmin, xmax, GMP_RNDN);
	mpfr_set_str(aux, "-1.", 10, GMP_RNDN);
	mpfr_div(deltax, deltax, aux, GMP_RNDN);

	mpfr_sub(x, xmax, deltax, GMP_RNDN);
	X.setfull(x, xmax);

	Xinit = X;

	coef = new INTERVAL[70];
	
	i = 0;
	do
	{
		flagevaluation = det_near0(X, PX);
		i++;
		if(flagevaluation > 0 && mpfi_has_zero(PX.x) > 0)
		{
			flagevaluation = -1;
		}
		if(flagevaluation <= 0)
		{
			mpfi_bisect(Xaux.x, Xaux2.x, X.x); 

			X = Xaux2;
			mpfr_div_d(deltax, deltax, 2., GMP_RNDN);
		}else
		{
			if(flagfirstevaluation == 0)
			{
				CorrectPX = PX;
				flagfirstevaluation = 1;
			}else
			{
				mpfi_put(CorrectPX.x, PX.x);
			}
			GoodXleft = X;
			mpfi_get_left(aux, X.x);
			mpfi_set_fr(X.x, aux);
			mpfr_mul_d(deltax, deltax, 1.01, GMP_RNDN);
			mpfr_sub(aux, aux, deltax, GMP_RNDN);
			mpfi_put_fr(X.x, aux);
		}
	}while(mpfr_cmpabs(deltax, tolstep) > 0);

	printf("I = \n");
	mpfi_put(CorrectX.x, GoodXleft.x);
	mpfi_out_str(stdout, 10, DIGITSPRINT, CorrectX.x);
	printf("\n");
	printf("\n");
	
	printf("P(I) = \n");
	mpfi_out_str(stdout, 10, DIGITSPRINT, CorrectPX.x);
	printf("\n");
	printf("\n");


	delete[] coef;

	return 0;
}

int det_near0(const INTERVAL& x0, INTERVAL& res)
{
	INTERVAL a, b, x, aux;
	int i;

	mpfi_set_d(a.x, 5.);
	mpfi_div_d(a.x, a.x, 3);
	mpfi_put_d(a.x, 1.);
	x = x0;

	b = exp(x*(-1.*a*x));
	mpfi_put(b.x, x.x);

	polynomial_determinantnear0(coef, a, b);

	res = x*coef[69];
	for(i = 68; i > 0; i--)
	{
		aux = res;
		res = x*(aux+coef[i]);
	}
	aux = res;
	res = aux+coef[0];
	
	return 1;
}

void polynomial_determinantnear0(INTERVAL *coef, const INTERVAL& a, const INTERVAL& b)
{
coef[0] = str_to_INTERVAL((char*)"377064171503616") + (str_to_INTERVAL((char*)"-273133143588864") + (str_to_INTERVAL((char*)"53597165322240") + (str_to_INTERVAL((char*)"7889989140480") + (str_to_INTERVAL((char*)"2365587456000") + (str_to_INTERVAL((char*)"-174550155264") - str_to_INTERVAL((char*)"2415919104") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-34394634977280") + (str_to_INTERVAL((char*)"45756501196800") + (str_to_INTERVAL((char*)"-22141898588160") + (str_to_INTERVAL((char*)"3891643023360") + (str_to_INTERVAL((char*)"-342255206400") + str_to_INTERVAL((char*)"15099494400") * b) * b) * b) * b) * b) * a;
coef[1] = str_to_INTERVAL((char*)"-100663296") * a * (str_to_INTERVAL((char*)"16856082") + (str_to_INTERVAL((char*)"-12210003") + (str_to_INTERVAL((char*)"2395980") + (str_to_INTERVAL((char*)"352710") + (str_to_INTERVAL((char*)"105750") + (-7803 - 108 * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1195880") + (str_to_INTERVAL((char*)"1590925") + (str_to_INTERVAL((char*)"-769860") + (str_to_INTERVAL((char*)"135310") + (str_to_INTERVAL((char*)"-11900") + 525 * b) * b) * b) * b) * b) * a);
coef[2] = str_to_INTERVAL((char*)"2769860245323776") + (str_to_INTERVAL((char*)"116110280097792") + (str_to_INTERVAL((char*)"-1069767368638464") + (str_to_INTERVAL((char*)"290453672951808") + (str_to_INTERVAL((char*)"65761385119744") + (str_to_INTERVAL((char*)"12138986864640") + (str_to_INTERVAL((char*)"-1015692656640") - str_to_INTERVAL((char*)"14495514624") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-2040335823798272") + (str_to_INTERVAL((char*)"2171578078986240") + (str_to_INTERVAL((char*)"-662663122124800") + (str_to_INTERVAL((char*)"-12020875264000") + (str_to_INTERVAL((char*)"7593367961600") + (str_to_INTERVAL((char*)"1025557659648") + str_to_INTERVAL((char*)"39258685440") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"4150598177914880") + (str_to_INTERVAL((char*)"-3274498551840768") + (str_to_INTERVAL((char*)"791315176488960") + (str_to_INTERVAL((char*)"27597681459200") + (str_to_INTERVAL((char*)"34900468039680") + (str_to_INTERVAL((char*)"-2636623380480") - str_to_INTERVAL((char*)"38050725888") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-281710260387840") + (str_to_INTERVAL((char*)"375463776092160") + (str_to_INTERVAL((char*)"-184253090365440") + (str_to_INTERVAL((char*)"34103214735360") + (str_to_INTERVAL((char*)"-3286656614400") + str_to_INTERVAL((char*)"154769817600") * b) * b) * b) * b) * b) * a) * a) * a;
coef[3] = str_to_INTERVAL((char*)"-8388608") * a * (str_to_INTERVAL((char*)"1816061896") + (str_to_INTERVAL((char*)"76127832") + (str_to_INTERVAL((char*)"-701394144") + (str_to_INTERVAL((char*)"190436268") + (str_to_INTERVAL((char*)"43116524") + (str_to_INTERVAL((char*)"7958940") + (str_to_INTERVAL((char*)"-665940") - 9504 * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1094521428") + (str_to_INTERVAL((char*)"1164925260") + (str_to_INTERVAL((char*)"-355480200") + (str_to_INTERVAL((char*)"-6448500") + (str_to_INTERVAL((char*)"4073400") + (str_to_INTERVAL((char*)"550152") + str_to_INTERVAL((char*)"21060") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"787823918") + (str_to_INTERVAL((char*)"-682467093") + (str_to_INTERVAL((char*)"195987540") + (str_to_INTERVAL((char*)"-8237110") + (str_to_INTERVAL((char*)"8639610") + (str_to_INTERVAL((char*)"-663117") - 9828 * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-48079800") + (str_to_INTERVAL((char*)"64169175") + (str_to_INTERVAL((char*)"-31815900") + (str_to_INTERVAL((char*)"6104250") + (str_to_INTERVAL((char*)"-622500") + str_to_INTERVAL((char*)"30375") * b) * b) * b) * b) * b) * a) * a) * a);
coef[4] = str_to_INTERVAL((char*)"2952885545467904") + (str_to_INTERVAL((char*)"9302604907544576") + (str_to_INTERVAL((char*)"-1568446357700608") + (str_to_INTERVAL((char*)"-2330705627447296") + (str_to_INTERVAL((char*)"685416550236160") + (str_to_INTERVAL((char*)"210121200238592") + (str_to_INTERVAL((char*)"30497354809344") + (str_to_INTERVAL((char*)"-2728856125440") - str_to_INTERVAL((char*)"40315650048") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1358292030849024") + (str_to_INTERVAL((char*)"1037337161105408") + (str_to_INTERVAL((char*)"3029967910731776") + (str_to_INTERVAL((char*)"-2141841587699712") + (str_to_INTERVAL((char*)"387913124675584") + (str_to_INTERVAL((char*)"30837336702976") + (str_to_INTERVAL((char*)"6841379586048") - str_to_INTERVAL((char*)"78441873408") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"36796456958427136") + (str_to_INTERVAL((char*)"3354302349312") + (str_to_INTERVAL((char*)"-12917835630116864") + (str_to_INTERVAL((char*)"3772835319250944") + (str_to_INTERVAL((char*)"659572171735040") + (str_to_INTERVAL((char*)"218731393318912") + (str_to_INTERVAL((char*)"-17005302251520") - str_to_INTERVAL((char*)"275414777856") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-17414065513037824") + (str_to_INTERVAL((char*)"19751724038225920") + (str_to_INTERVAL((char*)"-6625285352980480") + (str_to_INTERVAL((char*)"21911463526400") + (str_to_INTERVAL((char*)"151244253429760") + (str_to_INTERVAL((char*)"5410585051136") + str_to_INTERVAL((char*)"224227491840") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"6789637712379904") + (str_to_INTERVAL((char*)"-6372821034860544") + (str_to_INTERVAL((char*)"2091524586209280") + (str_to_INTERVAL((char*)"-198513388421120") + (str_to_INTERVAL((char*)"91404286033920") + (str_to_INTERVAL((char*)"-7043511484416") - str_to_INTERVAL((char*)"120191975424") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-335044694507520") + (str_to_INTERVAL((char*)"447283892060160") + (str_to_INTERVAL((char*)"-222211709337600") + (str_to_INTERVAL((char*)"42923584389120") + (str_to_INTERVAL((char*)"-4421635276800") + str_to_INTERVAL((char*)"217055232000") * b) * b) * b) * b) * b) * a) * a) * a) * a) * a;
coef[5] = str_to_INTERVAL((char*)"-4194304") * a * (str_to_INTERVAL((char*)"4576148044") + (str_to_INTERVAL((char*)"14416439986") + (str_to_INTERVAL((char*)"-2430653888") + (str_to_INTERVAL((char*)"-3611942906") + (str_to_INTERVAL((char*)"1062204260") + (str_to_INTERVAL((char*)"325629187") + (str_to_INTERVAL((char*)"47262384") + (str_to_INTERVAL((char*)"-4228965") - str_to_INTERVAL((char*)"62478") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1781131308") + (str_to_INTERVAL((char*)"1360262486") + (str_to_INTERVAL((char*)"3973203542") + (str_to_INTERVAL((char*)"-2808601554") + (str_to_INTERVAL((char*)"508671328") + (str_to_INTERVAL((char*)"40437067") + (str_to_INTERVAL((char*)"8971116") - str_to_INTERVAL((char*)"102861") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"12237386988") + (str_to_INTERVAL((char*)"-1138318704") + (str_to_INTERVAL((char*)"-3338421912") + (str_to_INTERVAL((char*)"1191269142") + (str_to_INTERVAL((char*)"60896310") + (str_to_INTERVAL((char*)"115289226") + (str_to_INTERVAL((char*)"-8255610") - str_to_INTERVAL((char*)"152928") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-4315895318") + (str_to_INTERVAL((char*)"5609487170") + (str_to_INTERVAL((char*)"-2210753720") + (str_to_INTERVAL((char*)"78470350") + (str_to_INTERVAL((char*)"88189640") + (str_to_INTERVAL((char*)"-619808") - 9450 * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1051760442") + (str_to_INTERVAL((char*)"-1069922043") + (str_to_INTERVAL((char*)"400882980") + (str_to_INTERVAL((char*)"-57385890") + (str_to_INTERVAL((char*)"17205150") + (str_to_INTERVAL((char*)"-1319643") - str_to_INTERVAL((char*)"28548") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-38012280") + (str_to_INTERVAL((char*)"50713935") + (str_to_INTERVAL((char*)"-25075620") + (str_to_INTERVAL((char*)"4765890") + (str_to_INTERVAL((char*)"-479100") + str_to_INTERVAL((char*)"23175") * b) * b) * b) * b) * b) * a) * a) * a) * a) * a);
coef[6] = str_to_INTERVAL((char*)"847566195392512") + (str_to_INTERVAL((char*)"8028244875935744") + (str_to_INTERVAL((char*)"15712085535096832") + (str_to_INTERVAL((char*)"-2629659871150080") + (str_to_INTERVAL((char*)"-3680624107847680") + (str_to_INTERVAL((char*)"895647272140800") + (str_to_INTERVAL((char*)"371609437208576") + (str_to_INTERVAL((char*)"49865639854080") + (str_to_INTERVAL((char*)"-4493722779648") - str_to_INTERVAL((char*)"68929191936") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"753518222770176") + (str_to_INTERVAL((char*)"9104951351443456") + (str_to_INTERVAL((char*)"12846652100444160") + (str_to_INTERVAL((char*)"-3947191278239744") + (str_to_INTERVAL((char*)"-3156082931269632") + (str_to_INTERVAL((char*)"1479660147834880") + (str_to_INTERVAL((char*)"144844156567552") + (str_to_INTERVAL((char*)"13524931706880") - str_to_INTERVAL((char*)"528067067904") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"54249979555872768") + (str_to_INTERVAL((char*)"169807535249817600") + (str_to_INTERVAL((char*)"-29209100038438912") + (str_to_INTERVAL((char*)"-39644763921580032") + (str_to_INTERVAL((char*)"11575029917548544") + (str_to_INTERVAL((char*)"3601770683039744") + (str_to_INTERVAL((char*)"684098246934528") + (str_to_INTERVAL((char*)"-56186860732416") - str_to_INTERVAL((char*)"952211865600") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-16900611760979968") + (str_to_INTERVAL((char*)"14777274951794688") + (str_to_INTERVAL((char*)"39678913902804992") + (str_to_INTERVAL((char*)"-28385974736125952") + (str_to_INTERVAL((char*)"4881566094327808") + (str_to_INTERVAL((char*)"606360523767808") + (str_to_INTERVAL((char*)"95393194967040") - str_to_INTERVAL((char*)"2110512955392") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"45289872327442432") + (str_to_INTERVAL((char*)"-10678775066918912") + (str_to_INTERVAL((char*)"-6062999055892480") + (str_to_INTERVAL((char*)"3418765303742464") + (str_to_INTERVAL((char*)"-600833569849344") + (str_to_INTERVAL((char*)"729285262311424") + (str_to_INTERVAL((char*)"-49628137390080") - str_to_INTERVAL((char*)"1066326294528") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-11050006354067456") + (str_to_INTERVAL((char*)"17963173483118592") + (str_to_INTERVAL((char*)"-8557725147463680") + (str_to_INTERVAL((char*)"590740589641728") + (str_to_INTERVAL((char*)"476691734462464") + (str_to_INTERVAL((char*)"-10728182382592") - str_to_INTERVAL((char*)"474501611520") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1741058308308992") + (str_to_INTERVAL((char*)"-1966336188088320") + (str_to_INTERVAL((char*)"866120034680832") + (str_to_INTERVAL((char*)"-169715414597632") + (str_to_INTERVAL((char*)"35976005025792") + (str_to_INTERVAL((char*)"-2706081054720") - str_to_INTERVAL((char*)"77309411328") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-40587944263680") + (str_to_INTERVAL((char*)"54082425323520") + (str_to_INTERVAL((char*)"-26491307950080") + (str_to_INTERVAL((char*)"4870971064320") + (str_to_INTERVAL((char*)"-464309452800") + str_to_INTERVAL((char*)"21705523200") * b) * b) * b) * b) * b) * a) * a) * a) * a) * a) * a) * a;
coef[7] = str_to_INTERVAL((char*)"-1048576") * a * (str_to_INTERVAL((char*)"6062265840") + (str_to_INTERVAL((char*)"57422482080") + (str_to_INTERVAL((char*)"112381593240") + (str_to_INTERVAL((char*)"-18808793100") + (str_to_INTERVAL((char*)"-26325875100") + (str_to_INTERVAL((char*)"6406168500") + (str_to_INTERVAL((char*)"2657957820") + (str_to_INTERVAL((char*)"356666850") + (str_to_INTERVAL((char*)"-32141610") - str_to_INTERVAL((char*)"493020") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"4670971344") + (str_to_INTERVAL((char*)"56440528664") + (str_to_INTERVAL((char*)"79634894040") + (str_to_INTERVAL((char*)"-24468177136") + (str_to_INTERVAL((char*)"-19564189008") + (str_to_INTERVAL((char*)"9172240220") + (str_to_INTERVAL((char*)"897871988") + (str_to_INTERVAL((char*)"83839470") - str_to_INTERVAL((char*)"3273426") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"83201952988") + (str_to_INTERVAL((char*)"256352608666") + (str_to_INTERVAL((char*)"-46259055744") + (str_to_INTERVAL((char*)"-49019574362") + (str_to_INTERVAL((char*)"13976460652") + (str_to_INTERVAL((char*)"4564354839") + (str_to_INTERVAL((char*)"1508693208") + (str_to_INTERVAL((char*)"-108637353") - str_to_INTERVAL((char*)"2245518") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-19095617916") + (str_to_INTERVAL((char*)"22609309566") + (str_to_INTERVAL((char*)"51087318654") + (str_to_INTERVAL((char*)"-37561341114") + (str_to_INTERVAL((char*)"5689269096") + (str_to_INTERVAL((char*)"1389105051") + (str_to_INTERVAL((char*)"140249700") - str_to_INTERVAL((char*)"5971509") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"24439500512") + (str_to_INTERVAL((char*)"-10132482892") + (str_to_INTERVAL((char*)"2801694280") + (str_to_INTERVAL((char*)"-161232652") + (str_to_INTERVAL((char*)"-727704180") + (str_to_INTERVAL((char*)"727887272") + (str_to_INTERVAL((char*)"-48839700") - str_to_INTERVAL((char*)"1236816") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-3617381616") + (str_to_INTERVAL((char*)"8477942800") + (str_to_INTERVAL((char*)"-4931299900") + (str_to_INTERVAL((char*)"503359820") + (str_to_INTERVAL((char*)"330662060") + (str_to_INTERVAL((char*)"-9110396") - str_to_INTERVAL((char*)"643680") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"356068544") + (str_to_INTERVAL((char*)"-481606488") + (str_to_INTERVAL((char*)"266070168") + (str_to_INTERVAL((char*)"-68758168") + (str_to_INTERVAL((char*)"10433928") + (str_to_INTERVAL((char*)"-745584") - str_to_INTERVAL((char*)"28800") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-4100160") + (str_to_INTERVAL((char*)"5454600") + (str_to_INTERVAL((char*)"-2639520") + (str_to_INTERVAL((char*)"463920") + (str_to_INTERVAL((char*)"-40800") + 1800 * b) * b) * b) * b) * b) * a) * a) * a) * a) * a) * a) * a);
coef[8] = str_to_INTERVAL((char*)"-441125252890624") + (str_to_INTERVAL((char*)"1373614142128128") + (str_to_INTERVAL((char*)"9861959436468224") + (str_to_INTERVAL((char*)"17341561197035520") + (str_to_INTERVAL((char*)"-1355257693077504") + (str_to_INTERVAL((char*)"-4276148868481024") + (str_to_INTERVAL((char*)"687393753530368") + (str_to_INTERVAL((char*)"416505353207808") + (str_to_INTERVAL((char*)"58881813774336") + (str_to_INTERVAL((char*)"-5078965813248") - str_to_INTERVAL((char*)"80999350272") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1088623516581888") + (str_to_INTERVAL((char*)"7010180625596416") + (str_to_INTERVAL((char*)"31647052014288896") + (str_to_INTERVAL((char*)"26732821997420544") + (str_to_INTERVAL((char*)"-14218126054916096") + (str_to_INTERVAL((char*)"-3885787404828672") + (str_to_INTERVAL((char*)"2582775330766848") + (str_to_INTERVAL((char*)"355015828963328") + (str_to_INTERVAL((char*)"15312026075136") - str_to_INTERVAL((char*)"1164140347392") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"20937415535362048") + (str_to_INTERVAL((char*)"204477515224317952") + (str_to_INTERVAL((char*)"405004836858232832") + (str_to_INTERVAL((char*)"-60828534827909120") + (str_to_INTERVAL((char*)"-94708232326479872") + (str_to_INTERVAL((char*)"21831723551555584") + (str_to_INTERVAL((char*)"9631285318254592") + (str_to_INTERVAL((char*)"1406408279982080") + (str_to_INTERVAL((char*)"-118840259248128") - str_to_INTERVAL((char*)"2040715542528") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"14068311281106944") + (str_to_INTERVAL((char*)"167551953130225664") + (str_to_INTERVAL((char*)"236740293318148096") + (str_to_INTERVAL((char*)"-70925424158310400") + (str_to_INTERVAL((char*)"-57189253871828992") + (str_to_INTERVAL((char*)"26202908091482112") + (str_to_INTERVAL((char*)"2884874589437952") + (str_to_INTERVAL((char*)"285189063311360") - str_to_INTERVAL((char*)"12083776192512") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"87839701574090752") + (str_to_INTERVAL((char*)"263575537643945984") + (str_to_INTERVAL((char*)"-50683000148459520") + (str_to_INTERVAL((char*)"-27695886070972416") + (str_to_INTERVAL((char*)"6218109102850048") + (str_to_INTERVAL((char*)"3023802235617280") + (str_to_INTERVAL((char*)"2674651893334016") + (str_to_INTERVAL((char*)"-168056749817856") - str_to_INTERVAL((char*)"4290370338816") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-12914469378195456") + (str_to_INTERVAL((char*)"24952931176415232") + (str_to_INTERVAL((char*)"46231258977533952") + (str_to_INTERVAL((char*)"-34850677375107072") + (str_to_INTERVAL((char*)"3696277548171264") + (str_to_INTERVAL((char*)"2349050587250688") + (str_to_INTERVAL((char*)"151033900695552") - str_to_INTERVAL((char*)"11277047955456") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"8829011530612736") + (str_to_INTERVAL((char*)"-5072768328531968") + (str_to_INTERVAL((char*)"5498057504325632") + (str_to_INTERVAL((char*)"-2592498614534144") + (str_to_INTERVAL((char*)"-166433879228416") + (str_to_INTERVAL((char*)"556648527560704") + (str_to_INTERVAL((char*)"-37978311229440") - str_to_INTERVAL((char*)"1134727004160") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-550684218884096") + (str_to_INTERVAL((char*)"2728891173044224") + (str_to_INTERVAL((char*)"-1955389092397056") + (str_to_INTERVAL((char*)"269833992142848") + (str_to_INTERVAL((char*)"140564800471040") + (str_to_INTERVAL((char*)"-3511139958784") - str_to_INTERVAL((char*)"497905827840") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"29472266911744") + (str_to_INTERVAL((char*)"-65027444834304") + (str_to_INTERVAL((char*)"51417452642304") + (str_to_INTERVAL((char*)"-17155777101824") + (str_to_INTERVAL((char*)"1830587203584") + (str_to_INTERVAL((char*)"-116203192320") - str_to_INTERVAL((char*)"6417285120") * b) * b) * b) * b) * b) * b) * a) * a) * a) * a) * a) * a) * a) * a;
coef[9] = str_to_INTERVAL((char*)"131072") * a * (str_to_INTERVAL((char*)"28606908032") + (str_to_INTERVAL((char*)"-89078675904") + (str_to_INTERVAL((char*)"-639546624832") + (str_to_INTERVAL((char*)"-1124597703360") + (str_to_INTERVAL((char*)"87888262872") + (str_to_INTERVAL((char*)"277307627732") + (str_to_INTERVAL((char*)"-44577384224") + (str_to_INTERVAL((char*)"-27010311144") + (str_to_INTERVAL((char*)"-3818476998") + (str_to_INTERVAL((char*)"329370189") + str_to_INTERVAL((char*)"5252796") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-62291537280") + (str_to_INTERVAL((char*)"-401125752960") + (str_to_INTERVAL((char*)"-1810858841760") + (str_to_INTERVAL((char*)"-1529664344640") + (str_to_INTERVAL((char*)"813567698760") + (str_to_INTERVAL((char*)"222346538820") + (str_to_INTERVAL((char*)"-147787589880") + (str_to_INTERVAL((char*)"-20314168680") + (str_to_INTERVAL((char*)"-876161160") + str_to_INTERVAL((char*)"66612645") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-302753801376") + (str_to_INTERVAL((char*)"-3172996477664") + (str_to_INTERVAL((char*)"-6448987655344") + (str_to_INTERVAL((char*)"734418228440") + (str_to_INTERVAL((char*)"1502475874744") + (str_to_INTERVAL((char*)"-305376631768") + (str_to_INTERVAL((char*)"-155126808824") + (str_to_INTERVAL((char*)"-26469705860") + (str_to_INTERVAL((char*)"1993566276") + str_to_INTERVAL((char*)"41381496") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-179284356064") + (str_to_INTERVAL((char*)"-2063993229584") + (str_to_INTERVAL((char*)"-2926147369904") + (str_to_INTERVAL((char*)"822949649632") + (str_to_INTERVAL((char*)"678107961344") + (str_to_INTERVAL((char*)"-292360640968") + (str_to_INTERVAL((char*)"-42041420344") + (str_to_INTERVAL((char*)"-4589135980") + str_to_INTERVAL((char*)"218993940") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-439825812624") + (str_to_INTERVAL((char*)"-1279871737272") + (str_to_INTERVAL((char*)"247904626944") + (str_to_INTERVAL((char*)"-83205819576") + (str_to_INTERVAL((char*)"64261955712") + (str_to_INTERVAL((char*)"-1884931476") + (str_to_INTERVAL((char*)"-26259831648") + (str_to_INTERVAL((char*)"1484422956") + str_to_INTERVAL((char*)"45555480") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"30923881296") + (str_to_INTERVAL((char*)"-126933054696") + (str_to_INTERVAL((char*)"-217245497448") + (str_to_INTERVAL((char*)"157450165656") + (str_to_INTERVAL((char*)"-244256880") + (str_to_INTERVAL((char*)"-19924399068") + (str_to_INTERVAL((char*)"-806581440") + str_to_INTERVAL((char*)"101234196") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-12195108864") + (str_to_INTERVAL((char*)"3461088992") + (str_to_INTERVAL((char*)"-24010529024") + (str_to_INTERVAL((char*)"18214843664") + (str_to_INTERVAL((char*)"-2129246080") + (str_to_INTERVAL((char*)"-2120884048") + (str_to_INTERVAL((char*)"150902880") + str_to_INTERVAL((char*)"5284224") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-282228016") + (str_to_INTERVAL((char*)"-3454455648") + (str_to_INTERVAL((char*)"3132157392") + (str_to_INTERVAL((char*)"-591788016") + (str_to_INTERVAL((char*)"-200575840") + (str_to_INTERVAL((char*)"688720") + str_to_INTERVAL((char*)"1598400") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"9430368") + (str_to_INTERVAL((char*)"19794432") + (str_to_INTERVAL((char*)"-31679712") + (str_to_INTERVAL((char*)"13396992") + (str_to_INTERVAL((char*)"-965472") + (str_to_INTERVAL((char*)"46080") + 4320 * b) * b) * b) * b) * b) * b) * a) * a) * a) * a) * a) * a) * a) * a);
coef[10] = str_to_INTERVAL((char*)"36391895433216") + (str_to_INTERVAL((char*)"-788130206580736") + (str_to_INTERVAL((char*)"581682411339776") + (str_to_INTERVAL((char*)"6982158862254080") + (str_to_INTERVAL((char*)"13385016375508992") + (str_to_INTERVAL((char*)"760530927091712") + (str_to_INTERVAL((char*)"-3553459339001856") + (str_to_INTERVAL((char*)"289805087801344") + (str_to_INTERVAL((char*)"313612982288384") + (str_to_INTERVAL((char*)"52478498832384") + (str_to_INTERVAL((char*)"-4182545793024") - str_to_INTERVAL((char*)"69278367744") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-430141310238720") + (str_to_INTERVAL((char*)"-303266617360384") + (str_to_INTERVAL((char*)"11807762635816960") + (str_to_INTERVAL((char*)"43849805612449792") + (str_to_INTERVAL((char*)"34480414087184384") + (str_to_INTERVAL((char*)"-17147267493396480") + (str_to_INTERVAL((char*)"-4812014553661440") + (str_to_INTERVAL((char*)"2606615419158528") + (str_to_INTERVAL((char*)"514186267066368") + (str_to_INTERVAL((char*)"12743952039936") - str_to_INTERVAL((char*)"1523642793984") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-13691273466609664") + (str_to_INTERVAL((char*)"49997098950590464") + (str_to_INTERVAL((char*)"344651160344330240") + (str_to_INTERVAL((char*)"613850435005972480") + (str_to_INTERVAL((char*)"-35820684576292864") + (str_to_INTERVAL((char*)"-155122541977665536") + (str_to_INTERVAL((char*)"23508726184935424") + (str_to_INTERVAL((char*)"15207022644953088") + (str_to_INTERVAL((char*)"2100933999853568") + (str_to_INTERVAL((char*)"-173158563250176") - str_to_INTERVAL((char*)"2993741365248") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"26994551960895488") + (str_to_INTERVAL((char*)"175319398779715584") + (str_to_INTERVAL((char*)"797223574277455872") + (str_to_INTERVAL((char*)"677230468243914752") + (str_to_INTERVAL((char*)"-354092834140717056") + (str_to_INTERVAL((char*)"-97995627102470144") + (str_to_INTERVAL((char*)"64173449505406976") + (str_to_INTERVAL((char*)"9098080818823168") + (str_to_INTERVAL((char*)"440004520706048") - str_to_INTERVAL((char*)"32009363324928") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"47606776824070144") + (str_to_INTERVAL((char*)"564115067832893440") + (str_to_INTERVAL((char*)"1193728049894391808") + (str_to_INTERVAL((char*)"-69611973707300864") + (str_to_INTERVAL((char*)"-275293566495883264") + (str_to_INTERVAL((char*)"44486137888112640") + (str_to_INTERVAL((char*)"28750580479426560") + (str_to_INTERVAL((char*)"6298337340293120") + (str_to_INTERVAL((char*)"-406404252303360") - str_to_INTERVAL((char*)"10687978930176") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"25330046860263424") + (str_to_INTERVAL((char*)"274799695008628736") + (str_to_INTERVAL((char*)"392227488878559232") + (str_to_INTERVAL((char*)"-96318025912811520") + (str_to_INTERVAL((char*)"-83110763071275008") + (str_to_INTERVAL((char*)"30321682364235776") + (str_to_INTERVAL((char*)"7577024316047360") + (str_to_INTERVAL((char*)"900557675954176") - str_to_INTERVAL((char*)"49324140527616") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"25159535480012800") + (str_to_INTERVAL((char*)"72968432282238976") + (str_to_INTERVAL((char*)"-9710846791909376") + (str_to_INTERVAL((char*)"29026392320507904") + (str_to_INTERVAL((char*)"-17203716950065152") + (str_to_INTERVAL((char*)"-658923473338368") + (str_to_INTERVAL((char*)"3413694024253440") + (str_to_INTERVAL((char*)"-182331434336256") - str_to_INTERVAL((char*)"6378403921920") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-253324601327616") + (str_to_INTERVAL((char*)"6460950285647872") + (str_to_INTERVAL((char*)"14301043373375488") + (str_to_INTERVAL((char*)"-8629752752504832") + (str_to_INTERVAL((char*)"-1841339274625024") + (str_to_INTERVAL((char*)"2027501837090816") + (str_to_INTERVAL((char*)"48863775817728") - str_to_INTERVAL((char*)"10576606003200") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"111068156264448") + (str_to_INTERVAL((char*)"680448539754496") + (str_to_INTERVAL((char*)"817263057829888") + (str_to_INTERVAL((char*)"-1070363681226752") + (str_to_INTERVAL((char*)"242641530781696") + (str_to_INTERVAL((char*)"90628977328128") + (str_to_INTERVAL((char*)"-6843090862080") - str_to_INTERVAL((char*)"283115520000") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"14973902258176") + (str_to_INTERVAL((char*)"23053182435328") + (str_to_INTERVAL((char*)"-32172704530432") + (str_to_INTERVAL((char*)"10847189467136") + (str_to_INTERVAL((char*)"-56245616640") + (str_to_INTERVAL((char*)"292332503040") - str_to_INTERVAL((char*)"47563407360") * b) * b) * b) * b) * b) * b) * a) * a) * a) * a) * a) * a) * a) * a) * a;
coef[11] = str_to_INTERVAL((char*)"32768") * a * (str_to_INTERVAL((char*)"-10550628864") + (str_to_INTERVAL((char*)"228492338944") + (str_to_INTERVAL((char*)"-168639615104") + (str_to_INTERVAL((char*)"-2024246496320") + (str_to_INTERVAL((char*)"-3880543687968") + (str_to_INTERVAL((char*)"-220490838848") + (str_to_INTERVAL((char*)"1030208243424") + (str_to_INTERVAL((char*)"-84019419376") + (str_to_INTERVAL((char*)"-90921732536") + (str_to_INTERVAL((char*)"-15214408536") + (str_to_INTERVAL((char*)"1212591096") + str_to_INTERVAL((char*)"20084976") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"111578403840") + (str_to_INTERVAL((char*)"78667182848") + (str_to_INTERVAL((char*)"-3062926709120") + (str_to_INTERVAL((char*)"-11374613882624") + (str_to_INTERVAL((char*)"-8944199210848") + (str_to_INTERVAL((char*)"4447991140560") + (str_to_INTERVAL((char*)"1248233755680") + (str_to_INTERVAL((char*)"-676154512416") + (str_to_INTERVAL((char*)"-133379616396") + (str_to_INTERVAL((char*)"-3305773692") + str_to_INTERVAL((char*)"395232048") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"845131164800") + (str_to_INTERVAL((char*)"-4317133727040") + (str_to_INTERVAL((char*)"-27720660351040") + (str_to_INTERVAL((char*)"-50531398286400") + (str_to_INTERVAL((char*)"1167643020600") + (str_to_INTERVAL((char*)"13320121984580") + (str_to_INTERVAL((char*)"-1814529671840") + (str_to_INTERVAL((char*)"-1319786370600") + (str_to_INTERVAL((char*)"-175387471230") + (str_to_INTERVAL((char*)"13283234745") + str_to_INTERVAL((char*)"264989340") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1575734336384") + (str_to_INTERVAL((char*)"-10442140072832") + (str_to_INTERVAL((char*)"-48281892888736") + (str_to_INTERVAL((char*)"-41538682293056") + (str_to_INTERVAL((char*)"20882921849608") + (str_to_INTERVAL((char*)"5949806483972") + (str_to_INTERVAL((char*)"-3763938229688") + (str_to_INTERVAL((char*)"-572341149224") + (str_to_INTERVAL((char*)"-34127393144") + str_to_INTERVAL((char*)"2308346469") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1142355472896") + (str_to_INTERVAL((char*)"-16384894279776") + (str_to_INTERVAL((char*)"-36601405944928") + (str_to_INTERVAL((char*)"-547488045456") + (str_to_INTERVAL((char*)"8207186960368") + (str_to_INTERVAL((char*)"-870358385248") + (str_to_INTERVAL((char*)"-838100047312") + (str_to_INTERVAL((char*)"-273313036920") + (str_to_INTERVAL((char*)"14815356600") + str_to_INTERVAL((char*)"499569840") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-566294765568") + (str_to_INTERVAL((char*)"-5618797472256") + (str_to_INTERVAL((char*)"-8127051611808") + (str_to_INTERVAL((char*)"1457976001728") + (str_to_INTERVAL((char*)"1400179378656") + (str_to_INTERVAL((char*)"-236173004064") + (str_to_INTERVAL((char*)"-253379741184") + (str_to_INTERVAL((char*)"-30265937304") + str_to_INTERVAL((char*)"1932190632") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-228051055360") + (str_to_INTERVAL((char*)"-729219659776") + (str_to_INTERVAL((char*)"-73015558528") + (str_to_INTERVAL((char*)"-738024451104") + (str_to_INTERVAL((char*)"498438719712") + (str_to_INTERVAL((char*)"122314224") + (str_to_INTERVAL((char*)"-79294740432") + (str_to_INTERVAL((char*)"4191871392") + str_to_INTERVAL((char*)"157976064") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-4029579072") + (str_to_INTERVAL((char*)"-41194630688") + (str_to_INTERVAL((char*)"-190297572512") + (str_to_INTERVAL((char*)"90971369952") + (str_to_INTERVAL((char*)"43676497472") + (str_to_INTERVAL((char*)"-33282244528") + (str_to_INTERVAL((char*)"-392659200") + str_to_INTERVAL((char*)"174630672") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-446614528") + (str_to_INTERVAL((char*)"-9242708800") + (str_to_INTERVAL((char*)"-1739644736") + (str_to_INTERVAL((char*)"7245148544") + (str_to_INTERVAL((char*)"-2307256704") + (str_to_INTERVAL((char*)"-529544512") + (str_to_INTERVAL((char*)"42317760") + str_to_INTERVAL((char*)"2211840") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"4920192") + (str_to_INTERVAL((char*)"62952192") + (str_to_INTERVAL((char*)"-41239296") + (str_to_INTERVAL((char*)"-13731840") + (str_to_INTERVAL((char*)"15878016") + (str_to_INTERVAL((char*)"-1785600") + str_to_INTERVAL((char*)"138240") * b) * b) * b) * b) * b) * b) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[12] = str_to_INTERVAL((char*)"-4602393001984") + (str_to_INTERVAL((char*)"47103980077056") + (str_to_INTERVAL((char*)"-506346210328576") + (str_to_INTERVAL((char*)"-207855378497536") + (str_to_INTERVAL((char*)"2994838507094016") + (str_to_INTERVAL((char*)"7273488759914496") + (str_to_INTERVAL((char*)"1601284218028032") + (str_to_INTERVAL((char*)"-2085068221710336") + (str_to_INTERVAL((char*)"35663849324544") + (str_to_INTERVAL((char*)"161267443761152") + (str_to_INTERVAL((char*)"35994311589888") + (str_to_INTERVAL((char*)"-2601808625664") - str_to_INTERVAL((char*)"44541739008") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"31766182100992") + (str_to_INTERVAL((char*)"-356823165042688") + (str_to_INTERVAL((char*)"-3013453107167232") + (str_to_INTERVAL((char*)"7670880251412480") + (str_to_INTERVAL((char*)"34570609157472256") + (str_to_INTERVAL((char*)"31089012145913856") + (str_to_INTERVAL((char*)"-11225283492839424") + (str_to_INTERVAL((char*)"-4784402694406144") + (str_to_INTERVAL((char*)"1609856083689472") + (str_to_INTERVAL((char*)"486430217535488") + (str_to_INTERVAL((char*)"9506515648512") - str_to_INTERVAL((char*)"1354322608128") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1340030626824192") + (str_to_INTERVAL((char*)"-33809003146379264") + (str_to_INTERVAL((char*)"29279328665600000") + (str_to_INTERVAL((char*)"327009432543166464") + (str_to_INTERVAL((char*)"626379830577332224") + (str_to_INTERVAL((char*)"44681418068459520") + (str_to_INTERVAL((char*)"-170143760174809088") + (str_to_INTERVAL((char*)"12871903780012032") + (str_to_INTERVAL((char*)"15357289707208704") + (str_to_INTERVAL((char*)"2364611922493440") + (str_to_INTERVAL((char*)"-181046821650432") - str_to_INTERVAL((char*)"3164615737344") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-13800789755559936") + (str_to_INTERVAL((char*)"-6722833803640832") + (str_to_INTERVAL((char*)"398272330811310080") + (str_to_INTERVAL((char*)"1466505834484727808") + (str_to_INTERVAL((char*)"1158848593691934720") + (str_to_INTERVAL((char*)"-566739270479642624") + (str_to_INTERVAL((char*)"-163844401427120128") + (str_to_INTERVAL((char*)"87251018846830592") + (str_to_INTERVAL((char*)"17305638650183680") + (str_to_INTERVAL((char*)"462891574820864") - str_to_INTERVAL((char*)"52514433662976") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-32592918006988800") + (str_to_INTERVAL((char*)"284899941650792448") + (str_to_INTERVAL((char*)"1691856837294424064") + (str_to_INTERVAL((char*)"3174552625416765440") + (str_to_INTERVAL((char*)"62839274668556288") + (str_to_INTERVAL((char*)"-875848129854832640") + (str_to_INTERVAL((char*)"104800483119464448") + (str_to_INTERVAL((char*)"87354881294729216") + (str_to_INTERVAL((char*)"11431660393594880") + (str_to_INTERVAL((char*)"-766867486015488") - str_to_INTERVAL((char*)"18602848419840") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"62954605171965952") + (str_to_INTERVAL((char*)"432777382756089856") + (str_to_INTERVAL((char*)"2061078970997145600") + (str_to_INTERVAL((char*)"1814201181533110272") + (str_to_INTERVAL((char*)"-850428496792780800") + (str_to_INTERVAL((char*)"-254474798225293312") + (str_to_INTERVAL((char*)"151010892103548928") + (str_to_INTERVAL((char*)"26212432370466816") + (str_to_INTERVAL((char*)"2065532826157056") - str_to_INTERVAL((char*)"129524186382336") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"19326796998639616") + (str_to_INTERVAL((char*)"367091746927017984") + (str_to_INTERVAL((char*)"876616075705319424") + (str_to_INTERVAL((char*)"91077553645158400") + (str_to_INTERVAL((char*)"-182612942000750592") + (str_to_INTERVAL((char*)"5652846154350592") + (str_to_INTERVAL((char*)"16277316580671488") + (str_to_INTERVAL((char*)"10279259956314112") + (str_to_INTERVAL((char*)"-473362323996672") - str_to_INTERVAL((char*)"19911443742720") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"9523649895727104") + (str_to_INTERVAL((char*)"84338626959245312") + (str_to_INTERVAL((char*)"124951424362610688") + (str_to_INTERVAL((char*)"-7044478137270272") + (str_to_INTERVAL((char*)"-11782357400944640") + (str_to_INTERVAL((char*)"-9565810366349312") + (str_to_INTERVAL((char*)"7574854057328640") + (str_to_INTERVAL((char*)"815843529981952") - str_to_INTERVAL((char*)"61558678093824") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1561174097264640") + (str_to_INTERVAL((char*)"6263936189988864") + (str_to_INTERVAL((char*)"3838147783819264") + (str_to_INTERVAL((char*)"13413685442641920") + (str_to_INTERVAL((char*)"-10542713099780096") + (str_to_INTERVAL((char*)"514230644113408") + (str_to_INTERVAL((char*)"1489648517382144") + (str_to_INTERVAL((char*)"-80573659348992") - str_to_INTERVAL((char*)"3130752761856") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"10237669015552") + (str_to_INTERVAL((char*)"252039442464768") + (str_to_INTERVAL((char*)"2099873380827136") + (str_to_INTERVAL((char*)"-1015500774572032") + (str_to_INTERVAL((char*)"-496079139241984") + (str_to_INTERVAL((char*)"388059096940544") + (str_to_INTERVAL((char*)"892653797376") - str_to_INTERVAL((char*)"2038714859520") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"5028987273216") + (str_to_INTERVAL((char*)"39510840705024") + (str_to_INTERVAL((char*)"-10984913633280") + (str_to_INTERVAL((char*)"-16109241630720") + (str_to_INTERVAL((char*)"6952914518016") + (str_to_INTERVAL((char*)"1471055659008") + (str_to_INTERVAL((char*)"-114410127360") - str_to_INTERVAL((char*)"10192158720") * b) * b) * b) * b) * b) * b) * b) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a;
coef[13] = str_to_INTERVAL((char*)"16384") * a * (str_to_INTERVAL((char*)"2949531648") + (str_to_INTERVAL((char*)"-30187487232") + (str_to_INTERVAL((char*)"324501660672") + (str_to_INTERVAL((char*)"133208097792") + (str_to_INTERVAL((char*)"-1919299580352") + (str_to_INTERVAL((char*)"-4661354490912") + (str_to_INTERVAL((char*)"-1026213640704") + (str_to_INTERVAL((char*)"1336255879392") + (str_to_INTERVAL((char*)"-22855860468") + (str_to_INTERVAL((char*)"-103351328094") + (str_to_INTERVAL((char*)"-23067643536") + (str_to_INTERVAL((char*)"1667418858") + str_to_INTERVAL((char*)"28545426") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-18419111936") + (str_to_INTERVAL((char*)"206898197504") + (str_to_INTERVAL((char*)"1747302521856") + (str_to_INTERVAL((char*)"-4447837059840") + (str_to_INTERVAL((char*)"-20045214050048") + (str_to_INTERVAL((char*)"-18026465782848") + (str_to_INTERVAL((char*)"6508800853392") + (str_to_INTERVAL((char*)"2774159277152") + (str_to_INTERVAL((char*)"-933449267276") + (str_to_INTERVAL((char*)"-282048771154") + (str_to_INTERVAL((char*)"-5512200846") + str_to_INTERVAL((char*)"785282274") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-157124246784") + (str_to_INTERVAL((char*)"5886961926272") + (str_to_INTERVAL((char*)"-6589461029696") + (str_to_INTERVAL((char*)"-66415539076896") + (str_to_INTERVAL((char*)"-127057393978288") + (str_to_INTERVAL((char*)"-11935634551632") + (str_to_INTERVAL((char*)"35729762976848") + (str_to_INTERVAL((char*)"-2392938208632") + (str_to_INTERVAL((char*)"-3330335543940") + (str_to_INTERVAL((char*)"-450823059024") + (str_to_INTERVAL((char*)"32084733012") + str_to_INTERVAL((char*)"617465160") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1854364039680") + (str_to_INTERVAL((char*)"-69218226560") + (str_to_INTERVAL((char*)"-59797536193600") + (str_to_INTERVAL((char*)"-216328540314880") + (str_to_INTERVAL((char*)"-172710818961680") + (str_to_INTERVAL((char*)"81513003828120") + (str_to_INTERVAL((char*)"25072664586240") + (str_to_INTERVAL((char*)"-12894166265520") + (str_to_INTERVAL((char*)"-2586708036810") + (str_to_INTERVAL((char*)"-79663999290") + str_to_INTERVAL((char*)"8229918060") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1178780680064") + (str_to_INTERVAL((char*)"-25307725271296") + (str_to_INTERVAL((char*)"-140741116313408") + (str_to_INTERVAL((char*)"-272454554864960") + (str_to_INTERVAL((char*)"-17693533539864") + (str_to_INTERVAL((char*)"78016863597244") + (str_to_INTERVAL((char*)"-8171497202336") + (str_to_INTERVAL((char*)"-7731656822344") + (str_to_INTERVAL((char*)"-1068413201466") + (str_to_INTERVAL((char*)"61167743871") + str_to_INTERVAL((char*)"1891627452") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-3094975182464") + (str_to_INTERVAL((char*)"-22627246200064") + (str_to_INTERVAL((char*)"-113067466986144") + (str_to_INTERVAL((char*)"-103482681159616") + (str_to_INTERVAL((char*)"43227256027560") + (str_to_INTERVAL((char*)"13881460007492") + (str_to_INTERVAL((char*)"-7336146611288") + (str_to_INTERVAL((char*)"-1630362956552") + (str_to_INTERVAL((char*)"-176459623120") + str_to_INTERVAL((char*)"10477922301") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-395504540160") + (str_to_INTERVAL((char*)"-11121602449920") + (str_to_INTERVAL((char*)"-28756906848960") + (str_to_INTERVAL((char*)"-5928707618880") + (str_to_INTERVAL((char*)"4907949120384") + (str_to_INTERVAL((char*)"507686874048") + (str_to_INTERVAL((char*)"-213121594512") + (str_to_INTERVAL((char*)"-585481183176") + (str_to_INTERVAL((char*)"23853237408") + str_to_INTERVAL((char*)"1181591712") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-212249583360") + (str_to_INTERVAL((char*)"-1689921217024") + (str_to_INTERVAL((char*)"-2578344179808") + (str_to_INTERVAL((char*)"-468926160896") + (str_to_INTERVAL((char*)"-123690068096") + (str_to_INTERVAL((char*)"790077263104") + (str_to_INTERVAL((char*)"-343401599136") + (str_to_INTERVAL((char*)"-30746540264") + str_to_INTERVAL((char*)"2771249976") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-13067202944") + (str_to_INTERVAL((char*)"-62849334848") + (str_to_INTERVAL((char*)"-128839860480") + (str_to_INTERVAL((char*)"-339235971328") + (str_to_INTERVAL((char*)"301769875824") + (str_to_INTERVAL((char*)"-29126465360") + (str_to_INTERVAL((char*)"-37866630432") + (str_to_INTERVAL((char*)"2143631472") + str_to_INTERVAL((char*)"83544912") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"25454592") + (str_to_INTERVAL((char*)"-6094777344") + (str_to_INTERVAL((char*)"-26620558848") + (str_to_INTERVAL((char*)"16715571264") + (str_to_INTERVAL((char*)"4499737344") + (str_to_INTERVAL((char*)"-4920330192") + (str_to_INTERVAL((char*)"16346400") + str_to_INTERVAL((char*)"27039600") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"24600960") + (str_to_INTERVAL((char*)"63831168") + (str_to_INTERVAL((char*)"-90708480") + (str_to_INTERVAL((char*)"32967936") + (str_to_INTERVAL((char*)"123264") + (str_to_INTERVAL((char*)"-178560") + str_to_INTERVAL((char*)"34560") * b) * b) * b) * b) * b) * b) * b * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[14] = str_to_INTERVAL((char*)"-54358179840") + (str_to_INTERVAL((char*)"-968632565760") + (str_to_INTERVAL((char*)"16702934876160") + (str_to_INTERVAL((char*)"-140933609816064") + (str_to_INTERVAL((char*)"-256292585209856") + (str_to_INTERVAL((char*)"748040398831616") + (str_to_INTERVAL((char*)"2745991802912768") + (str_to_INTERVAL((char*)"1140921492832256") + (str_to_INTERVAL((char*)"-865191061487616") + (str_to_INTERVAL((char*)"-24007571931136") + (str_to_INTERVAL((char*)"55264493764608") + (str_to_INTERVAL((char*)"19175091830784") + (str_to_INTERVAL((char*)"-1251926310912") - str_to_INTERVAL((char*)"21915795456") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-212701544448") + (str_to_INTERVAL((char*)"-11999266209792") + (str_to_INTERVAL((char*)"36914765758464") + (str_to_INTERVAL((char*)"-2570920086470656") + (str_to_INTERVAL((char*)"1487688819539968") + (str_to_INTERVAL((char*)"16712457696313344") + (str_to_INTERVAL((char*)"19425880812552192") + (str_to_INTERVAL((char*)"-3928790375333888") + (str_to_INTERVAL((char*)"-3270234270203904") + (str_to_INTERVAL((char*)"582061904953344") + (str_to_INTERVAL((char*)"316594443976704") + (str_to_INTERVAL((char*)"6727001702400") - str_to_INTERVAL((char*)"864419807232") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-229071527084032") + (str_to_INTERVAL((char*)"2453248921305088") + (str_to_INTERVAL((char*)"-27891287545872384") + (str_to_INTERVAL((char*)"-13629346268839936") + (str_to_INTERVAL((char*)"181813628084158464") + (str_to_INTERVAL((char*)"435181969526226944") + (str_to_INTERVAL((char*)"99820819228131328") + (str_to_INTERVAL((char*)"-125781465985187840") + (str_to_INTERVAL((char*)"1033461679751168") + (str_to_INTERVAL((char*)"10233643268259840") + (str_to_INTERVAL((char*)"2024863999590400") + (str_to_INTERVAL((char*)"-139819021811712") - str_to_INTERVAL((char*)"2483778207744") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1283822549204992") + (str_to_INTERVAL((char*)"-15394837549285376") + (str_to_INTERVAL((char*)"-124818358133587968") + (str_to_INTERVAL((char*)"336306739657834496") + (str_to_INTERVAL((char*)"1491022872682954752") + (str_to_INTERVAL((char*)"1341339573441527808") + (str_to_INTERVAL((char*)"-476458359901782016") + (str_to_INTERVAL((char*)"-210145365939453952") + (str_to_INTERVAL((char*)"70018152741240832") + (str_to_INTERVAL((char*)"21017253315395584") + (str_to_INTERVAL((char*)"417234796888064") - str_to_INTERVAL((char*)"58425107595264") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1780275809878016") + (str_to_INTERVAL((char*)"-192730967780098048") + (str_to_INTERVAL((char*)"291454469127798784") + (str_to_INTERVAL((char*)"2633531783040729088") + (str_to_INTERVAL((char*)"5024736825821888512") + (str_to_INTERVAL((char*)"592879874525757440") + (str_to_INTERVAL((char*)"-1459463034381008896") + (str_to_INTERVAL((char*)"84851526638632960") + (str_to_INTERVAL((char*)"140210040386224128") + (str_to_INTERVAL((char*)"16785763525033984") + (str_to_INTERVAL((char*)"-1082663274676224") - str_to_INTERVAL((char*)"23822592638976") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-42993437384900608") + (str_to_INTERVAL((char*)"44729767515127808") + (str_to_INTERVAL((char*)"1664844636030500864") + (str_to_INTERVAL((char*)"5871861611126325248") + (str_to_INTERVAL((char*)"4765255675888795648") + (str_to_INTERVAL((char*)"-2124228499195166720") + (str_to_INTERVAL((char*)"-718450088480473088") + (str_to_INTERVAL((char*)"350839566666825728") + (str_to_INTERVAL((char*)"71641553544970240") + (str_to_INTERVAL((char*)"2700107426070528") - str_to_INTERVAL((char*)"246552613060608") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1778250485334016") + (str_to_INTERVAL((char*)"435009741498875904") + (str_to_INTERVAL((char*)"2317897494996975616") + (str_to_INTERVAL((char*)"4640854653973037056") + (str_to_INTERVAL((char*)"520185003146477568") + (str_to_INTERVAL((char*)"-1360419882869194752") + (str_to_INTERVAL((char*)"125303285832220672") + (str_to_INTERVAL((char*)"130691248846012416") + (str_to_INTERVAL((char*)"21360056366530560") + (str_to_INTERVAL((char*)"-1003733494726656") - str_to_INTERVAL((char*)"41376247971840") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"26914612840497152") + (str_to_INTERVAL((char*)"218132205115801600") + (str_to_INTERVAL((char*)"1176579408723968000") + (str_to_INTERVAL((char*)"1149708992615809024") + (str_to_INTERVAL((char*)"-399123828925661184") + (str_to_INTERVAL((char*)"-139208892558409728") + (str_to_INTERVAL((char*)"57797952921272320") + (str_to_INTERVAL((char*)"21141611994546176") + (str_to_INTERVAL((char*)"3136255296798720") - str_to_INTERVAL((char*)"182596785684480") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1516369048764416") + (str_to_INTERVAL((char*)"66583808996016128") + (str_to_INTERVAL((char*)"189111775101190144") + (str_to_INTERVAL((char*)"59208626736201728") + (str_to_INTERVAL((char*)"-15798421054554112") + (str_to_INTERVAL((char*)"-11112729104613376") + (str_to_INTERVAL((char*)"-3227504879337472") + (str_to_INTERVAL((char*)"7222946996420608") + (str_to_INTERVAL((char*)"-274590034821120") - str_to_INTERVAL((char*)"14984990687232") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"887020884852736") + (str_to_INTERVAL((char*)"6732238569865216") + (str_to_INTERVAL((char*)"10229283765616640") + (str_to_INTERVAL((char*)"7472515525902336") + (str_to_INTERVAL((char*)"2868597586657280") + (str_to_INTERVAL((char*)"-8403694576992256") + (str_to_INTERVAL((char*)"3229921182220288") + (str_to_INTERVAL((char*)"231514450558976") - str_to_INTERVAL((char*)"25117467475968") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"8104511012864") + (str_to_INTERVAL((char*)"33131566465024") + (str_to_INTERVAL((char*)"851828375814144") + (str_to_INTERVAL((char*)"1541313177583616") + (str_to_INTERVAL((char*)"-1614668760612864") + (str_to_INTERVAL((char*)"242240217677824") + (str_to_INTERVAL((char*)"176210705907712") + (str_to_INTERVAL((char*)"-10620309602304") - str_to_INTERVAL((char*)"411296071680") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1016510545920") + (str_to_INTERVAL((char*)"19492591632384") + (str_to_INTERVAL((char*)"38520018173952") + (str_to_INTERVAL((char*)"-34370164359168") + (str_to_INTERVAL((char*)"-1381352079360") + (str_to_INTERVAL((char*)"6707331465216") + (str_to_INTERVAL((char*)"28460974080") - str_to_INTERVAL((char*)"49179525120") * b) * b) * b) * b) * b) * b) * b) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a;
coef[15] = 8192 * a * (str_to_INTERVAL((char*)"76308480") + (str_to_INTERVAL((char*)"1359774720") + (str_to_INTERVAL((char*)"-23447723520") + (str_to_INTERVAL((char*)"197843812608") + (str_to_INTERVAL((char*)"359785733632") + (str_to_INTERVAL((char*)"-1050105540352") + (str_to_INTERVAL((char*)"-3854846891296") + (str_to_INTERVAL((char*)"-1601635396432") + (str_to_INTERVAL((char*)"1214562647352") + (str_to_INTERVAL((char*)"33702035792") + (str_to_INTERVAL((char*)"-77580771276") + (str_to_INTERVAL((char*)"-26918158698") + (str_to_INTERVAL((char*)"1757464914") + str_to_INTERVAL((char*)"30765582") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"272627712") + (str_to_INTERVAL((char*)"15379918848") + (str_to_INTERVAL((char*)"-47315068416") + (str_to_INTERVAL((char*)"3295246692864") + (str_to_INTERVAL((char*)"-1906827710592") + (str_to_INTERVAL((char*)"-21420996803136") + (str_to_INTERVAL((char*)"-24898895084448") + (str_to_INTERVAL((char*)"5035681023072") + (str_to_INTERVAL((char*)"4191584452776") + (str_to_INTERVAL((char*)"-746051025636") + (str_to_INTERVAL((char*)"-405791218476") + (str_to_INTERVAL((char*)"-8622255600") + str_to_INTERVAL((char*)"1107959958") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"78843241472") + (str_to_INTERVAL((char*)"-933080002048") + (str_to_INTERVAL((char*)"11792860931584") + (str_to_INTERVAL((char*)"7369003442816") + (str_to_INTERVAL((char*)"-89287800682464") + (str_to_INTERVAL((char*)"-209447472879344") + (str_to_INTERVAL((char*)"-50765476095328") + (str_to_INTERVAL((char*)"61235198137280") + (str_to_INTERVAL((char*)"249065422702") + (str_to_INTERVAL((char*)"-5322044189195") + (str_to_INTERVAL((char*)"-887219164120") + (str_to_INTERVAL((char*)"56540446677") + str_to_INTERVAL((char*)"1072481049") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-392716559360") + (str_to_INTERVAL((char*)"5421839286784") + (str_to_INTERVAL((char*)"40398807241728") + (str_to_INTERVAL((char*)"-122111394798208") + (str_to_INTERVAL((char*)"-524775921558528") + (str_to_INTERVAL((char*)"-472421042713056") + (str_to_INTERVAL((char*)"162423234011816") + (str_to_INTERVAL((char*)"76564216035824") + (str_to_INTERVAL((char*)"-25044758548190") + (str_to_INTERVAL((char*)"-7422965452013") + (str_to_INTERVAL((char*)"-151799604211") + str_to_INTERVAL((char*)"20572358733") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"350422111488") + (str_to_INTERVAL((char*)"34196868228352") + (str_to_INTERVAL((char*)"-72061057719232") + (str_to_INTERVAL((char*)"-580341365431200") + (str_to_INTERVAL((char*)"-1101250024746944") + (str_to_INTERVAL((char*)"-155332616872104") + (str_to_INTERVAL((char*)"327312759411232") + (str_to_INTERVAL((char*)"-16246287556728") + (str_to_INTERVAL((char*)"-32248770958968") + (str_to_INTERVAL((char*)"-3542381814618") + (str_to_INTERVAL((char*)"202555678728") + str_to_INTERVAL((char*)"5306298768") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"4940761985536") + (str_to_INTERVAL((char*)"-14301625964288") + (str_to_INTERVAL((char*)"-250394299271168") + (str_to_INTERVAL((char*)"-857343794507200") + (str_to_INTERVAL((char*)"-712589788399424") + (str_to_INTERVAL((char*)"292302105990960") + (str_to_INTERVAL((char*)"112426506569416") + (str_to_INTERVAL((char*)"-51333239304672") + (str_to_INTERVAL((char*)"-10747804185456") + (str_to_INTERVAL((char*)"-528145659808") + str_to_INTERVAL((char*)"41897700234") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1600370027520") + (str_to_INTERVAL((char*)"-39733907811840") + (str_to_INTERVAL((char*)"-208641599367872") + (str_to_INTERVAL((char*)"-435292535538784") + (str_to_INTERVAL((char*)"-72263936647232") + (str_to_INTERVAL((char*)"128815986683296") + (str_to_INTERVAL((char*)"-10542510200992") + (str_to_INTERVAL((char*)"-11395950373280") + (str_to_INTERVAL((char*)"-2595248388920") + (str_to_INTERVAL((char*)"96273379752") + str_to_INTERVAL((char*)"5506373664") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1096958217216") + (str_to_INTERVAL((char*)"-10708289421312") + (str_to_INTERVAL((char*)"-65284783358976") + (str_to_INTERVAL((char*)"-70930815736320") + (str_to_INTERVAL((char*)"18387821633472") + (str_to_INTERVAL((char*)"6570615997920") + (str_to_INTERVAL((char*)"-1058084954688") + (str_to_INTERVAL((char*)"-1693096853760") + (str_to_INTERVAL((char*)"-325981722384") + str_to_INTERVAL((char*)"19280576664") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-47701956096") + (str_to_INTERVAL((char*)"-2271107525248") + (str_to_INTERVAL((char*)"-7118059579520") + (str_to_INTERVAL((char*)"-2813627498432") + (str_to_INTERVAL((char*)"-819811978912") + (str_to_INTERVAL((char*)"1051043114992") + (str_to_INTERVAL((char*)"450027159584") + (str_to_INTERVAL((char*)"-526814433328") + (str_to_INTERVAL((char*)"19605980208") + str_to_INTERVAL((char*)"1112725008") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-17210180096") + (str_to_INTERVAL((char*)"-129957992704") + (str_to_INTERVAL((char*)"-179597096128") + (str_to_INTERVAL((char*)"-487205802240") + (str_to_INTERVAL((char*)"-117705024288") + (str_to_INTERVAL((char*)"427016303936") + (str_to_INTERVAL((char*)"-166555320000") + (str_to_INTERVAL((char*)"-9340065744") + str_to_INTERVAL((char*)"1236732624") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"187668480") + (str_to_INTERVAL((char*)"-456523008") + (str_to_INTERVAL((char*)"-29377101312") + (str_to_INTERVAL((char*)"-27086651136") + (str_to_INTERVAL((char*)"39293967552") + (str_to_INTERVAL((char*)"-8497424064") + (str_to_INTERVAL((char*)"-3693885888") + (str_to_INTERVAL((char*)"239493120") + str_to_INTERVAL((char*)"9443520") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"49201920") + (str_to_INTERVAL((char*)"-3542784") + (str_to_INTERVAL((char*)"-42951168") + (str_to_INTERVAL((char*)"24961536") + (str_to_INTERVAL((char*)"-2699520") + str_to_INTERVAL((char*)"195840") * b) * b) * b) * b) * b) * b * b * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[16] = (str_to_INTERVAL((char*)"-212902871040") + (str_to_INTERVAL((char*)"2739174113280") + (str_to_INTERVAL((char*)"-14563225370624") + (str_to_INTERVAL((char*)"-83587314483200") + (str_to_INTERVAL((char*)"91966165483520") + (str_to_INTERVAL((char*)"707129549783040") + (str_to_INTERVAL((char*)"468797673308160") + (str_to_INTERVAL((char*)"-257797746655232") + (str_to_INTERVAL((char*)"-10050530639872") + (str_to_INTERVAL((char*)"11286198157312") + (str_to_INTERVAL((char*)"7972043513856") + (str_to_INTERVAL((char*)"-473769750528") - str_to_INTERVAL((char*)"8318287872") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-595675054080") + (str_to_INTERVAL((char*)"-2333551362048") + (str_to_INTERVAL((char*)"65030007554048") + (str_to_INTERVAL((char*)"-840863106203648") + (str_to_INTERVAL((char*)"-550425204883456") + (str_to_INTERVAL((char*)"4924317788274688") + (str_to_INTERVAL((char*)"8206277016551424") + (str_to_INTERVAL((char*)"-365102535540736") + (str_to_INTERVAL((char*)"-1481230898429952") + (str_to_INTERVAL((char*)"86938716372992") + (str_to_INTERVAL((char*)"145133020315648") + (str_to_INTERVAL((char*)"4101254209536") - str_to_INTERVAL((char*)"406393933824") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-3363412377600") + (str_to_INTERVAL((char*)"-58804641202176") + (str_to_INTERVAL((char*)"1015687264862208") + (str_to_INTERVAL((char*)"-9235491214852096") + (str_to_INTERVAL((char*)"-21066018881273856") + (str_to_INTERVAL((char*)"56049352696135680") + (str_to_INTERVAL((char*)"203887249275092992") + (str_to_INTERVAL((char*)"86248365845053440") + (str_to_INTERVAL((char*)"-63190226385469440") + (str_to_INTERVAL((char*)"-3028853190000640") + (str_to_INTERVAL((char*)"4454304039518208") + (str_to_INTERVAL((char*)"1321923798917120") + (str_to_INTERVAL((char*)"-81533131579392") - str_to_INTERVAL((char*)"1472900308992") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-10440318910464") + (str_to_INTERVAL((char*)"-591377284988928") + (str_to_INTERVAL((char*)"1690994170920960") + (str_to_INTERVAL((char*)"-135386551061315584") + (str_to_INTERVAL((char*)"80677041554849792") + (str_to_INTERVAL((char*)"904894697054666752") + (str_to_INTERVAL((char*)"1048143311819374592") + (str_to_INTERVAL((char*)"-206927404198920192") + (str_to_INTERVAL((char*)"-179634192900587520") + (str_to_INTERVAL((char*)"31989261303103488") + (str_to_INTERVAL((char*)"17101205503098880") + (str_to_INTERVAL((char*)"357521808687104") - str_to_INTERVAL((char*)"46063856148480") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1199957591719936") + (str_to_INTERVAL((char*)"16614651603189760") + (str_to_INTERVAL((char*)"-243847239787610112") + (str_to_INTERVAL((char*)"-185395700660961280") + (str_to_INTERVAL((char*)"2165659392762970112") + (str_to_INTERVAL((char*)"4971642187112906752") + (str_to_INTERVAL((char*)"1267456216715624448") + (str_to_INTERVAL((char*)"-1466303755480662016") + (str_to_INTERVAL((char*)"-23502621949067264") + (str_to_INTERVAL((char*)"135185066145136640") + (str_to_INTERVAL((char*)"19178559479939072") + (str_to_INTERVAL((char*)"-1106291784450048") - str_to_INTERVAL((char*)"22924641337344") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"5244532153122816") + (str_to_INTERVAL((char*)"-90414364258992128") + (str_to_INTERVAL((char*)"-593308224273252352") + (str_to_INTERVAL((char*)"2133389282719563776") + (str_to_INTERVAL((char*)"8781479083029037056") + (str_to_INTERVAL((char*)"7912136321168572416") + (str_to_INTERVAL((char*)"-2592430080487129088") + (str_to_INTERVAL((char*)"-1340736557891846144") + (str_to_INTERVAL((char*)"427904759084990464") + (str_to_INTERVAL((char*)"124799544758804480") + (str_to_INTERVAL((char*)"2688552860409856") - str_to_INTERVAL((char*)"345759279685632") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-9008035512975360") + (str_to_INTERVAL((char*)"-297120398007336960") + (str_to_INTERVAL((char*)"911322779734245376") + (str_to_INTERVAL((char*)"6479718095978233856") + (str_to_INTERVAL((char*)"12185044112789995520") + (str_to_INTERVAL((char*)"1999252942594834432") + (str_to_INTERVAL((char*)"-3667510845653778432") + (str_to_INTERVAL((char*)"149446639408840704") + (str_to_INTERVAL((char*)"369179933627449344") + (str_to_INTERVAL((char*)"39086270128111616") + (str_to_INTERVAL((char*)"-1928034639642624") - str_to_INTERVAL((char*)"63357298016256") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-25102997436497920") + (str_to_INTERVAL((char*)"164846985149415424") + (str_to_INTERVAL((char*)"1865194782141186048") + (str_to_INTERVAL((char*)"6204656730158137344") + (str_to_INTERVAL((char*)"5338875162007437312") + (str_to_INTERVAL((char*)"-1944292257030078464") + (str_to_INTERVAL((char*)"-884316360176500736") + (str_to_INTERVAL((char*)"371680814394638336") + (str_to_INTERVAL((char*)"80542756365844480") + (str_to_INTERVAL((char*)"5528296427028480") - str_to_INTERVAL((char*)"380300079513600") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"11237840719446016") + (str_to_INTERVAL((char*)"169899189793193984") + (str_to_INTERVAL((char*)"911088257152319488") + (str_to_INTERVAL((char*)"2019821443977838592") + (str_to_INTERVAL((char*)"476836469309833216") + (str_to_INTERVAL((char*)"-597032136635121664") + (str_to_INTERVAL((char*)"45241682122244096") + (str_to_INTERVAL((char*)"42541309496131584") + (str_to_INTERVAL((char*)"17532567406379008") + (str_to_INTERVAL((char*)"-497991376257024") - str_to_INTERVAL((char*)"40561287856128") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1631055543009280") + (str_to_INTERVAL((char*)"23956287010111488") + (str_to_INTERVAL((char*)"177359082655580160") + (str_to_INTERVAL((char*)"224618530915483648") + (str_to_INTERVAL((char*)"-37695232429522944") + (str_to_INTERVAL((char*)"-7104143642066944") + (str_to_INTERVAL((char*)"-10724678862110720") + (str_to_INTERVAL((char*)"7959966920081408") + (str_to_INTERVAL((char*)"1793034427826176") - str_to_INTERVAL((char*)"110663069663232") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"138257698390016") + (str_to_INTERVAL((char*)"4151568512319488") + (str_to_INTERVAL((char*)"13747150253981696") + (str_to_INTERVAL((char*)"5735016016904192") + (str_to_INTERVAL((char*)"7894871043211264") + (str_to_INTERVAL((char*)"-5185441547550720") + (str_to_INTERVAL((char*)"-1900514331328512") + (str_to_INTERVAL((char*)"1996344632934400") + (str_to_INTERVAL((char*)"-75393458110464") - str_to_INTERVAL((char*)"4260844929024") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"8868960665600") + (str_to_INTERVAL((char*)"47617222377472") + (str_to_INTERVAL((char*)"87142957580288") + (str_to_INTERVAL((char*)"1401095205158912") + (str_to_INTERVAL((char*)"28429842382848") + (str_to_INTERVAL((char*)"-931660387319808") + (str_to_INTERVAL((char*)"401228839780352") + (str_to_INTERVAL((char*)"16815354150912") - str_to_INTERVAL((char*)"2820213964800") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"127905300480") + (str_to_INTERVAL((char*)"4828541485056") + (str_to_INTERVAL((char*)"31168263094272") + (str_to_INTERVAL((char*)"8700230107136") + (str_to_INTERVAL((char*)"-28359772602368") + (str_to_INTERVAL((char*)"8414595645440") + (str_to_INTERVAL((char*)"2308522377216") + (str_to_INTERVAL((char*)"-159154176000") - str_to_INTERVAL((char*)"7419985920") * b) * b) * b) * b) * b) * b) * b) * b) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a;
coef[17] = 2048 * a * ((str_to_INTERVAL((char*)"1299456000") + (str_to_INTERVAL((char*)"-16718592000") + (str_to_INTERVAL((char*)"88886873600") + (str_to_INTERVAL((char*)"510176480000") + (str_to_INTERVAL((char*)"-561316928000") + (str_to_INTERVAL((char*)"-4315976256000") + (str_to_INTERVAL((char*)"-2861313924000") + (str_to_INTERVAL((char*)"1573472574800") + (str_to_INTERVAL((char*)"61343570800") + (str_to_INTERVAL((char*)"-68885486800") + (str_to_INTERVAL((char*)"-48657492150") + (str_to_INTERVAL((char*)"2891661075") + str_to_INTERVAL((char*)"50770800") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"3344855040") + (str_to_INTERVAL((char*)"13103437824") + (str_to_INTERVAL((char*)"-365158733824") + (str_to_INTERVAL((char*)"4721643418624") + (str_to_INTERVAL((char*)"3090766531328") + (str_to_INTERVAL((char*)"-27651198518144") + (str_to_INTERVAL((char*)"-46080168794112") + (str_to_INTERVAL((char*)"2050136307968") + (str_to_INTERVAL((char*)"8317458658176") + (str_to_INTERVAL((char*)"-488181268696") + (str_to_INTERVAL((char*)"-814955924624") + (str_to_INTERVAL((char*)"-23029503618") + str_to_INTERVAL((char*)"2281997187") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"5492551680") + (str_to_INTERVAL((char*)"92083332096") + (str_to_INTERVAL((char*)"-1596431574528") + (str_to_INTERVAL((char*)"16881983403264") + (str_to_INTERVAL((char*)"52597488355328") + (str_to_INTERVAL((char*)"-125646150745472") + (str_to_INTERVAL((char*)"-451673948559008") + (str_to_INTERVAL((char*)"-195539477744912") + (str_to_INTERVAL((char*)"136930680944232") + (str_to_INTERVAL((char*)"10338674815672") + (str_to_INTERVAL((char*)"-10889569238604") + (str_to_INTERVAL((char*)"-2632044912378") + (str_to_INTERVAL((char*)"147366946986") + str_to_INTERVAL((char*)"2813591214") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"13896370176") + (str_to_INTERVAL((char*)"795082125312") + (str_to_INTERVAL((char*)"-1850725102080") + (str_to_INTERVAL((char*)"210615872231936") + (str_to_INTERVAL((char*)"-132702813610368") + (str_to_INTERVAL((char*)"-1484183319333568") + (str_to_INTERVAL((char*)"-1708132857475808") + (str_to_INTERVAL((char*)"322015400435168") + (str_to_INTERVAL((char*)"302330047711320") + (str_to_INTERVAL((char*)"-53887890128172") + (str_to_INTERVAL((char*)"-27926659988860") + (str_to_INTERVAL((char*)"-566273982656") + str_to_INTERVAL((char*)"73333519290") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"749076998144") + (str_to_INTERVAL((char*)"-12785367546368") + (str_to_INTERVAL((char*)"228445628513280") + (str_to_INTERVAL((char*)"189931089118592") + (str_to_INTERVAL((char*)"-2360711828870080") + (str_to_INTERVAL((char*)"-5293084892813120") + (str_to_INTERVAL((char*)"-1408227861520608") + (str_to_INTERVAL((char*)"1566185746321136") + (str_to_INTERVAL((char*)"42610616960068") + (str_to_INTERVAL((char*)"-151752239768338") + (str_to_INTERVAL((char*)"-18905554174816") + (str_to_INTERVAL((char*)"978722654178") + str_to_INTERVAL((char*)"22633839882") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-2818720956416") + (str_to_INTERVAL((char*)"66189077208064") + (str_to_INTERVAL((char*)"366653888871936") + (str_to_INTERVAL((char*)"-1673228349744640") + (str_to_INTERVAL((char*)"-6533557816483968") + (str_to_INTERVAL((char*)"-5891021259788928") + (str_to_INTERVAL((char*)"1811085277206512") + (str_to_INTERVAL((char*)"1048500605351072") + (str_to_INTERVAL((char*)"-325147948094276") + (str_to_INTERVAL((char*)"-93344143957574") + (str_to_INTERVAL((char*)"-2199762188746") + str_to_INTERVAL((char*)"261209936574") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"5687427918848") + (str_to_INTERVAL((char*)"109640103725568") + (str_to_INTERVAL((char*)"-526143429370624") + (str_to_INTERVAL((char*)"-3276339108939520") + (str_to_INTERVAL((char*)"-6086051124376704") + (str_to_INTERVAL((char*)"-1155223218014528") + (str_to_INTERVAL((char*)"1838287266999456") + (str_to_INTERVAL((char*)"-55897178996672") + (str_to_INTERVAL((char*)"-188612576519328") + (str_to_INTERVAL((char*)"-20351767493512") + (str_to_INTERVAL((char*)"827218913352") + str_to_INTERVAL((char*)"36805189824") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"4624805965824") + (str_to_INTERVAL((char*)"-71896767810560") + (str_to_INTERVAL((char*)"-609706097837056") + (str_to_INTERVAL((char*)"-1984912477267456") + (str_to_INTERVAL((char*)"-1800353033380352") + (str_to_INTERVAL((char*)"549394847069376") + (str_to_INTERVAL((char*)"312891571421616") + (str_to_INTERVAL((char*)"-118664469455136") + (str_to_INTERVAL((char*)"-26979812375868") + (str_to_INTERVAL((char*)"-2788253429544") + str_to_INTERVAL((char*)"168933106188") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-2481156122624") + (str_to_INTERVAL((char*)"-28859292790272") + (str_to_INTERVAL((char*)"-167438970913280") + (str_to_INTERVAL((char*)"-412344307312128") + (str_to_INTERVAL((char*)"-138088180758720") + (str_to_INTERVAL((char*)"121123261836256") + (str_to_INTERVAL((char*)"-9865034292160") + (str_to_INTERVAL((char*)"-3934241152608") + (str_to_INTERVAL((char*)"-5893635975912") + (str_to_INTERVAL((char*)"127327491912") + str_to_INTERVAL((char*)"14700499272") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-35523291136") + (str_to_INTERVAL((char*)"-2317356315136") + (str_to_INTERVAL((char*)"-22092549895424") + (str_to_INTERVAL((char*)"-32883301209856") + (str_to_INTERVAL((char*)"3159177893312") + (str_to_INTERVAL((char*)"-3039883787936") + (str_to_INTERVAL((char*)"5374858567232") + (str_to_INTERVAL((char*)"-1975915760032") + (str_to_INTERVAL((char*)"-461211033280") + str_to_INTERVAL((char*)"30190299912") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-16504866816") + (str_to_INTERVAL((char*)"-329170274304") + (str_to_INTERVAL((char*)"-1012016244224") + (str_to_INTERVAL((char*)"-471733102080") + (str_to_INTERVAL((char*)"-1930534033408") + (str_to_INTERVAL((char*)"1177391833408") + (str_to_INTERVAL((char*)"275158381312") + (str_to_INTERVAL((char*)"-336563423232") + (str_to_INTERVAL((char*)"13259541312") + str_to_INTERVAL((char*)"721304640") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"212524032") + (str_to_INTERVAL((char*)"3514251264") + (str_to_INTERVAL((char*)"-15081985536") + (str_to_INTERVAL((char*)"-140185333760") + (str_to_INTERVAL((char*)"34030909184") + (str_to_INTERVAL((char*)"68104424704") + (str_to_INTERVAL((char*)"-35960720128") + (str_to_INTERVAL((char*)"-1033177600") + str_to_INTERVAL((char*)"237070080") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"98403840") + (str_to_INTERVAL((char*)"-94008960") + (str_to_INTERVAL((char*)"18869760") + (str_to_INTERVAL((char*)"7061760") + (str_to_INTERVAL((char*)"-921600") + str_to_INTERVAL((char*)"86400") * b) * b) * b) * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[18] = (str_to_INTERVAL((char*)"99656663040") + (str_to_INTERVAL((char*)"289278001152") + (str_to_INTERVAL((char*)"-11277961265152") + (str_to_INTERVAL((char*)"1741870333952") + (str_to_INTERVAL((char*)"122411406852096") + (str_to_INTERVAL((char*)"121317831081984") + (str_to_INTERVAL((char*)"-57648443523072") + (str_to_INTERVAL((char*)"1060991811584") + (str_to_INTERVAL((char*)"698819846144") + (str_to_INTERVAL((char*)"2591138856960") + (str_to_INTERVAL((char*)"-142707621888") - str_to_INTERVAL((char*)"2435088384") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-1303463854080") + (str_to_INTERVAL((char*)"17035048255488") + (str_to_INTERVAL((char*)"-103672049565696") + (str_to_INTERVAL((char*)"-308882163367936") + (str_to_INTERVAL((char*)"839798154592256") + (str_to_INTERVAL((char*)"2294792792965120") + (str_to_INTERVAL((char*)"265263198568448") + (str_to_INTERVAL((char*)"-441786559987712") + (str_to_INTERVAL((char*)"-18392116101120") + (str_to_INTERVAL((char*)"47024188596224") + (str_to_INTERVAL((char*)"1962234974208") - str_to_INTERVAL((char*)"141853409280") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-18220182405120") + (str_to_INTERVAL((char*)"205917555523584") + (str_to_INTERVAL((char*)"-1003360426131456") + (str_to_INTERVAL((char*)"-8262522361872384") + (str_to_INTERVAL((char*)"7518950909804544") + (str_to_INTERVAL((char*)"63228875292147712") + (str_to_INTERVAL((char*)"42772432464740352") + (str_to_INTERVAL((char*)"-21837658673922048") + (str_to_INTERVAL((char*)"-1772982131769344") + (str_to_INTERVAL((char*)"1165149588733952") + (str_to_INTERVAL((char*)"658710761490432") + (str_to_INTERVAL((char*)"-36532287611904") - str_to_INTERVAL((char*)"665542361088") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-36711589478400") + (str_to_INTERVAL((char*)"-134401071513600") + (str_to_INTERVAL((char*)"3958351488614400") + (str_to_INTERVAL((char*)"-53739921888772096") + (str_to_INTERVAL((char*)"-38833647455043584") + (str_to_INTERVAL((char*)"325661151971639296") + (str_to_INTERVAL((char*)"541523178740252672") + (str_to_INTERVAL((char*)"-20951925280079872") + (str_to_INTERVAL((char*)"-99408432184623104") + (str_to_INTERVAL((char*)"5938095091458048") + (str_to_INTERVAL((char*)"9569774695137280") + (str_to_INTERVAL((char*)"266299783313408") - str_to_INTERVAL((char*)"26327052112896") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-26186486906880") + (str_to_INTERVAL((char*)"-407273773989888") + (str_to_INTERVAL((char*)"7162335908069376") + (str_to_INTERVAL((char*)"-94990356524302336") + (str_to_INTERVAL((char*)"-393323590988595200") + (str_to_INTERVAL((char*)"875616621153615872") + (str_to_INTERVAL((char*)"3110017018463977472") + (str_to_INTERVAL((char*)"1373400301265715200") + (str_to_INTERVAL((char*)"-922751242550411264") + (str_to_INTERVAL((char*)"-93755022666801152") + (str_to_INTERVAL((char*)"81253914492428288") + (str_to_INTERVAL((char*)"16345102698631168") + (str_to_INTERVAL((char*)"-815828991062016") - str_to_INTERVAL((char*)"16710068809728") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-49411774218240") + (str_to_INTERVAL((char*)"-2893757779279872") + (str_to_INTERVAL((char*)"3281120413089792") + (str_to_INTERVAL((char*)"-976084296261435392") + (str_to_INTERVAL((char*)"671334197090320384") + (str_to_INTERVAL((char*)"7400723912435826688") + (str_to_INTERVAL((char*)"8441062217392390144") + (str_to_INTERVAL((char*)"-1492652631080304640") + (str_to_INTERVAL((char*)"-1554310984366669824") + (str_to_INTERVAL((char*)"277060691582918656") + (str_to_INTERVAL((char*)"138441936166715392") + (str_to_INTERVAL((char*)"2702060164792320") - str_to_INTERVAL((char*)"351603377270784") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1362949780275200") + (str_to_INTERVAL((char*)"29874109479387136") + (str_to_INTERVAL((char*)"-695589651585433600") + (str_to_INTERVAL((char*)"-549849821233545216") + (str_to_INTERVAL((char*)"8292803975220297728") + (str_to_INTERVAL((char*)"18092909087758352384") + (str_to_INTERVAL((char*)"4998287617112342528") + (str_to_INTERVAL((char*)"-5337691446563602432") + (str_to_INTERVAL((char*)"-205356883933593600") + (str_to_INTERVAL((char*)"540882188384468992") + (str_to_INTERVAL((char*)"61064494222114816") + (str_to_INTERVAL((char*)"-2822232546926592") - str_to_INTERVAL((char*)"74704820502528") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"4268073452306432") + (str_to_INTERVAL((char*)"-151671409999872000") + (str_to_INTERVAL((char*)"-671870910364385280") + (str_to_INTERVAL((char*)"4243565784300781568") + (str_to_INTERVAL((char*)"15584925370553729024") + (str_to_INTERVAL((char*)"14059227474082398208") + (str_to_INTERVAL((char*)"-3980240228102209536") + (str_to_INTERVAL((char*)"-2629228055720034304") + (str_to_INTERVAL((char*)"789322187909226496") + (str_to_INTERVAL((char*)"223878081944293376") + (str_to_INTERVAL((char*)"6096460501741568") - str_to_INTERVAL((char*)"646102837776384") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-9177463869407232") + (str_to_INTERVAL((char*)"-113169276740829184") + (str_to_INTERVAL((char*)"969973269826895872") + (str_to_INTERVAL((char*)"5277800922170523648") + (str_to_INTERVAL((char*)"9672761708521717760") + (str_to_INTERVAL((char*)"2153313899129405440") + (str_to_INTERVAL((char*)"-2912662294216048640") + (str_to_INTERVAL((char*)"50713767677460480") + (str_to_INTERVAL((char*)"303776371606618112") + (str_to_INTERVAL((char*)"35677586006441984") + (str_to_INTERVAL((char*)"-1092115294076928") - str_to_INTERVAL((char*)"74438540034048") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1467823398322176") + (str_to_INTERVAL((char*)"87130448762765312") + (str_to_INTERVAL((char*)"603172583536328704") + (str_to_INTERVAL((char*)"1955713961014329344") + (str_to_INTERVAL((char*)"1933666594228273152") + (str_to_INTERVAL((char*)"-443541337588498432") + (str_to_INTERVAL((char*)"-350745771141332992") + (str_to_INTERVAL((char*)"116385285702615040") + (str_to_INTERVAL((char*)"28399100487802880") + (str_to_INTERVAL((char*)"4788997478170624") - str_to_INTERVAL((char*)"263499378941952") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1308998183682048") + (str_to_INTERVAL((char*)"12378620137832448") + (str_to_INTERVAL((char*)"88835055337603072") + (str_to_INTERVAL((char*)"265411132153397248") + (str_to_INTERVAL((char*)"123820667673247744") + (str_to_INTERVAL((char*)"-76777654203056128") + (str_to_INTERVAL((char*)"9124799239421952") + (str_to_INTERVAL((char*)"-4223417955516416") + (str_to_INTERVAL((char*)"6921913628753920") + (str_to_INTERVAL((char*)"-117627186216960") - str_to_INTERVAL((char*)"18289578147840") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-18788544151552") + (str_to_INTERVAL((char*)"898229347549184") + (str_to_INTERVAL((char*)"9525137898995712") + (str_to_INTERVAL((char*)"14918160650600448") + (str_to_INTERVAL((char*)"-429873576148992") + (str_to_INTERVAL((char*)"5124454597328896") + (str_to_INTERVAL((char*)"-6050008951029760") + (str_to_INTERVAL((char*)"1757579046223872") + (str_to_INTERVAL((char*)"383385070338048") - str_to_INTERVAL((char*)"26881178222592") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"3312984260608") + (str_to_INTERVAL((char*)"48280530583552") + (str_to_INTERVAL((char*)"80025101533184") + (str_to_INTERVAL((char*)"218517421948928") + (str_to_INTERVAL((char*)"1150896530784256") + (str_to_INTERVAL((char*)"-785180668592128") + (str_to_INTERVAL((char*)"-75373052428288") + (str_to_INTERVAL((char*)"167275830837248") + (str_to_INTERVAL((char*)"-7045756354560") - str_to_INTERVAL((char*)"358382960640") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"6058672128") + (str_to_INTERVAL((char*)"665610878976") + (str_to_INTERVAL((char*)"9906913542144") + (str_to_INTERVAL((char*)"25327061434368") + (str_to_INTERVAL((char*)"-14172228943872") + (str_to_INTERVAL((char*)"-8183227613184") + (str_to_INTERVAL((char*)"6251132485632") + (str_to_INTERVAL((char*)"114787614720") - str_to_INTERVAL((char*)"39435632640") * b) * b) * b) * b) * b) * b) * b) * b) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a;
coef[19] = 512 * a * ((str_to_INTERVAL((char*)"-2627665920") + (str_to_INTERVAL((char*)"-7627447296") + (str_to_INTERVAL((char*)"297368119296") + (str_to_INTERVAL((char*)"-45928221696") + (str_to_INTERVAL((char*)"-3227644516608") + (str_to_INTERVAL((char*)"-3198809999232") + (str_to_INTERVAL((char*)"1520027319456") + (str_to_INTERVAL((char*)"-27975370032") + (str_to_INTERVAL((char*)"-18425913912") + (str_to_INTERVAL((char*)"-68321044080") + (str_to_INTERVAL((char*)"3762798624") + str_to_INTERVAL((char*)"64206432") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"31822848000") + (str_to_INTERVAL((char*)"-415894732800") + (str_to_INTERVAL((char*)"2531055897600") + (str_to_INTERVAL((char*)"7541068441600") + (str_to_INTERVAL((char*)"-20502884633600") + (str_to_INTERVAL((char*)"-56025214672000") + (str_to_INTERVAL((char*)"-6476152308800") + (str_to_INTERVAL((char*)"10785804687200") + (str_to_INTERVAL((char*)"449026272000") + (str_to_INTERVAL((char*)"-1148051479400") + (str_to_INTERVAL((char*)"-47906127300") + str_to_INTERVAL((char*)"3463218000") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"170142474240") + (str_to_INTERVAL((char*)"-1548880416768") + (str_to_INTERVAL((char*)"6181231078912") + (str_to_INTERVAL((char*)"91711526042368") + (str_to_INTERVAL((char*)"-65600371698688") + (str_to_INTERVAL((char*)"-626040185028224") + (str_to_INTERVAL((char*)"-434227170297504") + (str_to_INTERVAL((char*)"200975332858096") + (str_to_INTERVAL((char*)"28535623823088") + (str_to_INTERVAL((char*)"-13495422519504") + (str_to_INTERVAL((char*)"-5842282688814") + (str_to_INTERVAL((char*)"288483790983") + str_to_INTERVAL((char*)"5606878176") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"237766717440") + (str_to_INTERVAL((char*)"738342549504") + (str_to_INTERVAL((char*)"-24942685128704") + (str_to_INTERVAL((char*)"374955155391488") + (str_to_INTERVAL((char*)"320415114874624") + (str_to_INTERVAL((char*)"-2420313271374208") + (str_to_INTERVAL((char*)"-4009109819715840") + (str_to_INTERVAL((char*)"113957163730816") + (str_to_INTERVAL((char*)"757760854802112") + (str_to_INTERVAL((char*)"-46597425363608") + (str_to_INTERVAL((char*)"-70751932723024") + (str_to_INTERVAL((char*)"-1914682467810") + str_to_INTERVAL((char*)"188482681611") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"87863685120") + (str_to_INTERVAL((char*)"1197883312128") + (str_to_INTERVAL((char*)"-22157623796736") + (str_to_INTERVAL((char*)"395986477329920") + (str_to_INTERVAL((char*)"2039291867639296") + (str_to_INTERVAL((char*)"-4428425428978048") + (str_to_INTERVAL((char*)"-15532403085049280") + (str_to_INTERVAL((char*)"-6963193114649056") + (str_to_INTERVAL((char*)"4515221767692800") + (str_to_INTERVAL((char*)"561512546122648") + (str_to_INTERVAL((char*)"-430964003673656") + (str_to_INTERVAL((char*)"-74902626961092") + (str_to_INTERVAL((char*)"3327428482188") + str_to_INTERVAL((char*)"73688675004") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"107352907776") + (str_to_INTERVAL((char*)"6684234153984") + (str_to_INTERVAL((char*)"12040098017280") + (str_to_INTERVAL((char*)"3236622936355840") + (str_to_INTERVAL((char*)"-2536752447294464") + (str_to_INTERVAL((char*)"-26828737744827392") + (str_to_INTERVAL((char*)"-30244015404109568") + (str_to_INTERVAL((char*)"4946994864900032") + (str_to_INTERVAL((char*)"5797380310818336") + (str_to_INTERVAL((char*)"-1030871444776928") + (str_to_INTERVAL((char*)"-498397645571000") + (str_to_INTERVAL((char*)"-9378382944048") + str_to_INTERVAL((char*)"1220115006864") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1607211720704") + (str_to_INTERVAL((char*)"-46086562267136") + (str_to_INTERVAL((char*)"1576602682733568") + (str_to_INTERVAL((char*)"952341804833792") + (str_to_INTERVAL((char*)"-21657706670603008") + (str_to_INTERVAL((char*)"-45747875084774528") + (str_to_INTERVAL((char*)"-13102577002744896") + (str_to_INTERVAL((char*)"13368278148205280") + (str_to_INTERVAL((char*)"680749103389504") + (str_to_INTERVAL((char*)"-1416363503342080") + (str_to_INTERVAL((char*)"-148776301545760") + (str_to_INTERVAL((char*)"6076232813064") + str_to_INTERVAL((char*)"190903566528") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-3859075956736") + (str_to_INTERVAL((char*)"246475825516544") + (str_to_INTERVAL((char*)"785870602010624") + (str_to_INTERVAL((char*)"-7983460371248128") + (str_to_INTERVAL((char*)"-27349310670596096") + (str_to_INTERVAL((char*)"-24694689959070976") + (str_to_INTERVAL((char*)"6256226580726720") + (str_to_INTERVAL((char*)"4850458625903488") + (str_to_INTERVAL((char*)"-1402455515396624") + (str_to_INTERVAL((char*)"-395251458309032") + (str_to_INTERVAL((char*)"-13326478817912") + str_to_INTERVAL((char*)"1217611673928") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"9179873488896") + (str_to_INTERVAL((char*)"59623282104320") + (str_to_INTERVAL((char*)"-1278397819821056") + (str_to_INTERVAL((char*)"-6101994300728832") + (str_to_INTERVAL((char*)"-11071842684514304") + (str_to_INTERVAL((char*)"-2988009510724480") + (str_to_INTERVAL((char*)"3316254329201792") + (str_to_INTERVAL((char*)"-1785058571648") + (str_to_INTERVAL((char*)"-348906066481280") + (str_to_INTERVAL((char*)"-48586485285360") + (str_to_INTERVAL((char*)"908012920848") + str_to_INTERVAL((char*)"120640218336") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-770490163200") + (str_to_INTERVAL((char*)"-66263705440256") + (str_to_INTERVAL((char*)"-393186522105856") + (str_to_INTERVAL((char*)"-1321978188233728") + (str_to_INTERVAL((char*)"-1517809435590656") + (str_to_INTERVAL((char*)"206515103407872") + (str_to_INTERVAL((char*)"281826518482176") + (str_to_INTERVAL((char*)"-77604520283904") + (str_to_INTERVAL((char*)"-21310162362720") + (str_to_INTERVAL((char*)"-6365741563008") + str_to_INTERVAL((char*)"330350774976") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-331593244672") + (str_to_INTERVAL((char*)"-2245041958912") + (str_to_INTERVAL((char*)"-31640665938944") + (str_to_INTERVAL((char*)"-128826862286336") + (str_to_INTERVAL((char*)"-75342495621120") + (str_to_INTERVAL((char*)"33704405036800") + (str_to_INTERVAL((char*)"-7694299707392") + (str_to_INTERVAL((char*)"8852135277440") + (str_to_INTERVAL((char*)"-6375517841664") + (str_to_INTERVAL((char*)"92168008704") + str_to_INTERVAL((char*)"17458687872") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"814108672") + (str_to_INTERVAL((char*)"-341034833920") + (str_to_INTERVAL((char*)"-2960400199680") + (str_to_INTERVAL((char*)"-3829062009856") + (str_to_INTERVAL((char*)"-1070529030144") + (str_to_INTERVAL((char*)"-3845343578368") + (str_to_INTERVAL((char*)"4405067476480") + (str_to_INTERVAL((char*)"-1200173101184") + (str_to_INTERVAL((char*)"-225650191360") + str_to_INTERVAL((char*)"17133770880") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"47554560") + (str_to_INTERVAL((char*)"4229505024") + (str_to_INTERVAL((char*)"24562630656") + (str_to_INTERVAL((char*)"-143960444928") + (str_to_INTERVAL((char*)"-339012472320") + (str_to_INTERVAL((char*)"302653148928") + (str_to_INTERVAL((char*)"-10187063808") + (str_to_INTERVAL((char*)"-49674246144") + (str_to_INTERVAL((char*)"2284474368") + str_to_INTERVAL((char*)"106502400") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"98403840") + (str_to_INTERVAL((char*)"-118609920") + (str_to_INTERVAL((char*)"48522240") + (str_to_INTERVAL((char*)"-5068800") + str_to_INTERVAL((char*)"345600") * b) * b) * b) * b) *  pow( b,  4) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[20] = (str_to_INTERVAL((char*)"37371248640") + (str_to_INTERVAL((char*)"-551842480128") + (str_to_INTERVAL((char*)"-597112651776") + (str_to_INTERVAL((char*)"14248952725504") + (str_to_INTERVAL((char*)"20048955441152") + (str_to_INTERVAL((char*)"-10584988450816") + (str_to_INTERVAL((char*)"1921764433920") + (str_to_INTERVAL((char*)"-255966220288") + (str_to_INTERVAL((char*)"656612990976") + (str_to_INTERVAL((char*)"-34477658112") - str_to_INTERVAL((char*)"545218560") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"720953671680") + (str_to_INTERVAL((char*)"-364042518528") + (str_to_INTERVAL((char*)"-49303485677568") + (str_to_INTERVAL((char*)"76614486982656") + (str_to_INTERVAL((char*)"418922765484032") + (str_to_INTERVAL((char*)"119030274752512") + (str_to_INTERVAL((char*)"-87225209683968") + (str_to_INTERVAL((char*)"-10524932964352") + (str_to_INTERVAL((char*)"10593212489728") + (str_to_INTERVAL((char*)"710876092416") - str_to_INTERVAL((char*)"36522676224") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"4922104872960") + (str_to_INTERVAL((char*)"71851687280640") + (str_to_INTERVAL((char*)"-1277889273659392") + (str_to_INTERVAL((char*)"-217783383883776") + (str_to_INTERVAL((char*)"12750268576890880") + (str_to_INTERVAL((char*)"13126070983720960") + (str_to_INTERVAL((char*)"-5399179326316544") + (str_to_INTERVAL((char*)"-281779155021824") + (str_to_INTERVAL((char*)"119648965228544") + (str_to_INTERVAL((char*)"250755124492288") + (str_to_INTERVAL((char*)"-12761824284672") - str_to_INTERVAL((char*)"229302632448") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-102150061424640") + (str_to_INTERVAL((char*)"1280665442058240") + (str_to_INTERVAL((char*)"-7712757661827072") + (str_to_INTERVAL((char*)"-25632841726951424") + (str_to_INTERVAL((char*)"65540196520951808") + (str_to_INTERVAL((char*)"181411051743641600") + (str_to_INTERVAL((char*)"22653721718571008") + (str_to_INTERVAL((char*)"-35453169887240192") + (str_to_INTERVAL((char*)"-1488670127407104") + (str_to_INTERVAL((char*)"3706241206290432") + (str_to_INTERVAL((char*)"154246893446144") - str_to_INTERVAL((char*)"10994986063872") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-306522378731520") + (str_to_INTERVAL((char*)"2220770761113600") + (str_to_INTERVAL((char*)"-6203469735657472") + (str_to_INTERVAL((char*)"-191913099611865088") + (str_to_INTERVAL((char*)"110528411781955584") + (str_to_INTERVAL((char*)"1212647359009423360") + (str_to_INTERVAL((char*)"859117925590335488") + (str_to_INTERVAL((char*)"-363401764463173632") + (str_to_INTERVAL((char*)"-73854698402021376") + (str_to_INTERVAL((char*)"29453688239693824") + (str_to_INTERVAL((char*)"10196358656892928") + (str_to_INTERVAL((char*)"-438884314736640") - str_to_INTERVAL((char*)"9240806707200") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-279600074588160") + (str_to_INTERVAL((char*)"-611307947556864") + (str_to_INTERVAL((char*)"27977675429642240") + (str_to_INTERVAL((char*)"-493860440570855424") + (str_to_INTERVAL((char*)"-509913524220329984") + (str_to_INTERVAL((char*)"3465530043449016320") + (str_to_INTERVAL((char*)"5710092762027458560") + (str_to_INTERVAL((char*)"-89944465632296960") + (str_to_INTERVAL((char*)"-1116778494425382912") + (str_to_INTERVAL((char*)"70727970961338368") + (str_to_INTERVAL((char*)"100643648874700800") + (str_to_INTERVAL((char*)"2626144710579200") - str_to_INTERVAL((char*)"257139266950656") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-58931344834560") + (str_to_INTERVAL((char*)"-627328831979520") + (str_to_INTERVAL((char*)"13529578124869632") + (str_to_INTERVAL((char*)"-347173535757369344") + (str_to_INTERVAL((char*)"-2071644760537300992") + (str_to_INTERVAL((char*)"4572689944701632512") + (str_to_INTERVAL((char*)"15802384731058372608") + (str_to_INTERVAL((char*)"7162144335288385536") + (str_to_INTERVAL((char*)"-4499330231932829696") + (str_to_INTERVAL((char*)"-651052524710645760") + (str_to_INTERVAL((char*)"458918452884459520") + (str_to_INTERVAL((char*)"71200646908037120") + (str_to_INTERVAL((char*)"-2835605330933760") - str_to_INTERVAL((char*)"68152909768704") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-34573538820096") + (str_to_INTERVAL((char*)"-2646267723251712") + (str_to_INTERVAL((char*)"-27143689910353920") + (str_to_INTERVAL((char*)"-2170253691537850368") + (str_to_INTERVAL((char*)"2042666563992027136") + (str_to_INTERVAL((char*)"19944991601875091456") + (str_to_INTERVAL((char*)"22151502674723864576") + (str_to_INTERVAL((char*)"-3309477599479496704") + (str_to_INTERVAL((char*)"-4403912698695532544") + (str_to_INTERVAL((char*)"777931855038316544") + (str_to_INTERVAL((char*)"367632162240917504") + (str_to_INTERVAL((char*)"6746911528181760") - str_to_INTERVAL((char*)"867979624863744") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-312998623182848") + (str_to_INTERVAL((char*)"11007113736224768") + (str_to_INTERVAL((char*)"-729728794722041856") + (str_to_INTERVAL((char*)"-173028841693904896") + (str_to_INTERVAL((char*)"11661364119715840000") + (str_to_INTERVAL((char*)"23706456641036025856") + (str_to_INTERVAL((char*)"7051113094841925632") + (str_to_INTERVAL((char*)"-6811063667601162240") + (str_to_INTERVAL((char*)"-448292478614388736") + (str_to_INTERVAL((char*)"757512956569182208") + (str_to_INTERVAL((char*)"75690629942149120") + (str_to_INTERVAL((char*)"-2662319939776512") - str_to_INTERVAL((char*)"105253113526272") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"405438885527552") + (str_to_INTERVAL((char*)"-76800413573054464") + (str_to_INTERVAL((char*)"-126202791532167168") + (str_to_INTERVAL((char*)"3057435640030822400") + (str_to_INTERVAL((char*)"9692246547826737152") + (str_to_INTERVAL((char*)"8783944721987796992") + (str_to_INTERVAL((char*)"-1887885000033271808") + (str_to_INTERVAL((char*)"-1817225307640856576") + (str_to_INTERVAL((char*)"501773561790013440") + (str_to_INTERVAL((char*)"141256650411814912") + (str_to_INTERVAL((char*)"6383658181120000") - str_to_INTERVAL((char*)"489653921378304") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1536375885135872") + (str_to_INTERVAL((char*)"1943313464688640") + (str_to_INTERVAL((char*)"322768936876113920") + (str_to_INTERVAL((char*)"1361551536567091200") + (str_to_INTERVAL((char*)"2480278796516458496") + (str_to_INTERVAL((char*)"857809335441817600") + (str_to_INTERVAL((char*)"-744868400418521088") + (str_to_INTERVAL((char*)"-15279789133398016") + (str_to_INTERVAL((char*)"77046544879583232") + (str_to_INTERVAL((char*)"14381846065307648") + (str_to_INTERVAL((char*)"-68044607152128") - str_to_INTERVAL((char*)"43331316498432") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"251776812974080") + (str_to_INTERVAL((char*)"8231805213736960") + (str_to_INTERVAL((char*)"42515223506059264") + (str_to_INTERVAL((char*)"165088790840344576") + (str_to_INTERVAL((char*)"244607117085114368") + (str_to_INTERVAL((char*)"-9953325330595840") + (str_to_INTERVAL((char*)"-43351307384258560") + (str_to_INTERVAL((char*)"8753736830550016") + (str_to_INTERVAL((char*)"3160141526073344") + (str_to_INTERVAL((char*)"1793704069791744") - str_to_INTERVAL((char*)"90909931438080") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"5035876417536") + (str_to_INTERVAL((char*)"-25777303191552") + (str_to_INTERVAL((char*)"2594593115209728") + (str_to_INTERVAL((char*)"13521822379409408") + (str_to_INTERVAL((char*)"7710864437280768") + (str_to_INTERVAL((char*)"-1824746686054400") + (str_to_INTERVAL((char*)"1156212475297792") + (str_to_INTERVAL((char*)"-2218633975627776") + (str_to_INTERVAL((char*)"1235632893394944") + (str_to_INTERVAL((char*)"-16954843545600") - str_to_INTERVAL((char*)"3424859357184") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"368601726976") + (str_to_INTERVAL((char*)"18235125661696") + (str_to_INTERVAL((char*)"107445764489216") + (str_to_INTERVAL((char*)"45589863071744") + (str_to_INTERVAL((char*)"349752191614976") + (str_to_INTERVAL((char*)"404349841440768") + (str_to_INTERVAL((char*)"-558331514060800") + (str_to_INTERVAL((char*)"160693665333248") + (str_to_INTERVAL((char*)"24164628824064") - str_to_INTERVAL((char*)"2027080581120") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"36352032768") + (str_to_INTERVAL((char*)"1562118193152") + (str_to_INTERVAL((char*)"11427023683584") + (str_to_INTERVAL((char*)"9086860001280") + (str_to_INTERVAL((char*)"-13773463486464") + (str_to_INTERVAL((char*)"2179933667328") + (str_to_INTERVAL((char*)"1810181652480") + (str_to_INTERVAL((char*)"-91741224960") - str_to_INTERVAL((char*)"3981312000") * b) * b) * b) * b) * b) * b) * b) * b) * b * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a;
coef[21] = 256 * a * ((str_to_INTERVAL((char*)"-2116730880") + (str_to_INTERVAL((char*)"31256702976") + (str_to_INTERVAL((char*)"33820833792") + (str_to_INTERVAL((char*)"-807069587968") + (str_to_INTERVAL((char*)"-1135585366784") + (str_to_INTERVAL((char*)"599540361472") + (str_to_INTERVAL((char*)"-108849938640") + (str_to_INTERVAL((char*)"14498086696") + (str_to_INTERVAL((char*)"-37190970192") + (str_to_INTERVAL((char*)"1952836104") + str_to_INTERVAL((char*)"30881520") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-38019041280") + (str_to_INTERVAL((char*)"19197554688") + (str_to_INTERVAL((char*)"2599988502528") + (str_to_INTERVAL((char*)"-4040217086976") + (str_to_INTERVAL((char*)"-22091630211072") + (str_to_INTERVAL((char*)"-6276987145152") + (str_to_INTERVAL((char*)"4599766916928") + (str_to_INTERVAL((char*)"555025761792") + (str_to_INTERVAL((char*)"-558626439888") + (str_to_INTERVAL((char*)"-37487606436") + str_to_INTERVAL((char*)"1926000504") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"44326656000") + (str_to_INTERVAL((char*)"-2682076377600") + (str_to_INTERVAL((char*)"30182057600000") + (str_to_INTERVAL((char*)"15609511641600") + (str_to_INTERVAL((char*)"-272910218556800") + (str_to_INTERVAL((char*)"-294383684835200") + (str_to_INTERVAL((char*)"98962176768400") + (str_to_INTERVAL((char*)"16789412890600") + (str_to_INTERVAL((char*)"-3846093956500") + (str_to_INTERVAL((char*)"-4842456121100") + (str_to_INTERVAL((char*)"215499433800") + str_to_INTERVAL((char*)"4240720800") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"1661070274560") + (str_to_INTERVAL((char*)"-19267577487360") + (str_to_INTERVAL((char*)"113614393010688") + (str_to_INTERVAL((char*)"457697015325696") + (str_to_INTERVAL((char*)"-1057923129298432") + (str_to_INTERVAL((char*)"-2995004840222400") + (str_to_INTERVAL((char*)"-421841642916832") + (str_to_INTERVAL((char*)"600328834805968") + (str_to_INTERVAL((char*)"25563436355616") + (str_to_INTERVAL((char*)"-60870568084028") + (str_to_INTERVAL((char*)"-2521695954926") + str_to_INTERVAL((char*)"175299333588") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"3153374945280") + (str_to_INTERVAL((char*)"-19196310948864") + (str_to_INTERVAL((char*)"30354240333312") + (str_to_INTERVAL((char*)"2255284294733568") + (str_to_INTERVAL((char*)"-1072710915876096") + (str_to_INTERVAL((char*)"-13762145406199232") + (str_to_INTERVAL((char*)"-9909585688929696") + (str_to_INTERVAL((char*)"3888641753439744") + (str_to_INTERVAL((char*)"1005776721986512") + (str_to_INTERVAL((char*)"-364107813100864") + (str_to_INTERVAL((char*)"-106230125754186") + (str_to_INTERVAL((char*)"3981673388049") + str_to_INTERVAL((char*)"91151981928") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"1841922478080") + (str_to_INTERVAL((char*)"1484000446464") + (str_to_INTERVAL((char*)"-170833627774976") + (str_to_INTERVAL((char*)"3790044873944064") + (str_to_INTERVAL((char*)"4658670159581440") + (str_to_INTERVAL((char*)"-29255418011808896") + (str_to_INTERVAL((char*)"-47874734269907328") + (str_to_INTERVAL((char*)"119628426114752") + (str_to_INTERVAL((char*)"9676020493315168") + (str_to_INTERVAL((char*)"-625779452501240") + (str_to_INTERVAL((char*)"-843574525301296") + (str_to_INTERVAL((char*)"-21114250965362") + str_to_INTERVAL((char*)"2056456828575") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"234049167360") + (str_to_INTERVAL((char*)"1408420491264") + (str_to_INTERVAL((char*)"-48818867668992") + (str_to_INTERVAL((char*)"1896433458495488") + (str_to_INTERVAL((char*)"12284050259682816") + (str_to_INTERVAL((char*)"-28616325947510272") + (str_to_INTERVAL((char*)"-96958403647752576") + (str_to_INTERVAL((char*)"-44269961015580288") + (str_to_INTERVAL((char*)"26984496358093184") + (str_to_INTERVAL((char*)"4421513038422720") + (str_to_INTERVAL((char*)"-2917374339351184") + (str_to_INTERVAL((char*)"-414408545481200") + (str_to_INTERVAL((char*)"14941835736912") + str_to_INTERVAL((char*)"390965199912") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"7132594176") + (str_to_INTERVAL((char*)"4754047905792") + (str_to_INTERVAL((char*)"200711249633280") + (str_to_INTERVAL((char*)"8665943070749696") + (str_to_INTERVAL((char*)"-10347798672867328") + (str_to_INTERVAL((char*)"-89677367598669568") + (str_to_INTERVAL((char*)"-97745658765557248") + (str_to_INTERVAL((char*)"13131269264997568") + (str_to_INTERVAL((char*)"20043509476034400") + (str_to_INTERVAL((char*)"-3496680294876352") + (str_to_INTERVAL((char*)"-1639127847884680") + (str_to_INTERVAL((char*)"-29980150386576") + str_to_INTERVAL((char*)"3745001678580") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"233707814912") + (str_to_INTERVAL((char*)"-1664510754816") + (str_to_INTERVAL((char*)"1983987106709504") + (str_to_INTERVAL((char*)"-743875721059328") + (str_to_INTERVAL((char*)"-37769267794851840") + (str_to_INTERVAL((char*)"-73395445122731264") + (str_to_INTERVAL((char*)"-22800909854371584") + (str_to_INTERVAL((char*)"20544292704750464") + (str_to_INTERVAL((char*)"1748415652530976") + (str_to_INTERVAL((char*)"-2420351007108112") + (str_to_INTERVAL((char*)"-234725276141408") + (str_to_INTERVAL((char*)"6679696476336") + str_to_INTERVAL((char*)"368669418156") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"441656664064") + (str_to_INTERVAL((char*)"129192461246464") + (str_to_INTERVAL((char*)"-80820323291136") + (str_to_INTERVAL((char*)"-6856431695252480") + (str_to_INTERVAL((char*)"-19967318074806784") + (str_to_INTERVAL((char*)"-18323861041801216") + (str_to_INTERVAL((char*)"2962945917651456") + (str_to_INTERVAL((char*)"4036696533291840") + (str_to_INTERVAL((char*)"-1050537963720256") + (str_to_INTERVAL((char*)"-295321321820096") + (str_to_INTERVAL((char*)"-19542394518592") + str_to_INTERVAL((char*)"1245687018048") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1122719096832") + (str_to_INTERVAL((char*)"-19892466249728") + (str_to_INTERVAL((char*)"-436018586933248") + (str_to_INTERVAL((char*)"-1636094442971136") + (str_to_INTERVAL((char*)"-3115213638311936") + (str_to_INTERVAL((char*)"-1493083947864576") + (str_to_INTERVAL((char*)"970311066776320") + (str_to_INTERVAL((char*)"39052797592320") + (str_to_INTERVAL((char*)"-90198090665856") + (str_to_INTERVAL((char*)"-27492404451360") + (str_to_INTERVAL((char*)"-286817847072") + str_to_INTERVAL((char*)"100219915008") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-195525705728") + (str_to_INTERVAL((char*)"-4165898387456") + (str_to_INTERVAL((char*)"-17943892170752") + (str_to_INTERVAL((char*)"-114154923882496") + (str_to_INTERVAL((char*)"-242283242547200") + (str_to_INTERVAL((char*)"-2395837621248") + (str_to_INTERVAL((char*)"31986688769152") + (str_to_INTERVAL((char*)"-1515984695808") + (str_to_INTERVAL((char*)"-2840112844512") + (str_to_INTERVAL((char*)"-3092456079424") + str_to_INTERVAL((char*)"156969393120") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"2440650752") + (str_to_INTERVAL((char*)"279441408") + (str_to_INTERVAL((char*)"-1734733123584") + (str_to_INTERVAL((char*)"-7887134365696") + (str_to_INTERVAL((char*)"-2855249067008") + (str_to_INTERVAL((char*)"-2321832972288") + (str_to_INTERVAL((char*)"-312957470208") + (str_to_INTERVAL((char*)"2727328630528") + (str_to_INTERVAL((char*)"-1411630315904") + (str_to_INTERVAL((char*)"20671229952") + str_to_INTERVAL((char*)"3868850304") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"570654720") + (str_to_INTERVAL((char*)"18654105600") + (str_to_INTERVAL((char*)"34209054720") + (str_to_INTERVAL((char*)"-340383575040") + (str_to_INTERVAL((char*)"-119660252160") + (str_to_INTERVAL((char*)"314744420352") + (str_to_INTERVAL((char*)"-108335582976") + (str_to_INTERVAL((char*)"-12075806208") + str_to_INTERVAL((char*)"1158278400") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"19680768") + (str_to_INTERVAL((char*)"-23721984") + (str_to_INTERVAL((char*)"9704448") + (str_to_INTERVAL((char*)"-1013760") + str_to_INTERVAL((char*)"69120") * b) * b) * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[22] = (str_to_INTERVAL((char*)"-17694720000") + (str_to_INTERVAL((char*)"-54278356992") + (str_to_INTERVAL((char*)"1107812614144") + (str_to_INTERVAL((char*)"2058052927488") + (str_to_INTERVAL((char*)"-1739689754624") + (str_to_INTERVAL((char*)"698331480064") + (str_to_INTERVAL((char*)"-50633365504") + (str_to_INTERVAL((char*)"128141878272") + (str_to_INTERVAL((char*)"-6687682560") - str_to_INTERVAL((char*)"91680768") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"141274644480") + (str_to_INTERVAL((char*)"-2752876118016") + (str_to_INTERVAL((char*)"3754043899904") + (str_to_INTERVAL((char*)"50059058675712") + (str_to_INTERVAL((char*)"22070496854016") + (str_to_INTERVAL((char*)"-12219835392000") + (str_to_INTERVAL((char*)"-1096757694464") + (str_to_INTERVAL((char*)"1583488333824") + (str_to_INTERVAL((char*)"194956167168") - str_to_INTERVAL((char*)"6773317632") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"6009126912000") + (str_to_INTERVAL((char*)"-60761781043200") + (str_to_INTERVAL((char*)"-144555924258816") + (str_to_INTERVAL((char*)"1671799271456768") + (str_to_INTERVAL((char*)"2526916703813632") + (str_to_INTERVAL((char*)"-1065207566065664") + (str_to_INTERVAL((char*)"115607318435840") + (str_to_INTERVAL((char*)"-23433117086720") + (str_to_INTERVAL((char*)"72812534775808") + (str_to_INTERVAL((char*)"-3518226032640") - str_to_INTERVAL((char*)"59795380224") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"55770820116480") + (str_to_INTERVAL((char*)"45049777225728") + (str_to_INTERVAL((char*)"-4753780616724480") + (str_to_INTERVAL((char*)"6607747622240256") + (str_to_INTERVAL((char*)"38872501414592512") + (str_to_INTERVAL((char*)"11716978032123904") + (str_to_INTERVAL((char*)"-8170587815124992") + (str_to_INTERVAL((char*)"-1045823047081984") + (str_to_INTERVAL((char*)"978525402146816") + (str_to_INTERVAL((char*)"66395047306752") - str_to_INTERVAL((char*)"3335916672000") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-182222243758080") + (str_to_INTERVAL((char*)"3791934714544128") + (str_to_INTERVAL((char*)"-33888659133693952") + (str_to_INTERVAL((char*)"-27712758368305152") + (str_to_INTERVAL((char*)"287420333801930752") + (str_to_INTERVAL((char*)"323427824000737280") + (str_to_INTERVAL((char*)"-88790081452298240") + (str_to_INTERVAL((char*)"-27837685769107456") + (str_to_INTERVAL((char*)"5259984776790016") + (str_to_INTERVAL((char*)"4632808750049792") + (str_to_INTERVAL((char*)"-174952925279232") - str_to_INTERVAL((char*)"3864194095104") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-1297923141795840") + (str_to_INTERVAL((char*)"13683076887478272") + (str_to_INTERVAL((char*)"-78343530352607232") + (str_to_INTERVAL((char*)"-398713274516373504") + (str_to_INTERVAL((char*)"824516636985786368") + (str_to_INTERVAL((char*)"2403293821420290048") + (str_to_INTERVAL((char*)"384599670227673088") + (str_to_INTERVAL((char*)"-495924262055538688") + (str_to_INTERVAL((char*)"-21568392637288448") + (str_to_INTERVAL((char*)"48557130668440576") + (str_to_INTERVAL((char*)"1998251537371648") - str_to_INTERVAL((char*)"134673289191936") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-1614504756510720") + (str_to_INTERVAL((char*)"9110071129669632") + (str_to_INTERVAL((char*)"-7201100193333248") + (str_to_INTERVAL((char*)"-1319777304705892352") + (str_to_INTERVAL((char*)"523889891131129856") + (str_to_INTERVAL((char*)"8050658935365500928") + (str_to_INTERVAL((char*)"5867884642575499264") + (str_to_INTERVAL((char*)"-2163071067225309184") + (str_to_INTERVAL((char*)"-666111673876205568") + (str_to_INTERVAL((char*)"226764218443831296") + (str_to_INTERVAL((char*)"58144395683461120") + (str_to_INTERVAL((char*)"-1920091826463744") - str_to_INTERVAL((char*)"47530573461504") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-603580630302720") + (str_to_INTERVAL((char*)"717664204357632") + (str_to_INTERVAL((char*)"49481818131398656") + (str_to_INTERVAL((char*)"-1505617763889840128") + (str_to_INTERVAL((char*)"-2127789062915162112") + (str_to_INTERVAL((char*)"12853174684028534784") + (str_to_INTERVAL((char*)"20848337163151114240") + (str_to_INTERVAL((char*)"211968967133937664") + (str_to_INTERVAL((char*)"-4332082679182696448") + (str_to_INTERVAL((char*)"282281360488871936") + (str_to_INTERVAL((char*)"367860071297298432") + (str_to_INTERVAL((char*)"8806364859331072") - str_to_INTERVAL((char*)"853894343685888") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-47613235691520") + (str_to_INTERVAL((char*)"53210165280768") + (str_to_INTERVAL((char*)"8963270174048256") + (str_to_INTERVAL((char*)"-558758000051879936") + (str_to_INTERVAL((char*)"-3706410848505298944") + (str_to_INTERVAL((char*)"9463383420346040320") + (str_to_INTERVAL((char*)"31181136434343641088") + (str_to_INTERVAL((char*)"14291048749333970944") + (str_to_INTERVAL((char*)"-8449450888261304320") + (str_to_INTERVAL((char*)"-1544962158669799424") + (str_to_INTERVAL((char*)"964382157409443840") + (str_to_INTERVAL((char*)"127949876905185280") + (str_to_INTERVAL((char*)"-4208824104376320") - str_to_INTERVAL((char*)"120628354710528") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"19445242134528") + (str_to_INTERVAL((char*)"-234597758533632") + (str_to_INTERVAL((char*)"-63607990961307648") + (str_to_INTERVAL((char*)"-1769888989764648960") + (str_to_INTERVAL((char*)"2824827795331350528") + (str_to_INTERVAL((char*)"21062312063682805760") + (str_to_INTERVAL((char*)"22425794670798045184") + (str_to_INTERVAL((char*)"-2640023666140528640") + (str_to_INTERVAL((char*)"-4716816854380617728") + (str_to_INTERVAL((char*)"806021279586648064") + (str_to_INTERVAL((char*)"381787014320510976") + (str_to_INTERVAL((char*)"7188307457560576") - str_to_INTERVAL((char*)"848787618216960") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-2399708119040") + (str_to_INTERVAL((char*)"-3557326128676864") + (str_to_INTERVAL((char*)"-265680478784716800") + (str_to_INTERVAL((char*)"367397335962484736") + (str_to_INTERVAL((char*)"6293170316466585600") + (str_to_INTERVAL((char*)"11601581029639520256") + (str_to_INTERVAL((char*)"3812170143150014464") + (str_to_INTERVAL((char*)"-3125736307528220672") + (str_to_INTERVAL((char*)"-351774703471443968") + (str_to_INTERVAL((char*)"396883767530299392") + (str_to_INTERVAL((char*)"38110318939160576") + (str_to_INTERVAL((char*)"-742264091013120") - str_to_INTERVAL((char*)"71276433647616") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-107931431337984") + (str_to_INTERVAL((char*)"-9430968832098304") + (str_to_INTERVAL((char*)"45231032636538880") + (str_to_INTERVAL((char*)"755274409771270144") + (str_to_INTERVAL((char*)"2007891216731602944") + (str_to_INTERVAL((char*)"1909792535862706176") + (str_to_INTERVAL((char*)"-158205379321397248") + (str_to_INTERVAL((char*)"-459524497788092416") + (str_to_INTERVAL((char*)"110907946239868928") + (str_to_INTERVAL((char*)"30519221438488576") + (str_to_INTERVAL((char*)"3277269364563968") - str_to_INTERVAL((char*)"175584053551104") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-18478012563456") + (str_to_INTERVAL((char*)"1931950522630144") + (str_to_INTERVAL((char*)"25147306387439616") + (str_to_INTERVAL((char*)"83956495118237696") + (str_to_INTERVAL((char*)"187311817001795584") + (str_to_INTERVAL((char*)"134860238563246080") + (str_to_INTERVAL((char*)"-66073636353343488") + (str_to_INTERVAL((char*)"-2146387360546816") + (str_to_INTERVAL((char*)"3851066039566336") + (str_to_INTERVAL((char*)"2967833688997888") + (str_to_INTERVAL((char*)"73224766832640") - str_to_INTERVAL((char*)"12687727362048") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"3491334455296") + (str_to_INTERVAL((char*)"36657331961856") + (str_to_INTERVAL((char*)"98938314555392") + (str_to_INTERVAL((char*)"4561791692570624") + (str_to_INTERVAL((char*)"12421418953474048") + (str_to_INTERVAL((char*)"-417016320360448") + (str_to_INTERVAL((char*)"-62258280693760") + (str_to_INTERVAL((char*)"-760151252271104") + (str_to_INTERVAL((char*)"160576964796416") + (str_to_INTERVAL((char*)"275207578533888") - str_to_INTERVAL((char*)"14212197310464") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"16080961536") + (str_to_INTERVAL((char*)"2294525460480") + (str_to_INTERVAL((char*)"45050945863680") + (str_to_INTERVAL((char*)"127703288119296") + (str_to_INTERVAL((char*)"-9352482127872") + (str_to_INTERVAL((char*)"322831037104128") + (str_to_INTERVAL((char*)"-75728811196416") + (str_to_INTERVAL((char*)"-137269968961536") + (str_to_INTERVAL((char*)"76749800669184") + (str_to_INTERVAL((char*)"-1325600145408") - str_to_INTERVAL((char*)"203533516800") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"99968090112") + (str_to_INTERVAL((char*)"2174280007680") + (str_to_INTERVAL((char*)"7966304501760") + (str_to_INTERVAL((char*)"-1415992049664") + (str_to_INTERVAL((char*)"-4929079738368") + (str_to_INTERVAL((char*)"2326970302464") + (str_to_INTERVAL((char*)"179052871680") - str_to_INTERVAL((char*)"20652687360") * b) * b) * b) * b) * b) * b) * b) * b * b * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a;
coef[23] = 256 * a * ((str_to_INTERVAL((char*)"1071360000") + (str_to_INTERVAL((char*)"3286384896") + (str_to_INTERVAL((char*)"-67074591872") + (str_to_INTERVAL((char*)"-124608673344") + (str_to_INTERVAL((char*)"105332778112") + (str_to_INTERVAL((char*)"-42281788832") + (str_to_INTERVAL((char*)"3065692052") + (str_to_INTERVAL((char*)"-7758590286") + (str_to_INTERVAL((char*)"404918280") + str_to_INTERVAL((char*)"5550984") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-8001884160") + (str_to_INTERVAL((char*)"155924623872") + (str_to_INTERVAL((char*)"-212631392768") + (str_to_INTERVAL((char*)"-2835376370304") + (str_to_INTERVAL((char*)"-1250086735872") + (str_to_INTERVAL((char*)"692139114000") + (str_to_INTERVAL((char*)"62121041288") + (str_to_INTERVAL((char*)"-89689768908") + (str_to_INTERVAL((char*)"-11042439156") + str_to_INTERVAL((char*)"383644944") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-183533506560") + (str_to_INTERVAL((char*)"1235062259712") + (str_to_INTERVAL((char*)"5492353789440") + (str_to_INTERVAL((char*)"-37315905663744") + (str_to_INTERVAL((char*)"-61713494945280") + (str_to_INTERVAL((char*)"18402012469008") + (str_to_INTERVAL((char*)"761066451180") + (str_to_INTERVAL((char*)"322351322022") + (str_to_INTERVAL((char*)"-1496692391472") + (str_to_INTERVAL((char*)"62502776388") + str_to_INTERVAL((char*)"1207736244") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-663819840000") + (str_to_INTERVAL((char*)"-3239563699200") + (str_to_INTERVAL((char*)"91285483622400") + (str_to_INTERVAL((char*)"-103798834656000") + (str_to_INTERVAL((char*)"-701441388368000") + (str_to_INTERVAL((char*)"-232114597612400") + (str_to_INTERVAL((char*)"149800441826800") + (str_to_INTERVAL((char*)"21001683040400") + (str_to_INTERVAL((char*)"-17520628491100") + (str_to_INTERVAL((char*)"-1211366987325") + str_to_INTERVAL((char*)"58561528950") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"3527831646720") + (str_to_INTERVAL((char*)"-54567326356992") + (str_to_INTERVAL((char*)"430344352153088") + (str_to_INTERVAL((char*)"481096404188928") + (str_to_INTERVAL((char*)"-3574475955448448") + (str_to_INTERVAL((char*)"-4175482078098480") + (str_to_INTERVAL((char*)"951317370618040") + (str_to_INTERVAL((char*)"450325928642964") + (str_to_INTERVAL((char*)"-77732550844824") + (str_to_INTERVAL((char*)"-53454781264443") + (str_to_INTERVAL((char*)"1696495447398") + str_to_INTERVAL((char*)"42506097876") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"11827228815360") + (str_to_INTERVAL((char*)"-114038045915136") + (str_to_INTERVAL((char*)"632154729067008") + (str_to_INTERVAL((char*)"4074231944075776") + (str_to_INTERVAL((char*)"-7612268185322752") + (str_to_INTERVAL((char*)"-22953173869273984") + (str_to_INTERVAL((char*)"-4121398892437792") + (str_to_INTERVAL((char*)"4868647461254416") + (str_to_INTERVAL((char*)"218539015657368") + (str_to_INTERVAL((char*)"-461363859344640") + (str_to_INTERVAL((char*)"-18810942027178") + str_to_INTERVAL((char*)"1228028415075") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"9714615828480") + (str_to_INTERVAL((char*)"-56903099722752") + (str_to_INTERVAL((char*)"41568665640448") + (str_to_INTERVAL((char*)"9203864640332032") + (str_to_INTERVAL((char*)"-3072492835566080") + (str_to_INTERVAL((char*)"-57602458531543360") + (str_to_INTERVAL((char*)"-42351642716154912") + (str_to_INTERVAL((char*)"14799222959181264") + (str_to_INTERVAL((char*)"5212578680112984") + (str_to_INTERVAL((char*)"-1700225199044276") + (str_to_INTERVAL((char*)"-395386380514744") + (str_to_INTERVAL((char*)"11716533537036") + str_to_INTERVAL((char*)"310010114124") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"2333309829120") + (str_to_INTERVAL((char*)"-9412120535040") + (str_to_INTERVAL((char*)"-153544855846912") + (str_to_INTERVAL((char*)"7351589457047552") + (str_to_INTERVAL((char*)"11421265321548288") + (str_to_INTERVAL((char*)"-69534759693514752") + (str_to_INTERVAL((char*)"-111475333853911552") + (str_to_INTERVAL((char*)"-2416930301582848") + (str_to_INTERVAL((char*)"23649193778786048") + (str_to_INTERVAL((char*)"-1528183460438288") + (str_to_INTERVAL((char*)"-1973674203390528") + (str_to_INTERVAL((char*)"-45132565286860") + str_to_INTERVAL((char*)"4360473640242") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"115309854720") + (str_to_INTERVAL((char*)"-1394352119808") + (str_to_INTERVAL((char*)"-17884573937664") + (str_to_INTERVAL((char*)"2067314088617984") + (str_to_INTERVAL((char*)"13319644844656640") + (str_to_INTERVAL((char*)"-38870646748136448") + (str_to_INTERVAL((char*)"-123149791966921088") + (str_to_INTERVAL((char*)"-56433828023564480") + (str_to_INTERVAL((char*)"32290908314081216") + (str_to_INTERVAL((char*)"6547262656505488") + (str_to_INTERVAL((char*)"-3888289557380272") + (str_to_INTERVAL((char*)"-489945872708072") + (str_to_INTERVAL((char*)"14725164720804") + str_to_INTERVAL((char*)"468648373842") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-81542922240") + (str_to_INTERVAL((char*)"32475156480") + (str_to_INTERVAL((char*)"218324100194304") + (str_to_INTERVAL((char*)"4253350750172160") + (str_to_INTERVAL((char*)"-9554637197973504") + (str_to_INTERVAL((char*)"-60122986314796288") + (str_to_INTERVAL((char*)"-62193271240226048") + (str_to_INTERVAL((char*)"6097575756224576") + (str_to_INTERVAL((char*)"13360949646072288") + (str_to_INTERVAL((char*)"-2209272932089184") + (str_to_INTERVAL((char*)"-1082637603752952") + (str_to_INTERVAL((char*)"-21937573176896") + str_to_INTERVAL((char*)"2359716253596") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"16192421888") + (str_to_INTERVAL((char*)"11026773286912") + (str_to_INTERVAL((char*)"390222810783744") + (str_to_INTERVAL((char*)"-1242275767794688") + (str_to_INTERVAL((char*)"-12404381785438720") + (str_to_INTERVAL((char*)"-21515679808825088") + (str_to_INTERVAL((char*)"-7674078415446144") + (str_to_INTERVAL((char*)"5486628014399168") + (str_to_INTERVAL((char*)"857503376951296") + (str_to_INTERVAL((char*)"-775581928518304") + (str_to_INTERVAL((char*)"-75309319236256") + (str_to_INTERVAL((char*)"464476052976") + str_to_INTERVAL((char*)"179102388888") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"129549156352") + (str_to_INTERVAL((char*)"5873515208704") + (str_to_INTERVAL((char*)"-94096111804416") + (str_to_INTERVAL((char*)"-912763641266176") + (str_to_INTERVAL((char*)"-2203496631189504") + (str_to_INTERVAL((char*)"-2303316031828992") + (str_to_INTERVAL((char*)"-87474388581760") + (str_to_INTERVAL((char*)"633068956217984") + (str_to_INTERVAL((char*)"-140773517034656") + (str_to_INTERVAL((char*)"-35010993365264") + (str_to_INTERVAL((char*)"-6986555809168") + str_to_INTERVAL((char*)"324740760048") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-10696916992") + (str_to_INTERVAL((char*)"-1319579340800") + (str_to_INTERVAL((char*)"-12245907025920") + (str_to_INTERVAL((char*)"-36204790892544") + (str_to_INTERVAL((char*)"-132066705675264") + (str_to_INTERVAL((char*)"-142869445126144") + (str_to_INTERVAL((char*)"59906824978176") + (str_to_INTERVAL((char*)"-3460000669440") + (str_to_INTERVAL((char*)"1417867665472") + (str_to_INTERVAL((char*)"-4229711233024") + (str_to_INTERVAL((char*)"-149318713920") + str_to_INTERVAL((char*)"20080510848") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"157483008") + (str_to_INTERVAL((char*)"14127366144") + (str_to_INTERVAL((char*)"-41721464832") + (str_to_INTERVAL((char*)"-2800856819712") + (str_to_INTERVAL((char*)"-6505625573376") + (str_to_INTERVAL((char*)"1018936436736") + (str_to_INTERVAL((char*)"-2115009825024") + (str_to_INTERVAL((char*)"1698043703040") + (str_to_INTERVAL((char*)"-180593567040") + (str_to_INTERVAL((char*)"-284103420672") + str_to_INTERVAL((char*)"15119735616") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1569300480") + (str_to_INTERVAL((char*)"23689543680") + (str_to_INTERVAL((char*)"-8062248960") + (str_to_INTERVAL((char*)"-224182970880") + (str_to_INTERVAL((char*)"104066603520") + (str_to_INTERVAL((char*)"53246825472") + (str_to_INTERVAL((char*)"-41170344960") + (str_to_INTERVAL((char*)"891276288") + str_to_INTERVAL((char*)"103726080") * b) * b) * b) * b) * b) * b) * b) * b) * b * b * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[24] = (str_to_INTERVAL((char*)"-2866544640") + (str_to_INTERVAL((char*)"50156863488") + (str_to_INTERVAL((char*)"114763415552") + (str_to_INTERVAL((char*)"-248506654720") + (str_to_INTERVAL((char*)"139118436352") + (str_to_INTERVAL((char*)"9406242816") + (str_to_INTERVAL((char*)"18606508032") + (str_to_INTERVAL((char*)"-1030196736") - str_to_INTERVAL((char*)"11206656") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"-97657159680") + (str_to_INTERVAL((char*)"65470267392") + (str_to_INTERVAL((char*)"3897707003904") + (str_to_INTERVAL((char*)"1938273763328") + (str_to_INTERVAL((char*)"-1575928913920") + (str_to_INTERVAL((char*)"415990257664") + (str_to_INTERVAL((char*)"139758850048") + (str_to_INTERVAL((char*)"41235783168") - str_to_INTERVAL((char*)"858069504") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"-1843028951040") + (str_to_INTERVAL((char*)"-11804718759936") + (str_to_INTERVAL((char*)"144011202183168") + (str_to_INTERVAL((char*)"300134688841728") + (str_to_INTERVAL((char*)"-194193955659776") + (str_to_INTERVAL((char*)"72019085789184") + (str_to_INTERVAL((char*)"-6949139607040") + (str_to_INTERVAL((char*)"15981127685376") + (str_to_INTERVAL((char*)"-771335316480") - str_to_INTERVAL((char*)"11599746048") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"18358838231040") + (str_to_INTERVAL((char*)"-289893107957760") + (str_to_INTERVAL((char*)"305083852587008") + (str_to_INTERVAL((char*)"5347718180192256") + (str_to_INTERVAL((char*)"2537161295757312") + (str_to_INTERVAL((char*)"-1294375472334848") + (str_to_INTERVAL((char*)"-149849130193920") + (str_to_INTERVAL((char*)"167779330940416") + (str_to_INTERVAL((char*)"21172592317952") - str_to_INTERVAL((char*)"718252494336") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"238657182105600") + (str_to_INTERVAL((char*)"-1144693777367040") + (str_to_INTERVAL((char*)"-8644567381573632") + (str_to_INTERVAL((char*)"41024689531797504") + (str_to_INTERVAL((char*)"74145544952455168") + (str_to_INTERVAL((char*)"-14624768059901952") + (str_to_INTERVAL((char*)"-4396943463419904") + (str_to_INTERVAL((char*)"-153696921071104") + (str_to_INTERVAL((char*)"1530282582659584") + (str_to_INTERVAL((char*)"-52791106625280") - str_to_INTERVAL((char*)"1210627703808") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"226698081730560") + (str_to_INTERVAL((char*)"5335218432245760") + (str_to_INTERVAL((char*)"-85905022817402880") + (str_to_INTERVAL((char*)"74422772129660928") + (str_to_INTERVAL((char*)"618895687126171648") + (str_to_INTERVAL((char*)"226943332235825152") + (str_to_INTERVAL((char*)"-134666248616673280") + (str_to_INTERVAL((char*)"-20845434310666240") + (str_to_INTERVAL((char*)"15330802175381504") + (str_to_INTERVAL((char*)"1083165834164480") - str_to_INTERVAL((char*)"49996576544256") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-2850540455854080") + (str_to_INTERVAL((char*)"37426901577891840") + (str_to_INTERVAL((char*)"-273933290108485632") + (str_to_INTERVAL((char*)"-398645398442803200") + (str_to_INTERVAL((char*)"2310626036638466048") + (str_to_INTERVAL((char*)"2793789922006708224") + (str_to_INTERVAL((char*)"-536831258466363392") + (str_to_INTERVAL((char*)"-347029737802118144") + (str_to_INTERVAL((char*)"56794072716961792") + (str_to_INTERVAL((char*)"32791818242780928") + (str_to_INTERVAL((char*)"-879325641133056") - str_to_INTERVAL((char*)"24956738002944") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-5503541905981440") + (str_to_INTERVAL((char*)"49957029628870656") + (str_to_INTERVAL((char*)"-269723295301238784") + (str_to_INTERVAL((char*)"-2139770309396529152") + (str_to_INTERVAL((char*)"3682758528262602752") + (str_to_INTERVAL((char*)"11525748803646734336") + (str_to_INTERVAL((char*)"2278114247881474048") + (str_to_INTERVAL((char*)"-2501635554172452864") + (str_to_INTERVAL((char*)"-117306568101494784") + (str_to_INTERVAL((char*)"230842835911644160") + (str_to_INTERVAL((char*)"9296624299705856") - str_to_INTERVAL((char*)"589510931413248") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-2984295499038720") + (str_to_INTERVAL((char*)"20068118465347584") + (str_to_INTERVAL((char*)"-27378126147551232") + (str_to_INTERVAL((char*)"-3352493182146969600") + (str_to_INTERVAL((char*)"969301822899027968") + (str_to_INTERVAL((char*)"21947877770185146368") + (str_to_INTERVAL((char*)"16222545804140134400") + (str_to_INTERVAL((char*)"-5408456802488135680") + (str_to_INTERVAL((char*)"-2122016834408749056") + (str_to_INTERVAL((char*)"670507150394326016") + (str_to_INTERVAL((char*)"144891650917417472") + (str_to_INTERVAL((char*)"-3932209226728704") - str_to_INTERVAL((char*)"109593442114560") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-460751256944640") + (str_to_INTERVAL((char*)"3730929465950208") + (str_to_INTERVAL((char*)"18419991732813824") + (str_to_INTERVAL((char*)"-1914815798758604800") + (str_to_INTERVAL((char*)"-3104958906269827072") + (str_to_INTERVAL((char*)"20086186248154906624") + (str_to_INTERVAL((char*)"31692904535508254720") + (str_to_INTERVAL((char*)"1010137825941438464") + (str_to_INTERVAL((char*)"-6811497469443383296") + (str_to_INTERVAL((char*)"428731957184737280") + (str_to_INTERVAL((char*)"564309200536764416") + (str_to_INTERVAL((char*)"12328241341973504") - str_to_INTERVAL((char*)"1186514195931648") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-14130295603200") + (str_to_INTERVAL((char*)"410394386497536") + (str_to_INTERVAL((char*)"881324870074368") + (str_to_INTERVAL((char*)"-409526120348123136") + (str_to_INTERVAL((char*)"-2435762094700756992") + (str_to_INTERVAL((char*)"8548999722954784768") + (str_to_INTERVAL((char*)"25689195754154655744") + (str_to_INTERVAL((char*)"11718960921406259200") + (str_to_INTERVAL((char*)"-6456669615286075392") + (str_to_INTERVAL((char*)"-1454340490967977984") + (str_to_INTERVAL((char*)"823031815546408960") + (str_to_INTERVAL((char*)"99967591505889280") + (str_to_INTERVAL((char*)"-2718589896041472") - str_to_INTERVAL((char*)"98824509808128") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"12150223405056") + (str_to_INTERVAL((char*)"-94301372547072") + (str_to_INTERVAL((char*)"-35481095145259008") + (str_to_INTERVAL((char*)"-502183474670338048") + (str_to_INTERVAL((char*)"1690377478739591168") + (str_to_INTERVAL((char*)"8897917011954040832") + (str_to_INTERVAL((char*)"8887219131134115840") + (str_to_INTERVAL((char*)"-643167484586115072") + (str_to_INTERVAL((char*)"-1948060136717115392") + (str_to_INTERVAL((char*)"305553547212144640") + (str_to_INTERVAL((char*)"160048664765466624") + (str_to_INTERVAL((char*)"3699224953872384") - str_to_INTERVAL((char*)"345466086709248") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-4813504905216") + (str_to_INTERVAL((char*)"-1157534901600256") + (str_to_INTERVAL((char*)"-24628777388081152") + (str_to_INTERVAL((char*)"174995862875275264") + (str_to_INTERVAL((char*)"1213514050611970048") + (str_to_INTERVAL((char*)"1964162788566433792") + (str_to_INTERVAL((char*)"800933014788112384") + (str_to_INTERVAL((char*)"-462985272169594880") + (str_to_INTERVAL((char*)"-109165061960785920") + (str_to_INTERVAL((char*)"77443340452204544") + (str_to_INTERVAL((char*)"7777271851089920") + (str_to_INTERVAL((char*)"117268859129856") - str_to_INTERVAL((char*)"25354239906816") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-4586597253120") + (str_to_INTERVAL((char*)"-41720660099072") + (str_to_INTERVAL((char*)"6918078729289728") + (str_to_INTERVAL((char*)"48193694574051328") + (str_to_INTERVAL((char*)"105513817966903296") + (str_to_INTERVAL((char*)"140075682486419456") + (str_to_INTERVAL((char*)"29524414379556864") + (str_to_INTERVAL((char*)"-46268097400668160") + (str_to_INTERVAL((char*)"9844276620009472") + (str_to_INTERVAL((char*)"1717617863290880") + (str_to_INTERVAL((char*)"809768696221696") - str_to_INTERVAL((char*)"34067438456832") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"418394406912") + (str_to_INTERVAL((char*)"22494798938112") + (str_to_INTERVAL((char*)"111536583475200") + (str_to_INTERVAL((char*)"392402320490496") + (str_to_INTERVAL((char*)"5455397588041728") + (str_to_INTERVAL((char*)"7032984596971520") + (str_to_INTERVAL((char*)"-3080137908813824") + (str_to_INTERVAL((char*)"804646207881216") + (str_to_INTERVAL((char*)"-530707690569728") + (str_to_INTERVAL((char*)"335736485969920") + (str_to_INTERVAL((char*)"13592759500800") - str_to_INTERVAL((char*)"1659193491456") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"112566730752") + (str_to_INTERVAL((char*)"6562833235968") + (str_to_INTERVAL((char*)"65588009893888") + (str_to_INTERVAL((char*)"84593023647744") + (str_to_INTERVAL((char*)"-17122092122112") + (str_to_INTERVAL((char*)"166684904849408") + (str_to_INTERVAL((char*)"-127603304890368") + (str_to_INTERVAL((char*)"15188598472704") + (str_to_INTERVAL((char*)"13697885405184") - str_to_INTERVAL((char*)"765049651200") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"166613483520") + (str_to_INTERVAL((char*)"1980805939200") + (str_to_INTERVAL((char*)"3215859646464") + (str_to_INTERVAL((char*)"-2897252057088") + (str_to_INTERVAL((char*)"-294303891456") + (str_to_INTERVAL((char*)"701049028608") + (str_to_INTERVAL((char*)"-19297566720") - str_to_INTERVAL((char*)"1656668160") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a;
coef[25] = 128 * a * ((str_to_INTERVAL((char*)"369515520") + (str_to_INTERVAL((char*)"-6465533184") + (str_to_INTERVAL((char*)"-14793721536") + (str_to_INTERVAL((char*)"32034060960") + (str_to_INTERVAL((char*)"-17933235936") + (str_to_INTERVAL((char*)"-1212523488") + (str_to_INTERVAL((char*)"-2398495176") + (str_to_INTERVAL((char*)"132798798") + str_to_INTERVAL((char*)"1444608") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"11825671680") + (str_to_INTERVAL((char*)"-7928040192") + (str_to_INTERVAL((char*)"-471987957504") + (str_to_INTERVAL((char*)"-234712838528") + (str_to_INTERVAL((char*)"190835141920") + (str_to_INTERVAL((char*)"-50373820264") + (str_to_INTERVAL((char*)"-16923923248") + (str_to_INTERVAL((char*)"-4993395618") + str_to_INTERVAL((char*)"103906854") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"53433423360") + (str_to_INTERVAL((char*)"860727487104") + (str_to_INTERVAL((char*)"-6587953175872") + (str_to_INTERVAL((char*)"-15931375085472") + (str_to_INTERVAL((char*)"6725281213344") + (str_to_INTERVAL((char*)"-2027552681416") + (str_to_INTERVAL((char*)"342681873570") + (str_to_INTERVAL((char*)"-685366529139") + (str_to_INTERVAL((char*)"28664678220") + str_to_INTERVAL((char*)"509141052") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-928046315520") + (str_to_INTERVAL((char*)"10928161122048") + (str_to_INTERVAL((char*)"-5385257088768") + (str_to_INTERVAL((char*)"-206759729158848") + (str_to_INTERVAL((char*)"-110080301692032") + (str_to_INTERVAL((char*)"49306634733816") + (str_to_INTERVAL((char*)"7977149247852") + (str_to_INTERVAL((char*)"-6394565427714") + (str_to_INTERVAL((char*)"-841699512378") + str_to_INTERVAL((char*)"27413929818") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-6312906547200") + (str_to_INTERVAL((char*)"20653489789440") + (str_to_INTERVAL((char*)"295356477626880") + (str_to_INTERVAL((char*)"-1065410877760320") + (str_to_INTERVAL((char*)"-2105221280851360") + (str_to_INTERVAL((char*)"253021248449880") + (str_to_INTERVAL((char*)"198359978311500") + (str_to_INTERVAL((char*)"-122039930660") + (str_to_INTERVAL((char*)"-38065657032130") + (str_to_INTERVAL((char*)"1049868679635") + str_to_INTERVAL((char*)"29540543400") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"2811708357120") + (str_to_INTERVAL((char*)"-162162607856640") + (str_to_INTERVAL((char*)"1911086774676480") + (str_to_INTERVAL((char*)"-1163677841858304") + (str_to_INTERVAL((char*)"-13089293104087744") + (str_to_INTERVAL((char*)"-5290960879889776") + (str_to_INTERVAL((char*)"2901714486626880") + (str_to_INTERVAL((char*)"493852825051480") + (str_to_INTERVAL((char*)"-322243288518272") + (str_to_INTERVAL((char*)"-23232565327195") + str_to_INTERVAL((char*)"1023799762818") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"51207317821440") + (str_to_INTERVAL((char*)"-601229802087936") + (str_to_INTERVAL((char*)"4195253063166976") + (str_to_INTERVAL((char*)"7721788073094144") + (str_to_INTERVAL((char*)"-36867006889947712") + (str_to_INTERVAL((char*)"-46068121280125184") + (str_to_INTERVAL((char*)"7561175298275792") + (str_to_INTERVAL((char*)"6299213378841136") + (str_to_INTERVAL((char*)"-994036448919304") + (str_to_INTERVAL((char*)"-506278839192452") + (str_to_INTERVAL((char*)"11656199791230") + str_to_INTERVAL((char*)"370556460252") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"61530298859520") + (str_to_INTERVAL((char*)"-550278057959424") + (str_to_INTERVAL((char*)"2943250107738624") + (str_to_INTERVAL((char*)"27313523002832384") + (str_to_INTERVAL((char*)"-44313618347010560") + (str_to_INTERVAL((char*)"-144121132572950144") + (str_to_INTERVAL((char*)"-30759332113930464") + (str_to_INTERVAL((char*)"31821317996846384") + (str_to_INTERVAL((char*)"1576060217060912") + (str_to_INTERVAL((char*)"-2881213441032328") + (str_to_INTERVAL((char*)"-114278556558500") + str_to_INTERVAL((char*)"7071436029825") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"21951326453760") + (str_to_INTERVAL((char*)"-182542375219200") + (str_to_INTERVAL((char*)"444651156582400") + (str_to_INTERVAL((char*)"30101463353665024") + (str_to_INTERVAL((char*)"-8177187688434176") + (str_to_INTERVAL((char*)"-209229150046945408") + (str_to_INTERVAL((char*)"-154876379438217472") + (str_to_INTERVAL((char*)"49492040325780192") + (str_to_INTERVAL((char*)"21227495297558816") + (str_to_INTERVAL((char*)"-6543146372482944") + (str_to_INTERVAL((char*)"-1340035304200684") + (str_to_INTERVAL((char*)"33952270850286") + str_to_INTERVAL((char*)"981694764144") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"2163040174080") + (str_to_INTERVAL((char*)"-30214318399488") + (str_to_INTERVAL((char*)"10947208151040") + (str_to_INTERVAL((char*)"12429595780431872") + (str_to_INTERVAL((char*)"19771463159977472") + (str_to_INTERVAL((char*)"-144972300466096640") + (str_to_INTERVAL((char*)"-223792334361205760") + (str_to_INTERVAL((char*)"-9132635856836160") + (str_to_INTERVAL((char*)"48324209008364896") + (str_to_INTERVAL((char*)"-2898001955272304") + (str_to_INTERVAL((char*)"-4017364516306880") + (str_to_INTERVAL((char*)"-83992574731700") + str_to_INTERVAL((char*)"8031135728898") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"39967948800") + (str_to_INTERVAL((char*)"-2225975132160") + (str_to_INTERVAL((char*)"8961315078144") + (str_to_INTERVAL((char*)"1992921995284480") + (str_to_INTERVAL((char*)"10348796781027328") + (str_to_INTERVAL((char*)"-46774759962597888") + (str_to_INTERVAL((char*)"-131324431683883776") + (str_to_INTERVAL((char*)"-59365434799001344") + (str_to_INTERVAL((char*)"31182954446651904") + (str_to_INTERVAL((char*)"7904770019203200") + (str_to_INTERVAL((char*)"-4249643802988896") + (str_to_INTERVAL((char*)"-504572109958592") + (str_to_INTERVAL((char*)"12042479336784") + str_to_INTERVAL((char*)"528716918340") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-34978922496") + (str_to_INTERVAL((char*)"930810028032") + (str_to_INTERVAL((char*)"125722684268544") + (str_to_INTERVAL((char*)"1278250089125888") + (str_to_INTERVAL((char*)"-7132298034761728") + (str_to_INTERVAL((char*)"-31345660698629632") + (str_to_INTERVAL((char*)"-30035086423260160") + (str_to_INTERVAL((char*)"1045412678302336") + (str_to_INTERVAL((char*)"6758555232882432") + (str_to_INTERVAL((char*)"-970555841527808") + (str_to_INTERVAL((char*)"-571115003062528") + (str_to_INTERVAL((char*)"-16193867375888") + str_to_INTERVAL((char*)"1238470655148") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"15204548608") + (str_to_INTERVAL((char*)"2127446122496") + (str_to_INTERVAL((char*)"23409601314816") + (str_to_INTERVAL((char*)"-501817736437760") + (str_to_INTERVAL((char*)"-2633606342064128") + (str_to_INTERVAL((char*)"-3956058171928576") + (str_to_INTERVAL((char*)"-2039938004900352") + (str_to_INTERVAL((char*)"836367230268160") + (str_to_INTERVAL((char*)"335980913493440") + (str_to_INTERVAL((char*)"-184916996759072") + (str_to_INTERVAL((char*)"-19666385716160") + (str_to_INTERVAL((char*)"-989086766592") + str_to_INTERVAL((char*)"94424808024") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1062649856") + (str_to_INTERVAL((char*)"-201460482048") + (str_to_INTERVAL((char*)"-8375948410880") + (str_to_INTERVAL((char*)"-43890427836416") + (str_to_INTERVAL((char*)"-90534402834432") + (str_to_INTERVAL((char*)"-212164089026048") + (str_to_INTERVAL((char*)"-81321171038976") + (str_to_INTERVAL((char*)"83277265866880") + (str_to_INTERVAL((char*)"-18902572875648") + (str_to_INTERVAL((char*)"-760730283008") + (str_to_INTERVAL((char*)"-2357489419264") + str_to_INTERVAL((char*)"93324731616") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"2204762112") + (str_to_INTERVAL((char*)"73119940608") + (str_to_INTERVAL((char*)"-335005347840") + (str_to_INTERVAL((char*)"-6171644579840") + (str_to_INTERVAL((char*)"-6195408748544") + (str_to_INTERVAL((char*)"3035379010560") + (str_to_INTERVAL((char*)"-2045857779712") + (str_to_INTERVAL((char*)"1632144892160") + (str_to_INTERVAL((char*)"-663557298816") + (str_to_INTERVAL((char*)"-26343512064") + str_to_INTERVAL((char*)"3203280000") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"5231001600") + (str_to_INTERVAL((char*)"37198402560") + (str_to_INTERVAL((char*)"-67982192640") + (str_to_INTERVAL((char*)"-154960468992") + (str_to_INTERVAL((char*)"155107021824") + (str_to_INTERVAL((char*)"-27604832832") + (str_to_INTERVAL((char*)"-12716187264") + str_to_INTERVAL((char*)"766906560") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[26] = (str_to_INTERVAL((char*)"1468661760") + (str_to_INTERVAL((char*)"1822433280") + (str_to_INTERVAL((char*)"-27158077440") + (str_to_INTERVAL((char*)"17098887168") + (str_to_INTERVAL((char*)"5744328192") + (str_to_INTERVAL((char*)"1829960448") + (str_to_INTERVAL((char*)"-122191872") - str_to_INTERVAL((char*)"940032") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"-3370844160") + (str_to_INTERVAL((char*)"168903278592") + (str_to_INTERVAL((char*)"24121171968") + (str_to_INTERVAL((char*)"-235898609664") + (str_to_INTERVAL((char*)"171531407360") + (str_to_INTERVAL((char*)"4961184256") + (str_to_INTERVAL((char*)"6918179328") - str_to_INTERVAL((char*)"65350656") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"-594144460800") + (str_to_INTERVAL((char*)"7128458035200") + (str_to_INTERVAL((char*)"19945466974208") + (str_to_INTERVAL((char*)"-32288611563520") + (str_to_INTERVAL((char*)"17906147962880") + (str_to_INTERVAL((char*)"1114590421504") + (str_to_INTERVAL((char*)"2574500142080") + (str_to_INTERVAL((char*)"-134043146112") - str_to_INTERVAL((char*)"1621748736") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"-11159223828480") + (str_to_INTERVAL((char*)"1365042806784") + (str_to_INTERVAL((char*)"473318719537152") + (str_to_INTERVAL((char*)"264899169116160") + (str_to_INTERVAL((char*)"-184195066456064") + (str_to_INTERVAL((char*)"42728028102144") + (str_to_INTERVAL((char*)"16401281441792") + (str_to_INTERVAL((char*)"5090586948480") - str_to_INTERVAL((char*)"103890540672") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"-8068699422720") + (str_to_INTERVAL((char*)"-676267362336768") + (str_to_INTERVAL((char*)"3678980151599104") + (str_to_INTERVAL((char*)"10402663095644160") + (str_to_INTERVAL((char*)"-2502207725119488") + (str_to_INTERVAL((char*)"399233913163264") + (str_to_INTERVAL((char*)"-215369489317632") + (str_to_INTERVAL((char*)"365224010444416") + (str_to_INTERVAL((char*)"-12440220297216") - str_to_INTERVAL((char*)"279140152320") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"548797104783360") + (str_to_INTERVAL((char*)"-4920913491886080") + (str_to_INTERVAL((char*)"-1340553096167424") + (str_to_INTERVAL((char*)"97783792317456384") + (str_to_INTERVAL((char*)"59239935369830400") + (str_to_INTERVAL((char*)"-22900589078543360") + (str_to_INTERVAL((char*)"-5079026479856128") + (str_to_INTERVAL((char*)"2980228870880000") + (str_to_INTERVAL((char*)"412599285094912") - str_to_INTERVAL((char*)"12804857900928") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"1865572661329920") + (str_to_INTERVAL((char*)"-2524939650859008") + (str_to_INTERVAL((char*)"-125586412930990080") + (str_to_INTERVAL((char*)"358842421762883584") + (str_to_INTERVAL((char*)"779593576265564160") + (str_to_INTERVAL((char*)"-47175889055266816") + (str_to_INTERVAL((char*)"-93728233199494144") + (str_to_INTERVAL((char*)"1094161194069504") + (str_to_INTERVAL((char*)"12724814243300608") + (str_to_INTERVAL((char*)"-273964417963392") - str_to_INTERVAL((char*)"9708085854720") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-2706678551347200") + (str_to_INTERVAL((char*)"56660540032548864") + (str_to_INTERVAL((char*)"-551550785740603392") + (str_to_INTERVAL((char*)"202791142280429568") + (str_to_INTERVAL((char*)"3668305070691737600") + (str_to_INTERVAL((char*)"1616961249103824896") + (str_to_INTERVAL((char*)"-827146534311112704") + (str_to_INTERVAL((char*)"-153507818788273152") + (str_to_INTERVAL((char*)"90102916964234240") + (str_to_INTERVAL((char*)"6602050376025216") - str_to_INTERVAL((char*)"279171777740544") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-11552444372090880") + (str_to_INTERVAL((char*)"125818812255043584") + (str_to_INTERVAL((char*)"-851568226841591808") + (str_to_INTERVAL((char*)"-1925593924881383424") + (str_to_INTERVAL((char*)"7909860842124754944") + (str_to_INTERVAL((char*)"10204124693794353152") + (str_to_INTERVAL((char*)"-1445189312678729728") + (str_to_INTERVAL((char*)"-1493972546971847680") + (str_to_INTERVAL((char*)"229346820177641472") + (str_to_INTERVAL((char*)"106729051409612032") + (str_to_INTERVAL((char*)"-2158270103993856") - str_to_INTERVAL((char*)"75461519121408") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-8925211671920640") + (str_to_INTERVAL((char*)"83023013743165440") + (str_to_INTERVAL((char*)"-451799106843770880") + (str_to_INTERVAL((char*)"-4603138193216176128") + (str_to_INTERVAL((char*)"7219977784006606848") + (str_to_INTERVAL((char*)"24344674571883741184") + (str_to_INTERVAL((char*)"5513928106287566848") + (str_to_INTERVAL((char*)"-5433222780976613376") + (str_to_INTERVAL((char*)"-286549455936630784") + (str_to_INTERVAL((char*)"486414887068581888") + (str_to_INTERVAL((char*)"18961002355073536") - str_to_INTERVAL((char*)"1150564028715648") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-2079959128473600") + (str_to_INTERVAL((char*)"22478735189999616") + (str_to_INTERVAL((char*)"-83879776055984128") + (str_to_INTERVAL((char*)"-3603336550641565696") + (str_to_INTERVAL((char*)"1061929905274159104") + (str_to_INTERVAL((char*)"26923884233250177024") + (str_to_INTERVAL((char*)"19867069378528100352") + (str_to_INTERVAL((char*)"-6104329892343177216") + (str_to_INTERVAL((char*)"-2822324158682296320") + (str_to_INTERVAL((char*)"852562561455777792") + (str_to_INTERVAL((char*)"168316914056480768") + (str_to_INTERVAL((char*)"-4046068012511232") - str_to_INTERVAL((char*)"119656251380736") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-128431108915200") + (str_to_INTERVAL((char*)"2905861335810048") + (str_to_INTERVAL((char*)"-11744309525086208") + (str_to_INTERVAL((char*)"-1076032659136708608") + (str_to_INTERVAL((char*)"-1543098451871727616") + (str_to_INTERVAL((char*)"14045465888798605312") + (str_to_INTERVAL((char*)"21043162434230091776") + (str_to_INTERVAL((char*)"1025823437097066496") + (str_to_INTERVAL((char*)"-4524064368767307776") + (str_to_INTERVAL((char*)"250301182820083712") + (str_to_INTERVAL((char*)"382090468336926720") + (str_to_INTERVAL((char*)"7670454137075200") - str_to_INTERVAL((char*)"723751546141440") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1352725954560") + (str_to_INTERVAL((char*)"134371325509632") + (str_to_INTERVAL((char*)"-1474518152380416") + (str_to_INTERVAL((char*)"-124925319664107520") + (str_to_INTERVAL((char*)"-523436784694067200") + (str_to_INTERVAL((char*)"3388650331439628288") + (str_to_INTERVAL((char*)"8752597207590535168") + (str_to_INTERVAL((char*)"3905098843813363712") + (str_to_INTERVAL((char*)"-1917948871271104512") + (str_to_INTERVAL((char*)"-564802962362888192") + (str_to_INTERVAL((char*)"285720541311242240") + (str_to_INTERVAL((char*)"33616476048963584") + (str_to_INTERVAL((char*)"-651714107771904") - str_to_INTERVAL((char*)"38780191084032") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1044306198528") + (str_to_INTERVAL((char*)"-65595327381504") + (str_to_INTERVAL((char*)"-5022790739558400") + (str_to_INTERVAL((char*)"-32324515684155392") + (str_to_INTERVAL((char*)"374860410176143360") + (str_to_INTERVAL((char*)"1377035586378072064") + (str_to_INTERVAL((char*)"1259789326875688960") + (str_to_INTERVAL((char*)"34609695251152896") + (str_to_INTERVAL((char*)"-297730854993166336") + (str_to_INTERVAL((char*)"36606526471626752") + (str_to_INTERVAL((char*)"26237540730179584") + (str_to_INTERVAL((char*)"995774405462016") - str_to_INTERVAL((char*)"58473094445568") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-365823000576") + (str_to_INTERVAL((char*)"-32032658817024") + (str_to_INTERVAL((char*)"174541987381248") + (str_to_INTERVAL((char*)"15262800471916544") + (str_to_INTERVAL((char*)"63136745150087168") + (str_to_INTERVAL((char*)"88945224016920576") + (str_to_INTERVAL((char*)"69532394573365248") + (str_to_INTERVAL((char*)"-16768162146107392") + (str_to_INTERVAL((char*)"-13072178823823360") + (str_to_INTERVAL((char*)"5747178528210944") + (str_to_INTERVAL((char*)"667245355761664") + (str_to_INTERVAL((char*)"71761603645440") - str_to_INTERVAL((char*)"4927151591424") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"20289945600") + (str_to_INTERVAL((char*)"3054715797504") + (str_to_INTERVAL((char*)"66691563257856") + (str_to_INTERVAL((char*)"194044118171648") + (str_to_INTERVAL((char*)"697008432873472") + (str_to_INTERVAL((char*)"4583119516794880") + (str_to_INTERVAL((char*)"1835190522642432") + (str_to_INTERVAL((char*)"-1792859566620672") + (str_to_INTERVAL((char*)"492542442094592") + (str_to_INTERVAL((char*)"-68491299676160") + (str_to_INTERVAL((char*)"90936502689792") - str_to_INTERVAL((char*)"3480347160576") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"365841874944") + (str_to_INTERVAL((char*)"11402073866240") + (str_to_INTERVAL((char*)"61838539358208") + (str_to_INTERVAL((char*)"24800936460288") + (str_to_INTERVAL((char*)"3182184890368") + (str_to_INTERVAL((char*)"39609939591168") + (str_to_INTERVAL((char*)"-46614761422848") + (str_to_INTERVAL((char*)"16385666408448") + (str_to_INTERVAL((char*)"545749229568") - str_to_INTERVAL((char*)"72973762560") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + (str_to_INTERVAL((char*)"187440168960") + (str_to_INTERVAL((char*)"1223519109120") + (str_to_INTERVAL((char*)"483831447552") + (str_to_INTERVAL((char*)"-1248737624064") + (str_to_INTERVAL((char*)"370362654720") + (str_to_INTERVAL((char*)"91017953280") - str_to_INTERVAL((char*)"6154444800") * b) * b) * b) * b) * b) * b) *  pow( b,  4) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a;
coef[27] = 64 * a * ((str_to_INTERVAL((char*)"-401587200") + (str_to_INTERVAL((char*)"-498321600") + (str_to_INTERVAL((char*)"7426036800") + (str_to_INTERVAL((char*)"-4675476960") + (str_to_INTERVAL((char*)"-1570714740") + (str_to_INTERVAL((char*)"-500379810") + (str_to_INTERVAL((char*)"33411840") + str_to_INTERVAL((char*)"257040") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"869045760") + (str_to_INTERVAL((char*)"-43545376512") + (str_to_INTERVAL((char*)"-6218739648") + (str_to_INTERVAL((char*)"60817610304") + (str_to_INTERVAL((char*)"-44222940960") + (str_to_INTERVAL((char*)"-1279055316") + (str_to_INTERVAL((char*)"-1783593108") + str_to_INTERVAL((char*)"16848216") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"82801128960") + (str_to_INTERVAL((char*)"-657455277312") + (str_to_INTERVAL((char*)"-2384647488864") + (str_to_INTERVAL((char*)"2523600034320") + (str_to_INTERVAL((char*)"-1371683535008") + (str_to_INTERVAL((char*)"-69469317692") + (str_to_INTERVAL((char*)"-226960550728") + (str_to_INTERVAL((char*)"10507506513") + str_to_INTERVAL((char*)"153925416") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"813539255040") + (str_to_INTERVAL((char*)"840298316928") + (str_to_INTERVAL((char*)"-38798018557056") + (str_to_INTERVAL((char*)"-25982856416320") + (str_to_INTERVAL((char*)"14060599165552") + (str_to_INTERVAL((char*)"-2376364928612") + (str_to_INTERVAL((char*)"-1261946455696") + (str_to_INTERVAL((char*)"-429293740905") + str_to_INTERVAL((char*)"8471206791") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"-1115870618880") + (str_to_INTERVAL((char*)"46206933777792") + (str_to_INTERVAL((char*)"-191043334268352") + (str_to_INTERVAL((char*)"-642278578366944") + (str_to_INTERVAL((char*)"65309980357296") + (str_to_INTERVAL((char*)"16760577908412") + (str_to_INTERVAL((char*)"13453624341126") + (str_to_INTERVAL((char*)"-19033145936649") + (str_to_INTERVAL((char*)"491930908344") + str_to_INTERVAL((char*)"15012648864") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-28493727436800") + (str_to_INTERVAL((char*)"202906017999360") + (str_to_INTERVAL((char*)"264745925621760") + (str_to_INTERVAL((char*)"-4440003769057920") + (str_to_INTERVAL((char*)"-3057670649213760") + (str_to_INTERVAL((char*)"1020758989459600") + (str_to_INTERVAL((char*)"297396633545840") + (str_to_INTERVAL((char*)"-134184432460240") + (str_to_INTERVAL((char*)"-19547542925030") + str_to_INTERVAL((char*)"578457108945") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-44327837675520") + (str_to_INTERVAL((char*)"-106020698867712") + (str_to_INTERVAL((char*)"5128113002357760") + (str_to_INTERVAL((char*)"-11887078713339904") + (str_to_INTERVAL((char*)"-28673626527608320") + (str_to_INTERVAL((char*)"368799392503216") + (str_to_INTERVAL((char*)"4015268822798104") + (str_to_INTERVAL((char*)"-63017131360604") + (str_to_INTERVAL((char*)"-433444435565678") + (str_to_INTERVAL((char*)"7161171014937") + str_to_INTERVAL((char*)"326083659660") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"120693802152960") + (str_to_INTERVAL((char*)"-1820101275614208") + (str_to_INTERVAL((char*)"15659311099597824") + (str_to_INTERVAL((char*)"-2140923570820608") + (str_to_INTERVAL((char*)"-103362078923950592") + (str_to_INTERVAL((char*)"-49105274274132320") + (str_to_INTERVAL((char*)"23651422863516128") + (str_to_INTERVAL((char*)"4743528652399904") + (str_to_INTERVAL((char*)"-2544187306117336") + (str_to_INTERVAL((char*)"-188527856811322") + str_to_INTERVAL((char*)"7708241213892") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"249585864468480") + (str_to_INTERVAL((char*)"-2616677119558656") + (str_to_INTERVAL((char*)"17426294735511040") + (str_to_INTERVAL((char*)"46785999904397568") + (str_to_INTERVAL((char*)"-171828182579161088") + (str_to_INTERVAL((char*)"-228553174738450816") + (str_to_INTERVAL((char*)"28187753385046304") + (str_to_INTERVAL((char*)"35156645319529968") + (str_to_INTERVAL((char*)"-5277736957325024") + (str_to_INTERVAL((char*)"-2303668652693732") + (str_to_INTERVAL((char*)"41982374824164") + str_to_INTERVAL((char*)"1578798809496") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"125434034749440") + (str_to_INTERVAL((char*)"-1283089100070912") + (str_to_INTERVAL((char*)"7331748176695296") + (str_to_INTERVAL((char*)"76920035860492288") + (str_to_INTERVAL((char*)"-119972805160259072") + (str_to_INTERVAL((char*)"-416324097963274496") + (str_to_INTERVAL((char*)"-98548866008454528") + (str_to_INTERVAL((char*)"93294658358347328") + (str_to_INTERVAL((char*)"5266250929892896") + (str_to_INTERVAL((char*)"-8317173255894416") + (str_to_INTERVAL((char*)"-318366795041416") + str_to_INTERVAL((char*)"19021047385644") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"18844584837120") + (str_to_INTERVAL((char*)"-273767874650112") + (str_to_INTERVAL((char*)"1421083361538048") + (str_to_INTERVAL((char*)"42923922799552512") + (str_to_INTERVAL((char*)"-16158638698420736") + (str_to_INTERVAL((char*)"-348933595976928768") + (str_to_INTERVAL((char*)"-255190035439324544") + (str_to_INTERVAL((char*)"75508059110618944") + (str_to_INTERVAL((char*)"37248500570056928") + (str_to_INTERVAL((char*)"-11059206083779216") + (str_to_INTERVAL((char*)"-2138111656373120") + (str_to_INTERVAL((char*)"49354281524904") + str_to_INTERVAL((char*)"1474423500024") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"704758579200") + (str_to_INTERVAL((char*)"-25241891291136") + (str_to_INTERVAL((char*)"192241541513216") + (str_to_INTERVAL((char*)"9159312432291840") + (str_to_INTERVAL((char*)"10234600472398848") + (str_to_INTERVAL((char*)"-135596072273672192") + (str_to_INTERVAL((char*)"-195161859532982784") + (str_to_INTERVAL((char*)"-10972685021071488") + (str_to_INTERVAL((char*)"41340382061689152") + (str_to_INTERVAL((char*)"-1991006787042208") + (str_to_INTERVAL((char*)"-3603469919153728") + (str_to_INTERVAL((char*)"-69778667953384") + str_to_INTERVAL((char*)"6411426817068") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"3844177920") + (str_to_INTERVAL((char*)"-679651098624") + (str_to_INTERVAL((char*)"14488214986752") + (str_to_INTERVAL((char*)"725243807051776") + (str_to_INTERVAL((char*)"2075707605164032") + (str_to_INTERVAL((char*)"-23835707254142976") + (str_to_INTERVAL((char*)"-55761785568827392") + (str_to_INTERVAL((char*)"-24520384262013952") + (str_to_INTERVAL((char*)"10815938198176000") + (str_to_INTERVAL((char*)"3958154304004928") + (str_to_INTERVAL((char*)"-1848996519926656") + (str_to_INTERVAL((char*)"-218324727381664") + (str_to_INTERVAL((char*)"2738036216064") + str_to_INTERVAL((char*)"294098441400") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-2209480704") + (str_to_INTERVAL((char*)"308174045184") + (str_to_INTERVAL((char*)"15521106296832") + (str_to_INTERVAL((char*)"22599431507968") + (str_to_INTERVAL((char*)"-1756531293544448") + (str_to_INTERVAL((char*)"-5391729335917568") + (str_to_INTERVAL((char*)"-4729423565760512") + (str_to_INTERVAL((char*)"-684912281627392") + (str_to_INTERVAL((char*)"1249107449324160") + (str_to_INTERVAL((char*)"-116267386699136") + (str_to_INTERVAL((char*)"-114991147900256") + (str_to_INTERVAL((char*)"-6454387147584") + str_to_INTERVAL((char*)"274397271264") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"425852928") + (str_to_INTERVAL((char*)"9173762048") + (str_to_INTERVAL((char*)"-1679572443136") + (str_to_INTERVAL((char*)"-32910712647680") + (str_to_INTERVAL((char*)"-105619322572800") + (str_to_INTERVAL((char*)"-155160400594944") + (str_to_INTERVAL((char*)"-239387374882816") + (str_to_INTERVAL((char*)"33738169030656") + (str_to_INTERVAL((char*)"44607963108096") + (str_to_INTERVAL((char*)"-17162793795712") + (str_to_INTERVAL((char*)"-2359968279296") + (str_to_INTERVAL((char*)"-449002354944") + str_to_INTERVAL((char*)"26291395584") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"14330953728") + (str_to_INTERVAL((char*)"219573993472") + (str_to_INTERVAL((char*)"-1317842880512") + (str_to_INTERVAL((char*)"-9402960751616") + (str_to_INTERVAL((char*)"-1932633545728") + (str_to_INTERVAL((char*)"2303189961728") + (str_to_INTERVAL((char*)"-876314253184") + (str_to_INTERVAL((char*)"447164688576") + (str_to_INTERVAL((char*)"-334014821184") + str_to_INTERVAL((char*)"12609964800") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + (str_to_INTERVAL((char*)"11769753600") + (str_to_INTERVAL((char*)"34435805184") + (str_to_INTERVAL((char*)"-115532531712") + (str_to_INTERVAL((char*)"-21809949696") + (str_to_INTERVAL((char*)"85050521856") + (str_to_INTERVAL((char*)"-33289710336") + (str_to_INTERVAL((char*)"-749993472") + str_to_INTERVAL((char*)"130590720") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[28] = (str_to_INTERVAL((char*)"57507840") + (str_to_INTERVAL((char*)"-1886281728") + (str_to_INTERVAL((char*)"1306694656") + (str_to_INTERVAL((char*)"1218974720") + (str_to_INTERVAL((char*)"84734976") + (str_to_INTERVAL((char*)"-10478592") - str_to_INTERVAL((char*)"48384") * b) * b) * b) * b) * b) * b) *  pow( b,  14) + ((str_to_INTERVAL((char*)"5221048320") + (str_to_INTERVAL((char*)"-10210516992") + (str_to_INTERVAL((char*)"-31684919296") + (str_to_INTERVAL((char*)"29971960832") + (str_to_INTERVAL((char*)"60966400") + (str_to_INTERVAL((char*)"933452160") - str_to_INTERVAL((char*)"1746432") * b) * b) * b) * b) * b) * b) *  pow( b,  13) + ((str_to_INTERVAL((char*)"224456417280") + (str_to_INTERVAL((char*)"466163472384") + (str_to_INTERVAL((char*)"-4134229383168") + (str_to_INTERVAL((char*)"2531059225600") + (str_to_INTERVAL((char*)"885350431104") + (str_to_INTERVAL((char*)"278735952576") + (str_to_INTERVAL((char*)"-18004151040") - str_to_INTERVAL((char*)"154386432") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"-717443481600") + (str_to_INTERVAL((char*)"23016487772160") + (str_to_INTERVAL((char*)"6438467860480") + (str_to_INTERVAL((char*)"-31055148824576") + (str_to_INTERVAL((char*)"22375138229248") + (str_to_INTERVAL((char*)"550031058560") + (str_to_INTERVAL((char*)"950698740864") - str_to_INTERVAL((char*)"8785503360") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"-32252219228160") + (str_to_INTERVAL((char*)"182330475761664") + (str_to_INTERVAL((char*)"865897332084736") + (str_to_INTERVAL((char*)"-559232008458240") + (str_to_INTERVAL((char*)"299642563141120") + (str_to_INTERVAL((char*)"7215607706496") + (str_to_INTERVAL((char*)"61662187676544") + (str_to_INTERVAL((char*)"-2374441322304") - str_to_INTERVAL((char*)"45899078400") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"-167748022517760") + (str_to_INTERVAL((char*)"-513719701364736") + (str_to_INTERVAL((char*)"9694687134449664") + (str_to_INTERVAL((char*)"7902935668219904") + (str_to_INTERVAL((char*)"-3182632833486848") + (str_to_INTERVAL((char*)"229907596239616") + (str_to_INTERVAL((char*)"290897180309760") + (str_to_INTERVAL((char*)"111793816961856") - str_to_INTERVAL((char*)"2118173744448") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"540339778437120") + (str_to_INTERVAL((char*)"-9752521142280192") + (str_to_INTERVAL((char*)"31259339063742464") + (str_to_INTERVAL((char*)"129460252702783488") + (str_to_INTERVAL((char*)"1000440054278144") + (str_to_INTERVAL((char*)"-9461242588041472") + (str_to_INTERVAL((char*)"-2860301549596800") + (str_to_INTERVAL((char*)"3361271150363456") + (str_to_INTERVAL((char*)"-59787185031936") - str_to_INTERVAL((char*)"2740656464640") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"4442438774292480") + (str_to_INTERVAL((char*)"-25628606271553536") + (str_to_INTERVAL((char*)"-73832659868499968") + (str_to_INTERVAL((char*)"669036020182474752") + (str_to_INTERVAL((char*)"520587865461313536") + (str_to_INTERVAL((char*)"-151252170046436352") + (str_to_INTERVAL((char*)"-55630055830103552") + (str_to_INTERVAL((char*)"20277153391822848") + (str_to_INTERVAL((char*)"3092773081256320") - str_to_INTERVAL((char*)"87818365209024") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"2147969669529600") + (str_to_INTERVAL((char*)"36857689222840320") + (str_to_INTERVAL((char*)"-688946247754530816") + (str_to_INTERVAL((char*)"1316483130575372288") + (str_to_INTERVAL((char*)"3572082015337824256") + (str_to_INTERVAL((char*)"93481394140500992") + (str_to_INTERVAL((char*)"-555272726252155392") + (str_to_INTERVAL((char*)"9336710077412608") + (str_to_INTERVAL((char*)"51040767192013696") + (str_to_INTERVAL((char*)"-639714522449472") - str_to_INTERVAL((char*)"37990417070592") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-14747425797242880") + (str_to_INTERVAL((char*)"187929849222070272") + (str_to_INTERVAL((char*)"-1489830617868140544") + (str_to_INTERVAL((char*)"-131062981728632832") + (str_to_INTERVAL((char*)"9941831416225382400") + (str_to_INTERVAL((char*)"5036062835459901440") + (str_to_INTERVAL((char*)"-2302990325249699840") + (str_to_INTERVAL((char*)"-494739383776393216") + (str_to_INTERVAL((char*)"246179115936718848") + (str_to_INTERVAL((char*)"18365472698363008") - str_to_INTERVAL((char*)"731989839491712") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-17515564321996800") + (str_to_INTERVAL((char*)"184161461913649152") + (str_to_INTERVAL((char*)"-1224947653862719488") + (str_to_INTERVAL((char*)"-3755104265656123392") + (str_to_INTERVAL((char*)"12775208110969036800") + (str_to_INTERVAL((char*)"17481117064984080384") + (str_to_INTERVAL((char*)"-1895156376295102464") + (str_to_INTERVAL((char*)"-2785377110262896640") + (str_to_INTERVAL((char*)"410243564225533952") + (str_to_INTERVAL((char*)"171401043587055616") + (str_to_INTERVAL((char*)"-2888000355061248") - str_to_INTERVAL((char*)"114084102449664") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-5722328059084800") + (str_to_INTERVAL((char*)"67819798140026880") + (str_to_INTERVAL((char*)"-419932583847002112") + (str_to_INTERVAL((char*)"-4293108105764274176") + (str_to_INTERVAL((char*)"6878512646413778944") + (str_to_INTERVAL((char*)"24241098846716428288") + (str_to_INTERVAL((char*)"5914809548586319872") + (str_to_INTERVAL((char*)"-5415786734789537792") + (str_to_INTERVAL((char*)"-328255296533940224") + (str_to_INTERVAL((char*)"483922728027829248") + (str_to_INTERVAL((char*)"18201475353239552") - str_to_INTERVAL((char*)"1073253474743040") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-541893863669760") + (str_to_INTERVAL((char*)"10871291965341696") + (str_to_INTERVAL((char*)"-75014361891209216") + (str_to_INTERVAL((char*)"-1700683300978688000") + (str_to_INTERVAL((char*)"922816642856550400") + (str_to_INTERVAL((char*)"15251548393359474688") + (str_to_INTERVAL((char*)"10974138735921487872") + (str_to_INTERVAL((char*)"-3125056412874166272") + (str_to_INTERVAL((char*)"-1634289979370756096") + (str_to_INTERVAL((char*)"477550768584610816") + (str_to_INTERVAL((char*)"91873115622203392") + (str_to_INTERVAL((char*)"-2052053557925376") - str_to_INTERVAL((char*)"61273193283072") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-11591032504320") + (str_to_INTERVAL((char*)"660695411589120") + (str_to_INTERVAL((char*)"-7998600137146368") + (str_to_INTERVAL((char*)"-252183247534948352") + (str_to_INTERVAL((char*)"-146753080849399808") + (str_to_INTERVAL((char*)"4338735841401307136") + (str_to_INTERVAL((char*)"5923964755891748864") + (str_to_INTERVAL((char*)"380946335427911680") + (str_to_INTERVAL((char*)"-1220671976681795584") + (str_to_INTERVAL((char*)"44664606096558080") + (str_to_INTERVAL((char*)"112356704206440448") + (str_to_INTERVAL((char*)"2108750369918464") - str_to_INTERVAL((char*)"184106509652736") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-27462205440") + (str_to_INTERVAL((char*)"9293634994176") + (str_to_INTERVAL((char*)"-364444169797632") + (str_to_INTERVAL((char*)"-12368970369466368") + (str_to_INTERVAL((char*)"-11234363792424960") + (str_to_INTERVAL((char*)"532477770545561600") + (str_to_INTERVAL((char*)"1110001898665607168") + (str_to_INTERVAL((char*)"484500048568991744") + (str_to_INTERVAL((char*)"-174678503522926592") + (str_to_INTERVAL((char*)"-91823906628026368") + (str_to_INTERVAL((char*)"38412448473751552") + (str_to_INTERVAL((char*)"4587397848203264") + (str_to_INTERVAL((char*)"-4141489787904") - str_to_INTERVAL((char*)"7890442947072") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"8493465600") + (str_to_INTERVAL((char*)"-3186342494208") + (str_to_INTERVAL((char*)"-99951958818816") + (str_to_INTERVAL((char*)"1049802166239232") + (str_to_INTERVAL((char*)"23050473921052672") + (str_to_INTERVAL((char*)"58542521648480256") + (str_to_INTERVAL((char*)"51376292847353856") + (str_to_INTERVAL((char*)"20152234112368640") + (str_to_INTERVAL((char*)"-17463622915956736") + (str_to_INTERVAL((char*)"1086362249609216") + (str_to_INTERVAL((char*)"1599147908753408") + (str_to_INTERVAL((char*)"148833631137792") - str_to_INTERVAL((char*)"4441978377216") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"162319564800") + (str_to_INTERVAL((char*)"10386111725568") + (str_to_INTERVAL((char*)"120236316557312") + (str_to_INTERVAL((char*)"219515998830592") + (str_to_INTERVAL((char*)"808094296309760") + (str_to_INTERVAL((char*)"2662679456546816") + (str_to_INTERVAL((char*)"-436975088615424") + (str_to_INTERVAL((char*)"-365668874559488") + (str_to_INTERVAL((char*)"138273193431040") + (str_to_INTERVAL((char*)"31033770254336") + (str_to_INTERVAL((char*)"8632647696384") - str_to_INTERVAL((char*)"466131557376") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"731683749888") + (str_to_INTERVAL((char*)"13391768715264") + (str_to_INTERVAL((char*)"38676812857344") + (str_to_INTERVAL((char*)"-5620639924224") + (str_to_INTERVAL((char*)"11530149888000") + (str_to_INTERVAL((char*)"-2985194446848") + (str_to_INTERVAL((char*)"-5283255226368") + (str_to_INTERVAL((char*)"3668340264960") - str_to_INTERVAL((char*)"140103843840") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + (str_to_INTERVAL((char*)"149952135168") + (str_to_INTERVAL((char*)"506743750656") + (str_to_INTERVAL((char*)"-204443025408") + (str_to_INTERVAL((char*)"-222138925056") + (str_to_INTERVAL((char*)"138359439360") + (str_to_INTERVAL((char*)"1315307520") - str_to_INTERVAL((char*)"464486400") * b) * b) * b) * b) * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a;
coef[29] = 32 * a * ((str_to_INTERVAL((char*)"-33246720") + (str_to_INTERVAL((char*)"1090506624") + (str_to_INTERVAL((char*)"-755432848") + (str_to_INTERVAL((char*)"-704719760") + (str_to_INTERVAL((char*)"-48987408") + (str_to_INTERVAL((char*)"6057936") + str_to_INTERVAL((char*)"27972") * b) * b) * b) * b) * b) * b) *  pow( b,  14) + ((str_to_INTERVAL((char*)"-2855260800") + (str_to_INTERVAL((char*)"5583876480") + (str_to_INTERVAL((char*)"17327690240") + (str_to_INTERVAL((char*)"-16390916080") + (str_to_INTERVAL((char*)"-33341000") + (str_to_INTERVAL((char*)"-510481650") + str_to_INTERVAL((char*)"955080") * b) * b) * b) * b) * b) * b) *  pow( b,  13) + ((str_to_INTERVAL((char*)"-40638533760") + (str_to_INTERVAL((char*)"-147179401248") + (str_to_INTERVAL((char*)"743043144096") + (str_to_INTERVAL((char*)"-430763221680") + (str_to_INTERVAL((char*)"-162785159658") + (str_to_INTERVAL((char*)"-50152201077") + (str_to_INTERVAL((char*)"3035376300") + str_to_INTERVAL((char*)"31539024") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"203829454080") + (str_to_INTERVAL((char*)"-3949109014656") + (str_to_INTERVAL((char*)"-2090467914784") + (str_to_INTERVAL((char*)"4987159474976") + (str_to_INTERVAL((char*)"-3526431341072") + (str_to_INTERVAL((char*)"-54950815078") + (str_to_INTERVAL((char*)"-165607308750") + str_to_INTERVAL((char*)"1469906478") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"4379826885120") + (str_to_INTERVAL((char*)"-18287558058816") + (str_to_INTERVAL((char*)"-117516806240224") + (str_to_INTERVAL((char*)"41072643139120") + (str_to_INTERVAL((char*)"-22865312414968") + (str_to_INTERVAL((char*)"989832872870") + (str_to_INTERVAL((char*)"-6531383805902") + (str_to_INTERVAL((char*)"189056249928") + str_to_INTERVAL((char*)"5405248404") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"11473755183360") + (str_to_INTERVAL((char*)"81878852218752") + (str_to_INTERVAL((char*)"-926039793376512") + (str_to_INTERVAL((char*)"-917851788555264") + (str_to_INTERVAL((char*)"268349340343392") + (str_to_INTERVAL((char*)"18363774596868") + (str_to_INTERVAL((char*)"-25557010073676") + (str_to_INTERVAL((char*)"-11317872764925") + str_to_INTERVAL((char*)"205908137289") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"-65481812467200") + (str_to_INTERVAL((char*)"781923319998720") + (str_to_INTERVAL((char*)"-1913293938776960") + (str_to_INTERVAL((char*)"-10389644917071360") + (str_to_INTERVAL((char*)"-1019164260280880") + (str_to_INTERVAL((char*)"1126920109175920") + (str_to_INTERVAL((char*)"248621066287500") + (str_to_INTERVAL((char*)"-244626463176620") + (str_to_INTERVAL((char*)"2520168155100") + str_to_INTERVAL((char*)"206337419460") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-250768958607360") + (str_to_INTERVAL((char*)"1137243550459392") + (str_to_INTERVAL((char*)"6700803264461056") + (str_to_INTERVAL((char*)"-40628924790036864") + (str_to_INTERVAL((char*)"-35501074962436032") + (str_to_INTERVAL((char*)"9063077328921424") + (str_to_INTERVAL((char*)"4074872605617624") + (str_to_INTERVAL((char*)"-1252496816249416") + (str_to_INTERVAL((char*)"-198473637812650") + str_to_INTERVAL((char*)"5456409896493") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"58465176330240") + (str_to_INTERVAL((char*)"-2923637478130176") + (str_to_INTERVAL((char*)"36830476381918464") + (str_to_INTERVAL((char*)"-58635120652647424") + (str_to_INTERVAL((char*)"-181760162747593344") + (str_to_INTERVAL((char*)"-10664378960758880") + (str_to_INTERVAL((char*)"30466403502240160") + (str_to_INTERVAL((char*)"-503954828142744") + (str_to_INTERVAL((char*)"-2493920181053608") + (str_to_INTERVAL((char*)"23560260039804") + str_to_INTERVAL((char*)"1842414068268") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"656138568683520") + (str_to_INTERVAL((char*)"-7629586384069632") + (str_to_INTERVAL((char*)"57339900560707584") + (str_to_INTERVAL((char*)"17471310899503104") + (str_to_INTERVAL((char*)"-392073813442749568") + (str_to_INTERVAL((char*)"-209766342497594112") + (str_to_INTERVAL((char*)"91725369519937280") + (str_to_INTERVAL((char*)"20923679140043856") + (str_to_INTERVAL((char*)"-9792489255549072") + (str_to_INTERVAL((char*)"-733015007300688") + str_to_INTERVAL((char*)"28690120490058") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"477808075653120") + (str_to_INTERVAL((char*)"-5265738822131712") + (str_to_INTERVAL((char*)"35594667270308864") + (str_to_INTERVAL((char*)"119428735191161856") + (str_to_INTERVAL((char*)"-389410243565293568") + (str_to_INTERVAL((char*)"-546010127209919360") + (str_to_INTERVAL((char*)"52593328546972480") + (str_to_INTERVAL((char*)"89088792730480000") + (str_to_INTERVAL((char*)"-12901747668717728") + (str_to_INTERVAL((char*)"-5244926986957552") + (str_to_INTERVAL((char*)"83637984677064") + str_to_INTERVAL((char*)"3390599852856") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"100394802708480") + (str_to_INTERVAL((char*)"-1445184103022592") + (str_to_INTERVAL((char*)"9987715906265088") + (str_to_INTERVAL((char*)"95349459053737984") + (str_to_INTERVAL((char*)"-163030435090200064") + (str_to_INTERVAL((char*)"-572392461412818432") + (str_to_INTERVAL((char*)"-142090312637531136") + (str_to_INTERVAL((char*)"126470373757956928") + (str_to_INTERVAL((char*)"8259202767237728") + (str_to_INTERVAL((char*)"-11399339381425680") + (str_to_INTERVAL((char*)"-422231511610032") + str_to_INTERVAL((char*)"24575318693676") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"5777918853120") + (str_to_INTERVAL((char*)"-164592497516544") + (str_to_INTERVAL((char*)"1494879220723712") + (str_to_INTERVAL((char*)"26432434687207424") + (str_to_INTERVAL((char*)"-22039367821367296") + (str_to_INTERVAL((char*)"-266476788072732672") + (str_to_INTERVAL((char*)"-186887146443185408") + (str_to_INTERVAL((char*)"51001012922685952") + (str_to_INTERVAL((char*)"28246850161587712") + (str_to_INTERVAL((char*)"-8117271996639936") + (str_to_INTERVAL((char*)"-1582933370391536") + (str_to_INTERVAL((char*)"34310231056872") + str_to_INTERVAL((char*)"1014742108512") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"64192020480") + (str_to_INTERVAL((char*)"-6046920327168") + (str_to_INTERVAL((char*)"113723628552192") + (str_to_INTERVAL((char*)"2588660257103872") + (str_to_INTERVAL((char*)"-778818577670144") + (str_to_INTERVAL((char*)"-54020592301609984") + (str_to_INTERVAL((char*)"-68904610975407616") + (str_to_INTERVAL((char*)"-5190616208884992") + (str_to_INTERVAL((char*)"13584438500214272") + (str_to_INTERVAL((char*)"-221519114908288") + (str_to_INTERVAL((char*)"-1370444139888864") + (str_to_INTERVAL((char*)"-24988552457760") + str_to_INTERVAL((char*)"1977669389808") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"44236800") + (str_to_INTERVAL((char*)"-36962205696") + (str_to_INTERVAL((char*)"2819406938112") + (str_to_INTERVAL((char*)"66607886336000") + (str_to_INTERVAL((char*)"-182326509514752") + (str_to_INTERVAL((char*)"-4326128430942208") + (str_to_INTERVAL((char*)"-7883005934757888") + (str_to_INTERVAL((char*)"-3531866000787456") + (str_to_INTERVAL((char*)"781965888717824") + (str_to_INTERVAL((char*)"851835794352384") + (str_to_INTERVAL((char*)"-307460622376576") + (str_to_INTERVAL((char*)"-36691313515520") + (str_to_INTERVAL((char*)"-767113058880") + str_to_INTERVAL((char*)"91300688496") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"6804799488") + (str_to_INTERVAL((char*)"10518331392") + (str_to_INTERVAL((char*)"-8364668223488") + (str_to_INTERVAL((char*)"-88802368954368") + (str_to_INTERVAL((char*)"-178250417356800") + (str_to_INTERVAL((char*)"-193808494451712") + (str_to_INTERVAL((char*)"-176414830577664") + (str_to_INTERVAL((char*)"104799314054400") + (str_to_INTERVAL((char*)"-6627114297344") + (str_to_INTERVAL((char*)"-7984384749056") + (str_to_INTERVAL((char*)"-1454652095040") + str_to_INTERVAL((char*)"31162634064") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"57323814912") + (str_to_INTERVAL((char*)"408149692416") + (str_to_INTERVAL((char*)"-3211387914240") + (str_to_INTERVAL((char*)"-9647087861760") + (str_to_INTERVAL((char*)"2863853352960") + (str_to_INTERVAL((char*)"82850969856") + (str_to_INTERVAL((char*)"28634669184") + (str_to_INTERVAL((char*)"-191504586240") + (str_to_INTERVAL((char*)"-60363886848") + str_to_INTERVAL((char*)"3132619776") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + (str_to_INTERVAL((char*)"18831605760") + (str_to_INTERVAL((char*)"11354517504") + (str_to_INTERVAL((char*)"-103520507904") + (str_to_INTERVAL((char*)"53428455936") + (str_to_INTERVAL((char*)"11435742720") + (str_to_INTERVAL((char*)"-12692021760") + str_to_INTERVAL((char*)"509299200") * b) * b) * b) * b) * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[30] = -32 * b * ((str_to_INTERVAL((char*)"1615680") + (str_to_INTERVAL((char*)"-1895976") + (str_to_INTERVAL((char*)"-4625220") + (str_to_INTERVAL((char*)"149694") + (str_to_INTERVAL((char*)"17964") + 36 * b) * b) * b) * b) * b) *  pow( b,  15) + ((str_to_INTERVAL((char*)"7637760") + (str_to_INTERVAL((char*)"83070384") + (str_to_INTERVAL((char*)"-93365376") + (str_to_INTERVAL((char*)"-3093756") + (str_to_INTERVAL((char*)"-3029172") - 3852 * b) * b) * b) * b) * b) *  pow( b,  14) + ((str_to_INTERVAL((char*)"-507098880") + (str_to_INTERVAL((char*)"10392245760") + (str_to_INTERVAL((char*)"-6606889080") + (str_to_INTERVAL((char*)"-6653307660") + (str_to_INTERVAL((char*)"-440742276") + (str_to_INTERVAL((char*)"54916626") + str_to_INTERVAL((char*)"279846") * b) * b) * b) * b) * b) * b) *  pow( b,  13) + ((str_to_INTERVAL((char*)"-24671839680") + (str_to_INTERVAL((char*)"43614308160") + (str_to_INTERVAL((char*)"148871417728") + (str_to_INTERVAL((char*)"-140065815896") + (str_to_INTERVAL((char*)"58652348") + (str_to_INTERVAL((char*)"-4407561435") + str_to_INTERVAL((char*)"6865092") * b) * b) * b) * b) * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"-167530904640") + (str_to_INTERVAL((char*)"-1007640923520") + (str_to_INTERVAL((char*)"3118896586656") + (str_to_INTERVAL((char*)"-1697233435312") + (str_to_INTERVAL((char*)"-709078144894") + (str_to_INTERVAL((char*)"-213763873157") + (str_to_INTERVAL((char*)"11588426970") + str_to_INTERVAL((char*)"158993748") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"1254482277120") + (str_to_INTERVAL((char*)"-16022705095296") + (str_to_INTERVAL((char*)"-13977138585696") + (str_to_INTERVAL((char*)"18404064168960") + (str_to_INTERVAL((char*)"-12684673109408") + (str_to_INTERVAL((char*)"-4950397762") + (str_to_INTERVAL((char*)"-692047787434") + str_to_INTERVAL((char*)"5817640347") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"14192837452800") + (str_to_INTERVAL((char*)"-42423433240320") + (str_to_INTERVAL((char*)"-403408925583936") + (str_to_INTERVAL((char*)"49645945355552") + (str_to_INTERVAL((char*)"-37676440839856") + (str_to_INTERVAL((char*)"9392892970560") + (str_to_INTERVAL((char*)"-18328787052366") + (str_to_INTERVAL((char*)"327399866475") + str_to_INTERVAL((char*)"16995056004") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"13666285681920") + (str_to_INTERVAL((char*)"288325956186240") + (str_to_INTERVAL((char*)"-2277678944802048") + (str_to_INTERVAL((char*)"-2736141417150592") + (str_to_INTERVAL((char*)"564753082081088") + (str_to_INTERVAL((char*)"157327029194004") + (str_to_INTERVAL((char*)"-58473496881732") + (str_to_INTERVAL((char*)"-30143070446897") + str_to_INTERVAL((char*)"529368661089") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-173284501754880") + (str_to_INTERVAL((char*)"1583045621610240") + (str_to_INTERVAL((char*)"-2770482620310144") + (str_to_INTERVAL((char*)"-22130464025054016") + (str_to_INTERVAL((char*)"-3895603378215872") + (str_to_INTERVAL((char*)"3014974760602896") + (str_to_INTERVAL((char*)"581324771879632") + (str_to_INTERVAL((char*)"-486897600521056") + (str_to_INTERVAL((char*)"1775173335048") + str_to_INTERVAL((char*)"424898223732") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-332121950576640") + (str_to_INTERVAL((char*)"995231559237120") + (str_to_INTERVAL((char*)"14663954320594688") + (str_to_INTERVAL((char*)"-65967795497584256") + (str_to_INTERVAL((char*)"-64438579053801152") + (str_to_INTERVAL((char*)"14590157510429488") + (str_to_INTERVAL((char*)"7817224300229960") + (str_to_INTERVAL((char*)"-2099709887479880") + (str_to_INTERVAL((char*)"-342769358002646") + str_to_INTERVAL((char*)"9210047462307") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"282590376099840") + (str_to_INTERVAL((char*)"-5162293346396160") + (str_to_INTERVAL((char*)"51923174515040256") + (str_to_INTERVAL((char*)"-69269836985206528") + (str_to_INTERVAL((char*)"-249517200907642752") + (str_to_INTERVAL((char*)"-21452706618403936") + (str_to_INTERVAL((char*)"44236477518777456") + (str_to_INTERVAL((char*)"-688595371377960") + (str_to_INTERVAL((char*)"-3326674256914772") + (str_to_INTERVAL((char*)"23653783179702") + str_to_INTERVAL((char*)"2445806174760") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"724382557286400") + (str_to_INTERVAL((char*)"-8111153439470592") + (str_to_INTERVAL((char*)"59084132625549312") + (str_to_INTERVAL((char*)"29869795443969024") + (str_to_INTERVAL((char*)"-417346010418158208") + (str_to_INTERVAL((char*)"-233837098669587456") + (str_to_INTERVAL((char*)"98362588006862656") + (str_to_INTERVAL((char*)"23621420604744016") + (str_to_INTERVAL((char*)"-10523478131389856") + (str_to_INTERVAL((char*)"-788934473681240") + str_to_INTERVAL((char*)"30503208501630") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"331563125882880") + (str_to_INTERVAL((char*)"-4008939437408256") + (str_to_INTERVAL((char*)"28114469024088064") + (str_to_INTERVAL((char*)"98858038314012672") + (str_to_INTERVAL((char*)"-319762375614692864") + (str_to_INTERVAL((char*)"-456525989114464512") + (str_to_INTERVAL((char*)"39578005453676032") + (str_to_INTERVAL((char*)"75514571497320320") + (str_to_INTERVAL((char*)"-10768228584977408") + (str_to_INTERVAL((char*)"-4319671589940960") + (str_to_INTERVAL((char*)"66520993806432") + str_to_INTERVAL((char*)"2705628615984") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"43846927810560") + (str_to_INTERVAL((char*)"-800704469827584") + (str_to_INTERVAL((char*)"6360284003942400") + (str_to_INTERVAL((char*)"54909406841038848") + (str_to_INTERVAL((char*)"-104773128826870272") + (str_to_INTERVAL((char*)"-357750073594976768") + (str_to_INTERVAL((char*)"-89160021333079552") + (str_to_INTERVAL((char*)"77408193491121600") + (str_to_INTERVAL((char*)"5483320039540128") + (str_to_INTERVAL((char*)"-7089581709278960") + (str_to_INTERVAL((char*)"-259591543506224") + str_to_INTERVAL((char*)"14873531673492") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"1446437191680") + (str_to_INTERVAL((char*)"-60705535918080") + (str_to_INTERVAL((char*)"737393587912704") + (str_to_INTERVAL((char*)"10351280741646336") + (str_to_INTERVAL((char*)"-13706231513295872") + (str_to_INTERVAL((char*)"-120718355361552384") + (str_to_INTERVAL((char*)"-81535413646004480") + (str_to_INTERVAL((char*)"21081971791252096") + (str_to_INTERVAL((char*)"12477847729545024") + (str_to_INTERVAL((char*)"-3510855882576096") + (str_to_INTERVAL((char*)"-711176231705312") + (str_to_INTERVAL((char*)"14916961096224") + str_to_INTERVAL((char*)"433797351984") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"7002685440") + (str_to_INTERVAL((char*)"-1200291692544") + (str_to_INTERVAL((char*)"36371088179200") + (str_to_INTERVAL((char*)"612307494526976") + (str_to_INTERVAL((char*)"-1147996806795264") + (str_to_INTERVAL((char*)"-16736577852591104") + (str_to_INTERVAL((char*)"-19567859151341056") + (str_to_INTERVAL((char*)"-1857760743013120") + (str_to_INTERVAL((char*)"3610238842856704") + (str_to_INTERVAL((char*)"84363402767744") + (str_to_INTERVAL((char*)"-426249651600160") + (str_to_INTERVAL((char*)"-7508143280896") + str_to_INTERVAL((char*)"485175968208") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-2123366400") + (str_to_INTERVAL((char*)"403347013632") + (str_to_INTERVAL((char*)"5615863791616") + (str_to_INTERVAL((char*)"-80170453614592") + (str_to_INTERVAL((char*)"-782976241741824") + (str_to_INTERVAL((char*)"-1212369144114176") + (str_to_INTERVAL((char*)"-626023697217024") + (str_to_INTERVAL((char*)"-9805112109568") + (str_to_INTERVAL((char*)"210122902486656") + (str_to_INTERVAL((char*)"-64626866536320") + (str_to_INTERVAL((char*)"-7165794455360") + (str_to_INTERVAL((char*)"-476857972896") + str_to_INTERVAL((char*)"30177058032") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-19021824000") + (str_to_INTERVAL((char*)"-681326542848") + (str_to_INTERVAL((char*)"-4580812480512") + (str_to_INTERVAL((char*)"-5438785597440") + (str_to_INTERVAL((char*)"-20931489811456") + (str_to_INTERVAL((char*)"-30922787646464") + (str_to_INTERVAL((char*)"17673725546240") + (str_to_INTERVAL((char*)"-2268430491392") + (str_to_INTERVAL((char*)"-669358390144") + (str_to_INTERVAL((char*)"-388225219392") + str_to_INTERVAL((char*)"6443466480") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-31439536128") + (str_to_INTERVAL((char*)"-349422391296") + (str_to_INTERVAL((char*)"-477973721088") + (str_to_INTERVAL((char*)"251581403136") + (str_to_INTERVAL((char*)"-215802356736") + (str_to_INTERVAL((char*)"140143432960") + (str_to_INTERVAL((char*)"-35745668352") + (str_to_INTERVAL((char*)"-9479194368") + str_to_INTERVAL((char*)"490694400") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + (str_to_INTERVAL((char*)"-2733502464") + (str_to_INTERVAL((char*)"-4001918976") + (str_to_INTERVAL((char*)"4328492544") + (str_to_INTERVAL((char*)"-306685440") + (str_to_INTERVAL((char*)"-676154880") + str_to_INTERVAL((char*)"29836800") * b) * b) * b) * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[31] = 16 * b * a * ((str_to_INTERVAL((char*)"63011520") + (str_to_INTERVAL((char*)"-73943064") + (str_to_INTERVAL((char*)"-180383580") + (str_to_INTERVAL((char*)"5838066") + (str_to_INTERVAL((char*)"700596") + 1404 * b) * b) * b) * b) * b) *  pow( b,  15) + ((str_to_INTERVAL((char*)"282597120") + (str_to_INTERVAL((char*)"3073604208") + (str_to_INTERVAL((char*)"-3454518912") + (str_to_INTERVAL((char*)"-114468972") + (str_to_INTERVAL((char*)"-112079364") - str_to_INTERVAL((char*)"142524") * b) * b) * b) * b) * b) *  pow( b,  14) + ((str_to_INTERVAL((char*)"-10766649600") + (str_to_INTERVAL((char*)"134722210560") + (str_to_INTERVAL((char*)"-72600219720") + (str_to_INTERVAL((char*)"-84874618500") + (str_to_INTERVAL((char*)"-5138623980") + (str_to_INTERVAL((char*)"649915350") + str_to_INTERVAL((char*)"3920490") * b) * b) * b) * b) * b) * b) *  pow( b,  13) + ((str_to_INTERVAL((char*)"-280236939840") + (str_to_INTERVAL((char*)"395087267520") + (str_to_INTERVAL((char*)"1672478710144") + (str_to_INTERVAL((char*)"-1557070617608") + (str_to_INTERVAL((char*)"8170294484") + (str_to_INTERVAL((char*)"-49989458805") + str_to_INTERVAL((char*)"47948076") * b) * b) * b) * b) * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"-957754860480") + (str_to_INTERVAL((char*)"-9984562625664") + (str_to_INTERVAL((char*)"19754645380704") + (str_to_INTERVAL((char*)"-10305373109840") + (str_to_INTERVAL((char*)"-4780071665890") + (str_to_INTERVAL((char*)"-1428931365611") + (str_to_INTERVAL((char*)"63993353382") + str_to_INTERVAL((char*)"1303752492") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"10991499932160") + (str_to_INTERVAL((char*)"-100823205888000") + (str_to_INTERVAL((char*)"-132035955040448") + (str_to_INTERVAL((char*)"102179905899232") + (str_to_INTERVAL((char*)"-68557236640272") + (str_to_INTERVAL((char*)"1691662762764") + (str_to_INTERVAL((char*)"-4608060488660") + str_to_INTERVAL((char*)"36356710401") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"70122414044160") + (str_to_INTERVAL((char*)"-127539213552000") + (str_to_INTERVAL((char*)"-2244910641921216") + (str_to_INTERVAL((char*)"-142813179707616") + (str_to_INTERVAL((char*)"-52946186868288") + (str_to_INTERVAL((char*)"84339535593636") + (str_to_INTERVAL((char*)"-87237287891334") + (str_to_INTERVAL((char*)"491002777215") + str_to_INTERVAL((char*)"90897785604") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"-17854303564800") + (str_to_INTERVAL((char*)"1528615665219840") + (str_to_INTERVAL((char*)"-9157748671322880") + (str_to_INTERVAL((char*)"-13356767647648000") + (str_to_INTERVAL((char*)"1849629685631360") + (str_to_INTERVAL((char*)"1153569631937040") + (str_to_INTERVAL((char*)"-224179435429680") + (str_to_INTERVAL((char*)"-134692746580400") + str_to_INTERVAL((char*)"2302031006280") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-675586832394240") + (str_to_INTERVAL((char*)"5071768348926720") + (str_to_INTERVAL((char*)"-5136947311286912") + (str_to_INTERVAL((char*)"-79166613980678208") + (str_to_INTERVAL((char*)"-19454797933851456") + (str_to_INTERVAL((char*)"12589227752444128") + (str_to_INTERVAL((char*)"2293779942550616") + (str_to_INTERVAL((char*)"-1669417395607108") + (str_to_INTERVAL((char*)"-3444809214456") + str_to_INTERVAL((char*)"1506597681276") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-601711047659520") + (str_to_INTERVAL((char*)"42451563718656") + (str_to_INTERVAL((char*)"50764986979104256") + (str_to_INTERVAL((char*)"-180469166397658368") + (str_to_INTERVAL((char*)"-196560778532364288") + (str_to_INTERVAL((char*)"39815956288270560") + (str_to_INTERVAL((char*)"24898801881516112") + (str_to_INTERVAL((char*)"-6015931041148848") + (str_to_INTERVAL((char*)"-1003881461425796") + str_to_INTERVAL((char*)"26584607637666") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"1025944400977920") + (str_to_INTERVAL((char*)"-13949571100529664") + (str_to_INTERVAL((char*)"121731727267014144") + (str_to_INTERVAL((char*)"-136355178912497920") + (str_to_INTERVAL((char*)"-580411388191850368") + (str_to_INTERVAL((char*)"-63244976858289792") + (str_to_INTERVAL((char*)"107343744220560032") + (str_to_INTERVAL((char*)"-1530849274784688") + (str_to_INTERVAL((char*)"-7584132840900344") + (str_to_INTERVAL((char*)"40740896159172") + str_to_INTERVAL((char*)"5559377039640") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"1259834646528000") + (str_to_INTERVAL((char*)"-14236708499496960") + (str_to_INTERVAL((char*)"102546240899948544") + (str_to_INTERVAL((char*)"69488116587408384") + (str_to_INTERVAL((char*)"-750988671806198272") + (str_to_INTERVAL((char*)"-437075315926390528") + (str_to_INTERVAL((char*)"177780134505525760") + (str_to_INTERVAL((char*)"44560182134598720") + (str_to_INTERVAL((char*)"-19095397581348192") + (str_to_INTERVAL((char*)"-1433347717575904") + str_to_INTERVAL((char*)"54979954024608") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"363393523998720") + (str_to_INTERVAL((char*)"-5046798675050496") + (str_to_INTERVAL((char*)"37623774659448832") + (str_to_INTERVAL((char*)"132794702136293376") + (str_to_INTERVAL((char*)"-442225858427286016") + (str_to_INTERVAL((char*)"-636968231268067328") + (str_to_INTERVAL((char*)"50491580050278272") + (str_to_INTERVAL((char*)"105794706992071360") + (str_to_INTERVAL((char*)"-14864615139819648") + (str_to_INTERVAL((char*)"-5962489060575888") + (str_to_INTERVAL((char*)"90201005055264") + str_to_INTERVAL((char*)"3597327284112") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"29171099566080") + (str_to_INTERVAL((char*)"-705952792412160") + (str_to_INTERVAL((char*)"6658542020640768") + (str_to_INTERVAL((char*)"50599069791965184") + (str_to_INTERVAL((char*)"-113590817753484288") + (str_to_INTERVAL((char*)"-367102474764597248") + (str_to_INTERVAL((char*)"-90539589525482496") + (str_to_INTERVAL((char*)"76781592367443072") + (str_to_INTERVAL((char*)"5987241902625344") + (str_to_INTERVAL((char*)"-7217454966675808") + (str_to_INTERVAL((char*)"-262695031481216") + str_to_INTERVAL((char*)"14718602440728") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"500725186560") + (str_to_INTERVAL((char*)"-32830235983872") + (str_to_INTERVAL((char*)"554307547258880") + (str_to_INTERVAL((char*)"6162141311602688") + (str_to_INTERVAL((char*)"-13402138863501312") + (str_to_INTERVAL((char*)"-87174995303823360") + (str_to_INTERVAL((char*)"-55834240142433280") + (str_to_INTERVAL((char*)"13305050273762304") + (str_to_INTERVAL((char*)"8686179609944448") + (str_to_INTERVAL((char*)"-2364402228773952") + (str_to_INTERVAL((char*)"-516140321183616") + (str_to_INTERVAL((char*)"10358738408928") + str_to_INTERVAL((char*)"295431225120") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"707788800") + (str_to_INTERVAL((char*)"-289076281344") + (str_to_INTERVAL((char*)"15615940427776") + (str_to_INTERVAL((char*)"185152494764032") + (str_to_INTERVAL((char*)"-995036027056128") + (str_to_INTERVAL((char*)"-7752857658347520") + (str_to_INTERVAL((char*)"-8105563326181376") + (str_to_INTERVAL((char*)"-1125033974093312") + (str_to_INTERVAL((char*)"1362207976225024") + (str_to_INTERVAL((char*)"152763421051520") + (str_to_INTERVAL((char*)"-212028795775872") + (str_to_INTERVAL((char*)"-3524946987488") + str_to_INTERVAL((char*)"133473246480") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"50943098880") + (str_to_INTERVAL((char*)"-221468884992") + (str_to_INTERVAL((char*)"-28094106419200") + (str_to_INTERVAL((char*)"-172785050005504") + (str_to_INTERVAL((char*)"-214587015114752") + (str_to_INTERVAL((char*)"-182769002344448") + (str_to_INTERVAL((char*)"-58066046921728") + (str_to_INTERVAL((char*)"85043004460288") + (str_to_INTERVAL((char*)"-23877576943616") + (str_to_INTERVAL((char*)"-1993770780032") + (str_to_INTERVAL((char*)"-364812448704") + str_to_INTERVAL((char*)"17503423488") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"157640491008") + (str_to_INTERVAL((char*)"433209655296") + (str_to_INTERVAL((char*)"-5264966756352") + (str_to_INTERVAL((char*)"-5972338692096") + (str_to_INTERVAL((char*)"4198075788288") + (str_to_INTERVAL((char*)"-1096290502144") + (str_to_INTERVAL((char*)"158906529408") + (str_to_INTERVAL((char*)"-169154285312") + str_to_INTERVAL((char*)"2428089984") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + (str_to_INTERVAL((char*)"21970206720") + (str_to_INTERVAL((char*)"-15100968960") + (str_to_INTERVAL((char*)"-49490012160") + (str_to_INTERVAL((char*)"46440407040") + (str_to_INTERVAL((char*)"-10471280640") + (str_to_INTERVAL((char*)"-1832448000") + str_to_INTERVAL((char*)"99993600") * b) * b) * b) * b) * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[32] = -16 * b * b * ((str_to_INTERVAL((char*)"-108000") + (str_to_INTERVAL((char*)"-654648") + (str_to_INTERVAL((char*)"59646") + 939 * b) * b) * b) *  pow( b,  16) + ((str_to_INTERVAL((char*)"4896720") + (str_to_INTERVAL((char*)"-10711704") + (str_to_INTERVAL((char*)"-1497480") + (str_to_INTERVAL((char*)"-420612") - 531 * b) * b) * b) * b) *  pow( b,  15) + ((str_to_INTERVAL((char*)"628354800") + (str_to_INTERVAL((char*)"-635454900") + (str_to_INTERVAL((char*)"-1785864090") + (str_to_INTERVAL((char*)"56923131") + (str_to_INTERVAL((char*)"6875022") + str_to_INTERVAL((char*)"14922") * b) * b) * b) * b) * b) *  pow( b,  14) + ((str_to_INTERVAL((char*)"2243099520") + (str_to_INTERVAL((char*)"28366837752") + (str_to_INTERVAL((char*)"-31202000736") + (str_to_INTERVAL((char*)"-1074215406") + (str_to_INTERVAL((char*)"-1007218110") - str_to_INTERVAL((char*)"1682244") * b) * b) * b) * b) * b) *  pow( b,  13) + ((str_to_INTERVAL((char*)"-77320984320") + (str_to_INTERVAL((char*)"654965405472") + (str_to_INTERVAL((char*)"-279607339872") + (str_to_INTERVAL((char*)"-407178051294") + (str_to_INTERVAL((char*)"-21509549086") + (str_to_INTERVAL((char*)"2784241209") + str_to_INTERVAL((char*)"21278052") * b) * b) * b) * b) * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"-1164415901760") + (str_to_INTERVAL((char*)"1056171513024") + (str_to_INTERVAL((char*)"6864724053440") + (str_to_INTERVAL((char*)"-6323973796976") + (str_to_INTERVAL((char*)"81641135616") + (str_to_INTERVAL((char*)"-209332659537") + str_to_INTERVAL((char*)"14342904") * b) * b) * b) * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"-1577875619520") + (str_to_INTERVAL((char*)"-37829156005824") + (str_to_INTERVAL((char*)"46799794612896") + (str_to_INTERVAL((char*)"-25115693151952") + (str_to_INTERVAL((char*)"-12671149218930") + (str_to_INTERVAL((char*)"-3872021887593") + (str_to_INTERVAL((char*)"119434926348") + str_to_INTERVAL((char*)"4484858112") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"36035065658880") + (str_to_INTERVAL((char*)"-251639789562240") + (str_to_INTERVAL((char*)"-471823519091200") + (str_to_INTERVAL((char*)"214941787153760") + (str_to_INTERVAL((char*)"-141601040816672") + (str_to_INTERVAL((char*)"9699839478356") + (str_to_INTERVAL((char*)"-12684919922982") + str_to_INTERVAL((char*)"93557844819") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"131752274595840") + (str_to_INTERVAL((char*)"-51280425664128") + (str_to_INTERVAL((char*)"-5155251222137536") + (str_to_INTERVAL((char*)"-1180731940483264") + (str_to_INTERVAL((char*)"136980914753040") + (str_to_INTERVAL((char*)"268778599493208") + (str_to_INTERVAL((char*)"-178991612933318") + (str_to_INTERVAL((char*)"-1295017326867") + str_to_INTERVAL((char*)"209257849260") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-172325491614720") + (str_to_INTERVAL((char*)"3173576390506752") + (str_to_INTERVAL((char*)"-15169586457067008") + (str_to_INTERVAL((char*)"-27124938669319936") + (str_to_INTERVAL((char*)"2260986766152512") + (str_to_INTERVAL((char*)"2959916784288048") + (str_to_INTERVAL((char*)"-367671303235072") + (str_to_INTERVAL((char*)"-255804264796192") + str_to_INTERVAL((char*)"4291847899308") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-998225631912960") + (str_to_INTERVAL((char*)"6369582450761472") + (str_to_INTERVAL((char*)"-483081060295552") + (str_to_INTERVAL((char*)"-119946388987755072") + (str_to_INTERVAL((char*)"-37196398596187008") + (str_to_INTERVAL((char*)"21421422533046048") + (str_to_INTERVAL((char*)"3833081119141976") + (str_to_INTERVAL((char*)"-2476234586204724") + (str_to_INTERVAL((char*)"-16874897710440") + str_to_INTERVAL((char*)"2308643351388") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-257550373232640") + (str_to_INTERVAL((char*)"-3305516287196160") + (str_to_INTERVAL((char*)"71281092076170240") + (str_to_INTERVAL((char*)"-209142419679245312") + (str_to_INTERVAL((char*)"-253674214375902336") + (str_to_INTERVAL((char*)"46349349356405920") + (str_to_INTERVAL((char*)"33234270032905744") + (str_to_INTERVAL((char*)"-7392497971707504") + (str_to_INTERVAL((char*)"-1252912022322332") + str_to_INTERVAL((char*)"32918834105838") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"1254568776376320") + (str_to_INTERVAL((char*)"-15003770655854592") + (str_to_INTERVAL((char*)"119462368517882880") + (str_to_INTERVAL((char*)"-112427116248673024") + (str_to_INTERVAL((char*)"-573345241603678336") + (str_to_INTERVAL((char*)"-73517028255489472") + (str_to_INTERVAL((char*)"109448505175705888") + (str_to_INTERVAL((char*)"-1406626009818544") + (str_to_INTERVAL((char*)"-7385944533944352") + (str_to_INTERVAL((char*)"30264247584912") + str_to_INTERVAL((char*)"5402411993016") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"866446624604160") + (str_to_INTERVAL((char*)"-10337774415642624") + (str_to_INTERVAL((char*)"75162050254454784") + (str_to_INTERVAL((char*)"60532789054930944") + (str_to_INTERVAL((char*)"-571075795666692096") + (str_to_INTERVAL((char*)"-342300462328895744") + (str_to_INTERVAL((char*)"135245866564604416") + (str_to_INTERVAL((char*)"35074321620394880") + (str_to_INTERVAL((char*)"-14590185328808384") + (str_to_INTERVAL((char*)"-1098015263260704") + str_to_INTERVAL((char*)"41898589187472") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"155257108439040") + (str_to_INTERVAL((char*)"-2593801811607552") + (str_to_INTERVAL((char*)"21152341627498496") + (str_to_INTERVAL((char*)"71780204623005696") + (str_to_INTERVAL((char*)"-256906546520589824") + (str_to_INTERVAL((char*)"-368688103407921664") + (str_to_INTERVAL((char*)"27234924487472768") + (str_to_INTERVAL((char*)"60871685232194880") + (str_to_INTERVAL((char*)"-8422719289436032") + (str_to_INTERVAL((char*)"-3427700010442800") + (str_to_INTERVAL((char*)"51710872397664") + str_to_INTERVAL((char*)"1969678020528") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"7137227243520") + (str_to_INTERVAL((char*)"-240755799883776") + (str_to_INTERVAL((char*)"2806197695594496") + (str_to_INTERVAL((char*)"18227657171193856") + (str_to_INTERVAL((char*)"-51367423917159424") + (str_to_INTERVAL((char*)"-152763623787979776") + (str_to_INTERVAL((char*)"-36669320623988736") + (str_to_INTERVAL((char*)"30313979822745216") + (str_to_INTERVAL((char*)"2684733423711808") + (str_to_INTERVAL((char*)"-2972988941975520") + (str_to_INTERVAL((char*)"-108360518591904") + str_to_INTERVAL((char*)"5861823014376") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"53491138560") + (str_to_INTERVAL((char*)"-6120914681856") + (str_to_INTERVAL((char*)"153519588573184") + (str_to_INTERVAL((char*)"1305315987767296") + (str_to_INTERVAL((char*)"-4998707687589888") + (str_to_INTERVAL((char*)"-24456178882734080") + (str_to_INTERVAL((char*)"-14556255732914176") + (str_to_INTERVAL((char*)"2971546462558976") + (str_to_INTERVAL((char*)"2349383180986880") + (str_to_INTERVAL((char*)"-600916019415424") + (str_to_INTERVAL((char*)"-150128452075360") + (str_to_INTERVAL((char*)"2804304199440") + str_to_INTERVAL((char*)"79730828352") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-15925248000") + (str_to_INTERVAL((char*)"2018750496768") + (str_to_INTERVAL((char*)"12135926759424") + (str_to_INTERVAL((char*)"-244074740207616") + (str_to_INTERVAL((char*)"-1254734403133440") + (str_to_INTERVAL((char*)"-1132913152720896") + (str_to_INTERVAL((char*)"-294564313352704") + (str_to_INTERVAL((char*)"172508047913728") + (str_to_INTERVAL((char*)"58487479281280") + (str_to_INTERVAL((char*)"-43009057486848") + (str_to_INTERVAL((char*)"-646528932384") - str_to_INTERVAL((char*)"9095270256") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-88768512000") + (str_to_INTERVAL((char*)"-1973478162432") + (str_to_INTERVAL((char*)"-7928430231552") + (str_to_INTERVAL((char*)"-6449607548928") + (str_to_INTERVAL((char*)"-25264760536064") + (str_to_INTERVAL((char*)"-10456095374336") + (str_to_INTERVAL((char*)"12950107951104") + (str_to_INTERVAL((char*)"-3848047913472") + (str_to_INTERVAL((char*)"-135916823552") + (str_to_INTERVAL((char*)"-102606732288") + str_to_INTERVAL((char*)"4258528128") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-62879072256") + (str_to_INTERVAL((char*)"-423539128320") + (str_to_INTERVAL((char*)"-176659046400") + (str_to_INTERVAL((char*)"188383552512") + (str_to_INTERVAL((char*)"-124913556992") + (str_to_INTERVAL((char*)"69266665344") + (str_to_INTERVAL((char*)"-27776485632") + str_to_INTERVAL((char*)"396489600") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + (str_to_INTERVAL((char*)"-2343002112") + (str_to_INTERVAL((char*)"-682767360") + (str_to_INTERVAL((char*)"2216156160") + (str_to_INTERVAL((char*)"-794878464") + (str_to_INTERVAL((char*)"-79687680") + str_to_INTERVAL((char*)"4999680") * b) * b) * b) * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[33] = 8 * a * b * b * ((str_to_INTERVAL((char*)"-4428000") + (str_to_INTERVAL((char*)"-26840568") + (str_to_INTERVAL((char*)"2445486") + str_to_INTERVAL((char*)"38499") * b) * b) * b) *  pow( b,  16) + ((str_to_INTERVAL((char*)"190972080") + (str_to_INTERVAL((char*)"-417756456") + (str_to_INTERVAL((char*)"-58401720") + (str_to_INTERVAL((char*)"-16403868") - str_to_INTERVAL((char*)"20709") * b) * b) * b) * b) *  pow( b,  15) + ((str_to_INTERVAL((char*)"8483428080") + (str_to_INTERVAL((char*)"-6184506636") + (str_to_INTERVAL((char*)"-23807085750") + (str_to_INTERVAL((char*)"738102381") + (str_to_INTERVAL((char*)"90202818") + str_to_INTERVAL((char*)"223110") * b) * b) * b) * b) * b) *  pow( b,  14) + ((str_to_INTERVAL((char*)"19163088000") + (str_to_INTERVAL((char*)"347382437640") + (str_to_INTERVAL((char*)"-366621054240") + (str_to_INTERVAL((char*)"-13559055090") + (str_to_INTERVAL((char*)"-11715967410") - str_to_INTERVAL((char*)"28948500") * b) * b) * b) * b) * b) *  pow( b,  13) + ((str_to_INTERVAL((char*)"-799348746240") + (str_to_INTERVAL((char*)"4985644030752") + (str_to_INTERVAL((char*)"-1583970716328") + (str_to_INTERVAL((char*)"-3100191028242") + (str_to_INTERVAL((char*)"-133639538010") + (str_to_INTERVAL((char*)"17924818791") + str_to_INTERVAL((char*)"188736174") * b) * b) * b) * b) * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"-7419795549120") + (str_to_INTERVAL((char*)"1947936091968") + (str_to_INTERVAL((char*)"43435160588864") + (str_to_INTERVAL((char*)"-39960195478544") + (str_to_INTERVAL((char*)"973890220608") + (str_to_INTERVAL((char*)"-1380934856607") - str_to_INTERVAL((char*)"1577079864") * b) * b) * b) * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"1143311293440") + (str_to_INTERVAL((char*)"-227303976797760") + (str_to_INTERVAL((char*)"153070544851392") + (str_to_INTERVAL((char*)"-101714886330688") + (str_to_INTERVAL((char*)"-52480079076104") + (str_to_INTERVAL((char*)"-17373281492698") + (str_to_INTERVAL((char*)"179009949462") + str_to_INTERVAL((char*)"26389381140") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"182806356003840") + (str_to_INTERVAL((char*)"-1004314366862208") + (str_to_INTERVAL((char*)"-2667870227824704") + (str_to_INTERVAL((char*)"654234752057760") + (str_to_INTERVAL((char*)"-437363012429280") + (str_to_INTERVAL((char*)"68731059496464") + (str_to_INTERVAL((char*)"-59143883336190") + str_to_INTERVAL((char*)"408017702871") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"362225104588800") + (str_to_INTERVAL((char*)"697806422340480") + (str_to_INTERVAL((char*)"-19789105047724480") + (str_to_INTERVAL((char*)"-7634325011248000") + (str_to_INTERVAL((char*)"1274315071764000") + (str_to_INTERVAL((char*)"1339939776543720") + (str_to_INTERVAL((char*)"-640505936398520") + (str_to_INTERVAL((char*)"-12873439951680") + str_to_INTERVAL((char*)"835453891800") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-914904965928960") + (str_to_INTERVAL((char*)"10523800839649536") + (str_to_INTERVAL((char*)"-41486515853197824") + (str_to_INTERVAL((char*)"-92775962797909248") + (str_to_INTERVAL((char*)"3461199119701056") + (str_to_INTERVAL((char*)"11837848824079584") + (str_to_INTERVAL((char*)"-1047336188352176") + (str_to_INTERVAL((char*)"-833123627647396") + str_to_INTERVAL((char*)"13826882619144") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-2242308543482880") + (str_to_INTERVAL((char*)"12157597627901952") + (str_to_INTERVAL((char*)"17444100747616512") + (str_to_INTERVAL((char*)"-309729544282538496") + (str_to_INTERVAL((char*)"-114974796871184896") + (str_to_INTERVAL((char*)"60760953477082272") + (str_to_INTERVAL((char*)"10888393107127664") + (str_to_INTERVAL((char*)"-6369820454567408") + (str_to_INTERVAL((char*)"-67972370779008") + str_to_INTERVAL((char*)"6124678639056") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"442951042129920") + (str_to_INTERVAL((char*)"-13305142708678656") + (str_to_INTERVAL((char*)"164864214742743552") + (str_to_INTERVAL((char*)"-412313555913275136") + (str_to_INTERVAL((char*)"-556436615498276608") + (str_to_INTERVAL((char*)"92451926745229568") + (str_to_INTERVAL((char*)"74802622562703584") + (str_to_INTERVAL((char*)"-15602110289745168") + (str_to_INTERVAL((char*)"-2672764170652448") + str_to_INTERVAL((char*)"70011815262408") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"2290572590284800") + (str_to_INTERVAL((char*)"-26187693034647552") + (str_to_INTERVAL((char*)"197172100232171520") + (str_to_INTERVAL((char*)"-156577841130535424") + (str_to_INTERVAL((char*)"-962380800720721408") + (str_to_INTERVAL((char*)"-138674932683476864") + (str_to_INTERVAL((char*)"187950388372546496") + (str_to_INTERVAL((char*)"-2159373884543616") + (str_to_INTERVAL((char*)"-12265592951217152") + (str_to_INTERVAL((char*)"38957569301472") + str_to_INTERVAL((char*)"8948950005960") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"938886363095040") + (str_to_INTERVAL((char*)"-12375330159599616") + (str_to_INTERVAL((char*)"92942445560758272") + (str_to_INTERVAL((char*)"81144142347472896") + (str_to_INTERVAL((char*)"-732457580603832320") + (str_to_INTERVAL((char*)"-447778485831587840") + (str_to_INTERVAL((char*)"172560172228062720") + (str_to_INTERVAL((char*)"45890470884912768") + (str_to_INTERVAL((char*)"-18682062990220224") + (str_to_INTERVAL((char*)"-1414049055505584") + str_to_INTERVAL((char*)"53750387969856") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"101228843335680") + (str_to_INTERVAL((char*)"-2136027063803904") + (str_to_INTERVAL((char*)"19717551857590272") + (str_to_INTERVAL((char*)"61422119729111040") + (str_to_INTERVAL((char*)"-249347493623134208") + (str_to_INTERVAL((char*)"-350886755455067136") + (str_to_INTERVAL((char*)"24688548369735424") + (str_to_INTERVAL((char*)"56917644477815296") + (str_to_INTERVAL((char*)"-7733126592027520") + (str_to_INTERVAL((char*)"-3255072952460992") + (str_to_INTERVAL((char*)"49692316995744") + str_to_INTERVAL((char*)"1744920815328") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"2420142243840") + (str_to_INTERVAL((char*)"-122024418607104") + (str_to_INTERVAL((char*)"1852964859371520") + (str_to_INTERVAL((char*)"9846330983514112") + (str_to_INTERVAL((char*)"-38038559886723072") + (str_to_INTERVAL((char*)"-101260416892682240") + (str_to_INTERVAL((char*)"-23186544172417536") + (str_to_INTERVAL((char*)"18509727328543488") + (str_to_INTERVAL((char*)"1984408986487168") + (str_to_INTERVAL((char*)"-1953764328083776") + (str_to_INTERVAL((char*)"-71967099087360") + str_to_INTERVAL((char*)"3670504215840") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"5308416000") + (str_to_INTERVAL((char*)"-1402910539776") + (str_to_INTERVAL((char*)"58958911930368") + (str_to_INTERVAL((char*)"338064169181184") + (str_to_INTERVAL((char*)"-2739393903239168") + (str_to_INTERVAL((char*)"-10238949625831424") + (str_to_INTERVAL((char*)"-5514495923502080") + (str_to_INTERVAL((char*)"743572199707648") + (str_to_INTERVAL((char*)"985583944766976") + (str_to_INTERVAL((char*)"-221330497530368") + (str_to_INTERVAL((char*)"-69869772812480") + (str_to_INTERVAL((char*)"1134774772320") + str_to_INTERVAL((char*)"35235868032") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"237115146240") + (str_to_INTERVAL((char*)"-1597020438528") + (str_to_INTERVAL((char*)"-67778714304512") + (str_to_INTERVAL((char*)"-247779956211712") + (str_to_INTERVAL((char*)"-183616811968512") + (str_to_INTERVAL((char*)"-129896287144960") + (str_to_INTERVAL((char*)"32492287691776") + (str_to_INTERVAL((char*)"26976311645184") + (str_to_INTERVAL((char*)"-14548952469632") + (str_to_INTERVAL((char*)"-200066191872") - str_to_INTERVAL((char*)"21786055488") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"315280982016") + (str_to_INTERVAL((char*)"84643012608") + (str_to_INTERVAL((char*)"-5984349241344") + (str_to_INTERVAL((char*)"-1085626398720") + (str_to_INTERVAL((char*)"2363734812672") + (str_to_INTERVAL((char*)"-868748000768") + (str_to_INTERVAL((char*)"32009464320") + (str_to_INTERVAL((char*)"-43361900544") + str_to_INTERVAL((char*)"1636789248") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + (str_to_INTERVAL((char*)"18831605760") + (str_to_INTERVAL((char*)"-25612139520") + (str_to_INTERVAL((char*)"-5668331520") + (str_to_INTERVAL((char*)"16202267136") + (str_to_INTERVAL((char*)"-6005345280") + str_to_INTERVAL((char*)"98864640") * b) * b) * b) * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[34] = -4 *  pow( b,  3) * ((str_to_INTERVAL((char*)"-86400") + str_to_INTERVAL((char*)"10908") * b) *  pow( b,  17) + ((str_to_INTERVAL((char*)"-1248480") + (str_to_INTERVAL((char*)"-649440") - str_to_INTERVAL((char*)"56340") * b) * b) *  pow( b,  16) + ((str_to_INTERVAL((char*)"-79172640") + (str_to_INTERVAL((char*)"-553155696") + (str_to_INTERVAL((char*)"49490322") + str_to_INTERVAL((char*)"818853") * b) * b) * b) *  pow( b,  15) + ((str_to_INTERVAL((char*)"3702324240") + (str_to_INTERVAL((char*)"-7871192760") + (str_to_INTERVAL((char*)"-1152035576") + (str_to_INTERVAL((char*)"-309957828") - str_to_INTERVAL((char*)"468747") * b) * b) * b) * b) *  pow( b,  14) + ((str_to_INTERVAL((char*)"85631288400") + (str_to_INTERVAL((char*)"-32368167000") + (str_to_INTERVAL((char*)"-238582439332") + (str_to_INTERVAL((char*)"7194998766") + (str_to_INTERVAL((char*)"880382604") + str_to_INTERVAL((char*)"2595492") * b) * b) * b) * b) * b) *  pow( b,  13) + ((str_to_INTERVAL((char*)"72359066880") + (str_to_INTERVAL((char*)"3157289897424") + (str_to_INTERVAL((char*)"-3150082132032") + (str_to_INTERVAL((char*)"-127935569692") + (str_to_INTERVAL((char*)"-99030079032") - str_to_INTERVAL((char*)"374297706") * b) * b) * b) * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"-6283560199680") + (str_to_INTERVAL((char*)"30364478900736") + (str_to_INTERVAL((char*)"-6674883662432") + (str_to_INTERVAL((char*)"-19320375134272") + (str_to_INTERVAL((char*)"-600853604732") + (str_to_INTERVAL((char*)"85843473210") + str_to_INTERVAL((char*)"1411032312") * b) * b) * b) * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"-36886277612160") + (str_to_INTERVAL((char*)"-21366207055872") + (str_to_INTERVAL((char*)"217108112852992") + (str_to_INTERVAL((char*)"-204139391033184") + (str_to_INTERVAL((char*)"8402316895600") + (str_to_INTERVAL((char*)"-7477816419234") - str_to_INTERVAL((char*)"20220155568") * b) * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"52745090008320") + (str_to_INTERVAL((char*)"-1107713724958080") + (str_to_INTERVAL((char*)"210223440969024") + (str_to_INTERVAL((char*)"-356285425943584") + (str_to_INTERVAL((char*)"-161944718584936") + (str_to_INTERVAL((char*)"-65453569257108") + (str_to_INTERVAL((char*)"-1308206532312") + str_to_INTERVAL((char*)"134396284176") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"730268300021760") + (str_to_INTERVAL((char*)"-3185439557548800") + (str_to_INTERVAL((char*)"-12290372355505024") + (str_to_INTERVAL((char*)"1126267717489472") + (str_to_INTERVAL((char*)"-891223261222976") + (str_to_INTERVAL((char*)"365414691189120") + (str_to_INTERVAL((char*)"-237229489115204") + str_to_INTERVAL((char*)"1536703488270") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"626956167997440") + (str_to_INTERVAL((char*)"5007260079899136") + (str_to_INTERVAL((char*)"-64030786464281856") + (str_to_INTERVAL((char*)"-34839238112995712") + (str_to_INTERVAL((char*)"6062802132554944") + (str_to_INTERVAL((char*)"5447783166961120") + (str_to_INTERVAL((char*)"-2013471955755136") + (str_to_INTERVAL((char*)"-65084189004336") + str_to_INTERVAL((char*)"2903701133520") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-3166211670589440") + (str_to_INTERVAL((char*)"28102643678461440") + (str_to_INTERVAL((char*)"-93185545524480000") + (str_to_INTERVAL((char*)"-269563282151048704") + (str_to_INTERVAL((char*)"-602442261192064") + (str_to_INTERVAL((char*)"38567702272313344") + (str_to_INTERVAL((char*)"-2616703942685984") + (str_to_INTERVAL((char*)"-2339588563269832") + str_to_INTERVAL((char*)"38631445951680") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-3721945253253120") + (str_to_INTERVAL((char*)"15985421307546624") + (str_to_INTERVAL((char*)"78661856611334656") + (str_to_INTERVAL((char*)"-684189869308230912") + (str_to_INTERVAL((char*)"-294614965122569216") + (str_to_INTERVAL((char*)"145478779721451584") + (str_to_INTERVAL((char*)"26354852952518080") + (str_to_INTERVAL((char*)"-14213868461854656") + (str_to_INTERVAL((char*)"-195286483293696") + str_to_INTERVAL((char*)"14062023192672") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"2322816404520960") + (str_to_INTERVAL((char*)"-33552857334583296") + (str_to_INTERVAL((char*)"317070316215446528") + (str_to_INTERVAL((char*)"-693391364489841152") + (str_to_INTERVAL((char*)"-1039267800022757376") + (str_to_INTERVAL((char*)"158361956220666368") + (str_to_INTERVAL((char*)"142370336152807360") + (str_to_INTERVAL((char*)"-28253008098280224") + (str_to_INTERVAL((char*)"-4875794600732480") + str_to_INTERVAL((char*)"127757666045808") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"3224869054709760") + (str_to_INTERVAL((char*)"-37384650010263552") + (str_to_INTERVAL((char*)"274140800474431488") + (str_to_INTERVAL((char*)"-186705179538964480") + (str_to_INTERVAL((char*)"-1370153877947608064") + (str_to_INTERVAL((char*)"-214684030457372160") + (str_to_INTERVAL((char*)"271516448274192896") + (str_to_INTERVAL((char*)"-2790904385342720") + (str_to_INTERVAL((char*)"-17299635471418240") + (str_to_INTERVAL((char*)"43584309727680") + str_to_INTERVAL((char*)"12568340304000") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"792333696614400") + (str_to_INTERVAL((char*)"-12090219517231104") + (str_to_INTERVAL((char*)"96378481448583168") + (str_to_INTERVAL((char*)"83510323975987200") + (str_to_INTERVAL((char*)"-788618928419692544") + (str_to_INTERVAL((char*)"-485997815556696064") + (str_to_INTERVAL((char*)"183336076796716032") + (str_to_INTERVAL((char*)"49534039653883648") + (str_to_INTERVAL((char*)"-19884324253384320") + (str_to_INTERVAL((char*)"-1522063438591904") + str_to_INTERVAL((char*)"57639028793472") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"48622883143680") + (str_to_INTERVAL((char*)"-1371271043579904") + (str_to_INTERVAL((char*)"14947437249904640") + (str_to_INTERVAL((char*)"40320795086168064") + (str_to_INTERVAL((char*)"-200132901116635136") + (str_to_INTERVAL((char*)"-270790463361751040") + (str_to_INTERVAL((char*)"18569434775140864") + (str_to_INTERVAL((char*)"42509334259955456") + (str_to_INTERVAL((char*)"-5622882594522624") + (str_to_INTERVAL((char*)"-2525318072710464") + (str_to_INTERVAL((char*)"39675812446464") + str_to_INTERVAL((char*)"1213824298176") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"507555348480") + (str_to_INTERVAL((char*)"-43123506020352") + (str_to_INTERVAL((char*)"920809322692608") + (str_to_INTERVAL((char*)"3659170438119424") + (str_to_INTERVAL((char*)"-22422887932637184") + (str_to_INTERVAL((char*)"-52010629177241600") + (str_to_INTERVAL((char*)"-11088283098874880") + (str_to_INTERVAL((char*)"8303155838621184") + (str_to_INTERVAL((char*)"1223974798930688") + (str_to_INTERVAL((char*)"-1008713544840320") + (str_to_INTERVAL((char*)"-37881205259904") + str_to_INTERVAL((char*)"1738653693696") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-148635648000") + (str_to_INTERVAL((char*)"13977419710464") + (str_to_INTERVAL((char*)"25433700335616") + (str_to_INTERVAL((char*)"-1038793813811200") + (str_to_INTERVAL((char*)"-2983808886161408") + (str_to_INTERVAL((char*)"-1408370250426368") + (str_to_INTERVAL((char*)"-54259644380160") + (str_to_INTERVAL((char*)"334674395180544") + (str_to_INTERVAL((char*)"-57741596538112") + (str_to_INTERVAL((char*)"-26205742155520") + (str_to_INTERVAL((char*)"301343414016") + str_to_INTERVAL((char*)"14532038784") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-576995328000") + (str_to_INTERVAL((char*)"-8344801640448") + (str_to_INTERVAL((char*)"-19808517636096") + (str_to_INTERVAL((char*)"-12423231246336") + (str_to_INTERVAL((char*)"-43644717197312") + (str_to_INTERVAL((char*)"9159492562944") + (str_to_INTERVAL((char*)"7183445191680") + (str_to_INTERVAL((char*)"-3906677555968") + (str_to_INTERVAL((char*)"-68169561088") - str_to_INTERVAL((char*)"13721151744") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-188637216768") + (str_to_INTERVAL((char*)"-743178534912") + (str_to_INTERVAL((char*)"147842224128") + (str_to_INTERVAL((char*)"84911765504") + (str_to_INTERVAL((char*)"-66661517312") + (str_to_INTERVAL((char*)"19907394560") + (str_to_INTERVAL((char*)"-13047690240") + str_to_INTERVAL((char*)"461721600") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + (str_to_INTERVAL((char*)"-2928752640") + (str_to_INTERVAL((char*)"1202995200") + (str_to_INTERVAL((char*)"940987392") + (str_to_INTERVAL((char*)"-632924160") + str_to_INTERVAL((char*)"13224960") * b) * b) * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[35] = 2 * a *  pow( b,  3) * ((str_to_INTERVAL((char*)"-3715200") + str_to_INTERVAL((char*)"469044") * b) *  pow( b,  17) + ((str_to_INTERVAL((char*)"-51187680") + (str_to_INTERVAL((char*)"-26627040") - str_to_INTERVAL((char*)"2309940") * b) * b) *  pow( b,  16) + ((str_to_INTERVAL((char*)"-785172960") + (str_to_INTERVAL((char*)"-7615976784") + (str_to_INTERVAL((char*)"658469838") + str_to_INTERVAL((char*)"11915787") * b) * b) * b) *  pow( b,  15) + ((str_to_INTERVAL((char*)"47483748720") + (str_to_INTERVAL((char*)"-95445606408") + (str_to_INTERVAL((char*)"-15254376872") + (str_to_INTERVAL((char*)"-3780493500") - str_to_INTERVAL((char*)"7638021") * b) * b) * b) * b) *  pow( b,  14) + ((str_to_INTERVAL((char*)"674374060080") + (str_to_INTERVAL((char*)"9111670344") + (str_to_INTERVAL((char*)"-1902079750340") + (str_to_INTERVAL((char*)"56738447934") + (str_to_INTERVAL((char*)"6718739244") + str_to_INTERVAL((char*)"24772356") * b) * b) * b) * b) * b) *  pow( b,  13) + ((str_to_INTERVAL((char*)"-340110144000") + (str_to_INTERVAL((char*)"22549709836896") + (str_to_INTERVAL((char*)"-21100019140992") + (str_to_INTERVAL((char*)"-948865808352") + (str_to_INTERVAL((char*)"-646763446428") - str_to_INTERVAL((char*)"3763852290") * b) * b) * b) * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"-38953220405760") + (str_to_INTERVAL((char*)"149639786488320") + (str_to_INTERVAL((char*)"-20453893790624") + (str_to_INTERVAL((char*)"-100527207069184") + (str_to_INTERVAL((char*)"-1635900829252") + (str_to_INTERVAL((char*)"276381956070") + str_to_INTERVAL((char*)"9062007048") * b) * b) * b) * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"-142369942913280") + (str_to_INTERVAL((char*)"-256765705746048") + (str_to_INTERVAL((char*)"860149817092992") + (str_to_INTERVAL((char*)"-865061074716864") + (str_to_INTERVAL((char*)"56668777239984") + (str_to_INTERVAL((char*)"-34208746419036") - str_to_INTERVAL((char*)"158571138816") * b) * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"388416798927360") + (str_to_INTERVAL((char*)"-4436971776975744") + (str_to_INTERVAL((char*)"-1223962997319360") + (str_to_INTERVAL((char*)"-1141039878301728") + (str_to_INTERVAL((char*)"-303681260706480") + (str_to_INTERVAL((char*)"-209132041700232") + (str_to_INTERVAL((char*)"-13554793309608") + str_to_INTERVAL((char*)"596618515728") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"2306307101644800") + (str_to_INTERVAL((char*)"-7799010502448640") + (str_to_INTERVAL((char*)"-47044505577840640") + (str_to_INTERVAL((char*)"-1442961880441600") + (str_to_INTERVAL((char*)"-322508769168320") + (str_to_INTERVAL((char*)"1559604078527520") + (str_to_INTERVAL((char*)"-826924747607360") + str_to_INTERVAL((char*)"5057441426880") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"90383089489920") + (str_to_INTERVAL((char*)"20704669841253888") + (str_to_INTERVAL((char*)"-175573455200453888") + (str_to_INTERVAL((char*)"-125368709001530496") + (str_to_INTERVAL((char*)"21212625724434752") + (str_to_INTERVAL((char*)"18478729883317760") + (str_to_INTERVAL((char*)"-5583305223316848") + (str_to_INTERVAL((char*)"-242174844083208") + str_to_INTERVAL((char*)"8800323198480") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-8054106993561600") + (str_to_INTERVAL((char*)"60405245281637376") + (str_to_INTERVAL((char*)"-169235334481265664") + (str_to_INTERVAL((char*)"-669345879430070272") + (str_to_INTERVAL((char*)"-24807739570985728") + (str_to_INTERVAL((char*)"104726760904930368") + (str_to_INTERVAL((char*)"-5766635900331520") + (str_to_INTERVAL((char*)"-5683563477613888") + str_to_INTERVAL((char*)"93730307184144") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-4090900725411840") + (str_to_INTERVAL((char*)"8303602494068736") + (str_to_INTERVAL((char*)"221306562879179776") + (str_to_INTERVAL((char*)"-1295778659439843840") + (str_to_INTERVAL((char*)"-634074614005680128") + (str_to_INTERVAL((char*)"296115564402019968") + (str_to_INTERVAL((char*)"54420859335082304") + (str_to_INTERVAL((char*)"-27483603467699296") + (str_to_INTERVAL((char*)"-443723749792608") + str_to_INTERVAL((char*)"27897932475792") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"5317219415162880") + (str_to_INTERVAL((char*)"-63110943457099776") + (str_to_INTERVAL((char*)"509969152371896320") + (str_to_INTERVAL((char*)"-996865976263585792") + (str_to_INTERVAL((char*)"-1652387900955761664") + (str_to_INTERVAL((char*)"233095467563899904") + (str_to_INTERVAL((char*)"229194566535178240") + (str_to_INTERVAL((char*)"-43779041037597184") + (str_to_INTERVAL((char*)"-7596114548157696") + str_to_INTERVAL((char*)"199530453268800") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"3529371474984960") + (str_to_INTERVAL((char*)"-43652699259617280") + (str_to_INTERVAL((char*)"320598973574160384") + (str_to_INTERVAL((char*)"-194772100790259712") + (str_to_INTERVAL((char*)"-1648341583500827648") + (str_to_INTERVAL((char*)"-273388178268278784") + (str_to_INTERVAL((char*)"328605779364077824") + (str_to_INTERVAL((char*)"-3054570038241920") + (str_to_INTERVAL((char*)"-20599678146924608") + (str_to_INTERVAL((char*)"42418891963488") + str_to_INTERVAL((char*)"14852273621760") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"510024164474880") + (str_to_INTERVAL((char*)"-9477584310140928") + (str_to_INTERVAL((char*)"82886516810416128") + (str_to_INTERVAL((char*)"64012800635092992") + (str_to_INTERVAL((char*)"-707378898993033216") + (str_to_INTERVAL((char*)"-433089533125330944") + (str_to_INTERVAL((char*)"160414556482862080") + (str_to_INTERVAL((char*)"43605365224407552") + (str_to_INTERVAL((char*)"-17385637366799872") + (str_to_INTERVAL((char*)"-1358207455962944") + str_to_INTERVAL((char*)"51180565768128") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"16207735357440") + (str_to_INTERVAL((char*)"-659153895063552") + (str_to_INTERVAL((char*)"8967121267900416") + (str_to_INTERVAL((char*)"19053734053453824") + (str_to_INTERVAL((char*)"-130713074078674944") + (str_to_INTERVAL((char*)"-166057684497989632") + (str_to_INTERVAL((char*)"11316369815675904") + (str_to_INTERVAL((char*)"24691972678287872") + (str_to_INTERVAL((char*)"-3106648468862464") + (str_to_INTERVAL((char*)"-1580115765889664") + (str_to_INTERVAL((char*)"26146552059264") + str_to_INTERVAL((char*)"626602306176") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"49545216000") + (str_to_INTERVAL((char*)"-9456018259968") + (str_to_INTERVAL((char*)"322873008783360") + (str_to_INTERVAL((char*)"725697194655744") + (str_to_INTERVAL((char*)"-10091554511118336") + (str_to_INTERVAL((char*)"-19825617252327424") + (str_to_INTERVAL((char*)"-3838030556962816") + (str_to_INTERVAL((char*)"2451187992075264") + (str_to_INTERVAL((char*)"643634483932672") + (str_to_INTERVAL((char*)"-406059316034304") + (str_to_INTERVAL((char*)"-15508554924544") + str_to_INTERVAL((char*)"576880999488") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"1535364956160") + (str_to_INTERVAL((char*)"-11885194838016") + (str_to_INTERVAL((char*)"-243441919508480") + (str_to_INTERVAL((char*)"-525045695930368") + (str_to_INTERVAL((char*)"-216995416096768") + (str_to_INTERVAL((char*)"-129077227892736") + (str_to_INTERVAL((char*)"103178169834496") + (str_to_INTERVAL((char*)"-12797578594816") + (str_to_INTERVAL((char*)"-7928613157376") + (str_to_INTERVAL((char*)"10593565440") + str_to_INTERVAL((char*)"6669093888") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"945842946048") + (str_to_INTERVAL((char*)"-1086796136448") + (str_to_INTERVAL((char*)"-9368858972160") + (str_to_INTERVAL((char*)"3280202569728") + (str_to_INTERVAL((char*)"692257517568") + (str_to_INTERVAL((char*)"-637212328960") + (str_to_INTERVAL((char*)"-30451360768") - str_to_INTERVAL((char*)"5561008128") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + (str_to_INTERVAL((char*)"23539507200") + (str_to_INTERVAL((char*)"-38740500480") + (str_to_INTERVAL((char*)"15208550400") + (str_to_INTERVAL((char*)"2506272768") + (str_to_INTERVAL((char*)"-2455019520") + str_to_INTERVAL((char*)"85155840") * b) * b) * b) * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[36] = -2 * a *  pow( b,  4) * (str_to_INTERVAL((char*)"-61560") *  pow( b,  17) + ((str_to_INTERVAL((char*)"-39933000") + str_to_INTERVAL((char*)"5004882") * b) *  pow( b,  16) + ((str_to_INTERVAL((char*)"-505355760") + (str_to_INTERVAL((char*)"-269884944") - str_to_INTERVAL((char*)"23106282") * b) * b) *  pow( b,  15) + ((str_to_INTERVAL((char*)"-1939861440") + (str_to_INTERVAL((char*)"-39176444556") + (str_to_INTERVAL((char*)"3229291116") + str_to_INTERVAL((char*)"65909778") * b) * b) * b) *  pow( b,  14) + ((str_to_INTERVAL((char*)"225491409360") + (str_to_INTERVAL((char*)"-418036028280") + (str_to_INTERVAL((char*)"-75837795312") + (str_to_INTERVAL((char*)"-16697100540") - str_to_INTERVAL((char*)"47971017") * b) * b) * b) * b) *  pow( b,  13) + ((str_to_INTERVAL((char*)"2113567527600") + (str_to_INTERVAL((char*)"955106913240") + (str_to_INTERVAL((char*)"-6238270345068") + (str_to_INTERVAL((char*)"190125630446") + (str_to_INTERVAL((char*)"20544971532") + str_to_INTERVAL((char*)"100019988") * b) * b) * b) * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"-3722843773440") + (str_to_INTERVAL((char*)"65384536089984") + (str_to_INTERVAL((char*)"-57372592709808") + (str_to_INTERVAL((char*)"-2839997628112") + (str_to_INTERVAL((char*)"-1692035391748") - str_to_INTERVAL((char*)"15201745578") * b) * b) * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"-97345379888640") + (str_to_INTERVAL((char*)"299162227344960") + (str_to_INTERVAL((char*)"-21528377253888") + (str_to_INTERVAL((char*)"-221326746890496") + (str_to_INTERVAL((char*)"480066571196") + (str_to_INTERVAL((char*)"131995010358") + str_to_INTERVAL((char*)"25291189044") * b) * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"-205270534782720") + (str_to_INTERVAL((char*)"-819920137200768") + (str_to_INTERVAL((char*)"1316972624259584") + (str_to_INTERVAL((char*)"-1548276951994208") + (str_to_INTERVAL((char*)"155731908362000") + (str_to_INTERVAL((char*)"-67512752639756") - str_to_INTERVAL((char*)"467398428660") * b) * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"924183713733120") + (str_to_INTERVAL((char*)"-7352388025207680") + (str_to_INTERVAL((char*)"-5731840567160256") + (str_to_INTERVAL((char*)"-1781281212373216") + (str_to_INTERVAL((char*)"95473467176896") + (str_to_INTERVAL((char*)"-285433840343096") + (str_to_INTERVAL((char*)"-37545848812848") + str_to_INTERVAL((char*)"1158704323920") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"2846313695139840") + (str_to_INTERVAL((char*)"-6710877752326656") + (str_to_INTERVAL((char*)"-75810873633063168") + (str_to_INTERVAL((char*)"-10303191226205952") + (str_to_INTERVAL((char*)"3294996864724544") + (str_to_INTERVAL((char*)"2762384497565952") + (str_to_INTERVAL((char*)"-1260376171506640") + str_to_INTERVAL((char*)"7319673876816") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-1816960793978880") + (str_to_INTERVAL((char*)"31345722570097152") + (str_to_INTERVAL((char*)"-204557222822046464") + (str_to_INTERVAL((char*)"-185588094793835136") + (str_to_INTERVAL((char*)"29744824488920256") + (str_to_INTERVAL((char*)"26508712571809824") + (str_to_INTERVAL((char*)"-6838880451180688") + (str_to_INTERVAL((char*)"-361756516766184") + str_to_INTERVAL((char*)"11631551014512") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-7773062097530880") + (str_to_INTERVAL((char*)"51672768809548800") + (str_to_INTERVAL((char*)"-119570312992014336") + (str_to_INTERVAL((char*)"-712872807276838912") + (str_to_INTERVAL((char*)"-48790227028903936") + (str_to_INTERVAL((char*)"120041287113750336") + (str_to_INTERVAL((char*)"-5616612426734912") + (str_to_INTERVAL((char*)"-5980660506145024") + str_to_INTERVAL((char*)"98727003411264") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-711633362780160") + (str_to_INTERVAL((char*)"-11190025271549952") + (str_to_INTERVAL((char*)"234569718371036160") + (str_to_INTERVAL((char*)"-1052917030392345600") + (str_to_INTERVAL((char*)"-576855894160788736") + (str_to_INTERVAL((char*)"257067249604361088") + (str_to_INTERVAL((char*)"47917506996140480") + (str_to_INTERVAL((char*)"-22967511551899872") + (str_to_INTERVAL((char*)"-413788542797088") + str_to_INTERVAL((char*)"23847553015344") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"4146306947481600") + (str_to_INTERVAL((char*)"-46611465481617408") + (str_to_INTERVAL((char*)"343790543616155648") + (str_to_INTERVAL((char*)"-613572992163631104") + (str_to_INTERVAL((char*)"-1115639440662354944") + (str_to_INTERVAL((char*)"147286390102979072") + (str_to_INTERVAL((char*)"155668226686308096") + (str_to_INTERVAL((char*)"-28891955339907584") + (str_to_INTERVAL((char*)"-5036358366052352") + str_to_INTERVAL((char*)"132810167248704") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"1492828748513280") + (str_to_INTERVAL((char*)"-20699524682956800") + (str_to_INTERVAL((char*)"156832057992093696") + (str_to_INTERVAL((char*)"-91295692059602944") + (str_to_INTERVAL((char*)"-832680627090701312") + (str_to_INTERVAL((char*)"-142554126229007360") + (str_to_INTERVAL((char*)"165330360892430592") + (str_to_INTERVAL((char*)"-1417966561142912") + (str_to_INTERVAL((char*)"-10267148042026176") + (str_to_INTERVAL((char*)"17912999719968") + str_to_INTERVAL((char*)"7304471718336") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"120932869570560") + (str_to_INTERVAL((char*)"-2904445447471104") + (str_to_INTERVAL((char*)"29050148035952640") + (str_to_INTERVAL((char*)"16680524000722944") + (str_to_INTERVAL((char*)"-261277184934215680") + (str_to_INTERVAL((char*)"-155986448946196480") + (str_to_INTERVAL((char*)"56839575895624704") + (str_to_INTERVAL((char*)"15391472895817216") + (str_to_INTERVAL((char*)"-6137938699088896") + (str_to_INTERVAL((char*)"-497062616250432") + str_to_INTERVAL((char*)"18587900270592") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1673141944320") + (str_to_INTERVAL((char*)"-111261089857536") + (str_to_INTERVAL((char*)"2044310554263552") + (str_to_INTERVAL((char*)"2731280123355136") + (str_to_INTERVAL((char*)"-33885411942014976") + (str_to_INTERVAL((char*)"-39248918896066560") + (str_to_INTERVAL((char*)"2668797640029184") + (str_to_INTERVAL((char*)"5343481813414912") + (str_to_INTERVAL((char*)"-595653209771520") + (str_to_INTERVAL((char*)"-393879792714240") + (str_to_INTERVAL((char*)"7107282371328") + str_to_INTERVAL((char*)"103916737152") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-483065856000") + (str_to_INTERVAL((char*)"35484143321088") + (str_to_INTERVAL((char*)"-20473771229184") + (str_to_INTERVAL((char*)"-1623677309526016") + (str_to_INTERVAL((char*)"-2602980503633920") + (str_to_INTERVAL((char*)"-459999152046080") + (str_to_INTERVAL((char*)"158858612261888") + (str_to_INTERVAL((char*)"146282400336384") + (str_to_INTERVAL((char*)"-65303314142464") + (str_to_INTERVAL((char*)"-2401994380800") + str_to_INTERVAL((char*)"51663086784") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-1384788787200") + (str_to_INTERVAL((char*)"-13279015206912") + (str_to_INTERVAL((char*)"-17618037669888") + (str_to_INTERVAL((char*)"-10718345662464") + (str_to_INTERVAL((char*)"-25485467707392") + (str_to_INTERVAL((char*)"15122745915392") + (str_to_INTERVAL((char*)"-2050470335488") + (str_to_INTERVAL((char*)"-893246813184") + (str_to_INTERVAL((char*)"-22332446208") + str_to_INTERVAL((char*)"1593296640") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-215585390592") + (str_to_INTERVAL((char*)"-455314083840") + (str_to_INTERVAL((char*)"289585289216") + (str_to_INTERVAL((char*)"-68753858560") + (str_to_INTERVAL((char*)"2449564672") + (str_to_INTERVAL((char*)"-6444180480") - str_to_INTERVAL((char*)"755896320") * b) * b) * b) * b) * b) * b) *  pow( b,  3) + (str_to_INTERVAL((char*)"-1301667840") + (str_to_INTERVAL((char*)"1048903680") + (str_to_INTERVAL((char*)"-116576256") + (str_to_INTERVAL((char*)"-106782720") + str_to_INTERVAL((char*)"3870720") * b) * b) * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[37] = 0.-a * a *  pow( b,  4) * (str_to_INTERVAL((char*)"2647080") *  pow( b,  17) + ((str_to_INTERVAL((char*)"570990600") - str_to_INTERVAL((char*)"70584534") * b) *  pow( b,  16) + ((str_to_INTERVAL((char*)"6400077840") + (str_to_INTERVAL((char*)"3602482416") + str_to_INTERVAL((char*)"300560598") * b) * b) *  pow( b,  15) + ((str_to_INTERVAL((char*)"-4304011680") + (str_to_INTERVAL((char*)"318973758060") + (str_to_INTERVAL((char*)"-24780463398") - str_to_INTERVAL((char*)"584641995") * b) * b) * b) *  pow( b,  14) + ((str_to_INTERVAL((char*)"-1679706519120") + (str_to_INTERVAL((char*)"2810801724024") + (str_to_INTERVAL((char*)"600483149280") + (str_to_INTERVAL((char*)"113388621612") + str_to_INTERVAL((char*)"482637141") * b) * b) * b) * b) *  pow( b,  13) + ((str_to_INTERVAL((char*)"-10629008010720") + (str_to_INTERVAL((char*)"-10433448636336") + (str_to_INTERVAL((char*)"34428464551624") + (str_to_INTERVAL((char*)"-1118344286400") + (str_to_INTERVAL((char*)"-101677281024") - str_to_INTERVAL((char*)"696270960") * b) * b) * b) * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"31879566005760") + (str_to_INTERVAL((char*)"-314093873217408") + (str_to_INTERVAL((char*)"260683707402576") + (str_to_INTERVAL((char*)"13906901124080") + (str_to_INTERVAL((char*)"7247447205244") + str_to_INTERVAL((char*)"101212948854") * b) * b) * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"396915732126720") + (str_to_INTERVAL((char*)"-961811769980736") + (str_to_INTERVAL((char*)"35048716538208") + (str_to_INTERVAL((char*)"832492106135616") + (str_to_INTERVAL((char*)"-21157933043856") + (str_to_INTERVAL((char*)"1804727075016") - str_to_INTERVAL((char*)"123538235292") * b) * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"382026143093760") + (str_to_INTERVAL((char*)"3799493450706816") + (str_to_INTERVAL((char*)"-2815877852971008") + (str_to_INTERVAL((char*)"4745804353605984") + (str_to_INTERVAL((char*)"-715466547749136") + (str_to_INTERVAL((char*)"233556101098992") + str_to_INTERVAL((char*)"2220867596124") * b) * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"-3286111692787200") + (str_to_INTERVAL((char*)"20172322472540160") + (str_to_INTERVAL((char*)"28120426220472960") + (str_to_INTERVAL((char*)"5622761454897280") + (str_to_INTERVAL((char*)"-2242972756362400") + (str_to_INTERVAL((char*)"669467033563520") + (str_to_INTERVAL((char*)"154986453968880") - str_to_INTERVAL((char*)"3944812931280") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-5278754348881920") + (str_to_INTERVAL((char*)"4911737193189888") + (str_to_INTERVAL((char*)"207568125550345984") + (str_to_INTERVAL((char*)"48364713131977216") + (str_to_INTERVAL((char*)"-17579909238441152") + (str_to_INTERVAL((char*)"-8278506496185696") + (str_to_INTERVAL((char*)"3371927005484800") - str_to_INTERVAL((char*)"18689412482568") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"7159467406417920") + (str_to_INTERVAL((char*)"-74049976234884096") + (str_to_INTERVAL((char*)"405282655198253568") + (str_to_INTERVAL((char*)"461573233420735744") + (str_to_INTERVAL((char*)"-69212172400727552") + (str_to_INTERVAL((char*)"-64886993668347584") + (str_to_INTERVAL((char*)"14792989432693472") + (str_to_INTERVAL((char*)"898619948150928") - str_to_INTERVAL((char*)"26800117859232") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"11316247401922560") + (str_to_INTERVAL((char*)"-68179146911668224") + (str_to_INTERVAL((char*)"117599482049433600") + (str_to_INTERVAL((char*)"1305075870613509120") + (str_to_INTERVAL((char*)"126921822395001600") + (str_to_INTERVAL((char*)"-233991383959166720") + (str_to_INTERVAL((char*)"9664465495445312") + (str_to_INTERVAL((char*)"10902255513072160") - str_to_INTERVAL((char*)"180341243230032") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-2488131933880320") + (str_to_INTERVAL((char*)"39303229518299136") + (str_to_INTERVAL((char*)"-396400875627268096") + (str_to_INTERVAL((char*)"1467869471018496000") + (str_to_INTERVAL((char*)"889342228355571712") + (str_to_INTERVAL((char*)"-380918409919348480") + (str_to_INTERVAL((char*)"-71805755774773120") + (str_to_INTERVAL((char*)"33053014624078592") + (str_to_INTERVAL((char*)"643624877623680") - str_to_INTERVAL((char*)"34996019055264") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-4785119110103040") + (str_to_INTERVAL((char*)"55037507239526400") + (str_to_INTERVAL((char*)"-388175391129149440") + (str_to_INTERVAL((char*)"647285334372651008") + (str_to_INTERVAL((char*)"1273638089304505344") + (str_to_INTERVAL((char*)"-159432105755786240") + (str_to_INTERVAL((char*)"-177490472719666176") + (str_to_INTERVAL((char*)"32264198323047936") + (str_to_INTERVAL((char*)"5653027052829888") - str_to_INTERVAL((char*)"149813305640352") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-959005517414400") + (str_to_INTERVAL((char*)"15712913861197824") + (str_to_INTERVAL((char*)"-127066686343225344") + (str_to_INTERVAL((char*)"78355641000419328") + (str_to_INTERVAL((char*)"700108806683021312") + (str_to_INTERVAL((char*)"120419645725828096") + (str_to_INTERVAL((char*)"-136678962276860416") + (str_to_INTERVAL((char*)"1106377927719424") + (str_to_INTERVAL((char*)"8468497191095552") + (str_to_INTERVAL((char*)"-13030556476800") - str_to_INTERVAL((char*)"5888312839872") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-39811562864640") + (str_to_INTERVAL((char*)"1338022380994560") + (str_to_INTERVAL((char*)"-16171544271126528") + (str_to_INTERVAL((char*)"-3944504197693440") + (str_to_INTERVAL((char*)"156269459827617792") + (str_to_INTERVAL((char*)"88752491098122240") + (str_to_INTERVAL((char*)"-31800839092868096") + (str_to_INTERVAL((char*)"-8506082183303680") + (str_to_INTERVAL((char*)"3413811664488192") + (str_to_INTERVAL((char*)"294711899928832") - str_to_INTERVAL((char*)"10883950757568") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-161021952000") + (str_to_INTERVAL((char*)"23460039622656") + (str_to_INTERVAL((char*)"-664115003424768") + (str_to_INTERVAL((char*)"-101982839341056") + (str_to_INTERVAL((char*)"13420781297147904") + (str_to_INTERVAL((char*)"13631358269042688") + (str_to_INTERVAL((char*)"-879310036838400") + (str_to_INTERVAL((char*)"-1598264460920832") + (str_to_INTERVAL((char*)"114878890731520") + (str_to_INTERVAL((char*)"155881639012352") + (str_to_INTERVAL((char*)"-3224982625536") - str_to_INTERVAL((char*)"8534458368") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-3663695314944") + (str_to_INTERVAL((char*)"29592864817152") + (str_to_INTERVAL((char*)"331820808208384") + (str_to_INTERVAL((char*)"402732410650624") + (str_to_INTERVAL((char*)"82016221208576") + (str_to_INTERVAL((char*)"32067101020160") + (str_to_INTERVAL((char*)"-55794124376064") + (str_to_INTERVAL((char*)"18312289697792") + (str_to_INTERVAL((char*)"519819047424") + str_to_INTERVAL((char*)"6630614784") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-1080963366912") + (str_to_INTERVAL((char*)"2074922606592") + (str_to_INTERVAL((char*)"4739430330368") + (str_to_INTERVAL((char*)"-3369515061248") + (str_to_INTERVAL((char*)"712904521728") + (str_to_INTERVAL((char*)"97628078080") + (str_to_INTERVAL((char*)"15294099456") - str_to_INTERVAL((char*)"628531200") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + (str_to_INTERVAL((char*)"-10462003200") + (str_to_INTERVAL((char*)"17605226496") + (str_to_INTERVAL((char*)"-9839259648") + (str_to_INTERVAL((char*)"1801605120") + str_to_INTERVAL((char*)"119992320") * b) * b) * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[38] =  pow( a,  3) *  pow( b,  5) * (str_to_INTERVAL((char*)"27899100") *  pow( b,  16) + ((str_to_INTERVAL((char*)"3040763220") - str_to_INTERVAL((char*)"368795628") * b) *  pow( b,  15) + ((str_to_INTERVAL((char*)"29172262320") + (str_to_INTERVAL((char*)"17777132928") + str_to_INTERVAL((char*)"1428735330") * b) * b) *  pow( b,  14) + ((str_to_INTERVAL((char*)"-88245123840") + (str_to_INTERVAL((char*)"1062772867728") + (str_to_INTERVAL((char*)"-76956663396") - str_to_INTERVAL((char*)"2148799182") * b) * b) * b) *  pow( b,  13) + ((str_to_INTERVAL((char*)"-5069836472400") + (str_to_INTERVAL((char*)"7511373961560") + (str_to_INTERVAL((char*)"1961364120912") + (str_to_INTERVAL((char*)"305903629596") + str_to_INTERVAL((char*)"1997489853") * b) * b) * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"-21367310509440") + (str_to_INTERVAL((char*)"-36492783191712") + (str_to_INTERVAL((char*)"81246580315824") + (str_to_INTERVAL((char*)"-2938979628968") + (str_to_INTERVAL((char*)"-202721004744") - str_to_INTERVAL((char*)"2113268796") * b) * b) * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"93771203281920") + (str_to_INTERVAL((char*)"-632730404617728") + (str_to_INTERVAL((char*)"506065180509168") + (str_to_INTERVAL((char*)"27944334636304") + (str_to_INTERVAL((char*)"12844609973012") + str_to_INTERVAL((char*)"283163469138") * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"662351060874240") + (str_to_INTERVAL((char*)"-1203114846899712") + (str_to_INTERVAL((char*)"80380688771136") + (str_to_INTERVAL((char*)"1345574218682976") + (str_to_INTERVAL((char*)"-74412866512760") + (str_to_INTERVAL((char*)"7772677367124") - str_to_INTERVAL((char*)"264998504232") * b) * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"37611770065920") + (str_to_INTERVAL((char*)"6926702138817408") + (str_to_INTERVAL((char*)"-1077726274603264") + (str_to_INTERVAL((char*)"6297595298805408") + (str_to_INTERVAL((char*)"-1396884118858448") + (str_to_INTERVAL((char*)"358129506855696") + str_to_INTERVAL((char*)"4399601633748") * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-4560657470392320") + (str_to_INTERVAL((char*)"22758334873572864") + (str_to_INTERVAL((char*)"51067173956942592") + (str_to_INTERVAL((char*)"8789492315231104") + (str_to_INTERVAL((char*)"-5652555782599456") + (str_to_INTERVAL((char*)"677927941187120") + (str_to_INTERVAL((char*)"259001602285344") - str_to_INTERVAL((char*)"5889201424704") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-3237465190164480") + (str_to_INTERVAL((char*)"-7384896177984000") + (str_to_INTERVAL((char*)"242795342984750336") + (str_to_INTERVAL((char*)"79732317375835136") + (str_to_INTERVAL((char*)"-29360186703488320") + (str_to_INTERVAL((char*)"-10617173931698464") + (str_to_INTERVAL((char*)"3964906355296128") - str_to_INTERVAL((char*)"21061067214312") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"8537469847695360") + (str_to_INTERVAL((char*)"-69753448695297024") + (str_to_INTERVAL((char*)"340829944377970176") + (str_to_INTERVAL((char*)"487580039022829312") + (str_to_INTERVAL((char*)"-67939197288291968") + (str_to_INTERVAL((char*)"-68098851265668800") + (str_to_INTERVAL((char*)"14097451403203776") + (str_to_INTERVAL((char*)"943941214285536") - str_to_INTERVAL((char*)"26875013366880") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"5893772355993600") + (str_to_INTERVAL((char*)"-31691768775358464") + (str_to_INTERVAL((char*)"20525006901196800") + (str_to_INTERVAL((char*)"1027058157288559616") + (str_to_INTERVAL((char*)"127407183849543936") + (str_to_INTERVAL((char*)"-194546534103246336") + (str_to_INTERVAL((char*)"7324806700817600") + (str_to_INTERVAL((char*)"8592742803269152") - str_to_INTERVAL((char*)"142478689627440") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-3203060214988800") + (str_to_INTERVAL((char*)"35998651833925632") + (str_to_INTERVAL((char*)"-273162857634652160") + (str_to_INTERVAL((char*)"876362422065770496") + (str_to_INTERVAL((char*)"580250986747033600") + (str_to_INTERVAL((char*)"-240366311283744256") + (str_to_INTERVAL((char*)"-45583887136201216") + (str_to_INTERVAL((char*)"20366620052655872") + (str_to_INTERVAL((char*)"420007963921536") - str_to_INTERVAL((char*)"21923375002560") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-2083264926842880") + (str_to_INTERVAL((char*)"26023223494656000") + (str_to_INTERVAL((char*)"-182636618481229824") + (str_to_INTERVAL((char*)"292441686713872384") + (str_to_INTERVAL((char*)"610299231768118272") + (str_to_INTERVAL((char*)"-73614960822418432") + (str_to_INTERVAL((char*)"-84194195133811712") + (str_to_INTERVAL((char*)"15096200256086016") + (str_to_INTERVAL((char*)"2663952622798656") - str_to_INTERVAL((char*)"71022428197344") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-226336914800640") + (str_to_INTERVAL((char*)"4659937033519104") + (str_to_INTERVAL((char*)"-41937183234883584") + (str_to_INTERVAL((char*)"30539691787198464") + (str_to_INTERVAL((char*)"241758810366707712") + (str_to_INTERVAL((char*)"40416394121617408") + (str_to_INTERVAL((char*)"-45585551198515712") + (str_to_INTERVAL((char*)"349457481541376") + (str_to_INTERVAL((char*)"2844451167365248") + (str_to_INTERVAL((char*)"-4026228950208") - str_to_INTERVAL((char*)"1900581291648") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-4060973629440") + (str_to_INTERVAL((char*)"217368113872896") + (str_to_INTERVAL((char*)"-3436392344911872") + (str_to_INTERVAL((char*)"961700682645504") + (str_to_INTERVAL((char*)"36892351315005440") + (str_to_INTERVAL((char*)"19250173587490816") + (str_to_INTERVAL((char*)"-6736433319498752") + (str_to_INTERVAL((char*)"-1779855415962112") + (str_to_INTERVAL((char*)"720111052488448") + (str_to_INTERVAL((char*)"69933977602560") - str_to_INTERVAL((char*)"2530103916096") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1159358054400") + (str_to_INTERVAL((char*)"-68274414157824") + (str_to_INTERVAL((char*)"136398189821952") + (str_to_INTERVAL((char*)"1906623218835456") + (str_to_INTERVAL((char*)"1597669135511552") + (str_to_INTERVAL((char*)"-69239205990400") + (str_to_INTERVAL((char*)"-140963247464448") + (str_to_INTERVAL((char*)"-11158026946560") + (str_to_INTERVAL((char*)"25013787934720") + (str_to_INTERVAL((char*)"-626040307200") + str_to_INTERVAL((char*)"6044583168") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"2538779443200") + (str_to_INTERVAL((char*)"16146187419648") + (str_to_INTERVAL((char*)"10387542392832") + (str_to_INTERVAL((char*)"8680802844672") + (str_to_INTERVAL((char*)"8402237284352") + (str_to_INTERVAL((char*)"-8093752133632") + (str_to_INTERVAL((char*)"2402813657088") + (str_to_INTERVAL((char*)"23074225152") + str_to_INTERVAL((char*)"4715723520") * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"188637216768") + (str_to_INTERVAL((char*)"171937931264") + (str_to_INTERVAL((char*)"-187734925312") + (str_to_INTERVAL((char*)"69176811520") + (str_to_INTERVAL((char*)"-7315357696") + (str_to_INTERVAL((char*)"2819985408") - str_to_INTERVAL((char*)"87920640") * b) * b) * b) * b) * b) * b) *  pow( b,  3) + (str_to_INTERVAL((char*)"390500352") + (str_to_INTERVAL((char*)"-389615616") + (str_to_INTERVAL((char*)"115568640") + str_to_INTERVAL((char*)"3870720") * b) * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[39] =  pow( a,  4) *  pow( b,  5) * (str_to_INTERVAL((char*)"-192075570") *  pow( b,  16) + ((str_to_INTERVAL((char*)"-12788927190") + str_to_INTERVAL((char*)"1515531654") * b) *  pow( b,  15) + ((str_to_INTERVAL((char*)"-101680537680") + (str_to_INTERVAL((char*)"-69015781800") - str_to_INTERVAL((char*)"5289614646") * b) * b) *  pow( b,  14) + ((str_to_INTERVAL((char*)"465931664640") + (str_to_INTERVAL((char*)"-2960826266844") + (str_to_INTERVAL((char*)"197179301118") + str_to_INTERVAL((char*)"6706775157") * b) * b) * b) *  pow( b,  13) + ((str_to_INTERVAL((char*)"12646901044800") + (str_to_INTERVAL((char*)"-16267879684848") + (str_to_INTERVAL((char*)"-5411077462032") + (str_to_INTERVAL((char*)"-665696177520") - str_to_INTERVAL((char*)"6936814962") * b) * b) * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"33551520511680") + (str_to_INTERVAL((char*)"97239191119344") + (str_to_INTERVAL((char*)"-166079401588936") + (str_to_INTERVAL((char*)"6950286589820") + (str_to_INTERVAL((char*)"316285123644") + str_to_INTERVAL((char*)"5630125698") * b) * b) * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"-212029561716480") + (str_to_INTERVAL((char*)"1076667249555264") + (str_to_INTERVAL((char*)"-855992645910000") + (str_to_INTERVAL((char*)"-45634648584752") + (str_to_INTERVAL((char*)"-18841275697188") - str_to_INTERVAL((char*)"675512583498") * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"-899406356981760") + (str_to_INTERVAL((char*)"1052823590830656") + (str_to_INTERVAL((char*)"-344964896164512") + (str_to_INTERVAL((char*)"-1875065843307600") + (str_to_INTERVAL((char*)"177761789149464") + (str_to_INTERVAL((char*)"-19883733961284") + str_to_INTERVAL((char*)"500157871632") * b) * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"653441290214400") + (str_to_INTERVAL((char*)"-10268284262411520") + (str_to_INTERVAL((char*)"-3500202563303680") + (str_to_INTERVAL((char*)"-7308959572611840") + (str_to_INTERVAL((char*)"2344552111429600") + (str_to_INTERVAL((char*)"-490562380610040") - str_to_INTERVAL((char*)"7409641587480") * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"4997508160135680") + (str_to_INTERVAL((char*)"-20737977681678336") + (str_to_INTERVAL((char*)"-74630859315260928") + (str_to_INTERVAL((char*)"-12915076303850496") + (str_to_INTERVAL((char*)"9803475969638304") + (str_to_INTERVAL((char*)"-595940542055520") + (str_to_INTERVAL((char*)"-362797594553136") + str_to_INTERVAL((char*)"7710802621056") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"493461794027520") + (str_to_INTERVAL((char*)"18057160425773568") + (str_to_INTERVAL((char*)"-243490367987929344") + (str_to_INTERVAL((char*)"-104053797514804992") + (str_to_INTERVAL((char*)"37760860377800768") + (str_to_INTERVAL((char*)"11736521659543776") + (str_to_INTERVAL((char*)"-4098649976804464") + str_to_INTERVAL((char*)"20935920943008") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-7321640379494400") + (str_to_INTERVAL((char*)"52568065842656256") + (str_to_INTERVAL((char*)"-242357233525050880") + (str_to_INTERVAL((char*)"-440087338970105600") + (str_to_INTERVAL((char*)"56748856046262528") + (str_to_INTERVAL((char*)"61438561867919040") + (str_to_INTERVAL((char*)"-11798837893480480") + (str_to_INTERVAL((char*)"-845870663504880") + str_to_INTERVAL((char*)"23411321652960") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-1775958088089600") + (str_to_INTERVAL((char*)"6701266087845888") + (str_to_INTERVAL((char*)"29541486695940096") + (str_to_INTERVAL((char*)"-694074823192078336") + (str_to_INTERVAL((char*)"-103470848600970752") + (str_to_INTERVAL((char*)"138020165035222272") + (str_to_INTERVAL((char*)"-4870264149957632") + (str_to_INTERVAL((char*)"-5837237426417984") + str_to_INTERVAL((char*)"97016624905248") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"2212740743086080") + (str_to_INTERVAL((char*)"-23484136239722496") + (str_to_INTERVAL((char*)"154883483666958336") + (str_to_INTERVAL((char*)"-446812315502287872") + (str_to_INTERVAL((char*)"-319131896860655616") + (str_to_INTERVAL((char*)"128597190877004544") + (str_to_INTERVAL((char*)"24345776930290560") + (str_to_INTERVAL((char*)"-10666189107338816") + (str_to_INTERVAL((char*)"-230022727793856") + str_to_INTERVAL((char*)"11641324973472") * b) * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"679630634680320") + (str_to_INTERVAL((char*)"-9762728622686208") + (str_to_INTERVAL((char*)"70938927487991808") + (str_to_INTERVAL((char*)"-112815076167311360") + (str_to_INTERVAL((char*)"-243000443703526400") + (str_to_INTERVAL((char*)"28819058645322752") + (str_to_INTERVAL((char*)"32793871639607296") + (str_to_INTERVAL((char*)"-5839140323771904") + (str_to_INTERVAL((char*)"-1042431535529920") + str_to_INTERVAL((char*)"27997017072480") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"37025016053760") + (str_to_INTERVAL((char*)"-1039493667373056") + (str_to_INTERVAL((char*)"10998288385990656") + (str_to_INTERVAL((char*)"-10399036375535616") + (str_to_INTERVAL((char*)"-67309754166974464") + (str_to_INTERVAL((char*)"-10440363763720192") + (str_to_INTERVAL((char*)"11932772792617472") + (str_to_INTERVAL((char*)"-79498089451264") + (str_to_INTERVAL((char*)"-761678337838208") + (str_to_INTERVAL((char*)"1035618782400") + str_to_INTERVAL((char*)"473201024256") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"193226342400") + (str_to_INTERVAL((char*)"-22152219328512") + (str_to_INTERVAL((char*)"522989964754944") + (str_to_INTERVAL((char*)"-602261423210496") + (str_to_INTERVAL((char*)"-6615498776444928") + (str_to_INTERVAL((char*)"-3002626705913856") + (str_to_INTERVAL((char*)"998512737359872") + (str_to_INTERVAL((char*)"271033454722048") + (str_to_INTERVAL((char*)"-108438895766528") + (str_to_INTERVAL((char*)"-13224985567232") + str_to_INTERVAL((char*)"463074899328") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"3328381550592") + (str_to_INTERVAL((char*)"-26817651277824") + (str_to_INTERVAL((char*)"-172952961679360") + (str_to_INTERVAL((char*)"-104496302174208") + (str_to_INTERVAL((char*)"-8078597586944") + (str_to_INTERVAL((char*)"4405917388800") + (str_to_INTERVAL((char*)"5942051139584") + (str_to_INTERVAL((char*)"-3438306551808") + (str_to_INTERVAL((char*)"105479606784") - str_to_INTERVAL((char*)"1860092928") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"472921473024") + (str_to_INTERVAL((char*)"-1075243778048") + (str_to_INTERVAL((char*)"-580699772928") + (str_to_INTERVAL((char*)"750649067520") + (str_to_INTERVAL((char*)"-253045217280") + (str_to_INTERVAL((char*)"4786169856") - str_to_INTERVAL((char*)"1094307840") * b) * b) * b) * b) * b) * b) *  pow( b,  3) + (str_to_INTERVAL((char*)"1569300480") + (str_to_INTERVAL((char*)"-2446737408") + (str_to_INTERVAL((char*)"1360723968") + (str_to_INTERVAL((char*)"-287539200") + str_to_INTERVAL((char*)"7741440") * b) * b) * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[40] = -2 *  pow( a,  5) *  pow( b,  6) * (str_to_INTERVAL((char*)"-485394525") *  pow( b,  15) + ((str_to_INTERVAL((char*)"-21981306015") + str_to_INTERVAL((char*)"2536687422") * b) *  pow( b,  14) + ((str_to_INTERVAL((char*)"-140256693720") + (str_to_INTERVAL((char*)"-109475353980") - str_to_INTERVAL((char*)"7935902514") * b) * b) *  pow( b,  13) + ((str_to_INTERVAL((char*)"824148572400") + (str_to_INTERVAL((char*)"-3499042525764") + (str_to_INTERVAL((char*)"210075988908") + str_to_INTERVAL((char*)"9071608317") * b) * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"13192225336800") + (str_to_INTERVAL((char*)"-14377618516008") + (str_to_INTERVAL((char*)"-6410770020592") + (str_to_INTERVAL((char*)"-582436910928") - str_to_INTERVAL((char*)"10257849909") * b) * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"19044788515200") + (str_to_INTERVAL((char*)"104755412913624") + (str_to_INTERVAL((char*)"-148616359012892") + (str_to_INTERVAL((char*)"7401412891822") + (str_to_INTERVAL((char*)"173714254542") + str_to_INTERVAL((char*)"6606556803") * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"-192426458872320") + (str_to_INTERVAL((char*)"775895620358592") + (str_to_INTERVAL((char*)"-642456759606408") + (str_to_INTERVAL((char*)"-29201659547728") + (str_to_INTERVAL((char*)"-11258091760290") - str_to_INTERVAL((char*)"694574154909") * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"-486742978068480") + (str_to_INTERVAL((char*)"166068173204160") + (str_to_INTERVAL((char*)"-530216659374816") + (str_to_INTERVAL((char*)"-1126400417900760") + (str_to_INTERVAL((char*)"167826791844736") + (str_to_INTERVAL((char*)"-19473177177384") + str_to_INTERVAL((char*)"415689237522") * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"739225843038720") + (str_to_INTERVAL((char*)"-6279791965914240") + (str_to_INTERVAL((char*)"-5329499756019712") + (str_to_INTERVAL((char*)"-3749031445926336") + (str_to_INTERVAL((char*)"1704760177641280") + (str_to_INTERVAL((char*)"-301453447091004") - str_to_INTERVAL((char*)"5367938341824") * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"2140687676424960") + (str_to_INTERVAL((char*)"-7307615835793920") + (str_to_INTERVAL((char*)"-45387952965056256") + (str_to_INTERVAL((char*)"-8532298625926464") + (str_to_INTERVAL((char*)"6724148510912928") + (str_to_INTERVAL((char*)"-229057384274800") + (str_to_INTERVAL((char*)"-216790847036016") + str_to_INTERVAL((char*)"4425682038048") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-805347466583040") + (str_to_INTERVAL((char*)"11254980471549696") + (str_to_INTERVAL((char*)"-104841421287243392") + (str_to_INTERVAL((char*)"-56064455541767936") + (str_to_INTERVAL((char*)"19864344735862944") + (str_to_INTERVAL((char*)"5612789958153648") + (str_to_INTERVAL((char*)"-1860320654192952") + str_to_INTERVAL((char*)"9164629818864") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-2334140422594560") + (str_to_INTERVAL((char*)"15649176674230272") + (str_to_INTERVAL((char*)"-72342708474700288") + (str_to_INTERVAL((char*)"-170114369893265536") + (str_to_INTERVAL((char*)"20248308571364544") + (str_to_INTERVAL((char*)"23830728139726368") + (str_to_INTERVAL((char*)"-4317519344656768") + (str_to_INTERVAL((char*)"-324521942771904") + str_to_INTERVAL((char*)"8832984429456") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"83709940285440") + (str_to_INTERVAL((char*)"-2029999223648256") + (str_to_INTERVAL((char*)"19571594911973376") + (str_to_INTERVAL((char*)"-200846506785835008") + (str_to_INTERVAL((char*)"-34594480498376960") + (str_to_INTERVAL((char*)"41674958903132032") + (str_to_INTERVAL((char*)"-1412945558480896") + (str_to_INTERVAL((char*)"-1700292896443808") + str_to_INTERVAL((char*)"28320915044880") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"526016591585280") + (str_to_INTERVAL((char*)"-5831450526713856") + (str_to_INTERVAL((char*)"36149127353125888") + (str_to_INTERVAL((char*)"-96800576288027136") + (str_to_INTERVAL((char*)"-73449299677179904") + (str_to_INTERVAL((char*)"28953952077691776") + (str_to_INTERVAL((char*)"5412119747711680") + (str_to_INTERVAL((char*)"-2349878914826528") + (str_to_INTERVAL((char*)"-52639455272544") + str_to_INTERVAL((char*)"2594431082160") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"80815935651840") + (str_to_INTERVAL((char*)"-1423298653691904") + (str_to_INTERVAL((char*)"11197844301885440") + (str_to_INTERVAL((char*)"-18437592214611968") + (str_to_INTERVAL((char*)"-39624038095624704") + (str_to_INTERVAL((char*)"4738655514459136") + (str_to_INTERVAL((char*)"5140975621466880") + (str_to_INTERVAL((char*)"-915390726604032") + (str_to_INTERVAL((char*)"-166841770232736") + str_to_INTERVAL((char*)"4524119378064") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1875100631040") + (str_to_INTERVAL((char*)"-81906904203264") + (str_to_INTERVAL((char*)"1102687605104640") + (str_to_INTERVAL((char*)"-1461415486011392") + (str_to_INTERVAL((char*)"-7353646238371840") + (str_to_INTERVAL((char*)"-980242953971712") + (str_to_INTERVAL((char*)"1171426786404864") + (str_to_INTERVAL((char*)"-3497907111424") + (str_to_INTERVAL((char*)"-79021482192000") + (str_to_INTERVAL((char*)"108239980992") + str_to_INTERVAL((char*)"42338481120") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-531372441600") + (str_to_INTERVAL((char*)"25336687951872") + (str_to_INTERVAL((char*)"-72336247357440") + (str_to_INTERVAL((char*)"-423075756883968") + (str_to_INTERVAL((char*)"-150417382865920") + (str_to_INTERVAL((char*)"41905167892480") + (str_to_INTERVAL((char*)"14898672697344") + (str_to_INTERVAL((char*)"-5241463789568") + (str_to_INTERVAL((char*)"-1013339256576") + str_to_INTERVAL((char*)"33910521408") * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-906706944000") + (str_to_INTERVAL((char*)"-3761378869248") + (str_to_INTERVAL((char*)"-726778150912") + (str_to_INTERVAL((char*)"-1562990352384") + (str_to_INTERVAL((char*)"4308341760") + (str_to_INTERVAL((char*)"474888059904") + (str_to_INTERVAL((char*)"-206172194304") + (str_to_INTERVAL((char*)"7366855680") - str_to_INTERVAL((char*)"138219264") * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-31439536128") + (str_to_INTERVAL((char*)"-4197811200") + (str_to_INTERVAL((char*)"15993474048") + (str_to_INTERVAL((char*)"-7300101120") + (str_to_INTERVAL((char*)"570608640") - str_to_INTERVAL((char*)"79810560") * b) * b) * b) * b) * b) *  pow( b,  3) + (str_to_INTERVAL((char*)"-17750016") + (str_to_INTERVAL((char*)"18740736") + (str_to_INTERVAL((char*)"-6312960") + str_to_INTERVAL((char*)"161280") * b) * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[41] = 0.-pow( a,  6) *  pow( b,  6) * (str_to_INTERVAL((char*)"3836817855") *  pow( b,  15) + ((str_to_INTERVAL((char*)"126245568060") - str_to_INTERVAL((char*)"14144782725") * b) *  pow( b,  14) + ((str_to_INTERVAL((char*)"623835198720") + (str_to_INTERVAL((char*)"581997913668") + str_to_INTERVAL((char*)"39665568339") * b) * b) *  pow( b,  13) + ((str_to_INTERVAL((char*)"-4493789625600") + (str_to_INTERVAL((char*)"14176311285972") + (str_to_INTERVAL((char*)"-741400955262") - str_to_INTERVAL((char*)"43297137741") * b) * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"-46325245217760") + (str_to_INTERVAL((char*)"41181793990392") + (str_to_INTERVAL((char*)"26423866993936") + (str_to_INTERVAL((char*)"1581731649600") + str_to_INTERVAL((char*)"52265565783") * b) * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"-21589618033440") + (str_to_INTERVAL((char*)"-375507757055808") + (str_to_INTERVAL((char*)"470149922517416") + (str_to_INTERVAL((char*)"-28351379555736") + (str_to_INTERVAL((char*)"-131512872792") - str_to_INTERVAL((char*)"27367937478") * b) * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"571818065794560") + (str_to_INTERVAL((char*)"-1890841897479744") + (str_to_INTERVAL((char*)"1742260303888704") + (str_to_INTERVAL((char*)"51722828759592") + (str_to_INTERVAL((char*)"20796223775508") + str_to_INTERVAL((char*)"2482719653898") * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"794579660467200") + (str_to_INTERVAL((char*)"851819004192000") + (str_to_INTERVAL((char*)"2370343938882240") + (str_to_INTERVAL((char*)"2324535219814080") + (str_to_INTERVAL((char*)"-528009295209280") + (str_to_INTERVAL((char*)"62731573398960") - str_to_INTERVAL((char*)"1217833065720") * b) * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-2060763232354560") + (str_to_INTERVAL((char*)"12732148451289600") + (str_to_INTERVAL((char*)"18559095254768896") + (str_to_INTERVAL((char*)"6883718867333088") + (str_to_INTERVAL((char*)"-4318789487151760") + (str_to_INTERVAL((char*)"665835793010952") + str_to_INTERVAL((char*)"13482865486332") * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-2754080533025280") + (str_to_INTERVAL((char*)"7010831066164224") + (str_to_INTERVAL((char*)"93425652937145088") + (str_to_INTERVAL((char*)"19774124155411328") + (str_to_INTERVAL((char*)"-15293581469161184") + (str_to_INTERVAL((char*)"311636763934240") + (str_to_INTERVAL((char*)"446443344606720") - str_to_INTERVAL((char*)"8902022334912") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"2330410999910400") + (str_to_INTERVAL((char*)"-20067625803816960") + (str_to_INTERVAL((char*)"155056343723513600") + (str_to_INTERVAL((char*)"101663324162287232") + (str_to_INTERVAL((char*)"-35059869407031552") + (str_to_INTERVAL((char*)"-9302604801699200") + (str_to_INTERVAL((char*)"2959347511257968") - str_to_INTERVAL((char*)"14098651338240") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"2163417390120960") + (str_to_INTERVAL((char*)"-14169469759312896") + (str_to_INTERVAL((char*)"71658960621894144") + (str_to_INTERVAL((char*)"225256051766073600") + (str_to_INTERVAL((char*)"-24717914877490176") + (str_to_INTERVAL((char*)"-31733227655704512") + (str_to_INTERVAL((char*)"5496164163908320") + (str_to_INTERVAL((char*)"426553385619696") - str_to_INTERVAL((char*)"11502430483680") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-553150127554560") + (str_to_INTERVAL((char*)"5401694541342720") + (str_to_INTERVAL((char*)"-28651495338135552") + (str_to_INTERVAL((char*)"198212981827086336") + (str_to_INTERVAL((char*)"38275553680415488") + (str_to_INTERVAL((char*)"-42627395294410368") + (str_to_INTERVAL((char*)"1421848496322816") + (str_to_INTERVAL((char*)"1686773442525632") - str_to_INTERVAL((char*)"28153172363472") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-360301931274240") + (str_to_INTERVAL((char*)"4473733895847936") + (str_to_INTERVAL((char*)"-27611933640374272") + (str_to_INTERVAL((char*)"70759486504808448") + (str_to_INTERVAL((char*)"55984029863768064") + (str_to_INTERVAL((char*)"-21715736321208576") + (str_to_INTERVAL((char*)"-3943969555211392") + (str_to_INTERVAL((char*)"1718357052352768") + (str_to_INTERVAL((char*)"39939442488960") - str_to_INTERVAL((char*)"1915807942368") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-26513600348160") + (str_to_INTERVAL((char*)"622482822070272") + (str_to_INTERVAL((char*)"-5609697185050624") + (str_to_INTERVAL((char*)"10061296397830144") + (str_to_INTERVAL((char*)"20742386983351296") + (str_to_INTERVAL((char*)"-2581664531799552") + (str_to_INTERVAL((char*)"-2516482700728832") + (str_to_INTERVAL((char*)"451585885096448") + (str_to_INTERVAL((char*)"85664937574528") - str_to_INTERVAL((char*)"2354652242784") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-177124147200") + (str_to_INTERVAL((char*)"16231488159744") + (str_to_INTERVAL((char*)"-317475517218816") + (str_to_INTERVAL((char*)"635786551537664") + (str_to_INTERVAL((char*)"2421496565645312") + (str_to_INTERVAL((char*)"236083868315648") + (str_to_INTERVAL((char*)"-316501175336960") + (str_to_INTERVAL((char*)"-3694144651264") + (str_to_INTERVAL((char*)"24537124522752") + (str_to_INTERVAL((char*)"-36209040000") - str_to_INTERVAL((char*)"9029763072") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-2343123025920") + (str_to_INTERVAL((char*)"18392492556288") + (str_to_INTERVAL((char*)"68706353831936") + (str_to_INTERVAL((char*)"14035649118208") + (str_to_INTERVAL((char*)"312736427008") + (str_to_INTERVAL((char*)"-2904621415936") + (str_to_INTERVAL((char*)"540751878144") + (str_to_INTERVAL((char*)"262629480704") - str_to_INTERVAL((char*)"8422868736") * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-157640491008") + (str_to_INTERVAL((char*)"370411143168") + (str_to_INTERVAL((char*)"-60873787392") + (str_to_INTERVAL((char*)"-71977863168") + (str_to_INTERVAL((char*)"36998049792") + (str_to_INTERVAL((char*)"-1494933504") + str_to_INTERVAL((char*)"23224320") * b) * b) * b) * b) * b) * b) *  pow( b,  3) + (str_to_INTERVAL((char*)"-142663680") + (str_to_INTERVAL((char*)"190241280") + (str_to_INTERVAL((char*)"-88611840") + str_to_INTERVAL((char*)"13386240") * b) * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[42] =  pow( a,  7) *  pow( b,  7) * (str_to_INTERVAL((char*)"12330640665") *  pow( b,  14) + ((str_to_INTERVAL((char*)"307357604730") - str_to_INTERVAL((char*)"33324593940") * b) *  pow( b,  13) + ((str_to_INTERVAL((char*)"1123499721600") + (str_to_INTERVAL((char*)"1318763290860") + str_to_INTERVAL((char*)"84253217541") * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"-9949155415200") + (str_to_INTERVAL((char*)"24789241464984") + (str_to_INTERVAL((char*)"-1056697807764") - str_to_INTERVAL((char*)"92486333994") * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"-68545671660000") + (str_to_INTERVAL((char*)"46201453330344") + (str_to_INTERVAL((char*)"47835049471200") + (str_to_INTERVAL((char*)"1444209454512") + str_to_INTERVAL((char*)"115724953665") * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"25020855970560") + (str_to_INTERVAL((char*)"-568532383802640") + (str_to_INTERVAL((char*)"662943039868392") + (str_to_INTERVAL((char*)"-48754405778212") + (str_to_INTERVAL((char*)"506942517900") - str_to_INTERVAL((char*)"50093332146") * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"700545552318720") + (str_to_INTERVAL((char*)"-1932177855288192") + (str_to_INTERVAL((char*)"2168156378415840") + (str_to_INTERVAL((char*)"11655468609144") + (str_to_INTERVAL((char*)"12129209372100") + str_to_INTERVAL((char*)"3879890325378") * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"403719191608320") + (str_to_INTERVAL((char*)"2079654196000128") + (str_to_INTERVAL((char*)"4136731600649472") + (str_to_INTERVAL((char*)"2039714518325088") + (str_to_INTERVAL((char*)"-709571160453272") + (str_to_INTERVAL((char*)"85716142275780") - str_to_INTERVAL((char*)"1572855283584") * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-2118231689091840") + (str_to_INTERVAL((char*)"10659253350332160") + (str_to_INTERVAL((char*)"24470055792770304") + (str_to_INTERVAL((char*)"5739014866663200") + (str_to_INTERVAL((char*)"-4780953016546064") + (str_to_INTERVAL((char*)"660238074131672") + str_to_INTERVAL((char*)"14744313031428") * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-1166595845091840") + (str_to_INTERVAL((char*)"916960674014976") + (str_to_INTERVAL((char*)"82062352074320256") + (str_to_INTERVAL((char*)"19863637282366528") + (str_to_INTERVAL((char*)"-14721513412025184") + (str_to_INTERVAL((char*)"191235950243424") + (str_to_INTERVAL((char*)"398123451819648") - str_to_INTERVAL((char*)"7834730032224") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"1906334746398720") + (str_to_INTERVAL((char*)"-13727810557052928") + (str_to_INTERVAL((char*)"98322783900190464") + (str_to_INTERVAL((char*)"78224915099652480") + (str_to_INTERVAL((char*)"-26247858913060736") + (str_to_INTERVAL((char*)"-6673990940056960") + (str_to_INTERVAL((char*)"2055486792654736") - str_to_INTERVAL((char*)"9494754706656") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"650934140682240") + (str_to_INTERVAL((char*)"-4358874645762048") + (str_to_INTERVAL((char*)"28846511491600384") + (str_to_INTERVAL((char*)"127415139847438336") + (str_to_INTERVAL((char*)"-12895864398636288") + (str_to_INTERVAL((char*)"-18060880886089600") + (str_to_INTERVAL((char*)"3022139256113152") + (str_to_INTERVAL((char*)"239475170090112") - str_to_INTERVAL((char*)"6428001739968") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-354273460715520") + (str_to_INTERVAL((char*)"3370459481978880") + (str_to_INTERVAL((char*)"-15213231060234240") + (str_to_INTERVAL((char*)"82843619368351744") + (str_to_INTERVAL((char*)"17466040928220416") + (str_to_INTERVAL((char*)"-18312735156668032") + (str_to_INTERVAL((char*)"615650970598144") + (str_to_INTERVAL((char*)"705380457360512") - str_to_INTERVAL((char*)"11797711660656") * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-88143547023360") + (str_to_INTERVAL((char*)"1313023801331712") + (str_to_INTERVAL((char*)"-8513667341494272") + (str_to_INTERVAL((char*)"21569877713605632") + (str_to_INTERVAL((char*)"17392332631884800") + (str_to_INTERVAL((char*)"-6680658996781056") + (str_to_INTERVAL((char*)"-1150194375123200") + (str_to_INTERVAL((char*)"511394610733312") + (str_to_INTERVAL((char*)"12405517499712") - str_to_INTERVAL((char*)"575220023136") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-2684696002560") + (str_to_INTERVAL((char*)"96046803173376") + (str_to_INTERVAL((char*)"-1073592572350464") + (str_to_INTERVAL((char*)"2233659511322624") + (str_to_INTERVAL((char*)"4231166224724992") + (str_to_INTERVAL((char*)"-570235142872576") + (str_to_INTERVAL((char*)"-456653814966272") + (str_to_INTERVAL((char*)"83324474967552") + (str_to_INTERVAL((char*)"17202269893760") - str_to_INTERVAL((char*)"482887554912") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"759103488000") + (str_to_INTERVAL((char*)"-29189395611648") + (str_to_INTERVAL((char*)"98952352858112") + (str_to_INTERVAL((char*)"281314132301824") + (str_to_INTERVAL((char*)"11042313999360") + (str_to_INTERVAL((char*)"-23278240155648") + (str_to_INTERVAL((char*)"-2102504623104") + (str_to_INTERVAL((char*)"2774044572160") + (str_to_INTERVAL((char*)"-5019620352") - str_to_INTERVAL((char*)"49462272") * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"1020045312000") + (str_to_INTERVAL((char*)"2659406610432") + (str_to_INTERVAL((char*)"-274708008960") + (str_to_INTERVAL((char*)"912744662016") + (str_to_INTERVAL((char*)"-368811721216") + (str_to_INTERVAL((char*)"21175361536") + (str_to_INTERVAL((char*)"28948283136") - str_to_INTERVAL((char*)"917846784") * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"15719768064") + (str_to_INTERVAL((char*)"-5535068160") + (str_to_INTERVAL((char*)"-1993477120") + (str_to_INTERVAL((char*)"1680821760") + (str_to_INTERVAL((char*)"-88152576") + str_to_INTERVAL((char*)"967680") * b) * b) * b) * b) * b) *  pow( b,  3) + (str_to_INTERVAL((char*)"1479168") + (str_to_INTERVAL((char*)"-1520640") + str_to_INTERVAL((char*)"483840") * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[43] =  pow( a,  8) *  pow( b,  7) * (str_to_INTERVAL((char*)"-33076495395") *  pow( b,  14) + ((str_to_INTERVAL((char*)"-641014074270") + str_to_INTERVAL((char*)"66979953108") * b) *  pow( b,  13) + ((str_to_INTERVAL((char*)"-1616734017360") + (str_to_INTERVAL((char*)"-2578581402804") - str_to_INTERVAL((char*)"154621291572") * b) * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"18356662620720") + (str_to_INTERVAL((char*)"-37594730035140") + (str_to_INTERVAL((char*)"1120315497414") + str_to_INTERVAL((char*)"178738815099") * b) * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"85099422415680") + (str_to_INTERVAL((char*)"-36177017430024") + (str_to_INTERVAL((char*)"-76676593403336") + (str_to_INTERVAL((char*)"-142877263800") - str_to_INTERVAL((char*)"224279571483") * b) * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"-95690347297920") + (str_to_INTERVAL((char*)"733259362934304") + (str_to_INTERVAL((char*)"-840050904454368") + (str_to_INTERVAL((char*)"75207092739696") + (str_to_INTERVAL((char*)"-1640542098240") + str_to_INTERVAL((char*)"81118589724") * b) * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"-706157346124800") + (str_to_INTERVAL((char*)"1625153309872320") + (str_to_INTERVAL((char*)"-2504665584977520") + (str_to_INTERVAL((char*)"64537289778240") + (str_to_INTERVAL((char*)"2042331562020") - str_to_INTERVAL((char*)"5325627926070") * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"31575577835520") + (str_to_INTERVAL((char*)"-2845873153759872") + (str_to_INTERVAL((char*)"-5883150512720448") + (str_to_INTERVAL((char*)"-1490253992464992") + (str_to_INTERVAL((char*)"825981743783008") + (str_to_INTERVAL((char*)"-100988945817600") + str_to_INTERVAL((char*)"1791622319616") * b) * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"1673980853103360") + (str_to_INTERVAL((char*)"-7273431461808000") + (str_to_INTERVAL((char*)"-26259939107218816") + (str_to_INTERVAL((char*)"-4412822674730112") + (str_to_INTERVAL((char*)"4633325344054912") + (str_to_INTERVAL((char*)"-586333985607984") - str_to_INTERVAL((char*)"14075226070632") * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"117483020075520") + (str_to_INTERVAL((char*)"2154675240875520") + (str_to_INTERVAL((char*)"-61776630373802496") + (str_to_INTERVAL((char*)"-17206716414051584") + (str_to_INTERVAL((char*)"12114733233834720") + (str_to_INTERVAL((char*)"-108767997740608") + (str_to_INTERVAL((char*)"-308145933457824") + str_to_INTERVAL((char*)"6022938514176") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-1085233579499520") + (str_to_INTERVAL((char*)"7312785856622592") + (str_to_INTERVAL((char*)"-53312537417488384") + (str_to_INTERVAL((char*)"-51236695126993408") + (str_to_INTERVAL((char*)"16744610940541824") + (str_to_INTERVAL((char*)"4133564558114560") + (str_to_INTERVAL((char*)"-1240974213095328") + str_to_INTERVAL((char*)"5571681656976") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-59857471549440") + (str_to_INTERVAL((char*)"474098065560576") + (str_to_INTERVAL((char*)"-9077906284680704") + (str_to_INTERVAL((char*)"-61291566279962368") + (str_to_INTERVAL((char*)"5740273042546432") + (str_to_INTERVAL((char*)"8730569380192960") + (str_to_INTERVAL((char*)"-1423633677645984") + (str_to_INTERVAL((char*)"-114225302368464") + str_to_INTERVAL((char*)"3061092000960") * b) * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"138015306915840") + (str_to_INTERVAL((char*)"-1435787401156608") + (str_to_INTERVAL((char*)"6224275281131520") + (str_to_INTERVAL((char*)"-29051944367411200") + (str_to_INTERVAL((char*)"-6513629694541568") + (str_to_INTERVAL((char*)"6530156064699392") + (str_to_INTERVAL((char*)"-227243577033984") + (str_to_INTERVAL((char*)"-245339795803936") + str_to_INTERVAL((char*)"4113315301392") * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"14700708126720") + (str_to_INTERVAL((char*)"-286282326822912") + (str_to_INTERVAL((char*)"2072061675927552") + (str_to_INTERVAL((char*)"-5391671135818752") + (str_to_INTERVAL((char*)"-4307410396359680") + (str_to_INTERVAL((char*)"1650689180789760") + (str_to_INTERVAL((char*)"258485995145472") + (str_to_INTERVAL((char*)"-120464296132608") + (str_to_INTERVAL((char*)"-3103156524864") + str_to_INTERVAL((char*)"136781720064") * b) * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"126517248000") + (str_to_INTERVAL((char*)"-9317700108288") + (str_to_INTERVAL((char*)"147524252729344") + (str_to_INTERVAL((char*)"-387173618274304") + (str_to_INTERVAL((char*)"-643494870868992") + (str_to_INTERVAL((char*)"98810404751872") + (str_to_INTERVAL((char*)"55437943016192") + (str_to_INTERVAL((char*)"-10363759084800") + (str_to_INTERVAL((char*)"-2628782688640") + str_to_INTERVAL((char*)"76446823680") * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"1285857607680") + (str_to_INTERVAL((char*)"-9734631735296") + (str_to_INTERVAL((char*)"-20436871665664") + (str_to_INTERVAL((char*)"1285451668480") + (str_to_INTERVAL((char*)"-223628562432") + (str_to_INTERVAL((char*)"568361518592") + (str_to_INTERVAL((char*)"-241215702528") + (str_to_INTERVAL((char*)"671682048") - str_to_INTERVAL((char*)"153045504") * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"39410122752") + (str_to_INTERVAL((char*)"-88115933184") + (str_to_INTERVAL((char*)"40684687872") + (str_to_INTERVAL((char*)"-3799008512") + (str_to_INTERVAL((char*)"-2322952448") + str_to_INTERVAL((char*)"74833920") * b) * b) * b) * b) * b) *  pow( b,  3) + (str_to_INTERVAL((char*)"5944320") + (str_to_INTERVAL((char*)"-6220800") + str_to_INTERVAL((char*)"1935360") * b) * b) *  pow( b,  5) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[44] = 0.-pow( a,  9) *  pow( b,  8) * (str_to_INTERVAL((char*)"-75428001225") *  pow( b,  13) + ((str_to_INTERVAL((char*)"-1153636528680") + str_to_INTERVAL((char*)"115523945541") * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"-1763918620560") + (str_to_INTERVAL((char*)"-4388719497804") - str_to_INTERVAL((char*)"248804301798") * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"28630539644400") + (str_to_INTERVAL((char*)"-49575250222836") + (str_to_INTERVAL((char*)"584626106694") + str_to_INTERVAL((char*)"314507362419") * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"87480769245120") + (str_to_INTERVAL((char*)"-8735260169880") + (str_to_INTERVAL((char*)"-109548384702304") + (str_to_INTERVAL((char*)"2753829098592") - str_to_INTERVAL((char*)"382590205947") * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"-166623535550880") + (str_to_INTERVAL((char*)"808630666134480") + (str_to_INTERVAL((char*)"-963684757901104") + (str_to_INTERVAL((char*)"104021599307212") + (str_to_INTERVAL((char*)"-3135353770308") + str_to_INTERVAL((char*)"116352956196") * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-578006840563200") + (str_to_INTERVAL((char*)"1078023989387712") + (str_to_INTERVAL((char*)"-2697057611330736") + (str_to_INTERVAL((char*)"161256163920976") + (str_to_INTERVAL((char*)"17493424495700") - str_to_INTERVAL((char*)"6441057067878") * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"327664801704960") + (str_to_INTERVAL((char*)"-2881646297634240") + (str_to_INTERVAL((char*)"-6988919962970016") + (str_to_INTERVAL((char*)"-863808774202752") + (str_to_INTERVAL((char*)"839543946441672") + (str_to_INTERVAL((char*)"-103538846259660") + str_to_INTERVAL((char*)"1800004564548") * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"1014292419943680") + (str_to_INTERVAL((char*)"-3925293396278400") + (str_to_INTERVAL((char*)"-23605062479736576") + (str_to_INTERVAL((char*)"-3167324010374592") + (str_to_INTERVAL((char*)"3932003466702048") + (str_to_INTERVAL((char*)"-464557726702512") - str_to_INTERVAL((char*)"11739508365816") * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-278576850309120") + (str_to_INTERVAL((char*)"2610089992962816") + (str_to_INTERVAL((char*)"-39908135157999360") + (str_to_INTERVAL((char*)"-12813153567459264") + (str_to_INTERVAL((char*)"8558343108061824") + (str_to_INTERVAL((char*)"-58964940238128") + (str_to_INTERVAL((char*)"-206952502848480") + str_to_INTERVAL((char*)"4033071839376") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-439610406174720") + (str_to_INTERVAL((char*)"2988003920873472") + (str_to_INTERVAL((char*)"-24609275648036864") + (str_to_INTERVAL((char*)"-28546270827507200") + (str_to_INTERVAL((char*)"9102534556877184") + (str_to_INTERVAL((char*)"2199192420692864") + (str_to_INTERVAL((char*)"-647246041510592") + str_to_INTERVAL((char*)"2830529122992") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"54846592266240") + (str_to_INTERVAL((char*)"-370947396971520") + (str_to_INTERVAL((char*)"-2037608227431936") + (str_to_INTERVAL((char*)"-24894786240036608") + (str_to_INTERVAL((char*)"2174274319372032") + (str_to_INTERVAL((char*)"3550908395956352") + (str_to_INTERVAL((char*)"-568461593296928") + (str_to_INTERVAL((char*)"-45891607521840") + str_to_INTERVAL((char*)"1230200751168") * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"36161510400000") + (str_to_INTERVAL((char*)"-444245440874496") + (str_to_INTERVAL((char*)"1986836691271680") + (str_to_INTERVAL((char*)"-8434367792166912") + (str_to_INTERVAL((char*)"-1953341019777280") + (str_to_INTERVAL((char*)"1900263414868992") + (str_to_INTERVAL((char*)"-70633549032448") + (str_to_INTERVAL((char*)"-69649068292960") + str_to_INTERVAL((char*)"1171506529872") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1502708613120") + (str_to_INTERVAL((char*)"-43790506180608") + (str_to_INTERVAL((char*)"383834098145280") + (str_to_INTERVAL((char*)"-1077118109792256") + (str_to_INTERVAL((char*)"-823023500461568") + (str_to_INTERVAL((char*)"317724029316864") + (str_to_INTERVAL((char*)"41768510305536") + (str_to_INTERVAL((char*)"-21503963227008") + (str_to_INTERVAL((char*)"-612729608832") + str_to_INTERVAL((char*)"24749554176") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-426995712000") + (str_to_INTERVAL((char*)"12959755001856") + (str_to_INTERVAL((char*)"-48996759711744") + (str_to_INTERVAL((char*)"-67900765621248") + (str_to_INTERVAL((char*)"12740902665728") + (str_to_INTERVAL((char*)"3275717595392") + (str_to_INTERVAL((char*)"-606973504768") + (str_to_INTERVAL((char*)"-301874397312") + str_to_INTERVAL((char*)"9325663488") * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-453353472000") + (str_to_INTERVAL((char*)"-692457013248") + (str_to_INTERVAL((char*)"208203018240") + (str_to_INTERVAL((char*)"-201809473024") + (str_to_INTERVAL((char*)"90815400704") + (str_to_INTERVAL((char*)"-20769848320") + (str_to_INTERVAL((char*)"97225728") - str_to_INTERVAL((char*)"24192576") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-2858139648") + (str_to_INTERVAL((char*)"1807021568") + (str_to_INTERVAL((char*)"-306300672") + (str_to_INTERVAL((char*)"-89354496") + str_to_INTERVAL((char*)"2903040") * b) * b) * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[45] = 0.-pow( a,  10) *  pow( b,  8) * (str_to_INTERVAL((char*)"148173398550") *  pow( b,  13) + ((str_to_INTERVAL((char*)"1800562445190") - str_to_INTERVAL((char*)"171419689692") * b) *  pow( b,  12) + ((str_to_INTERVAL((char*)"1183535510040") + (str_to_INTERVAL((char*)"6541733839626") + str_to_INTERVAL((char*)"356232193077") * b) * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"-38041878605760") + (str_to_INTERVAL((char*)"56864921470392") + (str_to_INTERVAL((char*)"845239432512") - str_to_INTERVAL((char*)"504988465788") * b) * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"-72172897868880") + (str_to_INTERVAL((char*)"-29525177942928") + (str_to_INTERVAL((char*)"140262219811440") + (str_to_INTERVAL((char*)"-7123651780320") + str_to_INTERVAL((char*)"576983026548") * b) * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"208079586914400") + (str_to_INTERVAL((char*)"-762435865911600") + (str_to_INTERVAL((char*)"1007239704594560") + (str_to_INTERVAL((char*)"-128967760090040") + (str_to_INTERVAL((char*)"4644146362560") - str_to_INTERVAL((char*)"147994435260") * b) * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"371694496550400") + (str_to_INTERVAL((char*)"-498897167587488") + (str_to_INTERVAL((char*)"2697017331361824") + (str_to_INTERVAL((char*)"-248464209219784") + (str_to_INTERVAL((char*)"-29240815231640") + str_to_INTERVAL((char*)"6878055889152") * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-406502211432960") + (str_to_INTERVAL((char*)"2304290917285248") + (str_to_INTERVAL((char*)"7036752210684480") + (str_to_INTERVAL((char*)"343010034639744") + (str_to_INTERVAL((char*)"-748389344753248") + (str_to_INTERVAL((char*)"92840310064800") - str_to_INTERVAL((char*)"1594313781228") * b) * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-450371466800640") + (str_to_INTERVAL((char*)"1555600139585280") + (str_to_INTERVAL((char*)"18009580281223040") + (str_to_INTERVAL((char*)"2128435997040768") + (str_to_INTERVAL((char*)"-2918793445708416") + (str_to_INTERVAL((char*)"326816444139760") + str_to_INTERVAL((char*)"8549765662968") * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"268676011169280") + (str_to_INTERVAL((char*)"-1809089861842944") + (str_to_INTERVAL((char*)"22101829139834112") + (str_to_INTERVAL((char*)"8176623560464640") + (str_to_INTERVAL((char*)"-5191971085369184") + (str_to_INTERVAL((char*)"30878658003424") + (str_to_INTERVAL((char*)"120315112206144") - str_to_INTERVAL((char*)"2343069932112") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"118152660510720") + (str_to_INTERVAL((char*)"-885014429572608") + (str_to_INTERVAL((char*)"9609996075376896") + (str_to_INTERVAL((char*)"13476129041607168") + (str_to_INTERVAL((char*)"-4202147320939392") + (str_to_INTERVAL((char*)"-997635040413184") + (str_to_INTERVAL((char*)"289288446343296") - str_to_INTERVAL((char*)"1234079493552") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-34358776750080") + (str_to_INTERVAL((char*)"272834806440960") + (str_to_INTERVAL((char*)"224973693055488") + (str_to_INTERVAL((char*)"8449870580952320") + (str_to_INTERVAL((char*)"-698453558885632") + (str_to_INTERVAL((char*)"-1199091664900992") + (str_to_INTERVAL((char*)"189772317035936") + (str_to_INTERVAL((char*)"15336386606352") - str_to_INTERVAL((char*)"411762954240") * b) * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-6278162964480") + (str_to_INTERVAL((char*)"99405299017728") + (str_to_INTERVAL((char*)"-489824052105216") + (str_to_INTERVAL((char*)"1988587430566912") + (str_to_INTERVAL((char*)"459163515602432") + (str_to_INTERVAL((char*)"-440342855517440") + (str_to_INTERVAL((char*)"18184414436864") + (str_to_INTERVAL((char*)"15709120271648") - str_to_INTERVAL((char*)"265531771008") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-71165952000") + (str_to_INTERVAL((char*)"4197726314496") + (str_to_INTERVAL((char*)"-50911993792512") + (str_to_INTERVAL((char*)"165274603634688") + (str_to_INTERVAL((char*)"115334560250368") + (str_to_INTERVAL((char*)"-45508870821120") + (str_to_INTERVAL((char*)"-4103631642368") + (str_to_INTERVAL((char*)"2693962071168") + (str_to_INTERVAL((char*)"93718670976") - str_to_INTERVAL((char*)"3190489344") * b) * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-546487418880") + (str_to_INTERVAL((char*)"4009967902720") + (str_to_INTERVAL((char*)"4353474080768") + (str_to_INTERVAL((char*)"-1116028196864") + (str_to_INTERVAL((char*)"125329802752") + (str_to_INTERVAL((char*)"-40955516416") + (str_to_INTERVAL((char*)"27670593920") - str_to_INTERVAL((char*)"930174048") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-7165476864") + (str_to_INTERVAL((char*)"14264840704") + (str_to_INTERVAL((char*)"-7956849920") + (str_to_INTERVAL((char*)"1785797632") + (str_to_INTERVAL((char*)"-11791872") + str_to_INTERVAL((char*)"1797120") * b) * b) * b) * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[46] =  pow( a,  11) *  pow( b,  9) * (str_to_INTERVAL((char*)"253186457130") *  pow( b,  12) + ((str_to_INTERVAL((char*)"2444269967640") - str_to_INTERVAL((char*)"218612721564") * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"-278104433400") + (str_to_INTERVAL((char*)"8574994073790") + str_to_INTERVAL((char*)"461022474339") * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"-43201321766640") + (str_to_INTERVAL((char*)"56618144017164") + (str_to_INTERVAL((char*)"3195261931614") - str_to_INTERVAL((char*)"739287210537") * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"-43993187335440") + (str_to_INTERVAL((char*)"-65132755755360") + (str_to_INTERVAL((char*)"161656717589000") + (str_to_INTERVAL((char*)"-12121597697280") + str_to_INTERVAL((char*)"771844955364") * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"203471342262720") + (str_to_INTERVAL((char*)"-612213802069968") + (str_to_INTERVAL((char*)"963712084302200") + (str_to_INTERVAL((char*)"-143256761423092") + (str_to_INTERVAL((char*)"5732036928516") - str_to_INTERVAL((char*)"167081924610") * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"172000948711680") + (str_to_INTERVAL((char*)"-71639507640480") + (str_to_INTERVAL((char*)"2480768453907648") + (str_to_INTERVAL((char*)"-296426448225336") + (str_to_INTERVAL((char*)"-34068155967328") + str_to_INTERVAL((char*)"6491481508764") * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-321877180200960") + (str_to_INTERVAL((char*)"1482125287146432") + (str_to_INTERVAL((char*)"6055487056038752") + (str_to_INTERVAL((char*)"22327039468912") + (str_to_INTERVAL((char*)"-586169911309728") + (str_to_INTERVAL((char*)"72970087235040") - str_to_INTERVAL((char*)"1243547254656") * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-124280845032960") + (str_to_INTERVAL((char*)"342521733249792") + (str_to_INTERVAL((char*)"11730587870418688") + (str_to_INTERVAL((char*)"1327292416349056") + (str_to_INTERVAL((char*)"-1890562770615808") + (str_to_INTERVAL((char*)"202997855737152") + str_to_INTERVAL((char*)"5426287512408") * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"143136795479040") + (str_to_INTERVAL((char*)"-894196244143104") + (str_to_INTERVAL((char*)"10461648766433280") + (str_to_INTERVAL((char*)"4452660783106560") + (str_to_INTERVAL((char*)"-2697751928842912") + (str_to_INTERVAL((char*)"15431348788304") + (str_to_INTERVAL((char*)"60274664464992") - str_to_INTERVAL((char*)"1174560951744") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"14433035950080") + (str_to_INTERVAL((char*)"-156371539726848") + (str_to_INTERVAL((char*)"3147221313918720") + (str_to_INTERVAL((char*)"5352833287477248") + (str_to_INTERVAL((char*)"-1636436656947584") + (str_to_INTERVAL((char*)"-381843123674880") + (str_to_INTERVAL((char*)"109633813460096") - str_to_INTERVAL((char*)"456386973984") * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-10694971330560") + (str_to_INTERVAL((char*)"102189329722368") + (str_to_INTERVAL((char*)"-42913720214016") + (str_to_INTERVAL((char*)"2361839806853888") + (str_to_INTERVAL((char*)"-189383274670080") + (str_to_INTERVAL((char*)"-329918863875200") + (str_to_INTERVAL((char*)"51997901070336") + (str_to_INTERVAL((char*)"4187530813344") - str_to_INTERVAL((char*)"112715297280") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-657731543040") + (str_to_INTERVAL((char*)"15367296485376") + (str_to_INTERVAL((char*)"-90546428921856") + (str_to_INTERVAL((char*)"370205426414592") + (str_to_INTERVAL((char*)"81232071229440") + (str_to_INTERVAL((char*)"-78345586138624") + (str_to_INTERVAL((char*)"3787452892928") + (str_to_INTERVAL((char*)"2702439019744") - str_to_INTERVAL((char*)"46064794752") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"189775872000") + (str_to_INTERVAL((char*)"-4309921234944") + (str_to_INTERVAL((char*)"18251584106496") + (str_to_INTERVAL((char*)"10884987892736") + (str_to_INTERVAL((char*)"-4508866624768") + (str_to_INTERVAL((char*)"-81985438976") + (str_to_INTERVAL((char*)"198539026560") + (str_to_INTERVAL((char*)"11098886976") - str_to_INTERVAL((char*)"258217056") * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"158673715200") + (str_to_INTERVAL((char*)"123195211776") + (str_to_INTERVAL((char*)"-55219076096") + (str_to_INTERVAL((char*)"32313339392") + (str_to_INTERVAL((char*)"-10897211136") + (str_to_INTERVAL((char*)"2386058880") - str_to_INTERVAL((char*)"84338784") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"357267456") + (str_to_INTERVAL((char*)"-274014720") + (str_to_INTERVAL((char*)"77548032") + (str_to_INTERVAL((char*)"-732672") + str_to_INTERVAL((char*)"69120") * b) * b) * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[47] =  pow( a,  12) *  pow( b,  9) * (str_to_INTERVAL((char*)"-378990266040") *  pow( b,  12) + ((str_to_INTERVAL((char*)"-2888453921760") + str_to_INTERVAL((char*)"238215642174") * b) *  pow( b,  11) + ((str_to_INTERVAL((char*)"2329096797720") + (str_to_INTERVAL((char*)"-9909223995582") - str_to_INTERVAL((char*)"548388137997") * b) * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"41902446486960") + (str_to_INTERVAL((char*)"-48671597567304") + (str_to_INTERVAL((char*)"-6084089562672") + str_to_INTERVAL((char*)"984293906472") * b) * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"13831823887200") + (str_to_INTERVAL((char*)"85146404298600") + (str_to_INTERVAL((char*)"-168291916060600") + (str_to_INTERVAL((char*)"16413487117500") - str_to_INTERVAL((char*)"918093412935") * b) * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-159852177146880") + (str_to_INTERVAL((char*)"414948655418112") + (str_to_INTERVAL((char*)"-846046065453120") + (str_to_INTERVAL((char*)"142435150503248") + (str_to_INTERVAL((char*)"-6074861222724") + str_to_INTERVAL((char*)"167521214610") * b) * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-38864033936640") + (str_to_INTERVAL((char*)"-135482320252128") + (str_to_INTERVAL((char*)"-2073929870502960") + (str_to_INTERVAL((char*)"291671309035664") + (str_to_INTERVAL((char*)"31859057801944") - str_to_INTERVAL((char*)"5415258810060") * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"185361274644480") + (str_to_INTERVAL((char*)"-764580638736000") + (str_to_INTERVAL((char*)"-4473411368104768") + (str_to_INTERVAL((char*)"107740438824160") + (str_to_INTERVAL((char*)"403311785985200") + (str_to_INTERVAL((char*)"-50276552652120") + str_to_INTERVAL((char*)"852484389840") * b) * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"1430091210240") + (str_to_INTERVAL((char*)"63055258841088") + (str_to_INTERVAL((char*)"-6532607271779328") + (str_to_INTERVAL((char*)"-754292298551488") + (str_to_INTERVAL((char*)"1064312092456128") + (str_to_INTERVAL((char*)"-110602376819880") - str_to_INTERVAL((char*)"2990752884504") * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-50635629411840") + (str_to_INTERVAL((char*)"327142992562176") + (str_to_INTERVAL((char*)"-4210193701492224") + (str_to_INTERVAL((char*)"-2056749638301952") + (str_to_INTERVAL((char*)"1194134185692736") + (str_to_INTERVAL((char*)"-7135204712336") + (str_to_INTERVAL((char*)"-25841231766672") + str_to_INTERVAL((char*)"504285350976") * b) * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"3194473420800") + (str_to_INTERVAL((char*)"-3341293671936") + (str_to_INTERVAL((char*)"-854226231757056") + (str_to_INTERVAL((char*)"-1770010559377920") + (str_to_INTERVAL((char*)"532048550725888") + (str_to_INTERVAL((char*)"121517019718720") + (str_to_INTERVAL((char*)"-34736630090320") + str_to_INTERVAL((char*)"140910082200") * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"2025510727680") + (str_to_INTERVAL((char*)"-25024455687168") + (str_to_INTERVAL((char*)"28078367537664") + (str_to_INTERVAL((char*)"-532370311068416") + (str_to_INTERVAL((char*)"42987397619200") + (str_to_INTERVAL((char*)"71968629824128") + (str_to_INTERVAL((char*)"-11398774875904") + (str_to_INTERVAL((char*)"-910343783328") + str_to_INTERVAL((char*)"24589255680") * b) * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"31629312000") + (str_to_INTERVAL((char*)"-1475201654784") + (str_to_INTERVAL((char*)"11856257703936") + (str_to_INTERVAL((char*)"-52141429381120") + (str_to_INTERVAL((char*)"-10085296096512") + (str_to_INTERVAL((char*)"10100176864896") + (str_to_INTERVAL((char*)"-616125076096") + (str_to_INTERVAL((char*)"-331367516384") + str_to_INTERVAL((char*)"5742735408") * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"175017443328") + (str_to_INTERVAL((char*)"-1286698622976") + (str_to_INTERVAL((char*)"-585494506496") + (str_to_INTERVAL((char*)"271764247296") + (str_to_INTERVAL((char*)"-33445742080") + (str_to_INTERVAL((char*)"-3122870912") + (str_to_INTERVAL((char*)"-1075017408") + str_to_INTERVAL((char*)"9303552") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"895684608") + (str_to_INTERVAL((char*)"-1482421248") + (str_to_INTERVAL((char*)"799867008") + (str_to_INTERVAL((char*)"-180562176") + str_to_INTERVAL((char*)"6249600") * b) * b) * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[48] = 0.-pow( a,  13) *  pow( b,  10) * (str_to_INTERVAL((char*)"-499525027380") *  pow( b,  11) + ((str_to_INTERVAL((char*)"-2967026026140") + str_to_INTERVAL((char*)"218679709716") * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"4286814609960") + (str_to_INTERVAL((char*)"-10105937459046") - str_to_INTERVAL((char*)"609188242149") * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"34540338941280") + (str_to_INTERVAL((char*)"-35753813279628") + (str_to_INTERVAL((char*)"-8794347031542") + str_to_INTERVAL((char*)"1188121211865") * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-7915656480480") + (str_to_INTERVAL((char*)"84509433276216") + (str_to_INTERVAL((char*)"-158632415330736") + (str_to_INTERVAL((char*)"18743671009032") - str_to_INTERVAL((char*)"972558026505") * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-100978104958560") + (str_to_INTERVAL((char*)"233465926399920") + (str_to_INTERVAL((char*)"-681142753876864") + (str_to_INTERVAL((char*)"126568763201068") + (str_to_INTERVAL((char*)"-5609877043176") + str_to_INTERVAL((char*)"149166779850") * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"18411118248960") + (str_to_INTERVAL((char*)"-168555762530784") + (str_to_INTERVAL((char*)"-1557361859294256") + (str_to_INTERVAL((char*)"242812189524000") + (str_to_INTERVAL((char*)"25045564302320") - str_to_INTERVAL((char*)"3989299333716") * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"78573481367040") + (str_to_INTERVAL((char*)"-309312705265152") + (str_to_INTERVAL((char*)"-2840917262784192") + (str_to_INTERVAL((char*)"117262175628816") + (str_to_INTERVAL((char*)"243274474722384") + (str_to_INTERVAL((char*)"-30312954281736") + str_to_INTERVAL((char*)"512083505160") * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-18634225927680") + (str_to_INTERVAL((char*)"102782583724032") + (str_to_INTERVAL((char*)"-3104934311096832") + (str_to_INTERVAL((char*)"-382181867596160") + (str_to_INTERVAL((char*)"517849683542144") + (str_to_INTERVAL((char*)"-52455761800424") - str_to_INTERVAL((char*)"1424064377088") * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-11572873920000") + (str_to_INTERVAL((char*)"86427357651456") + (str_to_INTERVAL((char*)"-1429300377653760") + (str_to_INTERVAL((char*)"-799050302123776") + (str_to_INTERVAL((char*)"446510303149216") + (str_to_INTERVAL((char*)"-2950460218704") + (str_to_INTERVAL((char*)"-9388703678112") + str_to_INTERVAL((char*)"183584664288") * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"2019912837120") + (str_to_INTERVAL((char*)"-12178900511232") + (str_to_INTERVAL((char*)"-189072184577280") + (str_to_INTERVAL((char*)"-479776515292416") + (str_to_INTERVAL((char*)"142265156604544") + (str_to_INTERVAL((char*)"31494323608000") + (str_to_INTERVAL((char*)"-9026770077808") + str_to_INTERVAL((char*)"35526292200") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"223144796160") + (str_to_INTERVAL((char*)"-4052042440704") + (str_to_INTERVAL((char*)"7210242300928") + (str_to_INTERVAL((char*)"-93851927815936") + (str_to_INTERVAL((char*)"8036099967744") + (str_to_INTERVAL((char*)"11938001240576") + (str_to_INTERVAL((char*)"-1926065700384") + (str_to_INTERVAL((char*)"-151549819152") + str_to_INTERVAL((char*)"4113264960") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-66421555200") + (str_to_INTERVAL((char*)"982965215232") + (str_to_INTERVAL((char*)"-5182899697664") + (str_to_INTERVAL((char*)"-761625056512") + (str_to_INTERVAL((char*)"851329274496") + (str_to_INTERVAL((char*)"-73972198656") + (str_to_INTERVAL((char*)"-25336216096") + str_to_INTERVAL((char*)"457647696") * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-43274649600") + (str_to_INTERVAL((char*)"-11494060032") + (str_to_INTERVAL((char*)"7708710912") + (str_to_INTERVAL((char*)"-3583500288") + (str_to_INTERVAL((char*)"667723008") + (str_to_INTERVAL((char*)"-93028608") - str_to_INTERVAL((char*)"82944") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-27482112") + (str_to_INTERVAL((char*)"21965184") + (str_to_INTERVAL((char*)"-6614784") + str_to_INTERVAL((char*)"224640") * b) * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[49] = 0.-pow( a,  14) *  pow( b,  10) * (str_to_INTERVAL((char*)"581784002280") *  pow( b,  11) + ((str_to_INTERVAL((char*)"2637342059850") - str_to_INTERVAL((char*)"163795970709") * b) *  pow( b,  10) + ((str_to_INTERVAL((char*)"-5432398300080") + (str_to_INTERVAL((char*)"9093889050390") + str_to_INTERVAL((char*)"639385619490") * b) * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"-23933187914400") + (str_to_INTERVAL((char*)"22013336320680") + (str_to_INTERVAL((char*)"10558566040560") - str_to_INTERVAL((char*)"1296289056780") * b) * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"17009693705520") + (str_to_INTERVAL((char*)"-67842108224544") + (str_to_INTERVAL((char*)"135555228469904") + (str_to_INTERVAL((char*)"-18512617286388") + str_to_INTERVAL((char*)"918214679970") * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"50285965406400") + (str_to_INTERVAL((char*)"-105565418716608") + (str_to_INTERVAL((char*)"501161133813712") + (str_to_INTERVAL((char*)"-100298418296816") + (str_to_INTERVAL((char*)"4547000570784") - str_to_INTERVAL((char*)"117881750790") * b) * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-26312218225920") + (str_to_INTERVAL((char*)"119891738523264") + (str_to_INTERVAL((char*)"1039540178295984") + (str_to_INTERVAL((char*)"-172996832405672") + (str_to_INTERVAL((char*)"-16880534837740") + str_to_INTERVAL((char*)"2590194364758") * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-23112373109760") + (str_to_INTERVAL((char*)"92525811833856") + (str_to_INTERVAL((char*)"1549440149506176") + (str_to_INTERVAL((char*)"-80099753612688") + (str_to_INTERVAL((char*)"-128154041263664") + (str_to_INTERVAL((char*)"15936259811016") - str_to_INTERVAL((char*)"268390666368") * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"10514845543680") + (str_to_INTERVAL((char*)"-54622270886400") + (str_to_INTERVAL((char*)"1253800879140864") + (str_to_INTERVAL((char*)"168959396783968") + (str_to_INTERVAL((char*)"-216116666578512") + (str_to_INTERVAL((char*)"21457507196688") + str_to_INTERVAL((char*)"581554770444") * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"1282157683200") + (str_to_INTERVAL((char*)"-14746339665408") + (str_to_INTERVAL((char*)"404730520798464") + (str_to_INTERVAL((char*)"258015947199616") + (str_to_INTERVAL((char*)"-139341478988000") + (str_to_INTERVAL((char*)"1060433012720") + (str_to_INTERVAL((char*)"2851868343024") - str_to_INTERVAL((char*)"55907804160") * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-471726858240") + (str_to_INTERVAL((char*)"4127566454784") + (str_to_INTERVAL((char*)"33358506798848") + (str_to_INTERVAL((char*)"104255687522432") + (str_to_INTERVAL((char*)"-30615515478976") + (str_to_INTERVAL((char*)"-6448584409376") + (str_to_INTERVAL((char*)"1872570696480") - str_to_INTERVAL((char*)"7080516720") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-11070259200") + (str_to_INTERVAL((char*)"398200578048") + (str_to_INTERVAL((char*)"-1104389113856") + (str_to_INTERVAL((char*)"12348304228096") + (str_to_INTERVAL((char*)"-1197832875264") + (str_to_INTERVAL((char*)"-1404012915456") + (str_to_INTERVAL((char*)"236624002720") + (str_to_INTERVAL((char*)"18135685104") - str_to_INTERVAL((char*)"495832896") * b) * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-38866673664") + (str_to_INTERVAL((char*)"319935762432") + (str_to_INTERVAL((char*)"22020658688") + (str_to_INTERVAL((char*)"-37455169024") + (str_to_INTERVAL((char*)"5917373504") + (str_to_INTERVAL((char*)"806315072") - str_to_INTERVAL((char*)"17432640") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-68898816") + (str_to_INTERVAL((char*)"86660352") + (str_to_INTERVAL((char*)"-38571264") + str_to_INTERVAL((char*)"6405120") * b) * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[50] =  pow( a,  15) *  pow( b,  11) * (str_to_INTERVAL((char*)"600059304900") *  pow( b,  10) + ((str_to_INTERVAL((char*)"2010285907740") - str_to_INTERVAL((char*)"91968449472") * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"-5422697137440") + (str_to_INTERVAL((char*)"7209702679590") + str_to_INTERVAL((char*)"636514621002") * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-13644074957040") + (str_to_INTERVAL((char*)"10916182751100") + (str_to_INTERVAL((char*)"10905381435414") - str_to_INTERVAL((char*)"1274878847265") * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"16014540743280") + (str_to_INTERVAL((char*)"-45025419887280") + (str_to_INTERVAL((char*)"105004364261704") + (str_to_INTERVAL((char*)"-16012985607636") + str_to_INTERVAL((char*)"772592502330") * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"18668034178560") + (str_to_INTERVAL((char*)"-35598475386288") + (str_to_INTERVAL((char*)"334992404530568") + (str_to_INTERVAL((char*)"-70673291645844") + (str_to_INTERVAL((char*)"3244555657056") - str_to_INTERVAL((char*)"82556617428") * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-16387248049920") + (str_to_INTERVAL((char*)"61872135598848") + (str_to_INTERVAL((char*)"611224877357136") + (str_to_INTERVAL((char*)"-105974522663000") + (str_to_INTERVAL((char*)"-9838403020364") + str_to_INTERVAL((char*)"1477696347090") * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-3606663214080") + (str_to_INTERVAL((char*)"16945227670272") + (str_to_INTERVAL((char*)"723397050055104") + (str_to_INTERVAL((char*)"-41675904416064") + (str_to_INTERVAL((char*)"-58622753822312") + (str_to_INTERVAL((char*)"7265600102268") - str_to_INTERVAL((char*)"122018961168") * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"3286945785600") + (str_to_INTERVAL((char*)"-18541074435840") + (str_to_INTERVAL((char*)"426877360790784") + (str_to_INTERVAL((char*)"63871149944992") + (str_to_INTERVAL((char*)"-76577599755088") + (str_to_INTERVAL((char*)"7484851498032") + str_to_INTERVAL((char*)"201673978164") * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-122779584000") + (str_to_INTERVAL((char*)"-863074711296") + (str_to_INTERVAL((char*)"94063340391552") + (str_to_INTERVAL((char*)"68104146179776") + (str_to_INTERVAL((char*)"-35676222225536") + (str_to_INTERVAL((char*)"323627962768") + (str_to_INTERVAL((char*)"710729457696") - str_to_INTERVAL((char*)"13979890080") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-57464709120") + (str_to_INTERVAL((char*)"763453427712") + (str_to_INTERVAL((char*)"4535367228160") + (str_to_INTERVAL((char*)"17573411994496") + (str_to_INTERVAL((char*)"-5136443321280") + (str_to_INTERVAL((char*)"-994315633952") + (str_to_INTERVAL((char*)"297870553504") - str_to_INTERVAL((char*)"1059380640") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"18114969600") + (str_to_INTERVAL((char*)"-98899347456") + (str_to_INTERVAL((char*)"1122518068224") + (str_to_INTERVAL((char*)"-133826723712") + (str_to_INTERVAL((char*)"-101792789952") + (str_to_INTERVAL((char*)"19068371712") + (str_to_INTERVAL((char*)"1386344736") - str_to_INTERVAL((char*)"38417184") * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"9015552000") + (str_to_INTERVAL((char*)"-597467136") + (str_to_INTERVAL((char*)"-358243584") + (str_to_INTERVAL((char*)"250424256") + (str_to_INTERVAL((char*)"-17550912") + str_to_INTERVAL((char*)"11520") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"981504") + (str_to_INTERVAL((char*)"-746496") + str_to_INTERVAL((char*)"207360") * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[51] =  pow( a,  16) *  pow( b,  11) * (str_to_INTERVAL((char*)"-548647136910") *  pow( b,  10) + ((str_to_INTERVAL((char*)"-1291445472060") + str_to_INTERVAL((char*)"26482337802") * b) *  pow( b,  9) + ((str_to_INTERVAL((char*)"4448799757800") + (str_to_INTERVAL((char*)"-5021957085930") - str_to_INTERVAL((char*)"598277060325") * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"6116167992960") + (str_to_INTERVAL((char*)"-3929105789040") + (str_to_INTERVAL((char*)"-9856499054496") + str_to_INTERVAL((char*)"1127466292110") * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-10588266371520") + (str_to_INTERVAL((char*)"24838809679416") + (str_to_INTERVAL((char*)"-73624255586808") + (str_to_INTERVAL((char*)"12205638449400") - str_to_INTERVAL((char*)"578831712435") * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-4305152950560") + (str_to_INTERVAL((char*)"6838065324432") + (str_to_INTERVAL((char*)"-201867595716696") + (str_to_INTERVAL((char*)"44116279394772") + (str_to_INTERVAL((char*)"-2038398199704") + str_to_INTERVAL((char*)"51113825136") * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"6751364083200") + (str_to_INTERVAL((char*)"-24270313073280") + (str_to_INTERVAL((char*)"-313943234551728") + (str_to_INTERVAL((char*)"55839759113024") + (str_to_INTERVAL((char*)"4970887997420") - str_to_INTERVAL((char*)"737460688878") * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-396390620160") + (str_to_INTERVAL((char*)"216934046592") + (str_to_INTERVAL((char*)"-287490251936448") + (str_to_INTERVAL((char*)"17373141421152") + (str_to_INTERVAL((char*)"23103070968872") + (str_to_INTERVAL((char*)"-2850817932540") + str_to_INTERVAL((char*)"47744362944") * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-615105273600") + (str_to_INTERVAL((char*)"4282156789632") + (str_to_INTERVAL((char*)"-121153435681408") + (str_to_INTERVAL((char*)"-20237943999104") + (str_to_INTERVAL((char*)"22725904331968") + (str_to_INTERVAL((char*)"-2194363893256") - str_to_INTERVAL((char*)"58592020680") * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"70447587840") + (str_to_INTERVAL((char*)"-293906427648") + (str_to_INTERVAL((char*)"-17524183360128") + (str_to_INTERVAL((char*)"-14349216481472") + (str_to_INTERVAL((char*)"7313435722688") + (str_to_INTERVAL((char*)"-81982216848") + (str_to_INTERVAL((char*)"-141467003568") + str_to_INTERVAL((char*)"2795730912") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"3019161600") + (str_to_INTERVAL((char*)"-80146999296") + (str_to_INTERVAL((char*)"-449512659456") + (str_to_INTERVAL((char*)"-2183084045568") + (str_to_INTERVAL((char*)"639705180352") + (str_to_INTERVAL((char*)"105953727584") + (str_to_INTERVAL((char*)"-34036064096") + str_to_INTERVAL((char*)"108140616") * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"4033935360") + (str_to_INTERVAL((char*)"-60986870784") + (str_to_INTERVAL((char*)"9931095424") + (str_to_INTERVAL((char*)"2955955136") + (str_to_INTERVAL((char*)"-804358528") + (str_to_INTERVAL((char*)"-50328384") + str_to_INTERVAL((char*)"1449216") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"2460672") + (str_to_INTERVAL((char*)"-2018304") + str_to_INTERVAL((char*)"552960") * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[52] = 0.-pow( a,  17) *  pow( b,  12) * (str_to_INTERVAL((char*)"-444650879430") *  pow( b,  9) + ((str_to_INTERVAL((char*)"-674839062450") - str_to_INTERVAL((char*)"16397308773") * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"3046396511880") + (str_to_INTERVAL((char*)"-3060471951918") - str_to_INTERVAL((char*)"525485424897") * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"1904899397040") + (str_to_INTERVAL((char*)"-603028119876") + (str_to_INTERVAL((char*)"-7866360938262") + str_to_INTERVAL((char*)"894516416607") * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-5319651870720") + (str_to_INTERVAL((char*)"11336759896296") + (str_to_INTERVAL((char*)"-46587590938080") + (str_to_INTERVAL((char*)"8218411264104") - str_to_INTERVAL((char*)"385467069231") * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"41844448800") + (str_to_INTERVAL((char*)"-967484722032") + (str_to_INTERVAL((char*)"-108704423595800") + (str_to_INTERVAL((char*)"24283964798856") + (str_to_INTERVAL((char*)"-1125126698736") + str_to_INTERVAL((char*)"27879353784") * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"1909178519040") + (str_to_INTERVAL((char*)"-7189136561280") + (str_to_INTERVAL((char*)"-139691160681744") + (str_to_INTERVAL((char*)"25239711947984") + (str_to_INTERVAL((char*)"2173582665964") - str_to_INTERVAL((char*)"320026677282") * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-409383244800") + (str_to_INTERVAL((char*)"1368020233152") + (str_to_INTERVAL((char*)"-96440071418464") + (str_to_INTERVAL((char*)"5889033284128") + (str_to_INTERVAL((char*)"7761490945584") + (str_to_INTERVAL((char*)"-952783285944") + str_to_INTERVAL((char*)"15912250956") * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-52202707200") + (str_to_INTERVAL((char*)"628253232768") + (str_to_INTERVAL((char*)"-28193800490240") + (str_to_INTERVAL((char*)"-5257232607552") + (str_to_INTERVAL((char*)"5544679504704") + (str_to_INTERVAL((char*)"-530447060424") - str_to_INTERVAL((char*)"13998216960") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"10768343040") + (str_to_INTERVAL((char*)"-90989950464") + (str_to_INTERVAL((char*)"-2524202765184") + (str_to_INTERVAL((char*)"-2329674867712") + (str_to_INTERVAL((char*)"1157679499392") + (str_to_INTERVAL((char*)"-16768381664") + (str_to_INTERVAL((char*)"-21600958272") + str_to_INTERVAL((char*)"429983280") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-3773952000") + (str_to_INTERVAL((char*)"-29189035008") + (str_to_INTERVAL((char*)"-183243267840") + (str_to_INTERVAL((char*)"54498423616") + (str_to_INTERVAL((char*)"6377466656") + (str_to_INTERVAL((char*)"-2470043744") + str_to_INTERVAL((char*)"5887512") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1387008000") + (str_to_INTERVAL((char*)"364787712") + (str_to_INTERVAL((char*)"-62279424") + (str_to_INTERVAL((char*)"-7115328") + (str_to_INTERVAL((char*)"74880") + 1728 * b) * b) * b) * b) * b) * b * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[53] = 0.-pow( a,  18) *  pow( b,  12) * (str_to_INTERVAL((char*)"319054297545") *  pow( b,  9) + ((str_to_INTERVAL((char*)"262138785300") + str_to_INTERVAL((char*)"33419524230") * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-1746321129720") + (str_to_INTERVAL((char*)"1622220602742") + str_to_INTERVAL((char*)"426119123223") * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-189725405760") + (str_to_INTERVAL((char*)"-430232859492") + (str_to_INTERVAL((char*)"5569213029990") - str_to_INTERVAL((char*)"635104978221") * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"2047177875600") + (str_to_INTERVAL((char*)"-4219791516264") + (str_to_INTERVAL((char*)"26486547775752") + (str_to_INTERVAL((char*)"-4887309786762") + str_to_INTERVAL((char*)"227550311397") * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-546075367200") + (str_to_INTERVAL((char*)"1467245122704") + (str_to_INTERVAL((char*)"51813125604600") + (str_to_INTERVAL((char*)"-11720475433672") + (str_to_INTERVAL((char*)"543402189744") - str_to_INTERVAL((char*)"13332267216") * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-335150922240") + (str_to_INTERVAL((char*)"1524947620896") + (str_to_INTERVAL((char*)"53353777661184") + (str_to_INTERVAL((char*)"-9732057652440") + (str_to_INTERVAL((char*)"-818190191500") + str_to_INTERVAL((char*)"119800120374") * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"116677808640") + (str_to_INTERVAL((char*)"-505796910144") + (str_to_INTERVAL((char*)"26980493463584") + (str_to_INTERVAL((char*)"-1622377612256") + (str_to_INTERVAL((char*)"-2191535030384") + (str_to_INTERVAL((char*)"267488265624") - str_to_INTERVAL((char*)"4454598096") * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-3750727680") + (str_to_INTERVAL((char*)"-39766349952") + (str_to_INTERVAL((char*)"5250826512768") + (str_to_INTERVAL((char*)"1089013472256") + (str_to_INTERVAL((char*)"-1083480377136") + (str_to_INTERVAL((char*)"102974771280") + str_to_INTERVAL((char*)"2678556420") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-628992000") + (str_to_INTERVAL((char*)"11366433792") + (str_to_INTERVAL((char*)"264789254784") + (str_to_INTERVAL((char*)"275679967872") + (str_to_INTERVAL((char*)"-133633032000") + (str_to_INTERVAL((char*)"2661648288") + (str_to_INTERVAL((char*)"2369111568") - str_to_INTERVAL((char*)"47755008") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"942197760") + (str_to_INTERVAL((char*)"8751932928") + (str_to_INTERVAL((char*)"-2714820352") + (str_to_INTERVAL((char*)"-58185152") + (str_to_INTERVAL((char*)"82546112") + str_to_INTERVAL((char*)"21312") * b) * b) * b) * b) * b) * b * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[54] =  pow( a,  19) *  pow( b,  13) * (str_to_INTERVAL((char*)"202227145965") *  pow( b,  8) + ((str_to_INTERVAL((char*)"50623211520") + str_to_INTERVAL((char*)"31815205434") * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-833180265720") + (str_to_INTERVAL((char*)"741965903118") + str_to_INTERVAL((char*)"315521570661") * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"215763143040") + (str_to_INTERVAL((char*)"-462931575972") + (str_to_INTERVAL((char*)"3503560747170") - str_to_INTERVAL((char*)"402377625321") * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"582541794000") + (str_to_INTERVAL((char*)"-1244603357976") + (str_to_INTERVAL((char*)"13450186738704") + (str_to_INTERVAL((char*)"-2561017094358") + str_to_INTERVAL((char*)"118625017923") * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-266469428160") + (str_to_INTERVAL((char*)"690320849328") + (str_to_INTERVAL((char*)"21637688403416") + (str_to_INTERVAL((char*)"-4925196764276") + (str_to_INTERVAL((char*)"228267262188") - str_to_INTERVAL((char*)"5554535814") * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-17603032320") + (str_to_INTERVAL((char*)"193864339488") + (str_to_INTERVAL((char*)"17301431059680") + (str_to_INTERVAL((char*)"-3173578439208") + (str_to_INTERVAL((char*)"-262777519756") + str_to_INTERVAL((char*)"38278981494") * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"17023841280") + (str_to_INTERVAL((char*)"-103629248064") + (str_to_INTERVAL((char*)"6188163524832") + (str_to_INTERVAL((char*)"-359400663216") + (str_to_INTERVAL((char*)"-510206498232") + (str_to_INTERVAL((char*)"61895484276") - str_to_INTERVAL((char*)"1027818216") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1336366080") + (str_to_INTERVAL((char*)"3963860352") + (str_to_INTERVAL((char*)"754181646336") + (str_to_INTERVAL((char*)"173047020800") + (str_to_INTERVAL((char*)"-163149420624") + (str_to_INTERVAL((char*)"15448894240") + str_to_INTERVAL((char*)"394729596") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"580608000") + (str_to_INTERVAL((char*)"18062447616") + (str_to_INTERVAL((char*)"21567466752") + (str_to_INTERVAL((char*)"-10166009408") + (str_to_INTERVAL((char*)"307684384") + (str_to_INTERVAL((char*)"164715072") - str_to_INTERVAL((char*)"3409344") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"148608000") + (str_to_INTERVAL((char*)"-53007360") + (str_to_INTERVAL((char*)"12104448") + (str_to_INTERVAL((char*)"-397824") + str_to_INTERVAL((char*)"16128") * b) * b) * b) * b) * b * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[55] =  pow( a,  20) *  pow( b,  13) * (str_to_INTERVAL((char*)"-112831974000") *  pow( b,  8) + ((str_to_INTERVAL((char*)"23500235970") - str_to_INTERVAL((char*)"22136006196") * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"325852619040") + (str_to_INTERVAL((char*)"-289712425842") - str_to_INTERVAL((char*)"211434824070") * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-172628258400") + (str_to_INTERVAL((char*)"259652760924") + (str_to_INTERVAL((char*)"-1957327053378") + str_to_INTERVAL((char*)"226697453607") * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-108604044000") + (str_to_INTERVAL((char*)"273810530952") + (str_to_INTERVAL((char*)"-6056231759864") + (str_to_INTERVAL((char*)"1177606722180") - str_to_INTERVAL((char*)"54336418947") * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"73901816640") + (str_to_INTERVAL((char*)"-208390551984") + (str_to_INTERVAL((char*)"-7828790657752") + (str_to_INTERVAL((char*)"1786284078820") + (str_to_INTERVAL((char*)"-82725506880") + str_to_INTERVAL((char*)"1999476648") * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-7618752000") + (str_to_INTERVAL((char*)"-629161056") + (str_to_INTERVAL((char*)"-4697713931856") + (str_to_INTERVAL((char*)"864403211024") + (str_to_INTERVAL((char*)"71064376148") - str_to_INTERVAL((char*)"10293844770") * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-898836480") + (str_to_INTERVAL((char*)"12255955776") + (str_to_INTERVAL((char*)"-1135021844448") + (str_to_INTERVAL((char*)"62714130576") + (str_to_INTERVAL((char*)"95339206968") + (str_to_INTERVAL((char*)"-11493874644") + str_to_INTERVAL((char*)"190315440") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"96768000") + (str_to_INTERVAL((char*)"-999857664") + (str_to_INTERVAL((char*)"-78605462016") + (str_to_INTERVAL((char*)"-19860650880") + (str_to_INTERVAL((char*)"17797580160") + (str_to_INTERVAL((char*)"-1685934048") - str_to_INTERVAL((char*)"42061248") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-603279360") + (str_to_INTERVAL((char*)"-906304512") + (str_to_INTERVAL((char*)"408760960") + (str_to_INTERVAL((char*)"-23041664") + (str_to_INTERVAL((char*)"-5207808") + str_to_INTERVAL((char*)"117504") * b) * b) * b) * b) * b) * b * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[56] = 0.-pow( a,  21) *  pow( b,  14) * (str_to_INTERVAL((char*)"-55149360300") *  pow( b,  7) + ((str_to_INTERVAL((char*)"31108314060") - str_to_INTERVAL((char*)"12276953433") * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"101332743120") + (str_to_INTERVAL((char*)"-95175906822") - str_to_INTERVAL((char*)"127319502114") * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-75689359200") + (str_to_INTERVAL((char*)"104522182164") + (str_to_INTERVAL((char*)"-968571810282") + str_to_INTERVAL((char*)"113086042293") * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-6397621920") + (str_to_INTERVAL((char*)"37834252824") + (str_to_INTERVAL((char*)"-2396765329840") + (str_to_INTERVAL((char*)"472265670912") - str_to_INTERVAL((char*)"21725425989") * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"12432748320") + (str_to_INTERVAL((char*)"-43416896832") + (str_to_INTERVAL((char*)"-2422757367336") + (str_to_INTERVAL((char*)"552976640616") + (str_to_INTERVAL((char*)"-25588778628") + str_to_INTERVAL((char*)"615194604") * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-2134010880") + (str_to_INTERVAL((char*)"5337061344") + (str_to_INTERVAL((char*)"-1048422702000") + (str_to_INTERVAL((char*)"193219435504") + (str_to_INTERVAL((char*)"15882089692") - str_to_INTERVAL((char*)"2285615430") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"79695360") + (str_to_INTERVAL((char*)"585944064") + (str_to_INTERVAL((char*)"-160336817280") + (str_to_INTERVAL((char*)"8329724256") + (str_to_INTERVAL((char*)"13744405152") + (str_to_INTERVAL((char*)"-1646596896") + str_to_INTERVAL((char*)"27186300") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-62208000") + (str_to_INTERVAL((char*)"-5302632960") + (str_to_INTERVAL((char*)"-1478675328") + (str_to_INTERVAL((char*)"1256645184") + (str_to_INTERVAL((char*)"-120068544") - str_to_INTERVAL((char*)"2884608") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-9907200") + (str_to_INTERVAL((char*)"3679680") + (str_to_INTERVAL((char*)"-839040") + str_to_INTERVAL((char*)"31680") * b) * b) * b) * b * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[57] = 0.-pow( a,  22) *  pow( b,  14) * (str_to_INTERVAL((char*)"23461845750") *  pow( b,  7) + ((str_to_INTERVAL((char*)"-19365411960") + str_to_INTERVAL((char*)"5576564853") * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-23453906400") + (str_to_INTERVAL((char*)"25770313350") + str_to_INTERVAL((char*)"68482739037") * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"22850186400") + (str_to_INTERVAL((char*)"-32125747836") + (str_to_INTERVAL((char*)"422625981786") - str_to_INTERVAL((char*)"49677980493") * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-3251346480") + (str_to_INTERVAL((char*)"-397473912") + (str_to_INTERVAL((char*)"824958370320") + (str_to_INTERVAL((char*)"-163829178120") + str_to_INTERVAL((char*)"7518391587") * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-972583200") + (str_to_INTERVAL((char*)"5982798240") + (str_to_INTERVAL((char*)"631395783768") + (str_to_INTERVAL((char*)"-144027687792") + (str_to_INTERVAL((char*)"6660558648") - str_to_INTERVAL((char*)"159491304") * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"220872960") + (str_to_INTERVAL((char*)"-1083204864") + (str_to_INTERVAL((char*)"187400557824") + (str_to_INTERVAL((char*)"-34555722080") + (str_to_INTERVAL((char*)"-2855755424") + str_to_INTERVAL((char*)"407932200") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-10368000") + (str_to_INTERVAL((char*)"32924160") + (str_to_INTERVAL((char*)"16403750592") + (str_to_INTERVAL((char*)"-794608032") + (str_to_INTERVAL((char*)"-1434768384") + (str_to_INTERVAL((char*)"170818944") - str_to_INTERVAL((char*)"2812320") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"174173184") + (str_to_INTERVAL((char*)"55903776") + (str_to_INTERVAL((char*)"-43896160") + (str_to_INTERVAL((char*)"4351936") + str_to_INTERVAL((char*)"95616") * b) * b) * b) * b) * b * a) * a) * a) * a) * a) * a) * a) * a);
coef[58] =  pow( a,  23) *  pow( b,  15) * (str_to_INTERVAL((char*)"8614475280") *  pow( b,  6) + ((str_to_INTERVAL((char*)"-8610736590") + str_to_INTERVAL((char*)"2087719416") * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-3296304720") + (str_to_INTERVAL((char*)"5575494222") + str_to_INTERVAL((char*)"32715608895") * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"4825718640") + (str_to_INTERVAL((char*)"-7600217364") + (str_to_INTERVAL((char*)"161550559182") - str_to_INTERVAL((char*)"19086845865") * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-1180234800") + (str_to_INTERVAL((char*)"1274453064") + (str_to_INTERVAL((char*)"243840261960") + (str_to_INTERVAL((char*)"-48628514304") + str_to_INTERVAL((char*)"2227359933") * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"53472960") + (str_to_INTERVAL((char*)"442761120") + (str_to_INTERVAL((char*)"135836439840") + (str_to_INTERVAL((char*)"-30961530672") + (str_to_INTERVAL((char*)"1431229596") - str_to_INTERVAL((char*)"34176438") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"3801600") + (str_to_INTERVAL((char*)"-96580800") + (str_to_INTERVAL((char*)"25814438400") + (str_to_INTERVAL((char*)"-4759093504") + (str_to_INTERVAL((char*)"-397017472") + str_to_INTERVAL((char*)"56251440") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"4147200") + (str_to_INTERVAL((char*)"1083500352") + (str_to_INTERVAL((char*)"-48729408") + (str_to_INTERVAL((char*)"-96508704") + (str_to_INTERVAL((char*)"11419968") - str_to_INTERVAL((char*)"187488") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"309600") + (str_to_INTERVAL((char*)"-100800") + str_to_INTERVAL((char*)"21600") * b) * b) * b * a) * a) * a) * a) * a) * a) * a) * a);
coef[59] =  pow( a,  24) *  pow( b,  15) * (str_to_INTERVAL((char*)"-2699917515") *  pow( b,  6) + ((str_to_INTERVAL((char*)"2951614890") - str_to_INTERVAL((char*)"641114334") * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-65153160") + (str_to_INTERVAL((char*)"-914842242") - str_to_INTERVAL((char*)"13795747767") * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-654888240") + (str_to_INTERVAL((char*)"1360113876") + (str_to_INTERVAL((char*)"-53627834286") + str_to_INTERVAL((char*)"6358249815") * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"189928800") + (str_to_INTERVAL((char*)"-337220640") + (str_to_INTERVAL((char*)"-60925444416") + (str_to_INTERVAL((char*)"12174294408") - str_to_INTERVAL((char*)"556804782") * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-18014400") + (str_to_INTERVAL((char*)"3283200") + (str_to_INTERVAL((char*)"-23479185600") + (str_to_INTERVAL((char*)"5348389536") + (str_to_INTERVAL((char*)"-247190520") + str_to_INTERVAL((char*)"5892480") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"691200") + (str_to_INTERVAL((char*)"2430720") + (str_to_INTERVAL((char*)"-2575043760") + (str_to_INTERVAL((char*)"474356176") + (str_to_INTERVAL((char*)"40039456") - str_to_INTERVAL((char*)"5624640") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-34745472") + (str_to_INTERVAL((char*)"1451808") + (str_to_INTERVAL((char*)"3140544") + (str_to_INTERVAL((char*)"-369408") + 6048 * b) * b) * b) * b) * b * a) * a) * a) * a) * a) * a) * a);
coef[60] =  pow( a,  25) *  pow( b,  16) * (str_to_INTERVAL((char*)"711892125") *  pow( b,  5) + ((str_to_INTERVAL((char*)"-792070650") + str_to_INTERVAL((char*)"159324732") * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"174816360") + (str_to_INTERVAL((char*)"102070998") + str_to_INTERVAL((char*)"5098143927") * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"37670400") + (str_to_INTERVAL((char*)"-176816592") + (str_to_INTERVAL((char*)"15283963716") - str_to_INTERVAL((char*)"1816022820") * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-13932000") + (str_to_INTERVAL((char*)"46303200") + (str_to_INTERVAL((char*)"12608848320") + (str_to_INTERVAL((char*)"-2521426872") + str_to_INTERVAL((char*)"115198488") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1015200") + (str_to_INTERVAL((char*)"-3546000") + (str_to_INTERVAL((char*)"3134364000") + (str_to_INTERVAL((char*)"-713751472") + (str_to_INTERVAL((char*)"32987988") - str_to_INTERVAL((char*)"785664") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"129600") + (str_to_INTERVAL((char*)"165714480") + (str_to_INTERVAL((char*)"-30487472") + (str_to_INTERVAL((char*)"-2605472") + str_to_INTERVAL((char*)"362880") * b) * b) * b) * b) * a) * a) * a) * a) * a) * a);
coef[61] = 0.-pow( a,  26) *  pow( b,  16) * (str_to_INTERVAL((char*)"154854585") *  pow( b,  5) + ((str_to_INTERVAL((char*)"-164734290") + str_to_INTERVAL((char*)"31305132") * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"49114080") + (str_to_INTERVAL((char*)"5239152") + str_to_INTERVAL((char*)"1636354368") * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-3996000") + (str_to_INTERVAL((char*)"-15494400") + (str_to_INTERVAL((char*)"3684347040") - str_to_INTERVAL((char*)"438282060") * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"54000") + (str_to_INTERVAL((char*)"3506400") + (str_to_INTERVAL((char*)"2102514120") + (str_to_INTERVAL((char*)"-420520422") + str_to_INTERVAL((char*)"19199664") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-21600") + (str_to_INTERVAL((char*)"-212400") + (str_to_INTERVAL((char*)"303410880") + (str_to_INTERVAL((char*)"-69086496") + (str_to_INTERVAL((char*)"3193344") - str_to_INTERVAL((char*)"76032") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"5171040") + (str_to_INTERVAL((char*)"-949696") + (str_to_INTERVAL((char*)"-82096") + str_to_INTERVAL((char*)"11340") * b) * b) * b) * b * a) * a) * a) * a) * a) * a);
coef[62] = -3 *  pow( a,  27) *  pow( b,  17) * (str_to_INTERVAL((char*)"-9014175") *  pow( b,  4) + ((str_to_INTERVAL((char*)"8587260") - str_to_INTERVAL((char*)"1561854") * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-2538000") + (str_to_INTERVAL((char*)"129060") - str_to_INTERVAL((char*)"150346008") * b) * b) * b * b + ((str_to_INTERVAL((char*)"306000") + (str_to_INTERVAL((char*)"263400") + (str_to_INTERVAL((char*)"-245479192") + str_to_INTERVAL((char*)"29217210") * b) * b) * b) * b + (str_to_INTERVAL((char*)"-18000") + (str_to_INTERVAL((char*)"-38400") + (str_to_INTERVAL((char*)"-90464280") + (str_to_INTERVAL((char*)"18093366") - str_to_INTERVAL((char*)"825792") * b) * b) * b) * b + (str_to_INTERVAL((char*)"-6320160") + (str_to_INTERVAL((char*)"1439152") + (str_to_INTERVAL((char*)"-66528") + 1584 * b) * b) * b) * b * a) * a) * a) * a) * a);
coef[63] =  pow( a,  28) *  pow( b,  18) * (str_to_INTERVAL((char*)"-3642570") *  pow( b,  3) + ((str_to_INTERVAL((char*)"2859300") - str_to_INTERVAL((char*)"502200") * b) * b * b + ((str_to_INTERVAL((char*)"-675000") + (str_to_INTERVAL((char*)"84600") - str_to_INTERVAL((char*)"105200043") * b) * b) * b + (str_to_INTERVAL((char*)"54000") + (str_to_INTERVAL((char*)"16200") + (str_to_INTERVAL((char*)"-118756980") + str_to_INTERVAL((char*)"14137200") * b) * b) * b + ((str_to_INTERVAL((char*)"-25446960") + (str_to_INTERVAL((char*)"5089392") - str_to_INTERVAL((char*)"232254") * b) * b) * b + (str_to_INTERVAL((char*)"-574560") + (str_to_INTERVAL((char*)"130832") + (-6048 + 144 * b) * b) * b) * a) * a) * a) * a) * a);
coef[64] = 9 *  pow( a,  29) *  pow( b,  19) * (str_to_INTERVAL((char*)"39450") * b * b + ((3815 * b - str_to_INTERVAL((char*)"22350")) * b + (3000 + (-500 + str_to_INTERVAL((char*)"2262027") * b) * b + ((str_to_INTERVAL((char*)"1649340") - str_to_INTERVAL((char*)"196350") * b) * b + (str_to_INTERVAL((char*)"171360") + (str_to_INTERVAL((char*)"-34272") + 1564 * b) * b) * a) * a) * a) * a);
coef[65] = -9 *  pow( a,  30) *  pow( b,  19) * (2475 * b * b + ((-750 + 125 * b) * b + (str_to_INTERVAL((char*)"353430") * b * b + ((str_to_INTERVAL((char*)"149940") - str_to_INTERVAL((char*)"17850") * b) * b + (5040 + (-1008 + 46 * b) * b) * a) * a) * a) * a);
coef[66] = -135 *  pow( a,  31) *  pow( b,  20) * (-5 * b + (-2856 * b + (-588 + 70 * b) * a) * a * a);
coef[67] = 54 *  pow( a,  34) *  pow( b,  20) * (-630 * b + (5 * b - 42) * a);
coef[68] = 1944 *  pow( a,  35) *  pow( b,  21);
coef[69] = -54 *  pow( a,  36) *  pow( b,  21);
	
	return;
}
