/*	File: det_with_coef_mpfi.cc

	This code checks that the Wronskian W[l0, l1] does not vanish in the interval I_3, see Lemma
	4.12. 

	Uses: Interval.hpp, involution.cc

	Author: Jordi-Lluis Figueras, Jordi Villadelprat, Warwick Tucker
	Latest edit: Thu June 21 10:25:00 CET 2012
*/



#include<stdio.h>
#include<time.h>
#include<math.h>
#include<stdlib.h>
#include "Interval.hpp"
#include "involution.cc"

#define DIGITSPRINT 40

int det_near0(const INTERVAL& x, INTERVAL& res);
void polynomial_determinantnear0(INTERVAL *coef, const INTERVAL& a, const INTERVAL& b);

INTERVAL *coef;

int main(int argc, char* argv[])
{
	if(argc < 2)
	{
		printf("Usage:\n\t[1]Precision of the rigorous computations\n\n");
		printf("Execution:\n./det_mpfi_near0 256 \n\n");
		exit(1);
	}
	
	int flagevaluation, flagfirstevaluation = 0;
	int prec = atoi(argv[1]), i;
	mpfr_t xmin, xmax, x, deltax, tolstep, aux;

	mpfr_set_default_prec(prec);

	INTERVAL X, Xinit, Xaux, Xaux2, PX, GoodXleft, GoodXright, z, CorrectX, CorrectPX;

	mpfr_init(xmin);
	mpfr_init(xmax);
	mpfr_init(x);
	mpfr_init(deltax);
	mpfr_init(tolstep);
	mpfr_init(aux);

	mpfr_set_str(tolstep, "1.e-75", 10, GMP_RNDN);
	mpfr_set_str(xmin, "-1.e-1", 10, GMP_RNDN);
	mpfr_set_str(xmax, "0", 10, GMP_RNDN);

	mpfr_sub(deltax, xmin, xmax, GMP_RNDN);
	mpfr_set_str(aux, "-1.", 10, GMP_RNDN);
	mpfr_div(deltax, deltax, aux, GMP_RNDN);

	mpfr_sub(x, xmax, deltax, GMP_RNDN);
	X.setfull(x, xmax);

	Xinit = X;

	coef = new INTERVAL[40];
	
	i = 0;
	do
	{
		flagevaluation = det_near0(X, PX);
		i++;
		if(flagevaluation > 0 && mpfi_has_zero(PX.x) > 0)
		{
			flagevaluation = -1;
		}
		if(flagevaluation <= 0)
		{
			mpfi_bisect(Xaux.x, Xaux2.x, X.x); 

			X = Xaux2;
			mpfr_div_d(deltax, deltax, 2., GMP_RNDN);
		}else
		{
			if(flagfirstevaluation == 0)
			{
				CorrectPX = PX;
				flagfirstevaluation = 1;
			}else
			{
				mpfi_put(CorrectPX.x, PX.x);
			}
			GoodXleft = X;
			mpfi_get_left(aux, X.x);
			mpfi_set_fr(X.x, aux);
			mpfr_mul_d(deltax, deltax, 1.01, GMP_RNDN);
			mpfr_sub(aux, aux, deltax, GMP_RNDN);
			mpfi_put_fr(X.x, aux);
		}
	}while(mpfr_cmpabs(deltax, tolstep) > 0);

	printf("I = \n");
	mpfi_put(CorrectX.x, GoodXleft.x);
	mpfi_out_str(stdout, 10, DIGITSPRINT, CorrectX.x);
	printf("\n");
	printf("\n");
	
	printf("P(I) = \n");
	mpfi_out_str(stdout, 10, DIGITSPRINT, CorrectPX.x);
	printf("\n");
	printf("\n");


	delete[] coef;

	return 0;
}

int det_near0(const INTERVAL& x0, INTERVAL& res)
{
	INTERVAL a, b, x, aux;
	int i;

	mpfi_set_d(a.x, 5.);
	mpfi_div_d(a.x, a.x, 3);
	mpfi_put_d(a.x, 1.);
	x = x0;

	b = exp(x*(-1.*a*x));
	mpfi_put(b.x, x.x);

	polynomial_determinantnear0(coef, a, b);

	res = x*coef[39];
	for(i = 38; i > 0; i--)
	{
		aux = res;
		res = x*(aux+coef[i]);
	}
	aux = res;
	res = aux+coef[0];
	
	return 1;
}

void polynomial_determinantnear0(INTERVAL *coef, const INTERVAL& a, const INTERVAL& b)
{
coef[0] = str_to_INTERVAL((char*)"-172916736") + (str_to_INTERVAL((char*)"90832896") + (str_to_INTERVAL((char*)"-15138816") + (str_to_INTERVAL((char*)"-3538944") + str_to_INTERVAL((char*)"98304") * b) * b) * b) * b;
coef[1] = str_to_INTERVAL((char*)"-196608") * a * (str_to_INTERVAL((char*)"-1759") + (str_to_INTERVAL((char*)"924") + (str_to_INTERVAL((char*)"-154") + (-36 + b) * b) * b) * b);
coef[2] = str_to_INTERVAL((char*)"-520388608") + (str_to_INTERVAL((char*)"-268795904") + (str_to_INTERVAL((char*)"224821248") + (str_to_INTERVAL((char*)"-47316992") + (str_to_INTERVAL((char*)"-10420224") + str_to_INTERVAL((char*)"294912") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"204800000") + (str_to_INTERVAL((char*)"-200769536") + (str_to_INTERVAL((char*)"64094208") + (str_to_INTERVAL((char*)"-3506176") - str_to_INTERVAL((char*)"655360") * b) * b) * b) * b + (str_to_INTERVAL((char*)"-308043776") + (str_to_INTERVAL((char*)"162496512") + (str_to_INTERVAL((char*)"-21250048") + (str_to_INTERVAL((char*)"-9732096") + str_to_INTERVAL((char*)"368640") * b) * b) * b) * b) * a) * a;
coef[3] = str_to_INTERVAL((char*)"-8192") * a * (str_to_INTERVAL((char*)"-190572") + (str_to_INTERVAL((char*)"-98436") + (str_to_INTERVAL((char*)"82332") + (str_to_INTERVAL((char*)"-17328") + (str_to_INTERVAL((char*)"-3816") + str_to_INTERVAL((char*)"108") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"50000") + (str_to_INTERVAL((char*)"-49016") + (str_to_INTERVAL((char*)"15648") + (str_to_INTERVAL((char*)"-856") - str_to_INTERVAL((char*)"160") * b) * b) * b) * b + (str_to_INTERVAL((char*)"-16495") + (str_to_INTERVAL((char*)"8748") + (str_to_INTERVAL((char*)"-746") + (str_to_INTERVAL((char*)"-756") + 33 * b) * b) * b) * b) * a) * a);
coef[4] = str_to_INTERVAL((char*)"40960000") + (str_to_INTERVAL((char*)"-866287616") + (str_to_INTERVAL((char*)"-328695808") + (str_to_INTERVAL((char*)"296206336") + (str_to_INTERVAL((char*)"-55930880") + (str_to_INTERVAL((char*)"-13770752") + str_to_INTERVAL((char*)"380928") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-284819456") + (str_to_INTERVAL((char*)"-567001088") + (str_to_INTERVAL((char*)"101629952") + (str_to_INTERVAL((char*)"54509568") + (str_to_INTERVAL((char*)"-29523968") - str_to_INTERVAL((char*)"1294336") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1634082816") + (str_to_INTERVAL((char*)"-802824192") + (str_to_INTERVAL((char*)"595910656") + (str_to_INTERVAL((char*)"-89178112") + (str_to_INTERVAL((char*)"-45219840") + str_to_INTERVAL((char*)"1449984") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"230408192") + (str_to_INTERVAL((char*)"-315252736") + (str_to_INTERVAL((char*)"138412032") + (str_to_INTERVAL((char*)"-22077440") - str_to_INTERVAL((char*)"819200") * b) * b) * b) * b + (str_to_INTERVAL((char*)"-22974464") + (str_to_INTERVAL((char*)"12238848") + (str_to_INTERVAL((char*)"-581632") + (str_to_INTERVAL((char*)"-1327104") + str_to_INTERVAL((char*)"61440") * b) * b) * b) * b) * a) * a) * a) * a;
coef[5] = str_to_INTERVAL((char*)"8192") * a * (str_to_INTERVAL((char*)"-20000") + (str_to_INTERVAL((char*)"422992") + (str_to_INTERVAL((char*)"160496") + (str_to_INTERVAL((char*)"-144632") + (str_to_INTERVAL((char*)"27310") + (str_to_INTERVAL((char*)"6724") - str_to_INTERVAL((char*)"186") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"104304") + (str_to_INTERVAL((char*)"207642") + (str_to_INTERVAL((char*)"-37218") + (str_to_INTERVAL((char*)"-19962") + (str_to_INTERVAL((char*)"10812") + str_to_INTERVAL((char*)"474") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"81326") + (str_to_INTERVAL((char*)"31942") + (str_to_INTERVAL((char*)"-8266") + (str_to_INTERVAL((char*)"-7108") + (str_to_INTERVAL((char*)"4680") - str_to_INTERVAL((char*)"174") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-3126") + (str_to_INTERVAL((char*)"13975") + (str_to_INTERVAL((char*)"-9072") + (str_to_INTERVAL((char*)"2267") + 20 * b) * b) * b) * b) * a) * a) * a);
coef[6] = str_to_INTERVAL((char*)"-4079616") + (str_to_INTERVAL((char*)"73015296") + (str_to_INTERVAL((char*)"-567123968") + (str_to_INTERVAL((char*)"-322068480") + (str_to_INTERVAL((char*)"228466688") + (str_to_INTERVAL((char*)"-32106496") + (str_to_INTERVAL((char*)"-10688512") + str_to_INTERVAL((char*)"276480") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"44498944") + (str_to_INTERVAL((char*)"-399548416") + (str_to_INTERVAL((char*)"-1451081728") + (str_to_INTERVAL((char*)"406953984") + (str_to_INTERVAL((char*)"68415488") + (str_to_INTERVAL((char*)"-55130112") - str_to_INTERVAL((char*)"1220608") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"170016768") + (str_to_INTERVAL((char*)"-5975138304") + (str_to_INTERVAL((char*)"-2147500032") + (str_to_INTERVAL((char*)"1876799488") + (str_to_INTERVAL((char*)"-330368000") + (str_to_INTERVAL((char*)"-99981312") + str_to_INTERVAL((char*)"2903040") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-916914176") + (str_to_INTERVAL((char*)"-1760907264") + (str_to_INTERVAL((char*)"223817728") + (str_to_INTERVAL((char*)"213471232") + (str_to_INTERVAL((char*)"-97296384") - str_to_INTERVAL((char*)"5599232") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-62111744") + (str_to_INTERVAL((char*)"-22040576") + (str_to_INTERVAL((char*)"-100716544") + (str_to_INTERVAL((char*)"77631488") + (str_to_INTERVAL((char*)"-22474752") + str_to_INTERVAL((char*)"933888") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-6397952") + (str_to_INTERVAL((char*)"-16072704") + (str_to_INTERVAL((char*)"14573568") - str_to_INTERVAL((char*)"4423680") * b) * b) * b) * a) * a) * a) * a) * a;
coef[7] = str_to_INTERVAL((char*)"-1024") * a * (str_to_INTERVAL((char*)"-19920") + (str_to_INTERVAL((char*)"356520") + (str_to_INTERVAL((char*)"-2769160") + (str_to_INTERVAL((char*)"-1572600") + (str_to_INTERVAL((char*)"1115560") + (str_to_INTERVAL((char*)"-156770") + (str_to_INTERVAL((char*)"-52190") + str_to_INTERVAL((char*)"1350") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"173824") + (str_to_INTERVAL((char*)"-1560736") + (str_to_INTERVAL((char*)"-5668288") + (str_to_INTERVAL((char*)"1589664") + (str_to_INTERVAL((char*)"267248") + (str_to_INTERVAL((char*)"-215352") - str_to_INTERVAL((char*)"4768") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-61904") + (str_to_INTERVAL((char*)"-5661512") + (str_to_INTERVAL((char*)"-1797616") + (str_to_INTERVAL((char*)"1448740") + (str_to_INTERVAL((char*)"-203195") + (str_to_INTERVAL((char*)"-104642") + str_to_INTERVAL((char*)"3297") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-400128") + (str_to_INTERVAL((char*)"-670712") + (str_to_INTERVAL((char*)"-59096") + (str_to_INTERVAL((char*)"150776") + (str_to_INTERVAL((char*)"-45872") - str_to_INTERVAL((char*)"4616") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"10552") + (str_to_INTERVAL((char*)"-25004") + (str_to_INTERVAL((char*)"-21644") + (str_to_INTERVAL((char*)"24276") + (str_to_INTERVAL((char*)"-8316") + str_to_INTERVAL((char*)"360") * b) * b) * b) * b) * b) * a) * a) * a) * a);
coef[8] = str_to_INTERVAL((char*)"737280") + (str_to_INTERVAL((char*)"-7692288") + (str_to_INTERVAL((char*)"33083392") + (str_to_INTERVAL((char*)"-170180608") + (str_to_INTERVAL((char*)"-184371200") + (str_to_INTERVAL((char*)"102062080") + (str_to_INTERVAL((char*)"-9006080") + (str_to_INTERVAL((char*)"-5326848") + str_to_INTERVAL((char*)"123264") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-4571136") + (str_to_INTERVAL((char*)"24641536") + (str_to_INTERVAL((char*)"-76664832") + (str_to_INTERVAL((char*)"-1195347968") + (str_to_INTERVAL((char*)"213661696") + (str_to_INTERVAL((char*)"107487232") + (str_to_INTERVAL((char*)"-46236672") - str_to_INTERVAL((char*)"860160") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-30281728") + (str_to_INTERVAL((char*)"777955328") + (str_to_INTERVAL((char*)"-6904444928") + (str_to_INTERVAL((char*)"-3928934400") + (str_to_INTERVAL((char*)"2765137920") + (str_to_INTERVAL((char*)"-396671488") + (str_to_INTERVAL((char*)"-125596160") + str_to_INTERVAL((char*)"3279360") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"267550720") + (str_to_INTERVAL((char*)"-2737932288") + (str_to_INTERVAL((char*)"-9453281280") + (str_to_INTERVAL((char*)"2628265984") + (str_to_INTERVAL((char*)"415312896") + (str_to_INTERVAL((char*)"-344663552") - str_to_INTERVAL((char*)"9625600") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-207765504") + (str_to_INTERVAL((char*)"-3316862976") + (str_to_INTERVAL((char*)"-854972416") + (str_to_INTERVAL((char*)"539713536") + (str_to_INTERVAL((char*)"-1093120") + (str_to_INTERVAL((char*)"-76897280") + str_to_INTERVAL((char*)"2671104") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-60268544") + (str_to_INTERVAL((char*)"-48166912") + (str_to_INTERVAL((char*)"-131301376") + (str_to_INTERVAL((char*)"79017984") + (str_to_INTERVAL((char*)"-14565376") - str_to_INTERVAL((char*)"2310144") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-872448") + (str_to_INTERVAL((char*)"-10823680") + (str_to_INTERVAL((char*)"2881536") + (str_to_INTERVAL((char*)"2177024") + (str_to_INTERVAL((char*)"-1419264") + str_to_INTERVAL((char*)"61440") * b) * b) * b) * b) * b) * a) * a) * a) * a) * a) * a;
coef[9] = str_to_INTERVAL((char*)"256") * a * (str_to_INTERVAL((char*)"-17280") + (str_to_INTERVAL((char*)"180288") + (str_to_INTERVAL((char*)"-775392") + (str_to_INTERVAL((char*)"3988608") + (str_to_INTERVAL((char*)"4321200") + (str_to_INTERVAL((char*)"-2392080") + (str_to_INTERVAL((char*)"211080") + (str_to_INTERVAL((char*)"124848") - str_to_INTERVAL((char*)"2889") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"89280") + (str_to_INTERVAL((char*)"-481280") + (str_to_INTERVAL((char*)"1497360") + (str_to_INTERVAL((char*)"23346640") + (str_to_INTERVAL((char*)"-4173080") + (str_to_INTERVAL((char*)"-2099360") + (str_to_INTERVAL((char*)"903060") + str_to_INTERVAL((char*)"16800") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-4928") + (str_to_INTERVAL((char*)"-3599072") + (str_to_INTERVAL((char*)"41422112") + (str_to_INTERVAL((char*)"23647200") + (str_to_INTERVAL((char*)"-16431840") + (str_to_INTERVAL((char*)"2435512") + (str_to_INTERVAL((char*)"709880") - str_to_INTERVAL((char*)"18840") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-701824") + (str_to_INTERVAL((char*)"10234840") + (str_to_INTERVAL((char*)"31424608") + (str_to_INTERVAL((char*)"-8544696") + (str_to_INTERVAL((char*)"-1125476") + (str_to_INTERVAL((char*)"1024098") + str_to_INTERVAL((char*)"46048") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"463808") + (str_to_INTERVAL((char*)"3961280") + (str_to_INTERVAL((char*)"687216") + (str_to_INTERVAL((char*)"42160") + (str_to_INTERVAL((char*)"-326520") + (str_to_INTERVAL((char*)"154176") - str_to_INTERVAL((char*)"5832") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-8544") + (str_to_INTERVAL((char*)"-45848") + (str_to_INTERVAL((char*)"196208") + (str_to_INTERVAL((char*)"-113576") + (str_to_INTERVAL((char*)"22816") + str_to_INTERVAL((char*)"2320") * b) * b) * b) * b) * b) * a) * a) * a) * a) * a);
coef[10] = (str_to_INTERVAL((char*)"-737280") + (str_to_INTERVAL((char*)"2242560") + (str_to_INTERVAL((char*)"-23951360") + (str_to_INTERVAL((char*)"-56235008") + (str_to_INTERVAL((char*)"26416128") + (str_to_INTERVAL((char*)"-974336") + (str_to_INTERVAL((char*)"-1745280") + str_to_INTERVAL((char*)"34560") * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"2580480") + (str_to_INTERVAL((char*)"-23310336") + (str_to_INTERVAL((char*)"36612096") + (str_to_INTERVAL((char*)"-438253568") + (str_to_INTERVAL((char*)"-6982656") + (str_to_INTERVAL((char*)"77704704") + (str_to_INTERVAL((char*)"-20126208") - str_to_INTERVAL((char*)"505344") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"11243520") + (str_to_INTERVAL((char*)"-134559744") + (str_to_INTERVAL((char*)"573899776") + (str_to_INTERVAL((char*)"-3105832960") + (str_to_INTERVAL((char*)"-3601346048") + (str_to_INTERVAL((char*)"1965757952") + (str_to_INTERVAL((char*)"-183920384") + (str_to_INTERVAL((char*)"-95560192") + str_to_INTERVAL((char*)"2173344") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-47001600") + (str_to_INTERVAL((char*)"288473088") + (str_to_INTERVAL((char*)"-941392896") + (str_to_INTERVAL((char*)"-13298355200") + (str_to_INTERVAL((char*)"2363611648") + (str_to_INTERVAL((char*)"1181449216") + (str_to_INTERVAL((char*)"-508826880") - str_to_INTERVAL((char*)"9945600") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"45572096") + (str_to_INTERVAL((char*)"622290944") + (str_to_INTERVAL((char*)"-10781778944") + (str_to_INTERVAL((char*)"-6088233984") + (str_to_INTERVAL((char*)"4153346048") + (str_to_INTERVAL((char*)"-630451200") + (str_to_INTERVAL((char*)"-173417728") + str_to_INTERVAL((char*)"4727040") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"43528192") + (str_to_INTERVAL((char*)"-1481662464") + (str_to_INTERVAL((char*)"-3765586944") + (str_to_INTERVAL((char*)"969172992") + (str_to_INTERVAL((char*)"79419392") + (str_to_INTERVAL((char*)"-93148160") - str_to_INTERVAL((char*)"9598464") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-21495808") + (str_to_INTERVAL((char*)"-112869376") + (str_to_INTERVAL((char*)"-3278848") + (str_to_INTERVAL((char*)"-72364032") + (str_to_INTERVAL((char*)"52717056") + (str_to_INTERVAL((char*)"-15008768") + str_to_INTERVAL((char*)"591360") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-2181120") + (str_to_INTERVAL((char*)"-5902336") + (str_to_INTERVAL((char*)"4999168") + (str_to_INTERVAL((char*)"-1376256") - str_to_INTERVAL((char*)"61440") * b) * b) * b) * b) * b * a) * a) * a) * a) * a) * a) * a;
coef[11] = -32 * a * ((str_to_INTERVAL((char*)"-161280") + (str_to_INTERVAL((char*)"490560") + (str_to_INTERVAL((char*)"-5239360") + (str_to_INTERVAL((char*)"-12301408") + (str_to_INTERVAL((char*)"5778528") + (str_to_INTERVAL((char*)"-213136") + (str_to_INTERVAL((char*)"-381780") + str_to_INTERVAL((char*)"7560") * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"483840") + (str_to_INTERVAL((char*)"-4370688") + (str_to_INTERVAL((char*)"6864768") + (str_to_INTERVAL((char*)"-82172544") + (str_to_INTERVAL((char*)"-1309248") + (str_to_INTERVAL((char*)"14569632") + (str_to_INTERVAL((char*)"-3773664") - str_to_INTERVAL((char*)"94752") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"489600") + (str_to_INTERVAL((char*)"-7803840") + (str_to_INTERVAL((char*)"32809760") + (str_to_INTERVAL((char*)"-192788480") + (str_to_INTERVAL((char*)"-245822320") + (str_to_INTERVAL((char*)"131730480") + (str_to_INTERVAL((char*)"-13258360") + (str_to_INTERVAL((char*)"-5775760") + str_to_INTERVAL((char*)"127725") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1589760") + (str_to_INTERVAL((char*)"12957696") + (str_to_INTERVAL((char*)"-45800832") + (str_to_INTERVAL((char*)"-541655680") + (str_to_INTERVAL((char*)"95143616") + (str_to_INTERVAL((char*)"46911872") + (str_to_INTERVAL((char*)"-20256480") - str_to_INTERVAL((char*)"436800") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1457152") + (str_to_INTERVAL((char*)"5482048") + (str_to_INTERVAL((char*)"-223147744") + (str_to_INTERVAL((char*)"-120007776") + (str_to_INTERVAL((char*)"79215936") + (str_to_INTERVAL((char*)"-11980352") + (str_to_INTERVAL((char*)"-3395608") + str_to_INTERVAL((char*)"97080") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-147712") + (str_to_INTERVAL((char*)"-14406848") + (str_to_INTERVAL((char*)"-27970496") + (str_to_INTERVAL((char*)"5991488") + (str_to_INTERVAL((char*)"-65376") + (str_to_INTERVAL((char*)"-206928") - str_to_INTERVAL((char*)"163936") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"170880") + (str_to_INTERVAL((char*)"-93056") + (str_to_INTERVAL((char*)"-537088") + (str_to_INTERVAL((char*)"422848") + (str_to_INTERVAL((char*)"-123520") + str_to_INTERVAL((char*)"4800") * b) * b) * b) * b) * b) * b * a) * a) * a) * a) * a) * a);
coef[12] = (str_to_INTERVAL((char*)"230400") + (str_to_INTERVAL((char*)"-2055168") + (str_to_INTERVAL((char*)"-9070080") + (str_to_INTERVAL((char*)"3998208") + (str_to_INTERVAL((char*)"-83456") + (str_to_INTERVAL((char*)"-369408") + str_to_INTERVAL((char*)"5952") * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-2304000") + (str_to_INTERVAL((char*)"1397760") + (str_to_INTERVAL((char*)"-76769280") + (str_to_INTERVAL((char*)"-32162304") + (str_to_INTERVAL((char*)"26889216") + (str_to_INTERVAL((char*)"-4361088") - str_to_INTERVAL((char*)"212544") * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-11796480") + (str_to_INTERVAL((char*)"13309440") + (str_to_INTERVAL((char*)"-569735680") + (str_to_INTERVAL((char*)"-1582238976") + (str_to_INTERVAL((char*)"709074688") + (str_to_INTERVAL((char*)"-21892992") + (str_to_INTERVAL((char*)"-44877152") + str_to_INTERVAL((char*)"861312") * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"39352320") + (str_to_INTERVAL((char*)"-386006016") + (str_to_INTERVAL((char*)"595382784") + (str_to_INTERVAL((char*)"-7340780032") + (str_to_INTERVAL((char*)"-176282368") + (str_to_INTERVAL((char*)"1335830400") + (str_to_INTERVAL((char*)"-344478848") - str_to_INTERVAL((char*)"8347904") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"12902400") + (str_to_INTERVAL((char*)"-312336384") + (str_to_INTERVAL((char*)"1332477952") + (str_to_INTERVAL((char*)"-8522429440") + (str_to_INTERVAL((char*)"-11754300672") + (str_to_INTERVAL((char*)"6194725376") + (str_to_INTERVAL((char*)"-663159808") + (str_to_INTERVAL((char*)"-246089216") + str_to_INTERVAL((char*)"5286384") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-30633984") + (str_to_INTERVAL((char*)"384528384") + (str_to_INTERVAL((char*)"-1526166528") + (str_to_INTERVAL((char*)"-14465167872") + (str_to_INTERVAL((char*)"2508989184") + (str_to_INTERVAL((char*)"1187142912") + (str_to_INTERVAL((char*)"-517414144") - str_to_INTERVAL((char*)"13271680") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"19165184") + (str_to_INTERVAL((char*)"-50243584") + (str_to_INTERVAL((char*)"-2942505984") + (str_to_INTERVAL((char*)"-1400360448") + (str_to_INTERVAL((char*)"856926208") + (str_to_INTERVAL((char*)"-118134016") + (str_to_INTERVAL((char*)"-43901312") + str_to_INTERVAL((char*)"1368960") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-2641920") + (str_to_INTERVAL((char*)"-60188672") + (str_to_INTERVAL((char*)"-68664320") + (str_to_INTERVAL((char*)"-6421504") + (str_to_INTERVAL((char*)"-961536") + (str_to_INTERVAL((char*)"4175872") - str_to_INTERVAL((char*)"1868800") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-2181120") + (str_to_INTERVAL((char*)"-449536") + (str_to_INTERVAL((char*)"1233664") + (str_to_INTERVAL((char*)"-514560") + str_to_INTERVAL((char*)"19200") * b) * b) * b) * b) * b * b * a) * a) * a) * a) * a) * a) * a) * a;
coef[13] = 64 * a * ((str_to_INTERVAL((char*)"-28800") + (str_to_INTERVAL((char*)"256896") + (str_to_INTERVAL((char*)"1133760") + (str_to_INTERVAL((char*)"-499776") + (str_to_INTERVAL((char*)"10432") + (str_to_INTERVAL((char*)"46176") - str_to_INTERVAL((char*)"744") * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"252000") + (str_to_INTERVAL((char*)"-152880") + (str_to_INTERVAL((char*)"8396640") + (str_to_INTERVAL((char*)"3517752") + (str_to_INTERVAL((char*)"-2941008") + (str_to_INTERVAL((char*)"476994") + str_to_INTERVAL((char*)"23247") * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"57600") + (str_to_INTERVAL((char*)"1940880") + (str_to_INTERVAL((char*)"19356880") + (str_to_INTERVAL((char*)"68375752") + (str_to_INTERVAL((char*)"-28915320") + (str_to_INTERVAL((char*)"667084") + (str_to_INTERVAL((char*)"1725663") - str_to_INTERVAL((char*)"31608") * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-856800") + (str_to_INTERVAL((char*)"10124400") + (str_to_INTERVAL((char*)"-15050760") + (str_to_INTERVAL((char*)"196874280") + (str_to_INTERVAL((char*)"7771340") + (str_to_INTERVAL((char*)"-37584270") + (str_to_INTERVAL((char*)"9616450") + str_to_INTERVAL((char*)"217900") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-97920") + (str_to_INTERVAL((char*)"4042176") + (str_to_INTERVAL((char*)"-18967840") + (str_to_INTERVAL((char*)"129785152") + (str_to_INTERVAL((char*)"187309632") + (str_to_INTERVAL((char*)"-97230416") + (str_to_INTERVAL((char*)"10959928") + (str_to_INTERVAL((char*)"3546608") - str_to_INTERVAL((char*)"74340") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"153792") + (str_to_INTERVAL((char*)"-3434304") + (str_to_INTERVAL((char*)"16543248") + (str_to_INTERVAL((char*)"122380856") + (str_to_INTERVAL((char*)"-21215284") + (str_to_INTERVAL((char*)"-8820796") + (str_to_INTERVAL((char*)"3976356") + str_to_INTERVAL((char*)"139950") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-45056") + (str_to_INTERVAL((char*)"751456") + (str_to_INTERVAL((char*)"10686928") + (str_to_INTERVAL((char*)"3785184") + (str_to_INTERVAL((char*)"-1749664") + (str_to_INTERVAL((char*)"57144") + (str_to_INTERVAL((char*)"184936") - str_to_INTERVAL((char*)"6600") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-85440") + (str_to_INTERVAL((char*)"140848") + (str_to_INTERVAL((char*)"-38976") + (str_to_INTERVAL((char*)"-8520") + str_to_INTERVAL((char*)"6200") * b) * b) * b) * b) * b * b * a) * a) * a) * a) * a) * a) * a);
coef[14] = (str_to_INTERVAL((char*)"-751872") + (str_to_INTERVAL((char*)"384704") + (str_to_INTERVAL((char*)"-95200") + (str_to_INTERVAL((char*)"-47520") + str_to_INTERVAL((char*)"576") * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"691200") + (str_to_INTERVAL((char*)"-7494144") + (str_to_INTERVAL((char*)"-9251072") + (str_to_INTERVAL((char*)"4303360") + (str_to_INTERVAL((char*)"-287648") - str_to_INTERVAL((char*)"54944") * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"3513600") + (str_to_INTERVAL((char*)"-68315904") + (str_to_INTERVAL((char*)"-343650176") + (str_to_INTERVAL((char*)"129198464") + (str_to_INTERVAL((char*)"4487424") + (str_to_INTERVAL((char*)"-12758272") + str_to_INTERVAL((char*)"200688") * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-45734400") + (str_to_INTERVAL((char*)"-8805120") + (str_to_INTERVAL((char*)"-1776016896") + (str_to_INTERVAL((char*)"-809872000") + (str_to_INTERVAL((char*)"657425664") + (str_to_INTERVAL((char*)"-106467744") - str_to_INTERVAL((char*)"5010416") * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"30458880") + (str_to_INTERVAL((char*)"-472435200") + (str_to_INTERVAL((char*)"-1905135360") + (str_to_INTERVAL((char*)"-8466130432") + (str_to_INTERVAL((char*)"3413700224") + (str_to_INTERVAL((char*)"-53935616") + (str_to_INTERVAL((char*)"-194173248") + str_to_INTERVAL((char*)"3397968") * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"45158400") + (str_to_INTERVAL((char*)"-723228672") + (str_to_INTERVAL((char*)"1038213120") + (str_to_INTERVAL((char*)"-14665457920") + (str_to_INTERVAL((char*)"-870562048") + (str_to_INTERVAL((char*)"2960617856") + (str_to_INTERVAL((char*)"-750816768") - str_to_INTERVAL((char*)"15601760") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1658880") + (str_to_INTERVAL((char*)"-134977536") + (str_to_INTERVAL((char*)"790354944") + (str_to_INTERVAL((char*)"-5690634240") + (str_to_INTERVAL((char*)"-8300877312") + (str_to_INTERVAL((char*)"4241146880") + (str_to_INTERVAL((char*)"-503778304") + (str_to_INTERVAL((char*)"-142707200") + str_to_INTERVAL((char*)"2935968") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1400832") + (str_to_INTERVAL((char*)"68980736") + (str_to_INTERVAL((char*)"-471739392") + (str_to_INTERVAL((char*)"-2641816576") + (str_to_INTERVAL((char*)"474695936") + (str_to_INTERVAL((char*)"131795712") + (str_to_INTERVAL((char*)"-67286016") - str_to_INTERVAL((char*)"4402560") * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-9246720") + (str_to_INTERVAL((char*)"-68517888") + (str_to_INTERVAL((char*)"-4194816") + (str_to_INTERVAL((char*)"-12617216") + (str_to_INTERVAL((char*)"8716288") + (str_to_INTERVAL((char*)"-2606208") + str_to_INTERVAL((char*)"101760") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-1090560") + (str_to_INTERVAL((char*)"647680") + (str_to_INTERVAL((char*)"-92160") - str_to_INTERVAL((char*)"38400") * b) * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a) * a;
coef[15] = -16 * a * ((str_to_INTERVAL((char*)"-422928") + (str_to_INTERVAL((char*)"216396") + (str_to_INTERVAL((char*)"-53550") + (str_to_INTERVAL((char*)"-26730") + str_to_INTERVAL((char*)"324") * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"345600") + (str_to_INTERVAL((char*)"-3747072") + (str_to_INTERVAL((char*)"-4625536") + (str_to_INTERVAL((char*)"2151680") + (str_to_INTERVAL((char*)"-143824") - str_to_INTERVAL((char*)"27472") * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-478800") + (str_to_INTERVAL((char*)"-11905488") + (str_to_INTERVAL((char*)"-70983752") + (str_to_INTERVAL((char*)"21540008") + (str_to_INTERVAL((char*)"2693488") + (str_to_INTERVAL((char*)"-2349424") + str_to_INTERVAL((char*)"35721") * b) * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-4046400") + (str_to_INTERVAL((char*)"-11251680") + (str_to_INTERVAL((char*)"-229381056") + (str_to_INTERVAL((char*)"-120778896") + (str_to_INTERVAL((char*)"93602208") + (str_to_INTERVAL((char*)"-15121716") - str_to_INTERVAL((char*)"670062") * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"3942720") + (str_to_INTERVAL((char*)"-53087040") + (str_to_INTERVAL((char*)"-135345360") + (str_to_INTERVAL((char*)"-724921968") + (str_to_INTERVAL((char*)"281996088") + (str_to_INTERVAL((char*)"-2554616") + (str_to_INTERVAL((char*)"-15603010") + str_to_INTERVAL((char*)"263265") * b) * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"1370880") + (str_to_INTERVAL((char*)"-33976704") + (str_to_INTERVAL((char*)"49359936") + (str_to_INTERVAL((char*)"-749177728") + (str_to_INTERVAL((char*)"-59931808") + (str_to_INTERVAL((char*)"158663888") + (str_to_INTERVAL((char*)"-39930000") - str_to_INTERVAL((char*)"755384") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"11520") + (str_to_INTERVAL((char*)"-2539392") + (str_to_INTERVAL((char*)"21774784") + (str_to_INTERVAL((char*)"-167430016") + (str_to_INTERVAL((char*)"-238455936") + (str_to_INTERVAL((char*)"119519168") + (str_to_INTERVAL((char*)"-15239584") + (str_to_INTERVAL((char*)"-3648224") + str_to_INTERVAL((char*)"74160") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-2304") + (str_to_INTERVAL((char*)"560896") + (str_to_INTERVAL((char*)"-7632192") + (str_to_INTERVAL((char*)"-31125472") + (str_to_INTERVAL((char*)"6364976") + (str_to_INTERVAL((char*)"-49328") + (str_to_INTERVAL((char*)"-268912") - str_to_INTERVAL((char*)"98920") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"170880") + (str_to_INTERVAL((char*)"-282080") + (str_to_INTERVAL((char*)"150320") + (str_to_INTERVAL((char*)"-33920") + str_to_INTERVAL((char*)"1200") * b) * b) * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a);
coef[16] = 8 * b * ((str_to_INTERVAL((char*)"-1800") + (str_to_INTERVAL((char*)"4260") + (str_to_INTERVAL((char*)"-4206") + (str_to_INTERVAL((char*)"-404") + 3 * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-7200") + (str_to_INTERVAL((char*)"-127560") + (str_to_INTERVAL((char*)"23216") + (str_to_INTERVAL((char*)"5038") - str_to_INTERVAL((char*)"958") * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"100800") + (str_to_INTERVAL((char*)"-4673256") + (str_to_INTERVAL((char*)"1412854") + (str_to_INTERVAL((char*)"-190835") + (str_to_INTERVAL((char*)"-255535") + str_to_INTERVAL((char*)"3168") * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"2181600") + (str_to_INTERVAL((char*)"-29221584") + (str_to_INTERVAL((char*)"-38529096") + (str_to_INTERVAL((char*)"17539096") + (str_to_INTERVAL((char*)"-1144513") - str_to_INTERVAL((char*)"223011") * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-5587200") + (str_to_INTERVAL((char*)"-51445440") + (str_to_INTERVAL((char*)"-332441792") + (str_to_INTERVAL((char*)"82206808") + (str_to_INTERVAL((char*)"19402516") + (str_to_INTERVAL((char*)"-10148874") + str_to_INTERVAL((char*)"150039") * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-5011200") + (str_to_INTERVAL((char*)"-70695360") + (str_to_INTERVAL((char*)"-658342944") + (str_to_INTERVAL((char*)"-408961488") + (str_to_INTERVAL((char*)"302108400") + (str_to_INTERVAL((char*)"-48683470") - str_to_INTERVAL((char*)"2011599") * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"7476480") + (str_to_INTERVAL((char*)"-118201920") + (str_to_INTERVAL((char*)"-230678720") + (str_to_INTERVAL((char*)"-1425551792") + (str_to_INTERVAL((char*)"539057504") + (str_to_INTERVAL((char*)"-1415160") + (str_to_INTERVAL((char*)"-29552886") + str_to_INTERVAL((char*)"485571") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"725760") + (str_to_INTERVAL((char*)"-33257088") + (str_to_INTERVAL((char*)"54270912") + (str_to_INTERVAL((char*)"-861496576") + (str_to_INTERVAL((char*)"-84648656") + (str_to_INTERVAL((char*)"187418336") + (str_to_INTERVAL((char*)"-46948952") - str_to_INTERVAL((char*)"807256") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-732672") + (str_to_INTERVAL((char*)"11441664") + (str_to_INTERVAL((char*)"-102733632") + (str_to_INTERVAL((char*)"-137192544") + (str_to_INTERVAL((char*)"66974528") + (str_to_INTERVAL((char*)"-9641408") + (str_to_INTERVAL((char*)"-1724504") + str_to_INTERVAL((char*)"35154") * b) * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"-1733760") + (str_to_INTERVAL((char*)"-4927040") + (str_to_INTERVAL((char*)"1530880") + (str_to_INTERVAL((char*)"-800080") + (str_to_INTERVAL((char*)"248000") - str_to_INTERVAL((char*)"50800") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-34080") + (str_to_INTERVAL((char*)"27056") + (str_to_INTERVAL((char*)"-8064") + str_to_INTERVAL((char*)"240") * b) * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[17] = 8 * b * a * ((str_to_INTERVAL((char*)"18000") + (str_to_INTERVAL((char*)"-42600") + (str_to_INTERVAL((char*)"42060") + (str_to_INTERVAL((char*)"4040") - 30 * b) * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"64800") + (str_to_INTERVAL((char*)"1148040") + (str_to_INTERVAL((char*)"-208944") + (str_to_INTERVAL((char*)"-45342") + str_to_INTERVAL((char*)"8622") * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-806400") + (str_to_INTERVAL((char*)"18213312") + (str_to_INTERVAL((char*)"-1492880") + (str_to_INTERVAL((char*)"-900920") + (str_to_INTERVAL((char*)"832520") - str_to_INTERVAL((char*)"10656") * b) * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-3175200") + (str_to_INTERVAL((char*)"73403568") + (str_to_INTERVAL((char*)"107809912") + (str_to_INTERVAL((char*)"-47464872") + (str_to_INTERVAL((char*)"2977751") + str_to_INTERVAL((char*)"599557") * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"10591200") + (str_to_INTERVAL((char*)"92640096") + (str_to_INTERVAL((char*)"561761840") + (str_to_INTERVAL((char*)"-115119104") + (str_to_INTERVAL((char*)"-42587160") + (str_to_INTERVAL((char*)"16616284") - str_to_INTERVAL((char*)"242304") * b) * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-1080000") + (str_to_INTERVAL((char*)"118046880") + (str_to_INTERVAL((char*)"687354720") + (str_to_INTERVAL((char*)"501245728") + (str_to_INTERVAL((char*)"-355253712") + (str_to_INTERVAL((char*)"57132746") + str_to_INTERVAL((char*)"2205953") * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-3882240") + (str_to_INTERVAL((char*)"86732160") + (str_to_INTERVAL((char*)"145714880") + (str_to_INTERVAL((char*)"1009808192") + (str_to_INTERVAL((char*)"-372448544") + (str_to_INTERVAL((char*)"-1123872") + (str_to_INTERVAL((char*)"20440008") - str_to_INTERVAL((char*)"329088") * b) * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-80640") + (str_to_INTERVAL((char*)"9803904") + (str_to_INTERVAL((char*)"-20948928") + (str_to_INTERVAL((char*)"339629504") + (str_to_INTERVAL((char*)"37933424") + (str_to_INTERVAL((char*)"-73375456") + (str_to_INTERVAL((char*)"18408360") + str_to_INTERVAL((char*)"286432") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"27648") + (str_to_INTERVAL((char*)"-1324800") + (str_to_INTERVAL((char*)"18382464") + (str_to_INTERVAL((char*)"21707840") + (str_to_INTERVAL((char*)"-10197952") + (str_to_INTERVAL((char*)"1860064") + (str_to_INTERVAL((char*)"157104") - str_to_INTERVAL((char*)"3612") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-85440") + (str_to_INTERVAL((char*)"110416") + (str_to_INTERVAL((char*)"-50048") + str_to_INTERVAL((char*)"8560") * b) * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[18] = -2 * b * b * ((str_to_INTERVAL((char*)"-1440") + (str_to_INTERVAL((char*)"2196") + 40 * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"3600") + (str_to_INTERVAL((char*)"6168") + (str_to_INTERVAL((char*)"-1812") + str_to_INTERVAL((char*)"220") * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"369000") + (str_to_INTERVAL((char*)"-447540") + (str_to_INTERVAL((char*)"719526") + (str_to_INTERVAL((char*)"76532") - str_to_INTERVAL((char*)"669") * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"813600") + (str_to_INTERVAL((char*)"22074600") + (str_to_INTERVAL((char*)"-3572592") + (str_to_INTERVAL((char*)"-943494") + str_to_INTERVAL((char*)"164506") * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-10987200") + (str_to_INTERVAL((char*)"203533920") + (str_to_INTERVAL((char*)"17522240") + (str_to_INTERVAL((char*)"-24059900") + (str_to_INTERVAL((char*)"8113170") - str_to_INTERVAL((char*)"108090") * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-4216320") + (str_to_INTERVAL((char*)"532534464") + (str_to_INTERVAL((char*)"871471328") + (str_to_INTERVAL((char*)"-371111080") + (str_to_INTERVAL((char*)"22177224") + str_to_INTERVAL((char*)"4679386") * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"41500800") + (str_to_INTERVAL((char*)"545835456") + (str_to_INTERVAL((char*)"2798552352") + (str_to_INTERVAL((char*)"-477096192") + (str_to_INTERVAL((char*)"-258256624") + (str_to_INTERVAL((char*)"83714272") - str_to_INTERVAL((char*)"1221276") * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-15840000") + (str_to_INTERVAL((char*)"498201600") + (str_to_INTERVAL((char*)"2125506048") + (str_to_INTERVAL((char*)"1790510048") + (str_to_INTERVAL((char*)"-1226939936") + (str_to_INTERVAL((char*)"197087992") + str_to_INTERVAL((char*)"7156148") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-4262400") + (str_to_INTERVAL((char*)"166644480") + (str_to_INTERVAL((char*)"263021440") + (str_to_INTERVAL((char*)"2037228480") + (str_to_INTERVAL((char*)"-732688672") + (str_to_INTERVAL((char*)"-4780896") + (str_to_INTERVAL((char*)"40203680") - str_to_INTERVAL((char*)"633840") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"5944320") + (str_to_INTERVAL((char*)"-20167680") + (str_to_INTERVAL((char*)"345085440") + (str_to_INTERVAL((char*)"40601472") + (str_to_INTERVAL((char*)"-69944096") + (str_to_INTERVAL((char*)"17830912") + str_to_INTERVAL((char*)"246000") * b) * b) * b) * b) * b) * b + ((str_to_INTERVAL((char*)"5779200") + (str_to_INTERVAL((char*)"5440000") + (str_to_INTERVAL((char*)"-2482304") + (str_to_INTERVAL((char*)"756736") + (str_to_INTERVAL((char*)"-49056") + 96 * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"13632") + (str_to_INTERVAL((char*)"-10368") + str_to_INTERVAL((char*)"2880") * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[19] = -2 * b * b * a * ((str_to_INTERVAL((char*)"15840") + (str_to_INTERVAL((char*)"-24156") - str_to_INTERVAL((char*)"440") * b) * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-36000") + (str_to_INTERVAL((char*)"-61680") + (str_to_INTERVAL((char*)"18120") - str_to_INTERVAL((char*)"2200") * b) * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-1269000") + (str_to_INTERVAL((char*)"-828540") + (str_to_INTERVAL((char*)"-1680894") + (str_to_INTERVAL((char*)"-228228") + str_to_INTERVAL((char*)"2601") * b) * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-633600") + (str_to_INTERVAL((char*)"-72507840") + (str_to_INTERVAL((char*)"9636480") + (str_to_INTERVAL((char*)"3436944") - str_to_INTERVAL((char*)"534320") * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"20462400") + (str_to_INTERVAL((char*)"-418223904") + (str_to_INTERVAL((char*)"-89817952") + (str_to_INTERVAL((char*)"71368500") + (str_to_INTERVAL((char*)"-15480430") + str_to_INTERVAL((char*)"216342") * b) * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-14014080") + (str_to_INTERVAL((char*)"-742155456") + (str_to_INTERVAL((char*)"-1306407648") + (str_to_INTERVAL((char*)"541704656") + (str_to_INTERVAL((char*)"-30572228") - str_to_INTERVAL((char*)"6899160") * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-20275200") + (str_to_INTERVAL((char*)"-622723392") + (str_to_INTERVAL((char*)"-2605658208") + (str_to_INTERVAL((char*)"365596192") + (str_to_INTERVAL((char*)"283092992") + (str_to_INTERVAL((char*)"-81179976") + str_to_INTERVAL((char*)"1195584") * b) * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"10828800") + (str_to_INTERVAL((char*)"-349708800") + (str_to_INTERVAL((char*)"-1213036800") + (str_to_INTERVAL((char*)"-1161476096") + (str_to_INTERVAL((char*)"773507072") + (str_to_INTERVAL((char*)"-124241488") - str_to_INTERVAL((char*)"4253384") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"483840") + (str_to_INTERVAL((char*)"-49497600") + (str_to_INTERVAL((char*)"-79335040") + (str_to_INTERVAL((char*)"-707170624") + (str_to_INTERVAL((char*)"247043936") + (str_to_INTERVAL((char*)"1290400") + (str_to_INTERVAL((char*)"-13332480") + str_to_INTERVAL((char*)"203160") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-304128") + (str_to_INTERVAL((char*)"2219520") + (str_to_INTERVAL((char*)"-49529600") + (str_to_INTERVAL((char*)"-5525056") + (str_to_INTERVAL((char*)"8315776") + (str_to_INTERVAL((char*)"-2249056") - str_to_INTERVAL((char*)"25280") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"34176") + (str_to_INTERVAL((char*)"-28032") + str_to_INTERVAL((char*)"7680") * b) * b) *  pow( b,  3) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[20] = 0.- pow( b,  3) * (str_to_INTERVAL((char*)"180") *  pow( b,  9) + ((str_to_INTERVAL((char*)"-1080") + str_to_INTERVAL((char*)"372") * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"-162720") + (str_to_INTERVAL((char*)"260802") + str_to_INTERVAL((char*)"2640") * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"369000") + (str_to_INTERVAL((char*)"832380") + (str_to_INTERVAL((char*)"-224930") + str_to_INTERVAL((char*)"24130") * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"6156000") + (str_to_INTERVAL((char*)"19029120") + (str_to_INTERVAL((char*)"3800896") + (str_to_INTERVAL((char*)"970040") - str_to_INTERVAL((char*)"15495") * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-5189760") + (str_to_INTERVAL((char*)"349368720") + (str_to_INTERVAL((char*)"-36231520") + (str_to_INTERVAL((char*)"-18309212") + str_to_INTERVAL((char*)"2564006") * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-43626240") + (str_to_INTERVAL((char*)"1301817312") + (str_to_INTERVAL((char*)"415825472") + (str_to_INTERVAL((char*)"-277446928") + (str_to_INTERVAL((char*)"47150572") - str_to_INTERVAL((char*)"693162") * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"47571840") + (str_to_INTERVAL((char*)"1638219072") + (str_to_INTERVAL((char*)"2945420704") + (str_to_INTERVAL((char*)"-1199168880") + (str_to_INTERVAL((char*)"63237216") + str_to_INTERVAL((char*)"15653772") * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"3340800") + (str_to_INTERVAL((char*)"1055259648") + (str_to_INTERVAL((char*)"3640926208") + (str_to_INTERVAL((char*)"-409998656") + (str_to_INTERVAL((char*)"-455161680") + (str_to_INTERVAL((char*)"120102336") - str_to_INTERVAL((char*)"1791528") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-6681600") + (str_to_INTERVAL((char*)"325966080") + (str_to_INTERVAL((char*)"998020608") + (str_to_INTERVAL((char*)"1077060672") + (str_to_INTERVAL((char*)"-697481888") + (str_to_INTERVAL((char*)"112213024") + str_to_INTERVAL((char*)"3584840") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"15989760") + (str_to_INTERVAL((char*)"28377600") + (str_to_INTERVAL((char*)"314531456") + (str_to_INTERVAL((char*)"-105830464") + (str_to_INTERVAL((char*)"743200") + (str_to_INTERVAL((char*)"5369520") - str_to_INTERVAL((char*)"75480") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"5779200") + (str_to_INTERVAL((char*)"468864") + (str_to_INTERVAL((char*)"-491072") + (str_to_INTERVAL((char*)"179776") + str_to_INTERVAL((char*)"768") * b) * b) * b) * b) * b * a) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[21] = -2 * a *  pow( b,  3) * (str_to_INTERVAL((char*)"-1080") *  pow( b,  9) + ((str_to_INTERVAL((char*)"5940") - str_to_INTERVAL((char*)"2046") * b) *  pow( b,  8) + ((str_to_INTERVAL((char*)"259200") + (str_to_INTERVAL((char*)"-458550") + str_to_INTERVAL((char*)"2200") * b) * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-634500") + (str_to_INTERVAL((char*)"-1987830") + (str_to_INTERVAL((char*)"495765") - str_to_INTERVAL((char*)"45885") * b) * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"-5162400") + (str_to_INTERVAL((char*)"-35320560") + (str_to_INTERVAL((char*)"1160072") + (str_to_INTERVAL((char*)"-794864") + str_to_INTERVAL((char*)"18528") * b) * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"8688960") + (str_to_INTERVAL((char*)"-318214680") + (str_to_INTERVAL((char*)"24476816") + (str_to_INTERVAL((char*)"18324250") - str_to_INTERVAL((char*)"2359469") * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"11632320") + (str_to_INTERVAL((char*)"-786381696") + (str_to_INTERVAL((char*)"-318256384") + (str_to_INTERVAL((char*)"194752924") + (str_to_INTERVAL((char*)"-28713246") + str_to_INTERVAL((char*)"442656") * b) * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-17236800") + (str_to_INTERVAL((char*)"-718054176") + (str_to_INTERVAL((char*)"-1256348944") + (str_to_INTERVAL((char*)"506012384") + (str_to_INTERVAL((char*)"-24614332") - str_to_INTERVAL((char*)"6859924") * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"3139200") + (str_to_INTERVAL((char*)"-322283520") + (str_to_INTERVAL((char*)"-946006400") + (str_to_INTERVAL((char*)"81617728") + (str_to_INTERVAL((char*)"133609408") + (str_to_INTERVAL((char*)"-33267360") + str_to_INTERVAL((char*)"502080") * b) * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"403200") + (str_to_INTERVAL((char*)"-48134400") + (str_to_INTERVAL((char*)"-140335872") + (str_to_INTERVAL((char*)"-171980000") + (str_to_INTERVAL((char*)"107631920") + (str_to_INTERVAL((char*)"-17410816") - str_to_INTERVAL((char*)"498452") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-506880") + (str_to_INTERVAL((char*)"-1128960") + (str_to_INTERVAL((char*)"-19244736") + (str_to_INTERVAL((char*)"6119648") + (str_to_INTERVAL((char*)"-294912") + (str_to_INTERVAL((char*)"-253552") + str_to_INTERVAL((char*)"2592") * b) * b) * b) * b) * b) * b) * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[22] = 0.-a * a *  pow( b,  4) * (str_to_INTERVAL((char*)"12105") *  pow( b,  8) + ((str_to_INTERVAL((char*)"-61470") + str_to_INTERVAL((char*)"22653") * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-1138500") + (str_to_INTERVAL((char*)"2387706") - str_to_INTERVAL((char*)"62665") * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"3128400") + (str_to_INTERVAL((char*)"13453488") + (str_to_INTERVAL((char*)"-3175052") + str_to_INTERVAL((char*)"260343") * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"11322000") + (str_to_INTERVAL((char*)"167592720") + (str_to_INTERVAL((char*)"-18958328") + (str_to_INTERVAL((char*)"2128804") - str_to_INTERVAL((char*)"71988") * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-25284960") + (str_to_INTERVAL((char*)"893976960") + (str_to_INTERVAL((char*)"-47030368") + (str_to_INTERVAL((char*)"-56276608") + str_to_INTERVAL((char*)"6796028") * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"383040") + (str_to_INTERVAL((char*)"1498530336") + (str_to_INTERVAL((char*)"702187664") + (str_to_INTERVAL((char*)"-409917312") + (str_to_INTERVAL((char*)"55589924") - str_to_INTERVAL((char*)"892854") * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"11923200") + (str_to_INTERVAL((char*)"982762368") + (str_to_INTERVAL((char*)"1619561280") + (str_to_INTERVAL((char*)"-649091392") + (str_to_INTERVAL((char*)"28705760") + str_to_INTERVAL((char*)"9223180") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-2880000") + (str_to_INTERVAL((char*)"274195584") + (str_to_INTERVAL((char*)"712907840") + (str_to_INTERVAL((char*)"-42717632") + (str_to_INTERVAL((char*)"-112109728") + (str_to_INTERVAL((char*)"26759104") - str_to_INTERVAL((char*)"406920") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"16104960") + (str_to_INTERVAL((char*)"48347904") + (str_to_INTERVAL((char*)"70113472") + (str_to_INTERVAL((char*)"-41671616") + (str_to_INTERVAL((char*)"6852896") + str_to_INTERVAL((char*)"154312") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1733760") + (str_to_INTERVAL((char*)"-487424") + (str_to_INTERVAL((char*)"100000") + (str_to_INTERVAL((char*)"4032") + str_to_INTERVAL((char*)"288") * b) * b) * b) * b) * b * a) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[23] =  pow( a,  3) *  pow( b,  4) * (str_to_INTERVAL((char*)"42075") *  pow( b,  8) + ((str_to_INTERVAL((char*)"-198900") + str_to_INTERVAL((char*)"83310") * b) *  pow( b,  7) + ((str_to_INTERVAL((char*)"-1792260") + (str_to_INTERVAL((char*)"4990248") - str_to_INTERVAL((char*)"263025") * b) * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"5565600") + (str_to_INTERVAL((char*)"33451056") + (str_to_INTERVAL((char*)"-7607944") + str_to_INTERVAL((char*)"571104") * b) * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"6894000") + (str_to_INTERVAL((char*)"288547920") + (str_to_INTERVAL((char*)"-42770008") + (str_to_INTERVAL((char*)"2414844") - str_to_INTERVAL((char*)"113088") * b) * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-20894400") + (str_to_INTERVAL((char*)"982180560") + (str_to_INTERVAL((char*)"-30154016") + (str_to_INTERVAL((char*)"-67104844") + str_to_INTERVAL((char*)"7728166") * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"8568000") + (str_to_INTERVAL((char*)"1131038400") + (str_to_INTERVAL((char*)"575126320") + (str_to_INTERVAL((char*)"-327844120") + (str_to_INTERVAL((char*)"42334900") - str_to_INTERVAL((char*)"703260") * b) * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"898560") + (str_to_INTERVAL((char*)"509289984") + (str_to_INTERVAL((char*)"779425792") + (str_to_INTERVAL((char*)"-312052704") + (str_to_INTERVAL((char*)"12344856") + str_to_INTERVAL((char*)"4669568") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-403200") + (str_to_INTERVAL((char*)"76671360") + (str_to_INTERVAL((char*)"184867520") + (str_to_INTERVAL((char*)"-5678592") + (str_to_INTERVAL((char*)"-32074464") + (str_to_INTERVAL((char*)"7406672") - str_to_INTERVAL((char*)"112608") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1140480") + (str_to_INTERVAL((char*)"3845376") + (str_to_INTERVAL((char*)"7732928") + (str_to_INTERVAL((char*)"-4157152") + (str_to_INTERVAL((char*)"721936") + str_to_INTERVAL((char*)"6048") * b) * b) * b) * b) * b) * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[24] =  pow( a,  4) *  pow( b,  5) * (str_to_INTERVAL((char*)"-101520") *  pow( b,  7) + ((str_to_INTERVAL((char*)"450000") - str_to_INTERVAL((char*)"227946") * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"2009160") + (str_to_INTERVAL((char*)"-8823192") + str_to_INTERVAL((char*)"707895") * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-7077600") + (str_to_INTERVAL((char*)"-63277344") + (str_to_INTERVAL((char*)"14028176") - str_to_INTERVAL((char*)"990144") * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-273600") + (str_to_INTERVAL((char*)"-378466560") + (str_to_INTERVAL((char*)"62414504") + (str_to_INTERVAL((char*)"-2343638") + str_to_INTERVAL((char*)"142746") * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"9627840") + (str_to_INTERVAL((char*)"-851416320") + (str_to_INTERVAL((char*)"9949120") + (str_to_INTERVAL((char*)"62405248") - str_to_INTERVAL((char*)"6947900") * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-5040000") + (str_to_INTERVAL((char*)"-670389408") + (str_to_INTERVAL((char*)"-352327328") + (str_to_INTERVAL((char*)"199187320") + (str_to_INTERVAL((char*)"-25002796") + str_to_INTERVAL((char*)"426726") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"570240") + (str_to_INTERVAL((char*)"-191491776") + (str_to_INTERVAL((char*)"-272676128") + (str_to_INTERVAL((char*)"109237072") + (str_to_INTERVAL((char*)"-3790360") - str_to_INTERVAL((char*)"1723100") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-12586752") + (str_to_INTERVAL((char*)"-29611008") + (str_to_INTERVAL((char*)"-258272") + (str_to_INTERVAL((char*)"5683152") + (str_to_INTERVAL((char*)"-1275536") + str_to_INTERVAL((char*)"19056") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-288960") + (str_to_INTERVAL((char*)"109216") + (str_to_INTERVAL((char*)"-25088") + str_to_INTERVAL((char*)"1056") * b) * b) * b) * b * a) * a) * a) * a) * a) * a) * a) * a) * a);
coef[25] = 0.- pow( a,  5) *  pow( b,  5) * (str_to_INTERVAL((char*)"-180135") *  pow( b,  7) + ((str_to_INTERVAL((char*)"751410") - str_to_INTERVAL((char*)"493533") * b) *  pow( b,  6) + ((str_to_INTERVAL((char*)"1495440") + (str_to_INTERVAL((char*)"-13543464") + str_to_INTERVAL((char*)"1406700") * b) * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-6307200") + (str_to_INTERVAL((char*)"-93259728") + (str_to_INTERVAL((char*)"20308552") - str_to_INTERVAL((char*)"1374708") * b) * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"3272400") + (str_to_INTERVAL((char*)"-386949600") + (str_to_INTERVAL((char*)"66922624") + (str_to_INTERVAL((char*)"-1926120") + str_to_INTERVAL((char*)"144180") * b) * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"1712160") + (str_to_INTERVAL((char*)"-583318320") + (str_to_INTERVAL((char*)"-2118080") + (str_to_INTERVAL((char*)"45153172") - str_to_INTERVAL((char*)"4913286") * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-1128960") + (str_to_INTERVAL((char*)"-305077632") + (str_to_INTERVAL((char*)"-160175008") + (str_to_INTERVAL((char*)"90753608") + (str_to_INTERVAL((char*)"-11210036") + str_to_INTERVAL((char*)"195480") * b) * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"120960") + (str_to_INTERVAL((char*)"-48957888") + (str_to_INTERVAL((char*)"-65811744") + (str_to_INTERVAL((char*)"26357216") + (str_to_INTERVAL((char*)"-780240") - str_to_INTERVAL((char*)"437536") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-912384") + (str_to_INTERVAL((char*)"-2212224") + (str_to_INTERVAL((char*)"-204160") + (str_to_INTERVAL((char*)"493920") + (str_to_INTERVAL((char*)"-107968") + str_to_INTERVAL((char*)"1488") * b) * b) * b) * b) * b) * a) * a) * a) * a) * a) * a) * a) * a);
coef[26] = 0.- pow( a,  6) *  pow( b,  6) * (str_to_INTERVAL((char*)"242595") *  pow( b,  6) + ((str_to_INTERVAL((char*)"-949860") + str_to_INTERVAL((char*)"876528") * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"-538200") + (str_to_INTERVAL((char*)"18093372") - str_to_INTERVAL((char*)"2182410") * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"3776400") + (str_to_INTERVAL((char*)"108638640") + (str_to_INTERVAL((char*)"-23372420") + str_to_INTERVAL((char*)"1539130") * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-2775600") + (str_to_INTERVAL((char*)"311665080") + (str_to_INTERVAL((char*)"-55045164") + (str_to_INTERVAL((char*)"1317682") - str_to_INTERVAL((char*)"116124") * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"442080") + (str_to_INTERVAL((char*)"313482720") + (str_to_INTERVAL((char*)"4590176") + (str_to_INTERVAL((char*)"-25182956") + str_to_INTERVAL((char*)"2701256") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"20160") + (str_to_INTERVAL((char*)"102540192") + (str_to_INTERVAL((char*)"52731600") + (str_to_INTERVAL((char*)"-30131528") + (str_to_INTERVAL((char*)"3688296") - str_to_INTERVAL((char*)"65430") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"7572096") + (str_to_INTERVAL((char*)"9844224") + (str_to_INTERVAL((char*)"-3926768") + (str_to_INTERVAL((char*)"94832") + str_to_INTERVAL((char*)"68384") * b) * b) * b) * b + (str_to_INTERVAL((char*)"20640") + (str_to_INTERVAL((char*)"-6720") + str_to_INTERVAL((char*)"1440") * b) * b) * b * a) * a) * a) * a) * a) * a) * a) * a);
coef[27] = 2 *  pow( a,  7) *  pow( b,  6) * (str_to_INTERVAL((char*)"125955") *  pow( b,  6) + ((str_to_INTERVAL((char*)"-457920") + str_to_INTERVAL((char*)"651132") * b) *  pow( b,  5) + ((str_to_INTERVAL((char*)"108900") + (str_to_INTERVAL((char*)"10394844") - str_to_INTERVAL((char*)"1357415") * b) * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"682200") + (str_to_INTERVAL((char*)"50388564") + (str_to_INTERVAL((char*)"-10758506") + str_to_INTERVAL((char*)"696304") * b) * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-540000") + (str_to_INTERVAL((char*)"99049260") + (str_to_INTERVAL((char*)"-17633394") + (str_to_INTERVAL((char*)"369442") - str_to_INTERVAL((char*)"37074") * b) * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"126720") + (str_to_INTERVAL((char*)"64810200") + (str_to_INTERVAL((char*)"1383472") + (str_to_INTERVAL((char*)"-5315506") + str_to_INTERVAL((char*)"565725") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-10080") + (str_to_INTERVAL((char*)"11942208") + (str_to_INTERVAL((char*)"5976392") + (str_to_INTERVAL((char*)"-3451776") + (str_to_INTERVAL((char*)"420176") - str_to_INTERVAL((char*)"7560") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"266112") + (str_to_INTERVAL((char*)"345120") + (str_to_INTERVAL((char*)"-136272") + (str_to_INTERVAL((char*)"2480") + str_to_INTERVAL((char*)"2480") * b) * b) * b) * b) * a) * a) * a) * a) * a) * a) * a);
coef[28] =  pow( a,  8) *  pow( b,  7) * (str_to_INTERVAL((char*)"-202500") *  pow( b,  5) + ((str_to_INTERVAL((char*)"669600") - str_to_INTERVAL((char*)"1632195") * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-429120") + (str_to_INTERVAL((char*)"-20209938") + str_to_INTERVAL((char*)"2741390") * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"-185400") + (str_to_INTERVAL((char*)"-74590620") + (str_to_INTERVAL((char*)"15859670") - str_to_INTERVAL((char*)"1016054") * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"180000") + (str_to_INTERVAL((char*)"-98657760") + (str_to_INTERVAL((char*)"17592424") + (str_to_INTERVAL((char*)"-334882") + str_to_INTERVAL((char*)"37119") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"-28800") + (str_to_INTERVAL((char*)"-39804480") + (str_to_INTERVAL((char*)"-972288") + (str_to_INTERVAL((char*)"3290716") - str_to_INTERVAL((char*)"349110") * b) * b) * b) * b + (str_to_INTERVAL((char*)"-3430080") + (str_to_INTERVAL((char*)"-1679280") + (str_to_INTERVAL((char*)"978480") + (str_to_INTERVAL((char*)"-118560") + str_to_INTERVAL((char*)"2160") * b) * b) * b) * b) * a) * a) * a) * a) * a) * a);
coef[29] = 0.- pow( a,  9) *  pow( b,  7) * (str_to_INTERVAL((char*)"-125190") *  pow( b,  5) + ((str_to_INTERVAL((char*)"364500") - str_to_INTERVAL((char*)"1727139") * b) *  pow( b,  4) + ((str_to_INTERVAL((char*)"-281520") + (str_to_INTERVAL((char*)"-16362732") + str_to_INTERVAL((char*)"2255460") * b) * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"59400") + (str_to_INTERVAL((char*)"-43897524") + (str_to_INTERVAL((char*)"9316446") - str_to_INTERVAL((char*)"593634") * b) * b) * b) * b * b + ((str_to_INTERVAL((char*)"-3600") + (str_to_INTERVAL((char*)"-37806720") + (str_to_INTERVAL((char*)"6735248") + (str_to_INTERVAL((char*)"-120320") + str_to_INTERVAL((char*)"14280") * b) * b) * b) * b) * b + (str_to_INTERVAL((char*)"1440") + (str_to_INTERVAL((char*)"-8539440") + (str_to_INTERVAL((char*)"-214080") + (str_to_INTERVAL((char*)"706080") - str_to_INTERVAL((char*)"74880") * b) * b) * b) * b + (str_to_INTERVAL((char*)"-228096") + (str_to_INTERVAL((char*)"-110880") + (str_to_INTERVAL((char*)"64800") + (str_to_INTERVAL((char*)"-7808") + str_to_INTERVAL((char*)"144") * b) * b) * b) * b) * a) * a) * a) * a) * a) * a);
coef[30] = 0.- pow( a,  10) *  pow( b,  8) * (str_to_INTERVAL((char*)"58455") *  pow( b,  4) + ((str_to_INTERVAL((char*)"-142830") + str_to_INTERVAL((char*)"1536414") * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"102780") + (str_to_INTERVAL((char*)"10879746") - str_to_INTERVAL((char*)"1508285") * b) * b) * b * b + ((str_to_INTERVAL((char*)"-28800") + (str_to_INTERVAL((char*)"20333448") + (str_to_INTERVAL((char*)"-4313072") + str_to_INTERVAL((char*)"274143") * b) * b) * b) * b + (str_to_INTERVAL((char*)"3600") + (str_to_INTERVAL((char*)"10778880") + (str_to_INTERVAL((char*)"-1917640") + (str_to_INTERVAL((char*)"33090") - str_to_INTERVAL((char*)"4080") * b) * b) * b) * b + (str_to_INTERVAL((char*)"1140480") + (str_to_INTERVAL((char*)"28160") + (str_to_INTERVAL((char*)"-94080") + str_to_INTERVAL((char*)"9984") * b) * b) * b) * a) * a) * a) * a) * a);
coef[31] =  pow( a,  11) *  pow( b,  8) * (str_to_INTERVAL((char*)"19935") *  pow( b,  4) + ((str_to_INTERVAL((char*)"-37980") + str_to_INTERVAL((char*)"1140219") * b) *  pow( b,  3) + ((str_to_INTERVAL((char*)"20700") + (str_to_INTERVAL((char*)"5857992") - str_to_INTERVAL((char*)"813385") * b) * b) * b * b + ((str_to_INTERVAL((char*)"-3600") + (str_to_INTERVAL((char*)"7269480") + (str_to_INTERVAL((char*)"-1541940") + str_to_INTERVAL((char*)"97920") * b) * b) * b) * b + ((str_to_INTERVAL((char*)"2154240") + (str_to_INTERVAL((char*)"-382976") + (str_to_INTERVAL((char*)"6528") - str_to_INTERVAL((char*)"816") * b) * b) * b) * b + (str_to_INTERVAL((char*)"71280") + (str_to_INTERVAL((char*)"1760") + (str_to_INTERVAL((char*)"-5880") + str_to_INTERVAL((char*)"624") * b) * b) * b) * a) * a) * a) * a) * a);
coef[32] =  pow( a,  12) *  pow( b,  9) * (str_to_INTERVAL((char*)"-4680") *  pow( b,  3) + ((str_to_INTERVAL((char*)"6120") - str_to_INTERVAL((char*)"698712") * b) * b * b + ((str_to_INTERVAL((char*)"-1800") + (str_to_INTERVAL((char*)"-2511348") + str_to_INTERVAL((char*)"348795") * b) * b) * b + ((str_to_INTERVAL((char*)"-1938816") + (str_to_INTERVAL((char*)"411264") - str_to_INTERVAL((char*)"26112") * b) * b) * b + (str_to_INTERVAL((char*)"-269280") + (str_to_INTERVAL((char*)"47872") + (str_to_INTERVAL((char*)"-816") + str_to_INTERVAL((char*)"102") * b) * b) * b) * a) * a) * a) * a);
coef[33] = 0.- pow( a,  13) *  pow( b,  9) * (str_to_INTERVAL((char*)"-675") *  pow( b,  3) + ((str_to_INTERVAL((char*)"450") - str_to_INTERVAL((char*)"348915") * b) * b * b + ((str_to_INTERVAL((char*)"-837216") + str_to_INTERVAL((char*)"116280") * b) * b * b + ((str_to_INTERVAL((char*)"-363528") + (str_to_INTERVAL((char*)"77112") - str_to_INTERVAL((char*)"4896") * b) * b) * b + (str_to_INTERVAL((char*)"-15840") + (str_to_INTERVAL((char*)"2816") + (-48 + 6 * b) * b) * b) * a) * a) * a) * a);
coef[34] = 0.-9 *  pow( a,  14) *  pow( b,  10) * (5 * b * b + (str_to_INTERVAL((char*)"15504") * b * b + ((str_to_INTERVAL((char*)"23256") - str_to_INTERVAL((char*)"3230") * b) * b + (str_to_INTERVAL((char*)"4752") + (str_to_INTERVAL((char*)"-1008") + 64 * b) * b) * a) * a) * a);
coef[35] =  pow( a,  16) *  pow( b,  10) * (str_to_INTERVAL((char*)"43605") * b * b + ((str_to_INTERVAL((char*)"36936") - str_to_INTERVAL((char*)"5130") * b) * b + (str_to_INTERVAL((char*)"2376") + (str_to_INTERVAL((char*)"-504") + 32 * b) * b) * a) * a);
coef[36] = str_to_INTERVAL((char*)"114") *  pow( a,  17) *  pow( b,  11) * (-90 * b + (5 * b - 36) * a);
coef[37] = 0.-6 *  pow( a,  18) *  pow( b,  11) * (str_to_INTERVAL((char*)"-285") * b + (5 * b - 36) * a);
coef[38] = str_to_INTERVAL((char*)"-180") *  pow( a,  19) *  pow( b,  12);
coef[39] = 9 *  pow( a,  20) *  pow( b,  12);
	return;
}
